@@ -33,6 +33,10 @@
 #include <assert.h>
 #include <unistd.h>
 #include <sys/stat.h>
+#include <sys/types.h>
+#ifndef HAVE_W32_SYSTEM
+#include <sys/wait.h>
+#endif
 #ifdef USE_GNU_PTH
 # include <pth.h>
 #endif
@@ -48,11 +52,11 @@
 
 static ASSUAN_CONTEXT scd_ctx = NULL;
 #ifdef USE_GNU_PTH
-static pth_mutex_t scd_lock = PTH_MUTEX_INIT;
+static pth_mutex_t scd_lock;
 #endif
 /* We need to keep track of the connection currently using the SCD.
-   For a pipe server this is all a NOP becuase the connection will
-   always have the conenction indicator -1.  agent_reset_scd releases
+   For a pipe server this is all a NOP because the connection will
+   always have the connection indicator -1.  agent_reset_scd releases
    the active connection; i.e. sets it back to -1, so that a new
    connection can start using the SCD.  If we eventually allow
    multiple SCD session we will either make scdaemon multi-threaded or
@@ -78,77 +82,25 @@ struct inq_needpin_s {
   void *getpin_cb_arg;
 };
 
-struct membuf {
-  size_t len;
-  size_t size;
-  char *buf;
-  int out_of_core;
-};
-
 
 
-/* A simple implementation of a dynamic buffer.  Use init_membuf() to
-   create a buffer, put_membuf to append bytes and get_membuf to
-   release and return the buffer.  Allocation errors are detected but
-   only returned at the final get_membuf(), this helps not to clutter
-   the code with out of core checks.  */
-
-static void
-init_membuf (struct membuf *mb, int initiallen)
+/* This function must be called once to initialize this module.  This
+   has to be done before a second thread is spawned.  We can't do the
+   static initialization because Pth emulation code might not be able
+   to do a static init; in particualr, it is not possible for W32. */
+void
+initialize_module_call_scd (void)
 {
-  mb->len = 0;
-  mb->size = initiallen;
-  mb->out_of_core = 0;
-  mb->buf = xtrymalloc (initiallen);
-  if (!mb->buf)
-      mb->out_of_core = 1;
-}
+#ifdef USE_GNU_PTH
+  static int initialized;
 
-static void
-put_membuf (struct membuf *mb, const void *buf, size_t len)
-{
-  if (mb->out_of_core)
-    return;
-
-  if (mb->len + len >= mb->size)
-    {
-      char *p;
-      
-      mb->size += len + 1024;
-      p = xtryrealloc (mb->buf, mb->size);
-      if (!p)
-        {
-          mb->out_of_core = 1;
-          return;
-        }
-      mb->buf = p;
-    }
-  memcpy (mb->buf + mb->len, buf, len);
-  mb->len += len;
-}
-
-static void *
-get_membuf (struct membuf *mb, size_t *len)
-{
-  char *p;
-
-  if (mb->out_of_core)
-    {
-      xfree (mb->buf);
-      mb->buf = NULL;
-      return NULL;
-    }
-
-  p = mb->buf;
-  *len = mb->len;
-  mb->buf = NULL;
-  mb->out_of_core = 1; /* don't allow a reuse */
-  return p;
+  if (!initialized)
+    if (pth_mutex_init (&scd_lock))
+      initialized = 1;
+#endif /*USE_GNU_PTH*/
 }
 
 
-
-
 static int 
 unlock_scd (int rc)
 {
@@ -159,7 +111,7 @@ unlock_scd (int rc)
       if (!rc)
         rc = gpg_error (GPG_ERR_INTERNAL);
     }
-#endif
+#endif /*USE_GNU_PTH*/
   return rc;
 }
 
@@ -195,6 +147,8 @@ start_scd (ctrl_t ctrl)
 
   if (scd_ctx)
     {
+      pid_t pid;
+
       /* If we are not the connection currently using the SCD, return
          an error. */
       if (!active_connection)
@@ -205,8 +159,21 @@ start_scd (ctrl_t ctrl)
       else if (ctrl->connection_fd != active_connection_fd)
         return unlock_scd (gpg_error (GPG_ERR_CONFLICT));
       
-      /* Okay, we scdaemon already started and used by us. */
-      return 0; 
+      /* Okay, we already started the scdaemon and it is used by us.*/
+
+      /* We better do a sanity check now to see whether it has
+         accidently died. */
+#ifndef HAVE_W32_SYSTEM 
+      pid = assuan_get_pid (scd_ctx);
+      if (pid != (pid_t)(-1) && pid
+          && ((rc=waitpid (pid, NULL, WNOHANG))==-1 || (rc == pid)) )
+        {
+          assuan_disconnect (scd_ctx);
+          scd_ctx = NULL;
+        }
+      else
+#endif
+        return 0; 
     }
 
   if (opt.verbose)
@@ -255,6 +222,19 @@ start_scd (ctrl_t ctrl)
   if (DBG_ASSUAN)
     log_debug ("connection to SCdaemon established\n");
 
+  /* Tell the scdaemon that we want him to send us an event signal.
+     But only do this if we are running as a regular sever and not
+     simply as a pipe server. */
+  if (ctrl->connection_fd != -1)
+  {
+#ifndef HAVE_W32_SYSTEM
+    char buf[100];
+
+    sprintf (buf, "OPTION event-signal=%d", SIGUSR2);
+    assuan_transact (scd_ctx, buf, NULL, NULL, NULL, NULL, NULL, NULL);
+#endif
+  }
+
   return 0;
 }
 
@@ -419,7 +399,7 @@ agent_card_serialno (ctrl_t ctrl, char **r_serialno)
 static AssuanError
 membuf_data_cb (void *opaque, const void *buffer, size_t length)
 {
-  struct membuf *data = opaque;
+  membuf_t *data = opaque;
 
   if (buffer)
     put_membuf (data, buffer, length);
@@ -470,7 +450,7 @@ agent_card_pksign (ctrl_t ctrl,
 {
   int rc, i;
   char *p, line[ASSUAN_LINELENGTH];
-  struct membuf data;
+  membuf_t data;
   struct inq_needpin_s inqparm;
   size_t len;
   unsigned char *sigbuf;
@@ -542,7 +522,7 @@ agent_card_pkdecrypt (ctrl_t ctrl,
 {
   int rc, i;
   char *p, line[ASSUAN_LINELENGTH];
-  struct membuf data;
+  membuf_t data;
   struct inq_needpin_s inqparm;
   size_t len;
 
@@ -594,7 +574,7 @@ agent_card_readcert (ctrl_t ctrl,
 {
   int rc;
   char line[ASSUAN_LINELENGTH];
-  struct membuf data;
+  membuf_t data;
   size_t len;
 
   *r_buf = NULL;
@@ -630,7 +610,7 @@ agent_card_readkey (ctrl_t ctrl, const char *id, unsigned char **r_buf)
 {
   int rc;
   char line[ASSUAN_LINELENGTH];
-  struct membuf data;
+  membuf_t data;
   size_t len, buflen;
 
   *r_buf = NULL;
