@@ -50,7 +50,7 @@ struct server_local_s {
   ASSUAN_CONTEXT assuan_ctx;
   int message_fd;
   int use_cache_for_signing;
-  char *keydesc;  /* Allocated description fro the next key
+  char *keydesc;  /* Allocated description for the next key
                      operation. */
 };
 
@@ -58,6 +58,41 @@ struct server_local_s {
 
 
 
+/* Release the memory buffer MB but first wipe out the used memory. */
+static void
+clear_outbuf (membuf_t *mb)
+{
+  void *p;
+  size_t n;
+
+  p = get_membuf (mb, &n);
+  if (p)
+    {
+      memset (p, 0, n);
+      xfree (p);
+    }
+}
+
+
+/* Write the content of memory buffer MB as assuan data to CTX and
+   wipe the buffer out afterwards. */
+static gpg_error_t
+write_and_clear_outbuf (assuan_context_t ctx, membuf_t *mb)
+{
+  assuan_error_t ae;
+  void *p;
+  size_t n;
+
+  p = get_membuf (mb, &n);
+  if (!p)
+    return gpg_error (GPG_ERR_ENOMEM);
+  ae = assuan_send_data (ctx, p, n);
+  memset (p, 0, n);
+  xfree (p);
+  return map_assuan_err (ae);
+}
+
+
 static void
 reset_notify (ASSUAN_CONTEXT ctx)
 {
@@ -106,7 +141,7 @@ parse_hexstring (ASSUAN_CONTEXT ctx, const char *string, size_t *len)
   /* parse the hash value */
   for (p=string, n=0; hexdigitp (p); p++, n++)
     ;
-  if (*p)
+  if (*p != ' ' && *p != '\t' && *p)
     return set_error (Parameter_Error, "invalid hexstring");
   if ((n&1))
     return set_error (Parameter_Error, "odd number of digits");
@@ -369,14 +404,21 @@ cmd_pksign (ASSUAN_CONTEXT ctx, char *line)
   int rc;
   int ignore_cache = 0;
   ctrl_t ctrl = assuan_get_pointer (ctx);
+  membuf_t outbuf;
 
   if (opt.ignore_cache_for_signing)
     ignore_cache = 1;
   else if (!ctrl->server_local->use_cache_for_signing)
     ignore_cache = 1;
 
+  init_membuf (&outbuf, 512);
+
   rc = agent_pksign (ctrl, ctrl->server_local->keydesc,
-                     assuan_get_data_fp (ctx), ignore_cache);
+                     &outbuf, ignore_cache);
+  if (rc)
+    clear_outbuf (&outbuf);
+  else
+    rc = write_and_clear_outbuf (ctx, &outbuf);
   if (rc)
     log_error ("command pksign failed: %s\n", gpg_strerror (rc));
   xfree (ctrl->server_local->keydesc);
@@ -395,6 +437,7 @@ cmd_pkdecrypt (ASSUAN_CONTEXT ctx, char *line)
   ctrl_t ctrl = assuan_get_pointer (ctx);
   unsigned char *value;
   size_t valuelen;
+  membuf_t outbuf;
 
   /* First inquire the data to decrypt */
   rc = assuan_inquire (ctx, "CIPHERTEXT",
@@ -402,9 +445,15 @@ cmd_pkdecrypt (ASSUAN_CONTEXT ctx, char *line)
   if (rc)
     return rc;
 
+  init_membuf (&outbuf, 512);
+
   rc = agent_pkdecrypt (ctrl, ctrl->server_local->keydesc,
-                        value, valuelen, assuan_get_data_fp (ctx));
+                        value, valuelen, &outbuf);
   xfree (value);
+  if (rc)
+    clear_outbuf (&outbuf);
+  else
+    rc = write_and_clear_outbuf (ctx, &outbuf);
   if (rc)
     log_error ("command pkdecrypt failed: %s\n", gpg_strerror (rc));
   xfree (ctrl->server_local->keydesc);
@@ -434,14 +483,21 @@ cmd_genkey (ASSUAN_CONTEXT ctx, char *line)
   int rc;
   unsigned char *value;
   size_t valuelen;
+  membuf_t outbuf;
 
   /* First inquire the parameters */
   rc = assuan_inquire (ctx, "KEYPARAM", &value, &valuelen, MAXLEN_KEYPARAM);
   if (rc)
     return rc;
 
-  rc = agent_genkey (ctrl, value, valuelen, assuan_get_data_fp (ctx));
+  init_membuf (&outbuf, 512);
+
+  rc = agent_genkey (ctrl, value, valuelen, &outbuf);
   xfree (value);
+  if (rc)
+    clear_outbuf (&outbuf);
+  else
+    rc = write_and_clear_outbuf (ctx, &outbuf);
   if (rc)
     log_error ("command genkey failed: %s\n", gpg_strerror (rc));
   return map_to_assuan_status (rc);
@@ -685,6 +741,64 @@ cmd_passwd (ASSUAN_CONTEXT ctx, char *line)
   return map_to_assuan_status (rc);
 }
 
+/* PRESET_PASSPHRASE <hexstring_with_keygrip> <timeout> <passwd>
+  
+   Set the cached passphrase/PIN for the key identified by the keygrip
+   to passwd for the given time, where -1 means infinite and 0 means
+   the default (currently only a timeout of -1 is allowed, which means
+   to never expire it).  If passwd is not provided, ask for it via the
+   pinentry module.  */
+static int
+cmd_preset_passphrase (ASSUAN_CONTEXT ctx, char *line)
+{
+  int rc;
+  unsigned char grip[20];
+  char *grip_clear = NULL;
+  char *passphrase = NULL;
+  int ttl;
+
+  if (!opt.allow_preset_passphrase)
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
+  rc = parse_keygrip (ctx, line, grip);
+  if (rc)
+    return rc;
+
+  /* FIXME: parse_keygrip should return a tail pointer.  */
+  grip_clear = line;
+  while (*line && (*line != ' ' && *line != '\t'))
+    line++;
+  if (!*line)
+    return map_to_assuan_status (gpg_error (GPG_ERR_MISSING_VALUE));
+  *line = '\0';
+  line++;
+  while (*line && (*line == ' ' || *line == '\t'))
+    line++;
+  
+  /* Currently, only infinite timeouts are allowed.  */
+  ttl = -1;
+  if (line[0] != '-' || line[1] != '1')
+    return map_to_assuan_status (gpg_error (GPG_ERR_NOT_IMPLEMENTED));
+  line++;
+  line++;
+  while (!(*line != ' ' && *line != '\t'))
+    line++;
+
+  /* If there is a passphrase, use it.  Currently, a passphrase is
+     required.  */
+  if (*line)
+    passphrase = line;
+  else
+    return map_to_assuan_status (gpg_error (GPG_ERR_NOT_IMPLEMENTED));
+
+  rc = agent_put_cache (grip_clear, passphrase, ttl);
+
+  if (rc)
+    log_error ("command preset_passwd failed: %s\n", gpg_strerror (rc));
+
+  return map_to_assuan_status (rc);
+}
+
 
 /* SCD <commands to pass to the scdaemon>
   
@@ -781,6 +895,7 @@ register_commands (ASSUAN_CONTEXT ctx)
     { "PKDECRYPT",      cmd_pkdecrypt },
     { "GENKEY",         cmd_genkey },
     { "GET_PASSPHRASE", cmd_get_passphrase },
+    { "PRESET_PASSPHRASE", cmd_preset_passphrase },
     { "CLEAR_PASSPHRASE", cmd_clear_passphrase },
     { "GET_CONFIRMATION", cmd_get_confirmation },
     { "LISTTRUSTED",    cmd_listtrusted },
@@ -854,6 +969,7 @@ start_command_handler (int listen_fd, int fd)
   ctrl.server_local->assuan_ctx = ctx;
   ctrl.server_local->message_fd = -1;
   ctrl.server_local->use_cache_for_signing = 1;
+  ctrl.digest.raw_value = 0;
 
   if (DBG_ASSUAN)
     assuan_set_log_stream (ctx, log_get_stream ());
