@@ -29,8 +29,11 @@
 #include <assert.h>
 #include <time.h>
 #include <fcntl.h>
+#include <sys/stat.h>
+#ifndef HAVE_W32_SYSTEM
 #include <sys/socket.h>
 #include <sys/un.h>
+#endif /*HAVE_W32_SYSTEM*/
 #include <unistd.h>
 #include <signal.h>
 #ifdef USE_GNU_PTH
@@ -39,11 +42,13 @@
 
 #define JNLIB_NEED_LOG_LOGV
 #include "agent.h"
-#include <assuan.h> /* malloc hooks */
+#include <assuan.h> /* Malloc hooks */
 
 #include "i18n.h"
 #include "sysutils.h"
-
+#ifdef HAVE_W32_SYSTEM
+#include "../jnlib/w32-afunix.h"
+#endif
 
 enum cmd_and_opt_values 
 { aNull = 0,
@@ -76,14 +81,18 @@ enum cmd_and_opt_values
   oLCctype,
   oLCmessages,
   oScdaemonProgram,
-  oDefCacheTTL,
   oDisablePth,
+  oDefCacheTTL,
+  oMaxCacheTTL,
+  oUseStandardSocket,
+  oNoUseStandardSocket,
 
   oIgnoreCacheForSigning,
+  oAllowMarkTrusted,
+  oAllowPresetPassphrase,
   oKeepTTY,
-  oKeepDISPLAY,
-
-aTest };
+  oKeepDISPLAY
+};
 
 
 
@@ -99,7 +108,7 @@ static ARGPARSE_OPTS opts[] = {
   { oQuiet,	"quiet",     0, N_("be somewhat more quiet") },
   { oSh,	"sh",        0, N_("sh-style command output") },
   { oCsh,	"csh",       0, N_("csh-style command output") },
-  { oOptions, "options"  , 2, N_("read options from file")},
+  { oOptions, "options"  , 2, N_("|FILE|read options from FILE")},
   { oDebug,	"debug"     ,4|16, "@"},
   { oDebugAll, "debug-all"     ,0, "@"},
   { oDebugLevel, "debug-level" ,2, "@"},
@@ -108,26 +117,40 @@ static ARGPARSE_OPTS opts[] = {
   { oNoGrab, "no-grab"     ,0, N_("do not grab keyboard and mouse")},
   { oLogFile, "log-file"   ,2, N_("use a log file for the server")},
   { oDisablePth, "disable-pth", 0, N_("do not allow multiple connections")},
+  { oUseStandardSocket, "use-standard-socket", 0,
+                      N_("use a standard location for the socket")},
+  { oNoUseStandardSocket, "no-use-standard-socket", 0, "@"},
 
-  { oPinentryProgram, "pinentry-program", 2 , "path to PIN Entry program" },
-  { oDisplay,    "display",     2, "set the display" },
-  { oTTYname,    "ttyname",     2, "set the tty terminal node name" },
-  { oTTYtype,    "ttytype",     2, "set the tty terminal type" },
-  { oLCctype,    "lc-ctype",    2, "set the tty LC_CTYPE value" },
-  { oLCmessages, "lc-messages", 2, "set the tty LC_MESSAGES value" },
+  { oPinentryProgram, "pinentry-program", 2 ,
+                               N_("|PGM|use PGM as the PIN-Entry program") },
+  { oScdaemonProgram, "scdaemon-program", 2 ,
+                               N_("|PGM|use PGM as the SCdaemon program") },
 
-  { oScdaemonProgram, "scdaemon-program", 2 , "path to SCdaemon program" },
-  { oDefCacheTTL, "default-cache-ttl", 4,
-                                 "|N|expire cached PINs after N seconds"},
-  { oIgnoreCacheForSigning, "ignore-cache-for-signing", 0,
-                                 "do not use the PIN cache when signing"},
+  { oDisplay,    "display",     2, "@" },
+  { oTTYname,    "ttyname",     2, "@" },
+  { oTTYtype,    "ttytype",     2, "@" },
+  { oLCctype,    "lc-ctype",    2, "@" },
+  { oLCmessages, "lc-messages", 2, "@" },
   { oKeepTTY, "keep-tty", 0,  N_("ignore requests to change the TTY")},
   { oKeepDISPLAY, "keep-display",
                           0, N_("ignore requests to change the X display")},
+
+  { oDefCacheTTL, "default-cache-ttl", 4,
+                               N_("|N|expire cached PINs after N seconds")},
+  { oMaxCacheTTL, "max-cache-ttl", 4, "@" },
+  { oIgnoreCacheForSigning, "ignore-cache-for-signing", 0,
+                               N_("do not use the PIN cache when signing")},
+  { oAllowMarkTrusted, "allow-mark-trusted", 0,
+                             N_("allow clients to mark keys as \"trusted\"")},
+  { oAllowPresetPassphrase, "allow-preset-passphrase", 0,
+                             N_("allow presetting passphrase")},
   {0}
 };
 
 
+#define DEFAULT_CACHE_TTL (10*60)  /* 10 minutes */
+#define MAX_CACHE_TTL     (120*60) /* 2 hours */
+
 static volatile int caught_fatal_sig = 0;
 
 /* flag to indicate that a shutdown was requested */
@@ -138,7 +161,7 @@ static int shutdown_pending;
 static int maybe_setuid = 1;
 
 /* Name of the communication socket */
-static char socket_name[128];
+static char *socket_name;
 
 /* Default values for options passed to the pinentry. */
 static char *default_display;
@@ -153,11 +176,19 @@ static char *config_filename;
 /* Helper to implement --debug-level */
 static const char *debug_level;
 
+/* Keep track of the current log file so that we can avoid updating
+   the log file after a SIGHUP if it didn't changed. Malloced. */
+static char *current_logfile;
+
 /* Local prototypes. */
 static void create_directories (void);
 #ifdef USE_GNU_PTH
 static void handle_connections (int listen_fd);
-#endif
+/* Pth wrapper function definitions. */
+GCRY_THREAD_OPTION_PTH_IMPL;
+#endif /*USE_GNU_PTH*/
+
+static int check_for_running_agent (int);
 
 
 
@@ -191,12 +222,12 @@ static void
 i18n_init (void)
 {
 #ifdef USE_SIMPLE_GETTEXT
-    set_gettext_file( PACKAGE );
+    set_gettext_file( PACKAGE_GT );
 #else
 #ifdef ENABLE_NLS
     setlocale (LC_ALL, "");
-    bindtextdomain (PACKAGE, LOCALEDIR);
-    textdomain (PACKAGE);
+    bindtextdomain (PACKAGE_GT, LOCALEDIR);
+    textdomain (PACKAGE_GT);
 #endif
 #endif
 }
@@ -223,12 +254,12 @@ my_gcry_logger (void *dummy, int level, const char *fmt, va_list arg_ptr)
 }
 
 
-/* Setup the debugging.  With a LEVEL of NULL only the active debug
-   flags are propagated to the subsystems.  With LEVEL set, a specific
-   set of debug flags is set; thus overriding all flags already
-   set. Note that we don't fail here, because it is important to keep
-   gpg-agent running even after re-reading the options due to a
-   SIGHUP. */
+/* Setup the debugging.  With the global variable DEBUG_LEVEL set to NULL
+   only the active debug flags are propagated to the subsystems.  With
+   DEBUG_LEVEL set, a specific set of debug flags is set; thus overriding
+   all flags already set. Note that we don't fail here, because it is
+   important to keep gpg-agent running even after re-reading the
+   options due to a SIGHUP. */
 static void
 set_debug (void)
 {
@@ -268,7 +299,7 @@ set_debug (void)
 static void
 cleanup (void)
 {
-  if (*socket_name)
+  if (socket_name && *socket_name)
     {
       char *p;
 
@@ -311,9 +342,10 @@ cleanup_sh (int sig)
 /* Handle options which are allowed to be reset after program start.
    Return true when the current option in PARGS could be handled and
    false if not.  As a special feature, passing a value of NULL for
-   PARGS, resets the options to the default. */
+   PARGS, resets the options to the default.  REREAD should be set
+   true if it is not the initial option parsing. */
 static int
-parse_rereadable_options (ARGPARSE_ARGS *pargs)
+parse_rereadable_options (ARGPARSE_ARGS *pargs, int reread)
 {
   if (!pargs)
     { /* reset mode */
@@ -323,8 +355,10 @@ parse_rereadable_options (ARGPARSE_ARGS *pargs)
       opt.no_grab = 0;
       opt.pinentry_program = NULL;
       opt.scdaemon_program = NULL;
-      opt.def_cache_ttl = 10*60; /* default to 10 minutes */
+      opt.def_cache_ttl = DEFAULT_CACHE_TTL;
+      opt.max_cache_ttl = MAX_CACHE_TTL;
       opt.ignore_cache_for_signing = 0;
+      opt.allow_mark_trusted = 0;
       return 1;
     }
 
@@ -337,15 +371,32 @@ parse_rereadable_options (ARGPARSE_ARGS *pargs)
     case oDebugAll: opt.debug = ~0; break;
     case oDebugLevel: debug_level = pargs->r.ret_str; break;
 
+    case oLogFile:
+      if (!reread)
+        return 0; /* not handeld */
+      if (!current_logfile || !pargs->r.ret_str
+          || strcmp (current_logfile, pargs->r.ret_str))
+        {
+          log_set_file (pargs->r.ret_str);
+          xfree (current_logfile);
+          current_logfile = xtrystrdup (pargs->r.ret_str);
+        }
+      break;
+
     case oNoGrab: opt.no_grab = 1; break;
       
     case oPinentryProgram: opt.pinentry_program = pargs->r.ret_str; break;
     case oScdaemonProgram: opt.scdaemon_program = pargs->r.ret_str; break;
 
     case oDefCacheTTL: opt.def_cache_ttl = pargs->r.ret_ulong; break;
+    case oMaxCacheTTL: opt.max_cache_ttl = pargs->r.ret_ulong; break;
       
     case oIgnoreCacheForSigning: opt.ignore_cache_for_signing = 1; break;
 
+    case oAllowMarkTrusted: opt.allow_mark_trusted = 1; break;
+
+    case oAllowPresetPassphrase: opt.allow_preset_passphrase = 1; break;
+
     default:
       return 0; /* not handled */
     }
@@ -376,33 +427,36 @@ main (int argc, char **argv )
   int debug_wait = 0;
   int disable_pth = 0;
   int gpgconf_list = 0;
+  int standard_socket = 0;
+  gpg_error_t err;
+
 
   set_strusage (my_strusage);
   gcry_control (GCRYCTL_SUSPEND_SECMEM_WARN);
   /* Please note that we may running SUID(ROOT), so be very CAREFUL
      when adding any stuff between here and the call to INIT_SECMEM()
      somewhere after the option parsing */
-  log_set_prefix ("gpg-agent", 1|4); 
+  log_set_prefix ("gpg-agent", JNLIB_LOG_WITH_PREFIX|JNLIB_LOG_WITH_PID); 
 
   /* Try to auto set the character set.  */
   set_native_charset (NULL); 
 
   i18n_init ();
 
-  /* We need to initialize Pth before libgcrypt, because the libgcrypt
-     initialization done by gcry_check_version internally sets up its
-     mutex system.  Note that one must not link against pth if
-     USE_GNU_PTH is not defined. */
+  /* Libgcrypt requires us to register the threading model first.
+     Note that this will also do the pth_init. */
 #ifdef USE_GNU_PTH
-  if (!pth_init ())
+  err = gcry_control (GCRYCTL_SET_THREAD_CBS, &gcry_threads_pth);
+  if (err)
     {
-      log_error ("failed to initialize the Pth library\n");
-      exit (1);
+      log_fatal ("can't register GNU Pth with Libgcrypt: %s\n",
+                 gpg_strerror (err));
     }
 #endif /*USE_GNU_PTH*/
 
-  /* check that the libraries are suitable.  Do it here because
-     the option parsing may need services of the library */
+
+  /* Check that the libraries are suitable.  Do it here because
+     the option parsing may need services of the library. */
   if (!gcry_check_version (NEED_LIBGCRYPT_VERSION) )
     {
       log_fatal( _("libgcrypt is too old (need %s, have %s)\n"),
@@ -418,18 +472,21 @@ main (int argc, char **argv )
 
   may_coredump = disable_core_dumps ();
 
-  parse_rereadable_options (NULL); /* Reset them to default values. */
-
+  /* Set default options.  */
+  parse_rereadable_options (NULL, 0); /* Reset them to default values. */
+#ifdef HAVE_W32_SYSTEM
+  standard_socket = 1;  /* Under Windows we always use a standard
+                           socket.  */
+#endif
+  
   shell = getenv ("SHELL");
   if (shell && strlen (shell) >= 3 && !strcmp (shell+strlen (shell)-3, "csh") )
     csh_style = 1;
-  
-  opt.homedir = getenv("GNUPGHOME");
-  if (!opt.homedir || !*opt.homedir)
-    opt.homedir = GNUPG_DEFAULT_HOMEDIR;
+
+  opt.homedir = default_homedir ();
 
 
-  /* check whether we have a config file on the commandline */
+  /* Check whether we have a config file on the commandline */
   orig_argc = argc;
   orig_argv = argv;
   pargs.argc = &argc;
@@ -459,7 +516,6 @@ main (int argc, char **argv )
      Now we are now working under our real uid 
   */
 
-
   if (default_config)
     configname = make_filename (opt.homedir, "gpg-agent.conf", NULL );
   
@@ -497,7 +553,7 @@ main (int argc, char **argv )
 
   while (optfile_parse( configfp, configname, &configlineno, &pargs, opts) )
     {
-      if (parse_rereadable_options (&pargs))
+      if (parse_rereadable_options (&pargs, 0))
         continue; /* Already handled */
       switch (pargs.r_opt)
         {
@@ -531,7 +587,11 @@ main (int argc, char **argv )
         case oTTYname: default_ttyname = xstrdup (pargs.r.ret_str); break;
         case oTTYtype: default_ttytype = xstrdup (pargs.r.ret_str); break;
         case oLCctype: default_lc_ctype = xstrdup (pargs.r.ret_str); break;
-        case oLCmessages: default_lc_messages = xstrdup (pargs.r.ret_str); break;
+        case oLCmessages: default_lc_messages = xstrdup (pargs.r.ret_str);
+          break;
+
+        case oUseStandardSocket: standard_socket = 1; break;
+        case oNoUseStandardSocket: standard_socket = 0; break;
 
         case oKeepTTY: opt.keep_tty = 1; break;
         case oKeepDISPLAY: opt.keep_display = 1; break;
@@ -562,7 +622,9 @@ main (int argc, char **argv )
       fprintf (stderr, "%s\n", strusage(15) );
     }
 #ifdef IS_DEVELOPMENT_VERSION
-  log_info ("NOTE: this is a development version!\n");
+  /* We don't want to print it here because gpg-agent is useful of its
+     own and quite matured.  */
+  /*log_info ("NOTE: this is a development version!\n");*/
 #endif
 
   set_debug ();
@@ -574,6 +636,10 @@ main (int argc, char **argv )
       exit (1);
     }
 
+  initialize_module_query ();
+  initialize_module_call_scd ();
+  
+  /* Try to create missing directories. */
   create_directories ();
 
   if (debug_wait && pipe_server)
@@ -585,41 +651,82 @@ main (int argc, char **argv )
     }
   
   if (gpgconf_list)
-    { /* List options and default values in the GPG Conf format.  */
+    {
+      char *filename;
 
-      printf ("gpgconf-gpg-agent.conf:\"%s\n", config_filename);
-      
-      printf ("verbose:\n"
-              "quiet:\n"
-              "debug-level:none\n"
-              "log-file:\n"
-              "force:\n"
-              "faked-system-time:\n"
-              "no-greeting:\n");
+      /* List options and default values in the GPG Conf format.  */
+
+      /* The following list is taken from gnupg/tools/gpgconf-comp.c.  */
+      /* Option flags.  YOU MUST NOT CHANGE THE NUMBERS OF THE EXISTING
+         FLAGS, AS THEY ARE PART OF THE EXTERNAL INTERFACE.  */
+#define GC_OPT_FLAG_NONE	0UL
+      /* The RUNTIME flag for an option indicates that the option can be
+         changed at runtime.  */
+#define GC_OPT_FLAG_RUNTIME	(1UL << 3)
+      /* The DEFAULT flag for an option indicates that the option has a
+         default value.  */
+#define GC_OPT_FLAG_DEFAULT	(1UL << 4)
+      /* The DEF_DESC flag for an option indicates that the option has a
+         default, which is described by the value of the default field.  */
+#define GC_OPT_FLAG_DEF_DESC	(1UL << 5)
+      /* The NO_ARG_DESC flag for an option indicates that the argument has
+         a default, which is described by the value of the ARGDEF field.  */
+#define GC_OPT_FLAG_NO_ARG_DESC	(1UL << 6)
+
+      filename = make_filename (opt.homedir, "gpg-agent.conf", NULL );
+      printf ("gpgconf-gpg-agent.conf:%lu:\"%s\n",
+              GC_OPT_FLAG_DEFAULT, filename);
+      xfree (filename);
+
+      printf ("verbose:%lu:\n"
+              "quiet:%lu:\n"
+              "debug-level:%lu:\"none:\n"
+              "log-file:%lu:\n",
+              GC_OPT_FLAG_NONE|GC_OPT_FLAG_RUNTIME,
+              GC_OPT_FLAG_NONE|GC_OPT_FLAG_RUNTIME,
+              GC_OPT_FLAG_DEFAULT|GC_OPT_FLAG_RUNTIME,
+              GC_OPT_FLAG_NONE|GC_OPT_FLAG_RUNTIME );
+      printf ("default-cache-ttl:%lu:%d:\n",
+              GC_OPT_FLAG_DEFAULT|GC_OPT_FLAG_RUNTIME, DEFAULT_CACHE_TTL );
+      printf ("no-grab:%lu:\n", 
+              GC_OPT_FLAG_NONE|GC_OPT_FLAG_RUNTIME);
+      printf ("ignore-cache-for-signing:%lu:\n",
+              GC_OPT_FLAG_NONE|GC_OPT_FLAG_RUNTIME);
+      printf ("allow-mark-trusted:%lu:\n",
+              GC_OPT_FLAG_NONE|GC_OPT_FLAG_RUNTIME);
 
       agent_exit (0);
     }
 
+  /* If this has been called without any options, we merely check
+     whether an agent is already running.  We do this here so that we
+     don't clobber a logfile but print it directly to stderr. */
   if (!pipe_server && !is_daemon)
-    log_info (_("please use the option `--daemon'"
-                " to run the program in the background\n"));
+    {
+      log_set_prefix (NULL, JNLIB_LOG_WITH_PREFIX); 
+      check_for_running_agent (0);
+      agent_exit (0);
+    }
   
 #ifdef ENABLE_NLS
-  /* gpg-agent usdually does not ooutput any messages becuase it runs
-     in the background.  For log files it is acceptable to have
-     messages always encoded in utf-8.  We switch here to utf-8, so
-     that commands like --help still give native messages.  It is far
-     easier to swicthnonly once instead of for every message and it
-     actually helps when more then one thread is active (avoids
-     required an extra copy step). */
-    bind_textdomain_codeset (PACKAGE, "UTF-8");
+  /* gpg-agent usually does not output any messages because it runs in
+     the background.  For log files it is acceptable to have messages
+     always encoded in utf-8.  We switch here to utf-8, so that
+     commands like --help still give native messages.  It is far
+     easier to switch only once instead of for every message and it
+     actually helps when more then one thread is active (avoids an
+     extra copy step). */
+    bind_textdomain_codeset (PACKAGE_GT, "UTF-8");
 #endif
 
-  /* now start with logging to a file if this is desired */
+  /* Now start with logging to a file if this is desired. */
   if (logfile)
     {
       log_set_file (logfile);
-      log_set_prefix (NULL, 1|2|4);
+      log_set_prefix (NULL, (JNLIB_LOG_WITH_PREFIX
+                             |JNLIB_LOG_WITH_TIME
+                             |JNLIB_LOG_WITH_PID));
+      current_logfile = xstrdup (logfile);
     }
 
   /* Make sure that we have a default ttyname. */
@@ -628,15 +735,17 @@ main (int argc, char **argv )
   if (!default_ttytype && getenv ("TERM"))
     default_ttytype = xstrdup (getenv ("TERM"));
 
+
   if (pipe_server)
     { /* this is the simple pipe based server */
       start_command_handler (-1, -1);
     }
   else if (!is_daemon)
-    ;
+    ; /* NOTREACHED */
   else
-    { /* regular server mode */
+    { /* Regular server mode */
       int fd;
+      int rc;
       pid_t pid;
       int len;
       struct sockaddr_un serv_addr;
@@ -646,28 +755,33 @@ main (int argc, char **argv )
          default to a specific display.  There is still a default
          display when gpg-agent weas started using --display or a
          client requested this using an OPTION command. */
+#ifndef HAVE_W32_SYSTEM
       if (!opt.keep_display)
         unsetenv ("DISPLAY");
+#endif
 
-      *socket_name = 0;
-      snprintf (socket_name, DIM(socket_name)-1,
-                "/tmp/gpg-XXXXXX/S.gpg-agent");
-      socket_name[DIM(socket_name)-1] = 0;
-      p = strrchr (socket_name, '/');
-      if (!p)
-        BUG ();
-      *p = 0;;
-      if (!mkdtemp(socket_name))
+      /* Create the socket name . */
+      if (standard_socket)
+        socket_name = make_filename (opt.homedir, "S.gpg-agent", NULL);
+      else
         {
-          log_error ("can't create directory `%s': %s\n",
-	             socket_name, strerror(errno) );
-          exit (1);
+          socket_name = xstrdup ("/tmp/gpg-XXXXXX/S.gpg-agent");
+          p = strrchr (socket_name, '/');
+          if (!p)
+            BUG ();
+          *p = 0;;
+          if (!mkdtemp(socket_name))
+            {
+              log_error (_("can't create directory `%s': %s\n"),
+                         socket_name, strerror(errno) );
+              exit (1);
+            }
+          *p = '/';
         }
-      *p = '/';
 
-      if (strchr (socket_name, ':') )
+      if (strchr (socket_name, PATHSEP_C) )
         {
-          log_error ("colons are not allowed in the socket name\n");
+          log_error ("`%s' are not allowed in the socket name\n", PATHSEP_S);
           exit (1);
         }
       if (strlen (socket_name)+1 >= sizeof serv_addr.sun_path ) 
@@ -676,8 +790,11 @@ main (int argc, char **argv )
           exit (1);
         }
    
-
+#ifdef HAVE_W32_SYSTEM
+      fd = _w32_sock_new (AF_UNIX, SOCK_STREAM, 0);
+#else
       fd = socket (AF_UNIX, SOCK_STREAM, 0);
+#endif
       if (fd == -1)
         {
           log_error ("can't create socket: %s\n", strerror(errno) );
@@ -690,7 +807,22 @@ main (int argc, char **argv )
       len = (offsetof (struct sockaddr_un, sun_path)
              + strlen(serv_addr.sun_path) + 1);
 
-      if (bind (fd, (struct sockaddr*)&serv_addr, len) == -1)
+#ifdef HAVE_W32_SYSTEM
+      rc = _w32_sock_bind (fd, (struct sockaddr*)&serv_addr, len);
+      if (rc == -1 && standard_socket)
+        {
+          remove (socket_name);
+          rc = bind (fd, (struct sockaddr*)&serv_addr, len);
+        }
+#else
+      rc = bind (fd, (struct sockaddr*)&serv_addr, len);
+      if (rc == -1 && standard_socket && errno == EADDRINUSE)
+        {
+          remove (socket_name);
+          rc = bind (fd, (struct sockaddr*)&serv_addr, len);
+        }
+#endif
+      if (rc == -1)
         {
           log_error ("error binding socket to `%s': %s\n",
                      serv_addr.sun_path, strerror (errno) );
@@ -710,6 +842,10 @@ main (int argc, char **argv )
 
 
       fflush (NULL);
+#ifdef HAVE_W32_SYSTEM
+      pid = getpid ();
+      printf ("set GPG_AGENT_INFO=%s;%lu;1\n", socket_name, (ulong)pid);
+#else /*!HAVE_W32_SYSTEM*/
       pid = fork ();
       if (pid == (pid_t)-1) 
         {
@@ -717,7 +853,7 @@ main (int argc, char **argv )
           exit (1);
         }
       else if (pid) 
-        { /* we are the parent */
+        { /* We are the parent */
           char *infostr;
           
           close (fd);
@@ -764,19 +900,21 @@ main (int argc, char **argv )
             }
           /*NEVER REACHED*/
         } /* end parent */
-      
 
-      /* this is the child */
+      /* 
+         This is the child
+       */
 
-      /* detach from tty and put process into a new session */
+      /* Detach from tty and put process into a new session */
       if (!nodetach )
         { 
           int i;
+          unsigned int oldflags;
 
-          /* close stdin, stdout and stderr unless it is the log stream */
+          /* Close stdin, stdout and stderr unless it is the log stream */
           for (i=0; i <= 2; i++) 
             {
-              if ( log_get_fd () != i)
+              if (!log_test_fd (i) && i != fd )
                 close (i);
             }
           if (setsid() == -1)
@@ -785,6 +923,9 @@ main (int argc, char **argv )
               cleanup ();
               exit (1);
             }
+
+          log_get_prefix (&oldflags);
+          log_set_prefix (NULL, oldflags | JNLIB_LOG_RUN_DETACHED);
           opt.running_detached = 1;
         }
 
@@ -794,22 +935,27 @@ main (int argc, char **argv )
           exit (1);
         }
 
+#endif /*!HAVE_W32_SYSTEM*/
+
 
 #ifdef USE_GNU_PTH
       if (!disable_pth)
         {
+#ifndef HAVE_W32_SYSTEM  /* FIXME */
 	  struct sigaction sa;
 
 	  sa.sa_handler = SIG_IGN;
 	  sigemptyset (&sa.sa_mask);
 	  sa.sa_flags = 0;
 	  sigaction (SIGPIPE, &sa, NULL);
+#endif
           handle_connections (fd);
         }
       else
 #endif /*!USE_GNU_PTH*/
       /* setup signals */
         {
+#ifndef HAVE_W32_SYSTEM  /* FIXME */
           struct sigaction oact, nact;
           
           nact.sa_handler = cleanup_sh;
@@ -825,7 +971,7 @@ main (int argc, char **argv )
           nact.sa_handler = SIG_IGN;
           sigaction (SIGPIPE, &nact, NULL);
           sigaction (SIGINT, &nact, NULL);
-
+#endif
           start_command_handler (fd, -1);
         }
       close (fd);
@@ -912,7 +1058,7 @@ reread_configuration (void)
       return;
     }
 
-  parse_rereadable_options (NULL); /* Start from the default values. */
+  parse_rereadable_options (NULL, 1); /* Start from the default values. */
 
   memset (&pargs, 0, sizeof pargs);
   dummy = 0;
@@ -923,7 +1069,7 @@ reread_configuration (void)
       if (pargs.r_opt < -1)
         pargs.err = 1; /* Print a warning. */
       else /* Try to parse this option - ignore unchangeable ones. */
-        parse_rereadable_options (&pargs);
+        parse_rereadable_options (&pargs, 1);
     }
   fclose (fp);
   set_debug ();
@@ -939,9 +1085,15 @@ create_private_keys_directory (const char *home)
   fname = make_filename (home, GNUPG_PRIVATE_KEYS_DIR, NULL);
   if (stat (fname, &statbuf) && errno == ENOENT)
     {
+#ifdef HAVE_W32_SYSTEM  /*FIXME: Setup proper permissions.  */
+      if (!CreateDirectory (fname, NULL))
+        log_error (_("can't create directory `%s': %s\n"),
+                   fname, w32_strerror (-1) );
+#else
       if (mkdir (fname, S_IRUSR|S_IWUSR|S_IXUSR ))
         log_error (_("can't create directory `%s': %s\n"),
-                   fname,	strerror(errno) );
+                   fname, strerror (errno) );
+#endif
       else if (!opt.quiet)
         log_info (_("directory `%s' created\n"), fname);
     }
@@ -973,9 +1125,15 @@ create_directories (void)
                || (*defhome != '~' && !strcmp (home, defhome) )
                )
             {
+#ifdef HAVE_W32_SYSTEM
+              if (!CreateDirectory (home, NULL))
+                log_error (_("can't create directory `%s': %s\n"),
+                           home, w32_strerror (-1) );
+#else
               if (mkdir (home, S_IRUSR|S_IWUSR|S_IXUSR ))
                 log_error (_("can't create directory `%s': %s\n"),
-                           home, strerror(errno) );
+                           home, strerror (errno) );
+#endif
               else 
                 {
                   if (!opt.quiet)
@@ -1006,23 +1164,21 @@ handle_signal (int signo)
 {
   switch (signo)
     {
+#ifndef HAVE_W32_SYSTEM
     case SIGHUP:
       log_info ("SIGHUP received - "
                 "re-reading configuration and flushing cache\n");
       agent_flush_cache ();
       reread_configuration ();
+      agent_reload_trustlist ();
       break;
       
     case SIGUSR1:
-      if (opt.verbose < 5)
-        opt.verbose++;
-      log_info ("SIGUSR1 received - verbosity set to %d\n", opt.verbose);
+      log_info ("SIGUSR1 received - no action defined\n");
       break;
-
+      
     case SIGUSR2:
-      if (opt.verbose)
-        opt.verbose--;
-      log_info ("SIGUSR2 received - verbosity set to %d\n", opt.verbose );
+      log_info ("SIGUSR2 received - checking smartcard status\n");
       break;
 
     case SIGTERM:
@@ -1047,7 +1203,7 @@ handle_signal (int signo)
       cleanup ();
       agent_exit (0);
       break;
-
+#endif
     default:
       log_info ("signal %d received - no action defined\n", signo);
     }
@@ -1061,6 +1217,12 @@ start_connection_thread (void *arg)
 
   if (opt.verbose)
     log_info ("handler for fd %d started\n", fd);
+
+  /* FIXME: Move this housekeeping into a ticker function.  Calling it
+     for each connection should work but won't work anymore if our
+     clients start to keep connections. */
+  agent_trustlist_housekeeping ();
+
   start_command_handler (-1, fd);
   if (opt.verbose)
     log_info ("handler for fd %d terminated\n", fd);
@@ -1082,9 +1244,10 @@ handle_connections (int listen_fd)
 
   tattr = pth_attr_new();
   pth_attr_set (tattr, PTH_ATTR_JOINABLE, 0);
-  pth_attr_set (tattr, PTH_ATTR_STACK_SIZE, 32*1024);
+  pth_attr_set (tattr, PTH_ATTR_STACK_SIZE, 256*1024);
   pth_attr_set (tattr, PTH_ATTR_NAME, "gpg-agent");
 
+#ifndef HAVE_W32_SYSTEM /* fixme */
   sigemptyset (&sigs );
   sigaddset (&sigs, SIGHUP);
   sigaddset (&sigs, SIGUSR1);
@@ -1092,6 +1255,9 @@ handle_connections (int listen_fd)
   sigaddset (&sigs, SIGINT);
   sigaddset (&sigs, SIGTERM);
   ev = pth_event (PTH_EVENT_SIGS, &sigs, &signo);
+#else
+  ev = NULL;
+#endif
 
   for (;;)
     {
@@ -1139,3 +1305,75 @@ handle_connections (int listen_fd)
   log_info ("%s %s stopped\n", strusage(11), strusage(13));
 }
 #endif /*USE_GNU_PTH*/
+
+
+/* Figure out whether an agent is available and running. Prints an
+   error if not.  Usually started with MODE 0. */
+static int
+check_for_running_agent (int mode)
+{
+  int rc;
+  char *infostr, *p;
+  assuan_context_t ctx;
+  int prot, pid;
+
+  if (!mode)
+    {
+      infostr = getenv ("GPG_AGENT_INFO");
+      if (!infostr || !*infostr)
+        {
+          if (!check_for_running_agent (1))
+            return 0; /* Okay, its running on the standard socket. */
+          log_error (_("no gpg-agent running in this session\n"));
+          return -1;
+        }
+
+      infostr = xstrdup (infostr);
+      if ( !(p = strchr (infostr, PATHSEP_C)) || p == infostr)
+        {
+          xfree (infostr);
+          if (!check_for_running_agent (1))
+            return 0; /* Okay, its running on the standard socket. */
+          log_error (_("malformed GPG_AGENT_INFO environment variable\n"));
+          return -1;
+        }
+
+      *p++ = 0;
+      pid = atoi (p);
+      while (*p && *p != PATHSEP_C)
+        p++;
+      prot = *p? atoi (p+1) : 0;
+      if (prot != 1)
+        {
+          xfree (infostr);
+          log_error (_("gpg-agent protocol version %d is not supported\n"),
+                     prot);
+          if (!check_for_running_agent (1))
+            return 0; /* Okay, its running on the standard socket. */
+          return -1;
+        }
+    }
+  else /* MODE != 0 */
+    {
+      infostr = make_filename (opt.homedir, "S.gpg-agent", NULL);
+    }
+
+
+  rc = assuan_socket_connect (&ctx, infostr, pid);
+  xfree (infostr);
+  if (rc)
+    {
+      if (!mode && !check_for_running_agent (1))
+        return 0; /* Okay, its running on the standard socket. */
+
+      if (!mode)
+        log_error ("can't connect to the agent: %s\n", assuan_strerror (rc));
+      return -1;
+    }
+
+  if (!opt.quiet)
+    log_info ("gpg-agent running and available\n");
+
+  assuan_disconnect (ctx);
+  return 0;
+}
