@@ -29,12 +29,23 @@
 #include <assert.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
+#ifdef HAVE_LANGINFO_CODESET
+#include <langinfo.h>
+#endif
+#ifdef HAVE_DOSISH_SYSTEM
+#include <fcntl.h> /* for setmode() */
+#endif
 
 #define JNLIB_NEED_LOG_LOGV
 #include "agent.h"
 #include "minip12.h"
 #include "simple-pwquery.h"
 #include "i18n.h"
+#include "sysutils.h"
+
 
 enum cmd_and_opt_values 
 { aNull = 0,
@@ -58,6 +69,7 @@ enum cmd_and_opt_values
   oNoFailOnExist,
   oHomedir,
   oPrompt,
+  oStatusMsg, 
 
 aTest };
 
@@ -72,7 +84,7 @@ struct rsa_secret_key_s
   };
 
 
-static char *opt_homedir;
+static const char *opt_homedir;
 static int opt_armor;
 static int opt_store;
 static int opt_force;
@@ -80,8 +92,10 @@ static int opt_no_fail_on_exist;
 static int opt_have_cert;
 static const char *opt_passphrase;
 static char *opt_prompt;
+static int opt_status_msg;
 
 static char *get_passphrase (int promptno);
+static char *get_new_passphrase (int promptno);
 static void release_passphrase (char *pw);
 static int store_private_key (const unsigned char *grip,
                               const void *buffer, size_t length, int force);
@@ -108,6 +122,7 @@ static ARGPARSE_OPTS opts[] = {
   { oNoFailOnExist, "no-fail-on-exist", 0, "@" },
   { oHomedir, "homedir", 2, "@" }, 
   { oPrompt,  "prompt", 2, "|ESCSTRING|use ESCSTRING as prompt in pinentry"}, 
+  { oStatusMsg, "enable-status-msg", 0, "@"},
   {0}
 };
 
@@ -141,12 +156,12 @@ static void
 i18n_init (void)
 {
 #ifdef USE_SIMPLE_GETTEXT
-    set_gettext_file( PACKAGE );
+    set_gettext_file( PACKAGE_GT );
 #else
 #ifdef ENABLE_NLS
     setlocale (LC_ALL, "");
-    bindtextdomain (PACKAGE, LOCALEDIR);
-    textdomain (PACKAGE);
+    bindtextdomain (PACKAGE_GT, LOCALEDIR);
+    textdomain (PACKAGE_GT);
 #endif
 #endif
 }
@@ -252,6 +267,9 @@ read_file (const char *fname, size_t *r_length)
       size_t nread, bufsize = 0;
 
       fp = stdin;
+#ifdef HAVE_DOSISH_SYSTEM
+      setmode ( fileno(fp) , O_BINARY );
+#endif
       buf = NULL;
       buflen = 0;
 #define NCHUNK 8192
@@ -384,6 +402,8 @@ read_and_unprotect (const char *fname)
   xfree (key);
   if (rc)
     {
+      if (opt_status_msg)
+        log_info ("[PROTECT-TOOL:] bad-passphrase\n");
       log_error ("unprotecting the key failed: %s\n", gpg_strerror (rc));
       return;
     }
@@ -652,7 +672,7 @@ import_p12_file (const char *fname)
   if (!buf)
     return;
 
-  kparms = p12_parse (buf, buflen, (pw=get_passphrase (0)),
+  kparms = p12_parse (buf, buflen, (pw=get_passphrase (2)),
                       import_p12_cert_cb, NULL);
   release_passphrase (pw);
   xfree (buf);
@@ -728,7 +748,7 @@ import_p12_file (const char *fname)
   gcry_sexp_release (s_key);
 
 
-  rc = agent_protect (key, (pw=get_passphrase (0)), &result, &resultlen);
+  rc = agent_protect (key, (pw=get_new_passphrase (4)), &result, &resultlen);
   release_passphrase (pw);
   xfree (key);
   if (rc)
@@ -870,6 +890,8 @@ export_p12_file (const char *fname)
       release_passphrase (pw);
       if (rc)
         {
+          if (opt_status_msg && gpg_err_code (rc) == GPG_ERR_BAD_PASSPHRASE )
+            log_info ("[PROTECT-TOOL:] bad-passphrase\n");
           log_error ("unprotecting key `%s' failed: %s\n",
                      fname, gpg_strerror (rc));
           xfree (key);
@@ -952,7 +974,8 @@ export_p12_file (const char *fname)
   kparms[7] = sk.u;
   kparms[8] = NULL;
 
-  key = p12_build (kparms, cert, certlen, (pw=get_passphrase (0)), &keylen);
+  key = p12_build (kparms, cert, certlen,
+                   (pw=get_new_passphrase (3)), &keylen);
   release_passphrase (pw);
   xfree (cert);
   for (i=0; i < 8; i++)
@@ -960,6 +983,9 @@ export_p12_file (const char *fname)
   if (!key)
     return;
   
+#ifdef HAVE_DOSISH_SYSTEM
+  setmode ( fileno (stdout) , O_BINARY );
+#endif
   fwrite (key, keylen, 1, stdout);
   xfree (key);
 }
@@ -1041,14 +1067,7 @@ main (int argc, char **argv )
   gcry_control (GCRYCTL_INIT_SECMEM, 16384, 0);
 
 
-#ifdef __MINGW32__
-  opt_homedir = read_w32_registry_string ( NULL,
-                                           "Software\\GNU\\GnuPG", "HomeDir" );
-#else
-  opt_homedir = getenv ("GNUPGHOME");
-#endif
-  if (!opt_homedir || !*opt_homedir)
-    opt_homedir = GNUPG_DEFAULT_HOMEDIR;
+  opt_homedir = default_homedir ();
 
 
   pargs.argc = &argc;
@@ -1076,6 +1095,7 @@ main (int argc, char **argv )
         case oNoFailOnExist: opt_no_fail_on_exist = 1; break;
         case oHaveCert: opt_have_cert = 1; break;
         case oPrompt: opt_prompt = pargs.r.ret_str; break;
+        case oStatusMsg: opt_status_msg = 1; break;
           
         default : pargs.err = 2; break;
 	}
@@ -1125,6 +1145,11 @@ agent_exit (int rc)
    set from the command line  PROMPTNO select the prompt to display:
      0 = default
      1 = taken from the option --prompt
+     2 = for unprotecting a pkcs#12 object
+     3 = for protecting a new pkcs#12 object
+     4 = for protecting an imported pkcs#12 in our system
+     5 = reenter the passphrase
+   When adding 100 to the values, a "does not match - try again" errro message is shown.
 */
 static char *
 get_passphrase (int promptno)
@@ -1132,29 +1157,116 @@ get_passphrase (int promptno)
   char *pw;
   int err;
   const char *desc;
+#ifdef HAVE_LANGINFO_CODESET
+  char *orig_codeset = NULL;
+#endif
+  int error_msgno;
+  
 
   if (opt_passphrase)
     return xstrdup (opt_passphrase);
 
+  error_msgno = promptno / 100;
+  promptno %= 100;
+
+#ifdef ENABLE_NLS
+  /* The Assuan agent protocol requires us to transmit utf-8 strings */
+  orig_codeset = bind_textdomain_codeset (PACKAGE_GT, NULL);
+#ifdef HAVE_LANGINFO_CODESET
+  if (!orig_codeset)
+    orig_codeset = nl_langinfo (CODESET);
+#endif
+  if (orig_codeset && !strcmp (orig_codeset, "UTF-8"))
+    orig_codeset = NULL;
+  if (orig_codeset)
+    {
+      /* We only switch when we are able to restore the codeset later. */
+      orig_codeset = xstrdup (orig_codeset);
+      if (!bind_textdomain_codeset (PACKAGE_GT, "utf-8"))
+        orig_codeset = NULL; 
+    }
+#endif
+
   if (promptno == 1 && opt_prompt)
     desc = opt_prompt;
+  else if (promptno == 2)
+    desc = _("Please enter the passphrase to unprotect the "
+             "PKCS#12 object.");
+  else if (promptno == 3)
+    desc = _("Please enter the passphrase to protect the "
+             "new PKCS#12 object.");
+  else if (promptno == 4)
+    desc = _("Please enter the passphrase to protect the "
+             "imported object within the GnuPG system.");
+  else if (promptno == 5)
+    desc = _("Please re-enter this passphrase");
   else
     desc = _("Please enter the passphrase or the PIN\n"
              "needed to complete this operation.");
 
-  pw = simple_pwquery (NULL,NULL, _("Passphrase:"), desc, &err);
+  pw = simple_pwquery (NULL,
+                       error_msgno == 1? _("does not match - try again"):NULL,
+                       _("Passphrase:"), desc, &err);
+
+#ifdef ENABLE_NLS
+  if (orig_codeset)
+    {
+      bind_textdomain_codeset (PACKAGE_GT, orig_codeset);
+      xfree (orig_codeset);
+    }
+#endif
+
   if (!pw)
     {
       if (err)
-        log_error ("error while asking for the passphrase\n");
+        log_error (_("error while asking for the passphrase: %s\n"),
+                   gpg_strerror (err));
       else
-        log_info ("cancelled\n");
+        log_info (_("cancelled\n"));
       agent_exit (0);
     }
 
   return pw;
 }
 
+
+/* Same as get_passphrase but requests it a second time and compares
+   it to the one entered the first time. */
+static char *
+get_new_passphrase (int promptno)
+{
+  char *pw;
+  int i, secondpromptno;
+  
+  pw = get_passphrase (promptno);
+  if (!pw)
+    return NULL; /* Canceled. */
+  if (!*pw)
+    return pw; /* Empty passphrase - no need to as for repeating it. */
+
+  secondpromptno = 5;
+  for (i=0; i < 3; i++)
+    {
+      char *pw2 = get_passphrase (secondpromptno);
+      if (!pw2)
+        {
+          xfree (pw);
+          return NULL; /* Canceled.  */
+        }
+      if (!strcmp (pw, pw2))
+        {
+          xfree (pw2);
+          return pw; /* Okay. */
+        }
+      secondpromptno = 105;
+      xfree (pw2);
+    }
+  xfree (pw);
+  return NULL; /* 3 times repeated wrong - cancel.  */
+}
+
+
+
 static void
 release_passphrase (char *pw)
 {
@@ -1185,6 +1297,8 @@ store_private_key (const unsigned char *grip,
     {
       if (!access (fname, F_OK))
       {
+        if (opt_status_msg)
+          log_info ("[PROTECT-TOOL:] secretkey-exists\n");
         if (opt_no_fail_on_exist)
           log_info ("secret key file `%s' already exists\n", fname);
         else
@@ -1221,6 +1335,9 @@ store_private_key (const unsigned char *grip,
     }
   log_info ("secret key stored as `%s'\n", fname);
 
+  if (opt_status_msg)
+    log_info ("[PROTECT-TOOL:] secretkey-stored\n");
+
   xfree (fname);
   return 0;
 }
