@@ -1,5 +1,5 @@
 /* app-openpgp.c - The OpenPGP card application.
- *	Copyright (C) 2003, 2004, 2005 Free Software Foundation, Inc.
+ *	Copyright (C) 2003, 2004 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  *
- * $Id: app-openpgp.c,v 1.23 2005/01/25 14:18:55 wk Exp $
+ * $Id: app-openpgp.c,v 1.21 2004/12/09 16:57:30 wk Exp $
  */
 
 #include <config.h>
@@ -138,12 +138,10 @@ do_deinit (app_t app)
 
 
 /* Wrapper around iso7816_get_data which first tries to get the data
-   from the cache.  With GET_IMMEDIATE passed as true, the cache is
-   bypassed. */
+   from the cache. */
 static gpg_error_t
 get_cached_data (app_t app, int tag, 
-                 unsigned char **result, size_t *resultlen,
-                 int get_immediate)
+                 unsigned char **result, size_t *resultlen)
 {
   gpg_error_t err;
   int i;
@@ -154,25 +152,23 @@ get_cached_data (app_t app, int tag,
   *result = NULL;
   *resultlen = 0;
 
-  if (!get_immediate)
-    {
-      for (c=app->app_local->cache; c; c = c->next)
-        if (c->tag == tag)
+  for (c=app->app_local->cache; c; c = c->next)
+    if (c->tag == tag)
+      {
+        if(c->length)
           {
-            if(c->length)
-              {
-                p = xtrymalloc (c->length);
-                if (!p)
-                  return gpg_error (gpg_err_code_from_errno (errno));
-                memcpy (p, c->data, c->length);
-                *result = p;
-              }
-            
-            *resultlen = c->length;
-            
-            return 0;
+            p = xtrymalloc (c->length);
+            if (!p)
+              return gpg_error (gpg_err_code_from_errno (errno));
+            memcpy (p, c->data, c->length);
+            *result = p;
           }
-    }
+
+        *resultlen = c->length;
+
+        return 0;
+      }
+ 
   
   err = iso7816_get_data (app->slot, tag, &p, &len);
   if (err)
@@ -181,9 +177,6 @@ get_cached_data (app_t app, int tag,
   *resultlen = len;
 
   /* Check whether we should cache this object. */
-  if (get_immediate)
-    return 0;
-
   for (i=0; data_objects[i].tag; i++)
     if (data_objects[i].tag == tag)
       {
@@ -192,7 +185,8 @@ get_cached_data (app_t app, int tag,
         break;
       }
 
-  /* Okay, cache it. */
+  /* No, cache it. */
+
   for (c=app->app_local->cache; c; c = c->next)
     assert (c->tag != tag);
   
@@ -305,8 +299,7 @@ get_one_do (app_t app, int tag, unsigned char **result, size_t *nbytes)
   if (data_objects[i].tag && data_objects[i].get_from)
     {
       rc = get_cached_data (app, data_objects[i].get_from,
-                            &buffer, &buflen,
-                            data_objects[i].get_immediate_in_v11);
+                            &buffer, &buflen);
       if (!rc)
         {
           const unsigned char *s;
@@ -327,8 +320,7 @@ get_one_do (app_t app, int tag, unsigned char **result, size_t *nbytes)
 
   if (!value) /* Not in a constructed DO, try simple. */
     {
-      rc = get_cached_data (app, tag, &buffer, &buflen,
-                            data_objects[i].get_immediate_in_v11);
+      rc = get_cached_data (app, tag, &buffer, &buflen);
       if (!rc)
         {
           value = buffer;
@@ -434,7 +426,7 @@ count_bits (const unsigned char *a, size_t len)
    at any time and should be called after changing the login-data DO.
 
    Everything up to a LF is considered a mailbox or account name.  If
-   the first LF is followed by DC4 (0x14) control sequence are
+   the first LF is follewed by DC4 (0x14) control sequence are
    expected up to the next LF.  Control sequences are separated by FS
    (0x28) and consist of key=value pairs.  There is one key defined:
 
@@ -844,6 +836,8 @@ verify_chv3 (app_t app,
       void *relptr;
       unsigned char *value;
       size_t valuelen;
+      int reread_chv_status;
+      
 
       relptr = get_one_do (app, 0x00C4, &value, &valuelen);
       if (!relptr || valuelen < 7)
@@ -859,11 +853,13 @@ verify_chv3 (app_t app,
           return gpg_error (GPG_ERR_BAD_PIN);
         }
 
+      reread_chv_status = (value[6] < 3);
+
       log_info(_("%d Admin PIN attempts remaining before card"
                  " is permanently locked\n"), value[6]);
       xfree (relptr);
 
-      /* TRANSLATORS: Do not translate the "|A|" prefix but
+      /* Note to translators: Do not translate the "|A|" prefix but
          keep it at the start of the string.  We need this elsewhere
          to get some infos on the string. */
       rc = pincb (pincb_arg, _("|A|Admin PIN"), &pinvalue); 
@@ -890,6 +886,13 @@ verify_chv3 (app_t app,
           return rc;
         }
       app->did_chv3 = 1;
+      /* If the PIN has been entered wrongly before, we need to flush
+         the cached value so that the next read correctly reflects the
+         resetted retry counter.  Note that version 1.1 of the specs
+         allow direct reading of that DO, so that we could actually
+         flush it in all cases. */
+      if (reread_chv_status)
+        flush_cache_item (app, 0x00C4);
     }
   return rc;
 }
@@ -1224,7 +1227,7 @@ compare_fingerprint (app_t app, int keyno, unsigned char *sha1fpr)
   
   assert (keyno >= 1 && keyno <= 3);
 
-  rc = get_cached_data (app, 0x006E, &buffer, &buflen, 0);
+  rc = get_cached_data (app, 0x006E, &buffer, &buflen);
   if (rc)
     {
       log_error (_("error reading application data\n"));
