@@ -1,12 +1,12 @@
 /* armor.c - Armor flter
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
- *               2007 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003
+ *                                             Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,7 +15,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #include <config.h>
@@ -26,6 +27,7 @@
 #include <assert.h>
 #include <ctype.h>
 
+#include "gpg.h"
 #include "errors.h"
 #include "iobuf.h"
 #include "memory.h"
@@ -37,6 +39,12 @@
 #include "status.h"
 #include "i18n.h"
 
+#ifdef HAVE_DOSISH_SYSTEM
+#define LF "\r\n"
+#else
+#define LF "\n"
+#endif
+
 #define MAX_LINELEN 20000
 
 #define CRCINIT 0xB704CE
@@ -112,57 +120,6 @@ static char *tail_strings[] = {
 };
 
 
-/* Create a new context for armor filters.  */
-armor_filter_context_t *
-new_armor_context (void)
-{
-  armor_filter_context_t *afx;
-
-  afx = xcalloc (1, sizeof *afx);
-  afx->refcount = 1;
-
-  return afx;
-}
-
-/* Release an armor filter context.  Passing NULL is explicitly
-   allowed and a no-op.  */
-void
-release_armor_context (armor_filter_context_t *afx)
-{
-  if (!afx)
-    return;
-
-  /* In contrast to 2.0, we use in 1.4 heap based contexts only in a
-     very few places and in general keep the stack based contexts.  A
-     REFCOUNT of 0 indicates a stack based context and thus we don't
-     do anything in this case. */
-  if (!afx->refcount)
-    return;
-
-  if ( --afx->refcount )
-    return;
-  xfree (afx);
-}
-
-/* Push the armor filter onto the iobuf stream IOBUF.  */
-int
-push_armor_filter (armor_filter_context_t *afx, iobuf_t iobuf)
-{
-  int rc;
-
-  if (!afx->refcount)
-    return iobuf_push_filter (iobuf, armor_filter, afx);
-
-  afx->refcount++;
-  rc = iobuf_push_filter (iobuf, armor_filter, afx);
-  if (rc)
-    afx->refcount--;
-  return rc;
-}
-
-
-
-
 
 static void
 initialize(void)
@@ -236,7 +193,7 @@ is_armored( const byte *buf )
  *	   filter to do further processing.
  */
 int
-use_armor_filter( IOBUF a )
+use_armor_filter( iobuf_t a )
 {
     byte buf[1];
     int n;
@@ -292,14 +249,12 @@ parse_hash_header( const char *line )
 	    found |= 2;
 	else if( !strncmp( s, "MD5", s2-s ) )
 	    found |= 4;
-	else if( !strncmp( s, "SHA224", s2-s ) )
-	    found |= 8;
 	else if( !strncmp( s, "SHA256", s2-s ) )
-	    found |= 16;
+	    found |= 8;
 	else if( !strncmp( s, "SHA384", s2-s ) )
-	    found |= 32;
+	    found |= 16;
 	else if( !strncmp( s, "SHA512", s2-s ) )
-	    found |= 64;
+	    found |= 32;
 	else
 	    return 0;
 	for(; *s2 && (*s2==' ' || *s2 == '\t'); s2++ )
@@ -312,19 +267,7 @@ parse_hash_header( const char *line )
     return found;
 }
 
-/* Returns true if this is a valid armor tag as per RFC-2440bis-21. */
-static int
-is_armor_tag(const char *line)
-{
-  if(strncmp(line,"Version",7)==0
-     || strncmp(line,"Comment",7)==0
-     || strncmp(line,"MessageID",9)==0
-     || strncmp(line,"Hash",4)==0
-     || strncmp(line,"Charset",7)==0)
-    return 1;
 
-  return 0;
-}
 
 /****************
  * Check whether this is a armor line.
@@ -349,25 +292,17 @@ is_armor_header( byte *line, unsigned len )
     save_p = p;
     p += 5;
 
-    /* Some Windows environments seem to add whitespace to the end of
-       the line, so we strip it here.  This becomes strict if
-       --rfc2440 is set since 2440 reads "The header lines, therefore,
-       MUST start at the beginning of a line, and MUST NOT have text
-       following them on the same line."  It is unclear whether "text"
-       refers to all text or just non-whitespace text.  4880 clarified
-       this was only non-whitespace text. */
+    /* Some mail programs on Windows seem to add spaces to the end of
+       the line.  This becomes strict if --openpgp is set. */
 
-    if(RFC2440)
-      {
-	if( *p == '\r' )
-	  p++;
-	if( *p == '\n' )
-	  p++;
-      }
-    else
-      while(*p==' ' || *p=='\r' || *p=='\n' || *p=='\t')
+    if(!RFC2440)
+      while(*p==' ')
 	p++;
 
+    if( *p == '\r' )
+	p++;
+    if( *p == '\n' )
+	p++;
     if( *p )
 	return -1; /* garbage after dashes */
     save_c = *save_p; *save_p = 0;
@@ -385,32 +320,6 @@ is_armor_header( byte *line, unsigned len )
 }
 
 
-/* Helper to parse a "KEY <keyid> FAILED <code>" line and return the
-   error code.  LINEPTR points right behind "KEY ".  */
-int
-parse_key_failed_line (const void *lineptr, unsigned int len)
-{
-  const byte *line = lineptr;
-  int code = 0;
-
-  for (; len && !spacep (line); len--, line++)
-    ;
-  for (; len && spacep (line); len--, line++)
-    ;
-  if (len > 7 && !memcmp (line, "FAILED ", 7))
-    {
-      line += 7;
-      len -= 7;
-      for (; len && digitp (line); len--, line++)
-        {
-          code *= 10;
-          code += atoi_1 (line);
-        }
-    }
-
-  return code;
-}
-
 
 /****************
  * Parse a header lines
@@ -425,35 +334,21 @@ parse_header_line( armor_filter_context_t *afx, byte *line, unsigned int len )
     int hashes=0;
     unsigned int len2;
 
-    len2 = check_trailing_ws( line, len );
+    len2 = length_sans_trailing_ws( line, len );
     if( !len2 ) {
         afx->buffer_pos = len2;  /* (it is not the fine way to do it here) */
 	return 0; /* WS only: same as empty line */
     }
-
-    /*
-      This is fussy.  The spec says that a header line is delimited
-      with a colon-space pair.  This means that a line such as
-      "Comment: " (with nothing else) is actually legal as an empty
-      string comment.  However, email and cut-and-paste being what it
-      is, that trailing space may go away.  Therefore, we accept empty
-      headers delimited with only a colon.  --rfc2440, as always,
-      makes this strict and enforces the colon-space pair. -dms
-    */
+    len = len2;
+    line[len2] = 0;
 
     p = strchr( line, ':');
-    if( !p || (RFC2440 && p[1]!=' ')
-	|| (!RFC2440 && p[1]!=' ' && p[1]!='\n' && p[1]!='\r'))
-      {
+    if( !p || !p[1] ) {
 	log_error(_("invalid armor header: "));
 	print_string( stderr, line, len, 0 );
 	putc('\n', stderr);
 	return -1;
-      }
-
-    /* Chop off the whitespace we detected before */
-    len=len2;
-    line[len2]='\0';
+    }
 
     if( opt.verbose ) {
 	log_info(_("armor header: "));
@@ -461,32 +356,16 @@ parse_header_line( armor_filter_context_t *afx, byte *line, unsigned int len )
 	putc('\n', stderr);
     }
 
-    if( afx->in_cleartext )
-      {
+    if( afx->in_cleartext ) {
 	if( (hashes=parse_hash_header( line )) )
-	  afx->hashes |= hashes;
+	    afx->hashes |= hashes;
 	else if( strlen(line) > 15 && !memcmp( line, "NotDashEscaped:", 15 ) )
-	  afx->not_dash_escaped = 1;
-	else
-	  {
+	    afx->not_dash_escaped = 1;
+	else {
 	    log_error(_("invalid clearsig header\n"));
 	    return -1;
-	  }
-      }
-    else if(!is_armor_tag(line))
-      {
-	/* Section 6.2: "Unknown keys should be reported to the user,
-	   but OpenPGP should continue to process the message."  Note
-	   that in a clearsigned message this applies to the signature
-	   part (i.e. "BEGIN PGP SIGNATURE") and not the signed data
-	   ("BEGIN PGP SIGNED MESSAGE").  The only key allowed in the
-	   signed data section is "Hash". */
-
-	log_info(_("unknown armor header: "));
-	print_string( stderr, line, len, 0 );
-	putc('\n', stderr);
-      }
-
+	}
+    }
     return 1;
 }
 
@@ -494,7 +373,7 @@ parse_header_line( armor_filter_context_t *afx, byte *line, unsigned int len )
 
 /* figure out whether the data is armored or not */
 static int
-check_input( armor_filter_context_t *afx, IOBUF a )
+check_input( armor_filter_context_t *afx, iobuf_t a )
 {
     int rc = 0;
     int i;
@@ -531,20 +410,12 @@ check_input( armor_filter_context_t *afx, IOBUF a )
     /* find the armor header */
     while(len) {
 	i = is_armor_header( line, len );
-        if (i == -1 && afx->only_keyblocks
-            && !afx->key_failed_code
-            && len > 4 && !memcmp (line, "KEY ", 4))
-          {
-            /* This is probably input from a keyserver helper and we
-               have not yet seen an error line.  */
-            afx->key_failed_code = parse_key_failed_line (line+4, len-4);
-          }
 	if( i >= 0 && !(afx->only_keyblocks && i != 1 && i != 5 && i != 6 )) {
 	    hdr_line = i;
 	    if( hdr_line == BEGIN_SIGNED_MSG_IDX ) {
 		if( afx->in_cleartext ) {
 		    log_error(_("nested clear text signatures\n"));
-		    rc = G10ERR_INVALID_ARMOR;
+		    rc = GPG_ERR_INV_ARMOR;
 		}
 		afx->in_cleartext = 1;
 	    }
@@ -560,9 +431,9 @@ check_input( armor_filter_context_t *afx, IOBUF a )
 	} while( !maxlen );
     }
 
-    /* Parse the header lines.  */
+    /* parse the header lines */
     while(len) {
-	/* Read the next line (skip all truncated lines). */
+	/* read the next line (skip all truncated lines) */
 	do {
 	    maxlen = MAX_LINELEN;
 	    afx->buffer_len = iobuf_read_line( a, &afx->buffer,
@@ -573,8 +444,8 @@ check_input( armor_filter_context_t *afx, IOBUF a )
 
 	i = parse_header_line( afx, line, len );
 	if( i <= 0 ) {
-	    if (i && RFC2440)
-		rc = G10ERR_INVALID_ARMOR;
+	    if( i )
+		rc = GPG_ERR_INV_ARMOR;
 	    break;
 	}
     }
@@ -594,8 +465,7 @@ check_input( armor_filter_context_t *afx, IOBUF a )
     return rc;
 }
 
-#define PARTIAL_CHUNK 512
-#define PARTIAL_POW   9
+
 
 /****************
  * Fake a literal data packet and wait for the next armor line
@@ -603,7 +473,7 @@ check_input( armor_filter_context_t *afx, IOBUF a )
  *	  not implemented/checked.
  */
 static int
-fake_packet( armor_filter_context_t *afx, IOBUF a,
+fake_packet( armor_filter_context_t *afx, iobuf_t a,
 	     size_t *retn, byte *buf, size_t size  )
 {
     int rc = 0;
@@ -611,31 +481,19 @@ fake_packet( armor_filter_context_t *afx, IOBUF a,
     int lastline = 0;
     unsigned maxlen, n;
     byte *p;
-    byte tempbuf[PARTIAL_CHUNK];
-    size_t tempbuf_len=0;
 
-    while( !rc && size-len>=(PARTIAL_CHUNK+1)) {
+    len = 2;	/* reserve 2 bytes for the length header */
+    size -= 2;	/* and 2 for the terminating header */
+    while( !rc && len < size ) {
 	/* copy what we have in the line buffer */
 	if( afx->faked == 1 )
 	    afx->faked++; /* skip the first (empty) line */
-	else
-	  {
-	    /* It's full, so write this partial chunk */
-	    if(tempbuf_len==PARTIAL_CHUNK)
-	      {
-		buf[len++]=0xE0+PARTIAL_POW;
-		memcpy(&buf[len],tempbuf,PARTIAL_CHUNK);
-		len+=PARTIAL_CHUNK;
-		tempbuf_len=0;
+	else {
+	    while( len < size && afx->buffer_pos < afx->buffer_len )
+		buf[len++] = afx->buffer[afx->buffer_pos++];
+	    if( len >= size )
 		continue;
-	      }
-
-	    while( tempbuf_len < PARTIAL_CHUNK
-		   && afx->buffer_pos < afx->buffer_len )
-	      tempbuf[tempbuf_len++] = afx->buffer[afx->buffer_pos++];
-	    if( tempbuf_len==PARTIAL_CHUNK )
-	      continue;
-	  }
+	}
 
 	/* read the next line */
 	maxlen = MAX_LINELEN;
@@ -648,64 +506,15 @@ fake_packet( armor_filter_context_t *afx, IOBUF a,
 	}
 	if( !maxlen )
 	    afx->truncated++;
-
-	p = afx->buffer;
-	n = afx->buffer_len;
-
-	/* Armor header or dash-escaped line? */
-	if(p[0]=='-')
-	  {
-	    /* 2440bis-10: When reversing dash-escaping, an
-	       implementation MUST strip the string "- " if it occurs
-	       at the beginning of a line, and SHOULD warn on "-" and
-	       any character other than a space at the beginning of a
-	       line.  */
-
-	    if(p[1]==' ' && !afx->not_dash_escaped)
-	      {
-		/* It's a dash-escaped line, so skip over the
-		   escape. */
-		afx->buffer_pos = 2;
-	      }
-	    else if(p[1]=='-' && p[2]=='-' && p[3]=='-' && p[4]=='-')
-	      {
-		/* Five dashes in a row mean it's probably armor
-		   header. */
-		int type = is_armor_header( p, n );
-		if( afx->not_dash_escaped && type != BEGIN_SIGNATURE )
-		  ; /* this is okay */
-		else
-		  {
-		    if( type != BEGIN_SIGNATURE )
-		      {
-			log_info(_("unexpected armor: "));
-			print_string( stderr, p, n, 0 );
-			putc('\n', stderr);
-		      }
-
-		    lastline = 1;
-		    rc = -1;
-		  }
-	      }
-	    else if(!afx->not_dash_escaped)
-	      {
-		/* Bad dash-escaping. */
-		log_info(_("invalid dash escaped line: "));
-		print_string( stderr, p, n, 0 );
-		putc('\n', stderr);
-	      }
-	  }
-
-	/* Now handle the end-of-line canonicalization */
-	if( !afx->not_dash_escaped )
-	  {
-	    int crlf = n > 1 && p[n-2] == '\r' && p[n-1]=='\n';
+	if( !afx->not_dash_escaped ) {
+	    int crlf;
+	    p = afx->buffer;
+	    n = afx->buffer_len;
+	    crlf = n > 1 && p[n-2] == '\r' && p[n-1]=='\n';
 
 	    /* PGP2 does not treat a tab as white space character */
-	    afx->buffer_len=
-	      trim_trailing_chars( &p[afx->buffer_pos], n-afx->buffer_pos,
-				   afx->pgp2mode ? " \r\n" : " \t\r\n");
-	    afx->buffer_len+=afx->buffer_pos;
+	    afx->buffer_len = trim_trailing_chars( p, n,
+					 afx->pgp2mode ? " \r\n" : " \t\r\n");
 	    /* the buffer is always allocated with enough space to append
 	     * the removed [CR], LF and a Nul
 	     * The reason for this complicated procedure is to keep at least
@@ -717,23 +526,48 @@ fake_packet( armor_filter_context_t *afx, IOBUF a,
 	     * faked packet could do the job).
 	     */
 	    if( crlf )
-	      afx->buffer[afx->buffer_len++] = '\r';
+		afx->buffer[afx->buffer_len++] = '\r';
 	    afx->buffer[afx->buffer_len++] = '\n';
-	    afx->buffer[afx->buffer_len] = '\0';
-	  }
+	    afx->buffer[afx->buffer_len] = 0;
+	}
+	p = afx->buffer;
+	n = afx->buffer_len;
+
+	if( n > 2 && *p == '-' ) {
+	    /* check for dash escaped or armor header */
+	    if( p[1] == ' ' && !afx->not_dash_escaped ) {
+		/* issue a warning if it is not regular encoded */
+		if( p[2] != '-' && !( n > 6 && !memcmp(p+2, "From ", 5))) {
+		    log_info(_("invalid dash escaped line: "));
+		    print_string( stderr, p, n, 0 );
+		    putc('\n', stderr);
+		}
+		afx->buffer_pos = 2; /* skip */
+	    }
+	    else if( n >= 15 &&  p[1] == '-' && p[2] == '-' && p[3] == '-' ) {
+		int type = is_armor_header( p, n );
+		if( afx->not_dash_escaped && type != BEGIN_SIGNATURE )
+		    ; /* this is okay */
+		else {
+		    if( type != BEGIN_SIGNATURE ) {
+			log_info(_("unexpected armor:"));
+			print_string( stderr, p, n, 0 );
+			putc('\n', stderr);
+		    }
+		    lastline = 1;
+		    rc = -1;
+		}
+	    }
+	}
     }
 
+    buf[0] = (len-2) >> 8;
+    buf[1] = (len-2);
     if( lastline ) { /* write last (ending) length header */
-        if(tempbuf_len<192)
-	  buf[len++]=tempbuf_len;
-	else
-	  {
-	    buf[len++]=((tempbuf_len-192)/256) + 192;
-	    buf[len++]=(tempbuf_len-192) % 256;
-	  }
-	memcpy(&buf[len],tempbuf,tempbuf_len);
-	len+=tempbuf_len;
-
+	if( buf[0] || buf[1] ) { /* only if we have some text */
+	    buf[len++] = 0;
+	    buf[len++] = 0;
+	}
 	rc = 0;
 	afx->faked = 0;
 	afx->in_cleartext = 0;
@@ -778,12 +612,12 @@ invalid_crc(void)
     if ( opt.ignore_crc_error )
         return 0;
     log_inc_errorcount();
-    return G10ERR_INVALID_ARMOR;
+    return GPG_ERR_INV_ARMOR;
 }
 
 
 static int
-radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
+radix64_read( armor_filter_context_t *afx, iobuf_t a, size_t *retn,
 	      byte *buf, size_t size )
 {
     byte val;
@@ -791,7 +625,7 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
     int checkcrc=0;
     int rc = 0;
     size_t n = 0;
-    int  idx, i, onlypad=0;
+    int  idx, i;
     u32 crc;
 
     crc = afx->crc;
@@ -809,14 +643,14 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
 	    if( !maxlen )
 		afx->truncated++;
 	    if( !afx->buffer_len )
-                break; /* eof */
+		break; /* eof */
 	    continue;
 	}
 
       again:
 	if( c == '\n' || c == ' ' || c == '\r' || c == '\t' )
 	    continue;
-	else if( c == '=' ) { /* Pad character: stop or CRC sum starts. */
+	else if( c == '=' ) { /* pad character: stop */
 	    /* some mailers leave quoted-printable encoded characters
 	     * so we try to workaround this */
 	    if( afx->buffer_pos+2 < afx->buffer_len ) {
@@ -836,16 +670,13 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
 		}
 	    }
 
-	    if (!n)
-	      onlypad = 1;
-
 	    if( idx == 1 )
 		buf[n++] = val;
 	    checkcrc++;
 	    break;
 	}
 	else if( (c = asctobin[(c2=c)]) == 255 ) {
-	    log_error(_("invalid radix64 character %02X skipped\n"), c2);
+	    log_error(_("invalid radix64 character %02x skipped\n"), c2);
 	    continue;
 	}
 	switch(idx) {
@@ -924,17 +755,13 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
 	    if( c == -1 ) {
 		log_info(_("premature eof (in CRC)\n"));
 		rc = invalid_crc();
-	    }
-	    else if( idx == 0 ) {
-	        /* No CRC at all is legal ("MAY") */
-	        rc=0;
-	    }
+                	    }
 	    else if( idx != 4 ) {
 		log_info(_("malformed CRC\n"));
 		rc = invalid_crc();
 	    }
 	    else if( mycrc != afx->crc ) {
-                log_info (_("CRC error; %06lX - %06lX\n"),
+                log_info (_("CRC error; %06lx - %06lx\n"),
 				    (ulong)afx->crc, (ulong)mycrc);
                 rc = invalid_crc();
 	    }
@@ -954,19 +781,19 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
 		if( rc == -1 )
 		    rc = 0;
 		else if( rc == 2 ) {
-		    log_error(_("premature eof (in trailer)\n"));
-		    rc = G10ERR_INVALID_ARMOR;
+		    log_error(_("premature eof (in Trailer)\n"));
+		    rc = GPG_ERR_INV_ARMOR;
 		}
 		else {
 		    log_error(_("error in trailer line\n"));
-		    rc = G10ERR_INVALID_ARMOR;
+		    rc = GPG_ERR_INV_ARMOR;
 		}
 #endif
 	    }
 	}
     }
 
-    if( !n && !onlypad )
+    if( !n )
 	rc = -1;
 
     *retn = n;
@@ -978,7 +805,7 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
  */
 int
 armor_filter( void *opaque, int control,
-	     IOBUF a, byte *buf, size_t *ret_len)
+	     iobuf_t a, byte *buf, size_t *ret_len)
 {
     size_t size = *ret_len;
     armor_filter_context_t *afx = opaque;
@@ -1016,10 +843,9 @@ armor_filter( void *opaque, int control,
 	*ret_len = n;
     }
     else if( control == IOBUFCTRL_UNDERFLOW ) {
-        /* We need some space for the faked packet.  The minmum
-         * required size is the PARTIAL_CHUNK size plus a byte for the
-         * length itself */
-	if( size < PARTIAL_CHUNK+1 )
+        /* We need some space for the faked packet.  The minmum required
+         * size is ~18 + length of the session marker */
+	if( size < 50 ) 
 	    BUG(); /* supplied buffer too short */
 
 	if( afx->faked )
@@ -1038,7 +864,7 @@ armor_filter( void *opaque, int control,
 	        unsigned int hashes = afx->hashes;
                 const byte *sesmark;
                 size_t sesmarklen;
-
+                
                 sesmark = get_session_marker( &sesmarklen );
                 if ( sesmarklen > 20 )
                     BUG();
@@ -1056,11 +882,11 @@ armor_filter( void *opaque, int control,
 			afx->pgp2mode = 1;
 		}
 		n=0;
-                /* First a gpg control packet... */
+                /* first a gpg control packet */
                 buf[n++] = 0xff; /* new format, type 63, 1 length byte */
                 n++;   /* see below */
                 memcpy(buf+n, sesmark, sesmarklen ); n+= sesmarklen;
-                buf[n++] = CTRLPKT_CLEARSIGN_START;
+                buf[n++] = CTRLPKT_CLEARSIGN_START; 
                 buf[n++] = afx->not_dash_escaped? 0:1; /* sigclass */
                 if( hashes & 1 )
                     buf[n++] = DIGEST_ALGO_RMD160;
@@ -1069,25 +895,19 @@ armor_filter( void *opaque, int control,
                 if( hashes & 4 )
                     buf[n++] = DIGEST_ALGO_MD5;
                 if( hashes & 8 )
-                    buf[n++] = DIGEST_ALGO_SHA224;
-                if( hashes & 16 )
                     buf[n++] = DIGEST_ALGO_SHA256;
-                if( hashes & 32 )
+                if( hashes & 16 )
                     buf[n++] = DIGEST_ALGO_SHA384;
-                if( hashes & 64 )
+                if( hashes & 32 )
                     buf[n++] = DIGEST_ALGO_SHA512;
                 buf[1] = n - 2;
 
-		/* ...followed by an invented plaintext packet.
-		   Amusingly enough, this packet is not compliant with
-		   2440 as the initial partial length is less than 512
-		   bytes.  Of course, we'll accept it anyway ;) */
-
-		buf[n++] = 0xCB; /* new packet format, type 11 */
-		buf[n++] = 0xE1; /* 2^1 == 2 bytes */
+		/* followed by a plaintext packet */
+		buf[n++] = 0xaf; /* old packet format, type 11, var length */
+		buf[n++] = 0;	 /* set the length header */
+		buf[n++] = 6;
 		buf[n++] = 't';  /* canonical text mode */
 		buf[n++] = 0;	 /* namelength */
-		buf[n++] = 0xE2; /* 2^2 == 4 more bytes */
 		memset(buf+n, 0, 4); /* timestamp */
 		n += 4;
 	    }
@@ -1106,54 +926,35 @@ armor_filter( void *opaque, int control,
     else if( control == IOBUFCTRL_FLUSH && !afx->cancel ) {
 	if( !afx->status ) { /* write the header line */
 	    const char *s;
-	    STRLIST comment=opt.comments;
+            STRLIST comment = opt.comments;
 
 	    if( afx->what >= DIM(head_strings) )
 		log_bug("afx->what=%d", afx->what);
 	    iobuf_writestr(a, "-----");
 	    iobuf_writestr(a, head_strings[afx->what] );
-	    iobuf_writestr(a, "-----" );
-	    iobuf_writestr(a,afx->eol);
-	    if (opt.emit_version)
-	      {
-		iobuf_writestr (a, "Version: GnuPG v");
-                for (s=VERSION; *s && *s != '.'; s++)
-                  iobuf_writebyte (a, *s);
-                if (opt.emit_version > 1 && *s)
-                  {
-                    iobuf_writebyte (a, *s++);
-                    for (; *s && *s != '.'; s++)
-                      iobuf_writebyte (a, *s);
-                    if (opt.emit_version > 2)
-                      {
-                        for (; *s && *s != '-' && !spacep (s); s++)
-                          iobuf_writebyte (a, *s);
-                        if (opt.emit_version > 3)
-                          iobuf_writestr (a, " (" PRINTABLE_OS_NAME ")");
-                      }
-                  }
+	    iobuf_writestr(a, "-----" LF );
+	    if( !opt.no_version )
+		iobuf_writestr(a, "Version: GnuPG v"  VERSION " ("
+					      PRINTABLE_OS_NAME ")" LF );
 
-		iobuf_writestr(a,afx->eol);
-	      }
-
-	    /* write the comment strings */
-	    for(s=comment->d;comment;comment=comment->next,s=comment->d)
+	    /* Write the comment string. */
+	    for(s=comment? comment->d:NULL; comment;
+                comment=comment->next,s=comment->d)
 	      {
 		iobuf_writestr(a, "Comment: " );
-		for( ; *s; s++ )
-		  {
+		for ( ; *s; s++ )
+                  {
 		    if( *s == '\n' )
-		      iobuf_writestr(a, "\\n" );
+                      iobuf_writestr(a, "\\n" );
 		    else if( *s == '\r' )
-		      iobuf_writestr(a, "\\r" );
+                      iobuf_writestr(a, "\\r" );
 		    else if( *s == '\v' )
-		      iobuf_writestr(a, "\\v" );
+                      iobuf_writestr(a, "\\v" );
 		    else
-		      iobuf_put(a, *s );
-		  }
-
-		iobuf_writestr(a,afx->eol);
-	      }
+                      iobuf_put(a, *s );
+                  }
+		iobuf_writestr(a, LF );
+              }
 
 	    if ( afx->hdrlines ) {
                 for ( s = afx->hdrlines; *s; s++ ) {
@@ -1164,8 +965,7 @@ armor_filter( void *opaque, int control,
                     iobuf_put(a, *s );
                 }
             }
-
-	    iobuf_writestr(a,afx->eol);
+	    iobuf_writestr(a, LF );
 	    afx->status++;
 	    afx->idx = 0;
 	    afx->idx2 = 0;
@@ -1194,11 +994,10 @@ armor_filter( void *opaque, int control,
 		iobuf_put(a, c);
 		c = bintoasc[radbuf[2]&077];
 		iobuf_put(a, c);
-		if( ++idx2 >= (64/4) )
-		  { /* pgp doesn't like 72 here */
-		    iobuf_writestr(a,afx->eol);
+		if( ++idx2 >= (64/4) ) { /* pgp doesn't like 72 here */
+		    iobuf_writestr(a, LF );
 		    idx2=0;
-		  }
+		}
 	    }
 	}
 	for(i=0; i < idx; i++ )
@@ -1207,23 +1006,10 @@ armor_filter( void *opaque, int control,
 	afx->idx2 = idx2;
 	afx->crc  = crc;
     }
-    else if( control == IOBUFCTRL_INIT )
-      {
+    else if( control == IOBUFCTRL_INIT ) {
 	if( !is_initialized )
-	  initialize();
-
-	/* Figure out what we're using for line endings if the caller
-	   didn't specify. */
-	if(afx->eol[0]==0)
-	  {
-#ifdef HAVE_DOSISH_SYSTEM
-	    afx->eol[0]='\r';
-	    afx->eol[1]='\n';
-#else
-	    afx->eol[0]='\n';
-#endif
-	  }
-      }
+	    initialize();
+    }
     else if( control == IOBUFCTRL_CANCEL ) {
 	afx->cancel = 1;
     }
@@ -1252,15 +1038,14 @@ armor_filter( void *opaque, int control,
 		    iobuf_put(a, c);
 		    iobuf_put(a, '=');
 		}
-		if( ++idx2 >= (64/4) )
-		  { /* pgp doesn't like 72 here */
-		    iobuf_writestr(a,afx->eol);
+		if( ++idx2 >= (64/4) ) { /* pgp doesn't like 72 here */
+		    iobuf_writestr(a, LF );
 		    idx2=0;
-		  }
+		}
 	    }
 	    /* may need a linefeed */
 	    if( idx2 )
-	      iobuf_writestr(a,afx->eol);
+		iobuf_writestr(a, LF );
 	    /* write the CRC */
 	    iobuf_put(a, '=');
 	    radbuf[0] = crc >>16;
@@ -1274,14 +1059,13 @@ armor_filter( void *opaque, int control,
 	    iobuf_put(a, c);
 	    c = bintoasc[radbuf[2]&077];
 	    iobuf_put(a, c);
-	    iobuf_writestr(a,afx->eol);
+	    iobuf_writestr(a, LF );
 	    /* and the the trailer */
 	    if( afx->what >= DIM(tail_strings) )
 		log_bug("afx->what=%d", afx->what);
 	    iobuf_writestr(a, "-----");
 	    iobuf_writestr(a, tail_strings[afx->what] );
-	    iobuf_writestr(a, "-----" );
-	    iobuf_writestr(a,afx->eol);
+	    iobuf_writestr(a, "-----" LF );
 	}
 	else if( !afx->any_data && !afx->inp_bypass ) {
 	    log_error(_("no valid OpenPGP data found.\n"));
@@ -1295,9 +1079,8 @@ armor_filter( void *opaque, int control,
 	if( afx->qp_detected )
 	    log_error(_("quoted printable character in armor - "
 			"probably a buggy MTA has been used\n") );
-	xfree( afx->buffer );
+	xfree ( afx->buffer );
 	afx->buffer = NULL;
-        release_armor_context (afx);
     }
     else if( control == IOBUFCTRL_DESC )
 	*(char**)buf = "armor_filter";
@@ -1313,7 +1096,7 @@ make_radix64_string( const byte *data, size_t len )
 {
     char *buffer, *p;
 
-    buffer = p = xmalloc( (len+2)/3*4 + 1 );
+    buffer = p = xmalloc ( (len+2)/3*4 + 1 );
     for( ; len >= 3 ; len -= 3, data += 3 ) {
 	*p++ = bintoasc[(data[0] >> 2) & 077];
 	*p++ = bintoasc[(((data[0] <<4)&060)|((data[1] >> 4)&017))&077];
@@ -1336,7 +1119,7 @@ make_radix64_string( const byte *data, size_t len )
 
 /***********************************************
  *  For the pipemode command we can't use the armor filter for various
- *  reasons, so we use this new unarmor_pump stuff to remove the armor
+ *  reasons, so we use this new unarmor_pump stuff to remove the armor 
  */
 
 enum unarmor_state_e {
@@ -1344,7 +1127,7 @@ enum unarmor_state_e {
     STA_bypass,
     STA_wait_newline,
     STA_wait_dash,
-    STA_first_dash,
+    STA_first_dash, 
     STA_compare_header,
     STA_found_header_wait_newline,
     STA_skip_header_lines,
@@ -1373,7 +1156,7 @@ unarmor_pump_new (void)
 
     if( !is_initialized )
         initialize();
-    x = xmalloc_clear (sizeof *x);
+    x = xcalloc (1,sizeof *x);
     return x;
 }
 
@@ -1383,12 +1166,12 @@ unarmor_pump_release (UnarmorPump x)
     xfree (x);
 }
 
-/*
+/* 
  * Get the next character from the ascii armor taken from the IOBUF
  * created earlier by unarmor_pump_new().
  * Return:  c = Character
  *        256 = ignore this value
- *         -1 = End of current armor
+ *         -1 = End of current armor 
  *         -2 = Premature EOF (not used)
  *         -3 = Invalid armor
  */
@@ -1399,9 +1182,9 @@ unarmor_pump (UnarmorPump x, int c)
 
     switch (x->state) {
       case STA_init:
-        {
+        { 
             byte tmp[1];
-            tmp[0] = c;
+            tmp[0] = c; 
             if ( is_armored (tmp) )
                 x->state = c == '-'? STA_first_dash : STA_wait_newline;
             else {
@@ -1424,10 +1207,10 @@ unarmor_pump (UnarmorPump x, int c)
         x->state = STA_compare_header;
       case STA_compare_header:
         if ( "-----BEGIN PGP SIGNATURE-----"[++x->pos] == c ) {
-            if ( x->pos == 28 )
+            if ( x->pos == 28 ) 
                 x->state = STA_found_header_wait_newline;
         }
-        else
+        else 
             x->state = c == '\n'? STA_wait_dash : STA_wait_newline;
         break;
       case STA_found_header_wait_newline:
@@ -1470,11 +1253,11 @@ unarmor_pump (UnarmorPump x, int c)
         {
             int c2;
             if( (c = asctobin[(c2=c)]) == 255 ) {
-                log_error(_("invalid radix64 character %02X skipped\n"), c2);
+                log_error(_("invalid radix64 character %02x skipped\n"), c2);
                 break;
             }
         }
-
+        
         switch(x->pos) {
           case 0:
             x->val = c << 2;
@@ -1507,7 +1290,7 @@ unarmor_pump (UnarmorPump x, int c)
         if( (c = asctobin[c]) == 255 ) {
             rval = -1; /* ready */
             if( x->crc != x->mycrc ) {
-                log_info (_("CRC error; %06lX - %06lX\n"),
+                log_info (_("CRC error; %06lx - %06lx\n"),
                           (ulong)x->crc, (ulong)x->mycrc);
                 if ( invalid_crc() )
                     rval = -3;
@@ -1515,7 +1298,7 @@ unarmor_pump (UnarmorPump x, int c)
             x->state = STA_ready; /* not sure whether this is correct */
             break;
         }
-
+        
         switch(x->pos) {
           case 0:
             x->val = c << 2;
