@@ -171,6 +171,10 @@ start_agent (void)
   if (rc)
     return map_assuan_err (rc);
 
+#ifdef __GNUC__
+#warning put this code into common/asshelp.c
+#endif
+
   dft_display = getenv ("DISPLAY");
   if (opt.display || dft_display)
     {
@@ -461,6 +465,7 @@ agent_release_card_info (struct agent_card_info_s *info)
   xfree (info->disp_lang); info->disp_lang = NULL;
   xfree (info->pubkey_url); info->pubkey_url = NULL;
   xfree (info->login_data); info->login_data = NULL;
+  info->cafpr1valid = info->cafpr2valid = info->cafpr3valid = 0;
   info->fpr1valid = info->fpr2valid = info->fpr3valid = 0;
 }
 
@@ -557,6 +562,20 @@ learn_status_cb (void *opaque, const char *line)
       else if (no == 3)
         parm->fpr3valid = unhexify_fpr (line, parm->fpr3);
     }
+  else if (keywordlen == 6 && !memcmp (keyword, "CA-FPR", keywordlen))
+    {
+      int no = atoi (line);
+      while (*line && !spacep (line))
+        line++;
+      while (spacep (line))
+        line++;
+      if (no == 1)
+        parm->cafpr1valid = unhexify_fpr (line, parm->cafpr1);
+      else if (no == 2)
+        parm->cafpr2valid = unhexify_fpr (line, parm->cafpr2);
+      else if (no == 3)
+        parm->cafpr3valid = unhexify_fpr (line, parm->cafpr3);
+    }
   
   return 0;
 }
@@ -870,7 +889,7 @@ agent_scd_change_pin (int chvno)
 }
 
 
-/* Perform a CHECKPIN operation.  SERIALNO should be the seriial
+/* Perform a CHECKPIN operation.  SERIALNO should be the serial
    number of the card - optioanlly followed by the fingerprint;
    however the fingerprint is ignored here. */
 int
