@@ -1,5 +1,5 @@
 /* comment.c - write comment stuff
- *	Copyright (C) 1998, 2000 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 2003 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -29,7 +29,7 @@
 #include "packet.h"
 #include "errors.h"
 #include "iobuf.h"
-#include <gcrypt.h>
+#include "memory.h"
 #include "util.h"
 #include "main.h"
 #include "keydb.h"
@@ -37,7 +37,7 @@
 
 
 int
-write_comment( IOBUF out, const char *s )
+write_comment( iobuf_t out, const char *s )
 {
     PACKET pkt;
     size_t n = strlen(s);
@@ -45,25 +45,25 @@ write_comment( IOBUF out, const char *s )
 
     pkt.pkttype = PKT_COMMENT;
     if( *s != '#' ) {
-       pkt.pkt.comment = gcry_xmalloc( sizeof *pkt.pkt.comment + n );
+       pkt.pkt.comment = xmalloc ( sizeof *pkt.pkt.comment + n );
        pkt.pkt.comment->len = n+1;
        *pkt.pkt.comment->data = '#';
        strcpy(pkt.pkt.comment->data+1, s);
     }
     else {
-       pkt.pkt.comment = gcry_xmalloc( sizeof *pkt.pkt.comment + n - 1 );
+       pkt.pkt.comment = xmalloc ( sizeof *pkt.pkt.comment + n - 1 );
        pkt.pkt.comment->len = n;
        strcpy(pkt.pkt.comment->data, s);
     }
     if( (rc = build_packet( out, &pkt )) )
-	log_error("build_packet(comment) failed: %s\n", gpg_errstr(rc) );
+	log_error("build_packet(comment) failed: %s\n", gpg_strerror (rc) );
     free_packet( &pkt );
     return rc;
 }
 
 
 KBNODE
-make_comment_node_from_buffer( const char *s, size_t n )
+make_comment_node_from_buffer (const char *s, size_t n)
 {
     PACKET *pkt;
 
@@ -78,8 +78,35 @@ make_comment_node_from_buffer( const char *s, size_t n )
 KBNODE
 make_comment_node( const char *s )
 {
-    return make_comment_node_from_buffer ( s, strlen (s) );
+  return make_comment_node_from_buffer (s, strlen (s));
 }
 
 
+KBNODE
+make_mpi_comment_node( const char *s, gcry_mpi_t a )
+{
+    PACKET *pkt;
+    byte *buf, *pp;
+    size_t n1, nb1;
+    size_t n = strlen(s);
+
+    nb1 = mpi_get_nbits( a );
+    if (gcry_mpi_print (GCRYMPI_FMT_PGP, NULL, 0, &n1, a))
+      BUG ();
+    /* fixme: allocate it on the stack */
+    buf = xmalloc (n1);
+    if (gcry_mpi_print (GCRYMPI_FMT_PGP, buf, n1, &n1, a))
+      BUG ();
+
+    pkt = xcalloc (1, sizeof *pkt );
+    pkt->pkttype = PKT_COMMENT;
+    pkt->pkt.comment = xmalloc ( sizeof *pkt->pkt.comment + n + 2 + n1 );
+    pkt->pkt.comment->len = n+1+2+n1;
+    pp = pkt->pkt.comment->data;
+    memcpy(pp, s, n+1);
+    memcpy(pp+n+1, buf, n1 );
+    xfree (buf);
+    return new_kbnode( pkt );
+}
+
 
