@@ -15,8 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #ifndef GPG_GLOBAL_H
@@ -27,4 +26,6 @@
 typedef struct kbnode_struct *KBNODE;
 typedef struct keydb_search_desc KEYDB_SEARCH_DESC;
 
+#include "gpg.h"
+
 #endif /*GPG_GLOBAL_H*/
