@@ -116,17 +116,12 @@ static void
 i18n_init(void)
 {
 #ifdef USE_SIMPLE_GETTEXT
-    set_gettext_file( PACKAGE );
+    set_gettext_file( PACKAGE_GT );
 #else
 #ifdef ENABLE_NLS
-#ifdef HAVE_LC_MESSAGES
-       setlocale( LC_TIME, "" );
-       setlocale( LC_MESSAGES, "" );
-#else
-       setlocale( LC_ALL, "" );
-#endif
-    bindtextdomain( PACKAGE, LOCALEDIR );
-    textdomain( PACKAGE );
+    setlocale( LC_ALL, "" );
+    bindtextdomain( PACKAGE_GT, LOCALEDIR );
+    textdomain( PACKAGE_GT );
 #endif
 #endif
 }
@@ -155,15 +150,8 @@ main( int argc, char **argv )
     opt.trust_model = TM_ALWAYS;
     opt.batch = 1;
 
-#if defined (_WIN32)
-    opt.homedir = read_w32_registry_string( NULL, "Software\\GNU\\GnuPG",
-                                            "HomeDir" );
-#else
-    opt.homedir = getenv("GNUPGHOME");
-#endif
-    if( !opt.homedir || !*opt.homedir ) {
-	opt.homedir = GNUPG_DEFAULT_HOMEDIR;
-    }
+    opt.homedir = default_homedir ();
+
     tty_no_terminal(1);
     tty_batchmode(1);
     disable_dotlock();
