@@ -1,11 +1,11 @@
 /* keydb.c - key database dispatcher
- * Copyright (C) 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
+ * Copyright (C) 2001, 2002, 2003 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -14,7 +14,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #include <config.h>
@@ -27,12 +28,13 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
+#include "gpg.h"
 #include "util.h"
 #include "options.h"
 #include "main.h" /*try_make_homedir ()*/
 #include "packet.h"
 #include "keyring.h"
-#include "keydb.h"
+#include "keydb.h" 
 #include "i18n.h"
 
 static int active_handles;
@@ -69,124 +71,6 @@ static int lock_all (KEYDB_HANDLE hd);
 static void unlock_all (KEYDB_HANDLE hd);
 
 
-/* Handle the creation of a keyring if it does not yet exist.  Take
-   into acount that other processes might have the keyring already
-   locked.  This lock check does not work if the directory itself is
-   not yet available. */
-static int
-maybe_create_keyring (char *filename, int force)
-{
-  dotlock_t lockhd = NULL;
-  IOBUF iobuf;
-  int rc;
-  mode_t oldmask;
-  char *last_slash_in_filename;
-
-  /* A quick test whether the filename already exists. */
-  if (!access (filename, F_OK))
-    return 0;
-
-  /* If we don't want to create a new file at all, there is no need to
-     go any further - bail out right here.  */
-  if (!force)
-    return G10ERR_OPEN_FILE;
-
-  /* First of all we try to create the home directory.  Note, that we
-     don't do any locking here because any sane application of gpg
-     would create the home directory by itself and not rely on gpg's
-     tricky auto-creation which is anyway only done for some home
-     directory name patterns. */
-  last_slash_in_filename = strrchr (filename, DIRSEP_C);
-  *last_slash_in_filename = 0;
-  if (access(filename, F_OK))
-    {
-      static int tried;
-
-      if (!tried)
-        {
-          tried = 1;
-          try_make_homedir (filename);
-        }
-      if (access (filename, F_OK))
-        {
-          rc = G10ERR_OPEN_FILE;
-          *last_slash_in_filename = DIRSEP_C;
-          goto leave;
-        }
-    }
-  *last_slash_in_filename = DIRSEP_C;
-
-
-  /* To avoid races with other instances of gpg trying to create or
-     update the keyring (it is removed during an update for a short
-     time), we do the next stuff in a locked state. */
-  lockhd = dotlock_create (filename, 0);
-  if (!lockhd)
-    {
-      /* A reason for this to fail is that the directory is not
-         writable. However, this whole locking stuff does not make
-         sense if this is the case. An empty non-writable directory
-         with no keyring is not really useful at all. */
-      if (opt.verbose)
-        log_info ("can't allocate lock for `%s'\n", filename );
-
-      if (!force)
-        return G10ERR_OPEN_FILE;
-      else
-        return G10ERR_GENERAL;
-    }
-
-  if ( dotlock_take (lockhd, -1) )
-    {
-      /* This is something bad.  Probably a stale lockfile.  */
-      log_info ("can't lock `%s'\n", filename );
-      rc = G10ERR_GENERAL;
-      goto leave;
-    }
-
-  /* Now the real test while we are locked. */
-  if (!access(filename, F_OK))
-    {
-      rc = 0;  /* Okay, we may access the file now.  */
-      goto leave;
-    }
-
-  /* The file does not yet exist, create it now. */
-  oldmask = umask (077);
-  if (is_secured_filename (filename))
-    {
-      iobuf = NULL;
-      errno = EPERM;
-    }
-  else
-    iobuf = iobuf_create (filename);
-  umask (oldmask);
-  if (!iobuf)
-    {
-      log_error ( _("error creating keyring `%s': %s\n"),
-                  filename, strerror(errno));
-      rc = G10ERR_OPEN_FILE;
-      goto leave;
-    }
-
-  if (!opt.quiet)
-    log_info (_("keyring `%s' created\n"), filename);
-
-  iobuf_close (iobuf);
-  /* Must invalidate that ugly cache */
-  iobuf_ioctl (NULL, 2, 0, filename);
-  rc = 0;
-
- leave:
-  if (lockhd)
-    {
-      dotlock_release (lockhd);
-      dotlock_destroy (lockhd);
-    }
-  return rc;
-}
-
-
 /*
  * Register a resource (which currently may only be a keyring file).
  * The first keyring which is added by this function is
@@ -194,25 +78,20 @@ maybe_create_keyring (char *filename, int force)
  * Note: this function may be called before secure memory is
  * available.
  * Flag 1 == force
- * Flag 2 == mark resource as primary
- * Flag 4 == This is a default resources
- * Flag 8 == Readonly
+ * Flag 2 == default
  */
 int
 keydb_add_resource (const char *url, int flags, int secret)
 {
     static int any_secret, any_public;
     const char *resname = url;
+    iobuf_t iobuf = NULL;
     char *filename = NULL;
     int force=(flags&1);
-    int read_only=!!(flags&8);
     int rc = 0;
     KeydbResourceType rt = KEYDB_RESOURCE_TYPE_NONE;
     void *token;
 
-    if (read_only)
-      force = 0;
-
     /* Do we have an URL?
      *	gnupg-ring:filename  := this is a plain keyring
      *	filename := See what is is, but create as plain keyring.
@@ -225,7 +104,7 @@ keydb_add_resource (const char *url, int flags, int secret)
 #if !defined(HAVE_DRIVE_LETTERS) && !defined(__riscos__)
 	else if (strchr (resname, ':')) {
 	    log_error ("invalid key resource URL `%s'\n", url );
-	    rc = G10ERR_GENERAL;
+	    rc = GPG_ERR_GENERAL;
 	    goto leave;
 	}
 #endif /* !HAVE_DRIVE_LETTERS && !__riscos__ */
@@ -240,7 +119,7 @@ keydb_add_resource (const char *url, int flags, int secret)
     else
 	filename = xstrdup (resname);
 
-    if (!force && !read_only)
+    if (!force)
 	force = secret? !any_secret : !any_public;
 
     /* see whether we can determine the filetype */
@@ -267,19 +146,66 @@ keydb_add_resource (const char *url, int flags, int secret)
     switch (rt) {
       case KEYDB_RESOURCE_TYPE_NONE:
 	log_error ("unknown type of key resource `%s'\n", url );
-	rc = G10ERR_GENERAL;
+	rc = GPG_ERR_GENERAL;
 	goto leave;
 
       case KEYDB_RESOURCE_TYPE_KEYRING:
-        rc = maybe_create_keyring (filename, force);
-        if (rc)
-          goto leave;
+        if (access(filename, F_OK))
+          { /* file does not exist */
+	    mode_t oldmask;
+	    char *last_slash_in_filename;
+
+            if (!force) 
+              {
+                rc = gpg_error_from_errno (errno);
+                goto leave;
+              }
+
+	    last_slash_in_filename = strrchr (filename, DIRSEP_C);
+	    *last_slash_in_filename = 0;
+	    if (access(filename, F_OK))
+              { /* On the first time we try to create the default
+		   homedir and check again. */
+                static int tried;
+                
+                if (!tried)
+                  {
+                    tried = 1;
+                    try_make_homedir (filename);
+                  }
+         	if (access (filename, F_OK))
+                  {
+                    rc = gpg_error_from_errno (errno);
+                    *last_slash_in_filename = DIRSEP_C;
+                    goto leave;
+                  }
+              }
+	    *last_slash_in_filename = DIRSEP_C;
+
+	    oldmask=umask(077);
+	    iobuf = iobuf_create (filename);
+	    umask(oldmask);
+	    if (!iobuf) 
+              {
+		log_error ( _("error creating keyring `%s': %s\n"),
+                            filename, strerror(errno));
+                rc = gpg_error_from_errno (errno);
+		goto leave;
+              }
+
+            if (!opt.quiet)
+              log_info (_("keyring `%s' created\n"), filename);
+            iobuf_close (iobuf);
+            iobuf = NULL;
+            /* must invalidate that ugly cache */
+            iobuf_ioctl (NULL, 2, 0, (char*)filename);
+          } /* end file creation */
 
         if(keyring_register_filename (filename, secret, &token))
 	  {
 	    if (used_resources >= MAX_KEYDB_RESOURCES)
-	      rc = G10ERR_RESOURCE_LIMIT;
-	    else
+	      rc = GPG_ERR_RESOURCE_LIMIT;
+	    else 
 	      {
 		if(flags&2)
 		  primary_keyring=token;
@@ -302,7 +228,7 @@ keydb_add_resource (const char *url, int flags, int secret)
 
       default:
 	log_error ("resource type of `%s' not supported\n", url);
-	rc = G10ERR_GENERAL;
+	rc = GPG_ERR_GENERAL;
 	goto leave;
     }
 
@@ -310,18 +236,7 @@ keydb_add_resource (const char *url, int flags, int secret)
 
   leave:
     if (rc)
-      {
-        /* Secret keyrings are not required in all cases.  To avoid
-           having gpg return failure we use log_info here if the
-           rewsource is a secret one and marked as default
-           resource.  */
-        if ((flags&4) && secret)
-          log_info (_("keyblock resource `%s': %s\n"),
-                    filename, g10_errstr(rc));
-        else
-          log_error (_("keyblock resource `%s': %s\n"),
-                     filename, g10_errstr(rc));
-      }
+	log_error ("keyblock resource `%s': %s\n", filename, gpg_strerror (rc));
     else if (secret)
 	any_secret = 1;
     else
@@ -338,10 +253,10 @@ keydb_new (int secret)
 {
   KEYDB_HANDLE hd;
   int i, j;
-
-  hd = xmalloc_clear (sizeof *hd);
+  
+  hd = xcalloc (1,sizeof *hd);
   hd->found = -1;
-
+  
   assert (used_resources <= MAX_KEYDB_RESOURCES);
   for (i=j=0; i < used_resources; i++)
     {
@@ -365,12 +280,12 @@ keydb_new (int secret)
         }
     }
   hd->used = j;
-
+  
   active_handles++;
   return hd;
 }
 
-void
+void 
 keydb_release (KEYDB_HANDLE hd)
 {
     int i;
@@ -409,19 +324,19 @@ keydb_get_resource_name (KEYDB_HANDLE hd)
     int idx;
     const char *s = NULL;
 
-    if (!hd)
+    if (!hd) 
         return NULL;
 
-    if ( hd->found >= 0 && hd->found < hd->used)
+    if ( hd->found >= 0 && hd->found < hd->used) 
         idx = hd->found;
-    else if ( hd->current >= 0 && hd->current < hd->used)
+    else if ( hd->current >= 0 && hd->current < hd->used) 
         idx = hd->current;
     else
         idx = 0;
 
     switch (hd->active[idx].type) {
       case KEYDB_RESOURCE_TYPE_NONE:
-        s = NULL;
+        s = NULL; 
         break;
       case KEYDB_RESOURCE_TYPE_KEYRING:
         s = keyring_get_resource_name (hd->active[idx].u.kr);
@@ -433,7 +348,7 @@ keydb_get_resource_name (KEYDB_HANDLE hd)
 
 
 
-static int
+static int 
 lock_all (KEYDB_HANDLE hd)
 {
     int i, rc = 0;
@@ -490,7 +405,7 @@ unlock_all (KEYDB_HANDLE hd)
 /*
  * Return the last found keyring.  Caller must free it.
  * The returned keyblock has the kbode flag bit 0 set for the node with
- * the public key used to locate the keyblock or flag bit 1 set for
+ * the public key used to locate the keyblock or flag bit 1 set for 
  * the user ID node.
  */
 int
@@ -499,14 +414,14 @@ keydb_get_keyblock (KEYDB_HANDLE hd, KBNODE *ret_kb)
     int rc = 0;
 
     if (!hd)
-        return G10ERR_INV_ARG;
+        return GPG_ERR_INV_ARG;
 
-    if ( hd->found < 0 || hd->found >= hd->used)
+    if ( hd->found < 0 || hd->found >= hd->used) 
         return -1; /* nothing found */
 
     switch (hd->active[hd->found].type) {
       case KEYDB_RESOURCE_TYPE_NONE:
-        rc = G10ERR_GENERAL; /* oops */
+        rc = GPG_ERR_GENERAL; /* oops */
         break;
       case KEYDB_RESOURCE_TYPE_KEYRING:
         rc = keyring_get_keyblock (hd->active[hd->found].u.kr, ret_kb);
@@ -516,7 +431,7 @@ keydb_get_keyblock (KEYDB_HANDLE hd, KBNODE *ret_kb)
     return rc;
 }
 
-/*
+/* 
  * update the current keyblock with KB
  */
 int
@@ -525,9 +440,9 @@ keydb_update_keyblock (KEYDB_HANDLE hd, KBNODE kb)
     int rc = 0;
 
     if (!hd)
-        return G10ERR_INV_ARG;
+        return GPG_ERR_INV_ARG;
 
-    if ( hd->found < 0 || hd->found >= hd->used)
+    if ( hd->found < 0 || hd->found >= hd->used) 
         return -1; /* nothing found */
 
     if( opt.dry_run )
@@ -539,7 +454,7 @@ keydb_update_keyblock (KEYDB_HANDLE hd, KBNODE kb)
 
     switch (hd->active[hd->found].type) {
       case KEYDB_RESOURCE_TYPE_NONE:
-        rc = G10ERR_GENERAL; /* oops */
+        rc = GPG_ERR_GENERAL; /* oops */
         break;
       case KEYDB_RESOURCE_TYPE_KEYRING:
         rc = keyring_update_keyblock (hd->active[hd->found].u.kr, kb);
@@ -551,8 +466,8 @@ keydb_update_keyblock (KEYDB_HANDLE hd, KBNODE kb)
 }
 
 
-/*
- * Insert a new KB into one of the resources.
+/* 
+ * Insert a new KB into one of the resources. 
  */
 int
 keydb_insert_keyblock (KEYDB_HANDLE hd, KBNODE kb)
@@ -560,18 +475,18 @@ keydb_insert_keyblock (KEYDB_HANDLE hd, KBNODE kb)
     int rc = -1;
     int idx;
 
-    if (!hd)
-        return G10ERR_INV_ARG;
+    if (!hd) 
+        return GPG_ERR_INV_ARG;
 
     if( opt.dry_run )
 	return 0;
 
-    if ( hd->found >= 0 && hd->found < hd->used)
+    if ( hd->found >= 0 && hd->found < hd->used) 
         idx = hd->found;
-    else if ( hd->current >= 0 && hd->current < hd->used)
+    else if ( hd->current >= 0 && hd->current < hd->used) 
         idx = hd->current;
     else
-        return G10ERR_GENERAL;
+        return GPG_ERR_GENERAL;
 
     rc = lock_all (hd);
     if (rc)
@@ -579,7 +494,7 @@ keydb_insert_keyblock (KEYDB_HANDLE hd, KBNODE kb)
 
     switch (hd->active[idx].type) {
       case KEYDB_RESOURCE_TYPE_NONE:
-        rc = G10ERR_GENERAL; /* oops */
+        rc = GPG_ERR_GENERAL; /* oops */
         break;
       case KEYDB_RESOURCE_TYPE_KEYRING:
         rc = keyring_insert_keyblock (hd->active[idx].u.kr, kb);
@@ -591,7 +506,7 @@ keydb_insert_keyblock (KEYDB_HANDLE hd, KBNODE kb)
 }
 
 
-/*
+/* 
  * The current keyblock will be deleted.
  */
 int
@@ -600,9 +515,9 @@ keydb_delete_keyblock (KEYDB_HANDLE hd)
     int rc = -1;
 
     if (!hd)
-        return G10ERR_INV_ARG;
+        return GPG_ERR_INV_ARG;
 
-    if ( hd->found < 0 || hd->found >= hd->used)
+    if ( hd->found < 0 || hd->found >= hd->used) 
         return -1; /* nothing found */
 
     if( opt.dry_run )
@@ -614,7 +529,7 @@ keydb_delete_keyblock (KEYDB_HANDLE hd)
 
     switch (hd->active[hd->found].type) {
       case KEYDB_RESOURCE_TYPE_NONE:
-        rc = G10ERR_GENERAL; /* oops */
+        rc = GPG_ERR_GENERAL; /* oops */
         break;
       case KEYDB_RESOURCE_TYPE_KEYRING:
         rc = keyring_delete_keyblock (hd->active[hd->found].u.kr);
@@ -629,18 +544,16 @@ keydb_delete_keyblock (KEYDB_HANDLE hd)
 /*
  * Locate the default writable key resource, so that the next
  * operation (which is only relevant for inserts) will be done on this
- * resource.
+ * resource.  
  */
 int
 keydb_locate_writable (KEYDB_HANDLE hd, const char *reserved)
 {
   int rc;
-
-  (void)reserved;
-
+  
   if (!hd)
-    return G10ERR_INV_ARG;
-
+    return GPG_ERR_INV_ARG;
+  
   rc = keydb_search_reset (hd); /* this does reset hd->current */
   if (rc)
     return rc;
@@ -664,9 +577,9 @@ keydb_locate_writable (KEYDB_HANDLE hd, const char *reserved)
 	return rc;
     }
 
-  for ( ; hd->current >= 0 && hd->current < hd->used; hd->current++)
+  for ( ; hd->current >= 0 && hd->current < hd->used; hd->current++) 
     {
-      switch (hd->active[hd->current].type)
+      switch (hd->active[hd->current].type) 
         {
         case KEYDB_RESOURCE_TYPE_NONE:
           BUG();
@@ -677,7 +590,7 @@ keydb_locate_writable (KEYDB_HANDLE hd, const char *reserved)
           break;
         }
     }
-
+  
   return -1;
 }
 
@@ -685,10 +598,10 @@ keydb_locate_writable (KEYDB_HANDLE hd, const char *reserved)
  * Rebuild the caches of all key resources.
  */
 void
-keydb_rebuild_caches (int noisy)
+keydb_rebuild_caches (void)
 {
   int i, rc;
-
+  
   for (i=0; i < used_resources; i++)
     {
       if (all_resources[i].secret)
@@ -698,10 +611,10 @@ keydb_rebuild_caches (int noisy)
         case KEYDB_RESOURCE_TYPE_NONE: /* ignore */
           break;
         case KEYDB_RESOURCE_TYPE_KEYRING:
-          rc = keyring_rebuild_cache (all_resources[i].token,noisy);
+          rc = keyring_rebuild_cache (all_resources[i].token);
           if (rc)
             log_error (_("failed to rebuild keyring cache: %s\n"),
-                       g10_errstr (rc));
+                       gpg_strerror (rc));
           break;
         }
     }
@@ -709,18 +622,18 @@ keydb_rebuild_caches (int noisy)
 
 
 
-/*
+/* 
  * Start the next search on this handle right at the beginning
  */
-int
+int 
 keydb_search_reset (KEYDB_HANDLE hd)
 {
     int i, rc = 0;
 
     if (!hd)
-        return G10ERR_INV_ARG;
+        return GPG_ERR_INV_ARG;
 
-    hd->current = 0;
+    hd->current = 0; 
     hd->found = -1;
     /* and reset all resources */
     for (i=0; !rc && i < hd->used; i++) {
@@ -732,22 +645,22 @@ keydb_search_reset (KEYDB_HANDLE hd)
             break;
         }
     }
-    return rc;
+    return rc; 
 }
 
 
-/*
+/* 
  * Search through all keydb resources, starting at the current position,
  * for a keyblock which contains one of the keys described in the DESC array.
  */
-int
+int 
 keydb_search2 (KEYDB_HANDLE hd, KEYDB_SEARCH_DESC *desc,
 	       size_t ndesc, size_t *descindex)
 {
     int rc = -1;
 
     if (!hd)
-        return G10ERR_INV_ARG;
+        return GPG_ERR_INV_ARG;
 
     while (rc == -1 && hd->current >= 0 && hd->current < hd->used) {
         switch (hd->active[hd->current].type) {
@@ -760,12 +673,12 @@ keydb_search2 (KEYDB_HANDLE hd, KEYDB_SEARCH_DESC *desc,
             break;
         }
         if (rc == -1) /* EOF -> switch to next resource */
-            hd->current++;
+            hd->current++; 
         else if (!rc)
             hd->found = hd->current;
     }
 
-    return rc;
+    return rc; 
 }
 
 int
