@@ -1,11 +1,11 @@
 /* keyring.c - keyring file handling
- * Copyright (C) 2001, 2004, 2007, 2009 Free Software Foundation, Inc.
+ * Copyright (C) 2001, 2003 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -14,7 +14,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #include <config.h>
@@ -27,6 +28,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#include "gpg.h"
 #include "util.h"
 #include "keyring.h"
 #include "packet.h"
@@ -71,7 +73,7 @@ struct keyring_handle {
   int secret;             /* this is for a secret keyring */
   struct {
     CONST_KR_NAME kr;
-    IOBUF iobuf;
+    iobuf_t iobuf;
     int eof;
     int error;
   } current;
@@ -100,7 +102,7 @@ new_offset_item (void)
 {
   struct off_item *k;
   
-  k = xmalloc_clear (sizeof *k);
+  k = xcalloc (1,sizeof *k);
   return k;
 }
 
@@ -123,7 +125,7 @@ new_offset_hash_table (void)
 {
   struct off_item **tbl;
 
-  tbl = xmalloc_clear (2048 * sizeof *tbl);
+  tbl = xcalloc (2048, sizeof *tbl);
   return tbl;
 }
 
@@ -205,16 +207,13 @@ keyring_register_filename (const char *fname, int secret, void **ptr)
 
     for (kr=kr_names; kr; kr = kr->next)
       {
-        if (same_file_p (kr->fname, fname) )
+        if ( !compare_filenames (kr->fname, fname) )
 	  {
             *ptr=kr;
 	    return 0; /* already registered */
 	  }
       }
 
-    if (secret)
-      register_secured_file (fname);
-
     kr = xmalloc (sizeof *kr + strlen (fname));
     strcpy (kr->fname, fname);
     kr->secret = !!secret;
@@ -256,7 +255,7 @@ keyring_new (void *token, int secret)
 
   assert (resource && !resource->secret == !secret);
   
-  hd = xmalloc_clear (sizeof *hd);
+  hd = xcalloc (1,sizeof *hd);
   hd->resource = resource;
   hd->secret = !!secret;
   active_handles++;
@@ -305,7 +304,7 @@ keyring_lock (KEYRING_HANDLE hd, int yes)
                 kr->lockhd = create_dotlock( kr->fname );
                 if (!kr->lockhd) {
                     log_info ("can't allocate lock for `%s'\n", kr->fname );
-                    rc = G10ERR_GENERAL;
+                    rc = GPG_ERR_GENERAL;
                 }
             }
         }
@@ -320,7 +319,7 @@ keyring_lock (KEYRING_HANDLE hd, int yes)
                 ;
             else if (make_dotlock (kr->lockhd, -1) ) {
                 log_info ("can't lock `%s'\n", kr->fname );
-                rc = G10ERR_GENERAL;
+                rc = GPG_ERR_GENERAL;
             }
             else 
                 kr->is_locked = 1;
@@ -357,7 +356,7 @@ keyring_get_keyblock (KEYRING_HANDLE hd, KBNODE *ret_kb)
     PACKET *pkt;
     int rc;
     KBNODE keyblock = NULL, node, lastnode;
-    IOBUF a;
+    iobuf_t a;
     int in_cert = 0;
     int pk_no = 0;
     int uid_no = 0;
@@ -370,16 +369,15 @@ keyring_get_keyblock (KEYRING_HANDLE hd, KBNODE *ret_kb)
         return -1; /* no successful search */
 
     a = iobuf_open (hd->found.kr->fname);
-    if (!a)
-      {
-	log_error(_("can't open `%s'\n"), hd->found.kr->fname);
-	return G10ERR_KEYRING_OPEN;
-      }
+    if (!a) {
+	log_error ("can't open `%s'\n", hd->found.kr->fname);
+	return GPG_ERR_KEYRING_OPEN;
+    }
 
     if (iobuf_seek (a, hd->found.offset) ) {
         log_error ("can't seek `%s'\n", hd->found.kr->fname);
 	iobuf_close(a);
-	return G10ERR_KEYRING_OPEN;
+	return GPG_ERR_KEYRING_OPEN;
     }
 
     pkt = xmalloc (sizeof *pkt);
@@ -389,15 +387,15 @@ keyring_get_keyblock (KEYRING_HANDLE hd, KBNODE *ret_kb)
     save_mode = set_packet_list_mode(0);
     while ((rc=parse_packet (a, pkt)) != -1) {
         hd->found.n_packets++;
-        if (rc == G10ERR_UNKNOWN_PACKET) {
+        if (gpg_err_code (rc) == GPG_ERR_UNKNOWN_PACKET) {
 	    free_packet (pkt);
 	    init_packet (pkt);
 	    continue;
 	}
 	if (rc) {  
             log_error ("keyring_get_keyblock: read error: %s\n",
-                       g10_errstr(rc) );
-            rc = G10ERR_INV_KEYRING;
+                       gpg_strerror (rc) );
+            rc = GPG_ERR_INV_KEYRING;
             break;
         }
 	if (pkt->pkttype == PKT_COMPRESSED) {
@@ -419,43 +417,38 @@ keyring_get_keyblock (KEYRING_HANDLE hd, KBNODE *ret_kb)
             if ( lastnode 
                  && lastnode->pkt->pkttype == PKT_SIGNATURE
                  && (pkt->pkt.ring_trust->sigcache & 1) ) {
-                /* This is a ring trust packet with a checked signature 
+                /* this is a ring trust packet with a checked signature 
                  * status cache following directly a signature paket.
-                 * Set the cache status into that signature packet.  */
+                 * Set the cache status into that signature packet */
                 PKT_signature *sig = lastnode->pkt->pkt.signature;
                 
                 sig->flags.checked = 1;
                 sig->flags.valid = !!(pkt->pkt.ring_trust->sigcache & 2);
             }
-            /* Reset LASTNODE, so that we set the cache status only
-             * from the ring trust packets immediately following
-             * signature packets.  */
+            /* reset lastnode, so that we set the cache status only from
+             * the ring trust packet immediately folling a signature */
             lastnode = NULL;
-	    free_packet(pkt);
-	    init_packet(pkt);
-            continue;
+        }
+        else {
+            node = lastnode = new_kbnode (pkt);
+            if (!keyblock)
+                keyblock = node;
+            else
+                add_kbnode (keyblock, node);
+
+            if ( pkt->pkttype == PKT_PUBLIC_KEY
+                 || pkt->pkttype == PKT_PUBLIC_SUBKEY
+                 || pkt->pkttype == PKT_SECRET_KEY
+                 || pkt->pkttype == PKT_SECRET_SUBKEY) {
+                if (++pk_no == hd->found.pk_no)
+                    node->flag |= 1;
+            }
+            else if ( pkt->pkttype == PKT_USER_ID) {
+                if (++uid_no == hd->found.uid_no)
+                    node->flag |= 2;
+            }
         }
 
-        node = lastnode = new_kbnode (pkt);
-        if (!keyblock)
-          keyblock = node;
-        else
-          add_kbnode (keyblock, node);
-        
-        if ( pkt->pkttype == PKT_PUBLIC_KEY
-             || pkt->pkttype == PKT_PUBLIC_SUBKEY
-             || pkt->pkttype == PKT_SECRET_KEY
-             || pkt->pkttype == PKT_SECRET_SUBKEY) 
-          {
-            if (++pk_no == hd->found.pk_no)
-              node->flag |= 1;
-          }
-        else if ( pkt->pkttype == PKT_USER_ID) 
-          {
-            if (++uid_no == hd->found.uid_no)
-              node->flag |= 2;
-          }
-        
         pkt = xmalloc (sizeof *pkt);
         init_packet(pkt);
     }
@@ -485,7 +478,7 @@ keyring_get_keyblock (KEYRING_HANDLE hd, KBNODE *ret_kb)
     /* Make sure that future search operations fail immediately when
      * we know that we are working on a invalid keyring 
      */
-    if (rc == G10ERR_INV_KEYRING)
+    if (gpg_err_code (rc) == GPG_ERR_INV_KEYRING)
         hd->current.error = rc;
 
     return rc;
@@ -503,7 +496,7 @@ keyring_update_keyblock (KEYRING_HANDLE hd, KBNODE kb)
         /* need to know the number of packets - do a dummy get_keyblock*/
         rc = keyring_get_keyblock (hd, NULL);
         if (rc) {
-            log_error ("re-reading keyblock failed: %s\n", g10_errstr (rc));
+            log_error ("re-reading keyblock failed: %s\n", gpg_strerror (rc));
             return rc;
         }
         if (!hd->found.n_packets)
@@ -547,7 +540,7 @@ keyring_insert_keyblock (KEYRING_HANDLE hd, KBNODE kb)
         fname = hd->resource? hd->resource->fname:NULL;
 
     if (!fname)
-        return G10ERR_GENERAL; 
+        return GPG_ERR_GENERAL; 
 
     /* close this one otherwise we will lose the position for
      * a next search.  Fixme: it would be better to adjust the position
@@ -579,7 +572,7 @@ keyring_delete_keyblock (KEYRING_HANDLE hd)
         /* need to know the number of packets - do a dummy get_keyblock*/
         rc = keyring_get_keyblock (hd, NULL);
         if (rc) {
-            log_error ("re-reading keyblock failed: %s\n", g10_errstr (rc));
+            log_error ("re-reading keyblock failed: %s\n", gpg_strerror (rc));
             return rc;
         }
         if (!hd->found.n_packets)
@@ -636,7 +629,7 @@ prepare_search (KEYRING_HANDLE hd)
 
     if (hd->current.kr && !hd->current.eof) {
         if ( !hd->current.iobuf )
-            return G10ERR_GENERAL; /* position invalid after a modify */
+            return GPG_ERR_GENERAL; /* position invalid after a modify */
         return 0; /* okay */
     }
 
@@ -661,11 +654,11 @@ prepare_search (KEYRING_HANDLE hd)
 
     hd->current.eof = 0;
     hd->current.iobuf = iobuf_open (hd->current.kr->fname);
-    if (!hd->current.iobuf)
-      {
-        log_error(_("can't open `%s'\n"), hd->current.kr->fname );
-        return (hd->current.error = G10ERR_OPEN_FILE);
-      }
+    if (!hd->current.iobuf) {
+        hd->current.error = gpg_error_from_errno (errno);
+        log_error ("can't open `%s'\n", hd->current.kr->fname );
+        return hd->current.error;
+    }
 
     return 0;
 }
@@ -783,7 +776,7 @@ prepare_word_match (const byte *name)
     int c;
 
     /* the original length is always enough for the pattern */
-    p = pattern = xmalloc(strlen(name)+1);
+    p = pattern = xmalloc (strlen(name)+1);
     do {
 	/* skip leading delimiters */
 	while( *name && !word_match_chars[*name] )
@@ -1078,7 +1071,7 @@ keyring_search (KEYRING_HANDLE hd, KEYDB_SEARCH_DESC *desc,
               goto found;
             break;
           default: 
-            rc = G10ERR_INV_ARG;
+            rc = GPG_ERR_INV_ARG;
             goto found;
           }
 	}
@@ -1092,7 +1085,7 @@ keyring_search (KEYRING_HANDLE hd, KEYDB_SEARCH_DESC *desc,
       for (n=any_skip?0:ndesc; n < ndesc; n++) 
         {
           if (desc[n].skipfnc
-              && desc[n].skipfnc (desc[n].skipfncvalue, aki, uid))
+              && desc[n].skipfnc (desc[n].skipfncvalue, aki))
             break;
         }
       if (n == ndesc)
@@ -1148,7 +1141,7 @@ keyring_search (KEYRING_HANDLE hd, KEYDB_SEARCH_DESC *desc,
 
 static int
 create_tmp_file (const char *template,
-                 char **r_bakfname, char **r_tmpfname, IOBUF *r_fp)
+                 char **r_bakfname, char **r_tmpfname, iobuf_t *r_fp)
 {  
   char *bakfname, *tmpfname;
   mode_t oldmask;
@@ -1191,21 +1184,15 @@ create_tmp_file (const char *template,
 
     /* Create the temp file with limited access */
     oldmask=umask(077);
-    if (is_secured_filename (tmpfname))
-      {
-        *r_fp = NULL;
-        errno = EPERM;
-      }
-    else
-      *r_fp = iobuf_create (tmpfname);
+    *r_fp = iobuf_create (tmpfname);
     umask(oldmask);
-    if (!*r_fp)
-      {
-	log_error(_("can't create `%s': %s\n"), tmpfname, strerror(errno) );
+    if (!*r_fp) {
+        int tmperr = gpg_error_from_errno (errno);
+	log_error ("can't create `%s': %s\n", tmpfname, strerror(errno) );
         xfree (tmpfname);
         xfree (bakfname);
-	return G10ERR_OPEN_FILE;
-      }
+	return tmperr;
+    }
     
     *r_bakfname = bakfname;
     *r_tmpfname = tmpfname;
@@ -1217,17 +1204,10 @@ static int
 rename_tmp_file (const char *bakfname, const char *tmpfname,
                  const char *fname, int secret )
 {
-  int rc=G10ERR_GENERAL;
-
-  /* It's a secret keyring, so let's force a fsync just to be safe on
-     filesystems that may not sync data and metadata together
-     (ext4). */
-  if(secret && iobuf_ioctl(NULL,4,0,(char*)tmpfname)!=0)
-    goto fail;
+  int rc=0;
 
   /* invalidate close caches*/
-  if(iobuf_ioctl (NULL, 2, 0, (char*)tmpfname )!=0)
-    goto fail;
+  iobuf_ioctl (NULL, 2, 0, (char*)tmpfname );
   iobuf_ioctl (NULL, 2, 0, (char*)bakfname );
   iobuf_ioctl (NULL, 2, 0, (char*)fname );
 
@@ -1239,9 +1219,10 @@ rename_tmp_file (const char *bakfname, const char *tmpfname,
 #endif
       if (rename (fname, bakfname) )
         {
+          int tmperr = gpg_error_from_errno (errno);
           log_error ("renaming `%s' to `%s' failed: %s\n",
                      fname, bakfname, strerror(errno) );
-          return G10ERR_RENAME_FILE;
+          return tmperr;
 	}
     }
   
@@ -1249,15 +1230,20 @@ rename_tmp_file (const char *bakfname, const char *tmpfname,
 #if defined(HAVE_DOSISH_SYSTEM) || defined(__riscos__)
   remove( fname );
 #endif
-  if (secret)
-    unregister_secured_file (fname);
   if (rename (tmpfname, fname) )
     {
-      log_error (_("renaming `%s' to `%s' failed: %s\n"),
+      rc = gpg_error_from_errno (errno);
+      log_error ("renaming `%s' to `%s' failed: %s\n",
                  tmpfname, fname, strerror(errno) );
-      register_secured_file (fname);
-      rc = G10ERR_RENAME_FILE;
-      goto fail;
+      if (secret)
+        {
+          log_info(_("WARNING: 2 files with confidential"
+                     " information exists.\n"));
+          log_info(_("%s is the unchanged one\n"), fname );
+          log_info(_("%s is the new one\n"), tmpfname );
+          log_info(_("Please fix this possible security flaw\n"));
+	}
+      return rc;
     }
 
   /* Now make sure the file has the same permissions as the original */
@@ -1279,22 +1265,11 @@ rename_tmp_file (const char *bakfname, const char *tmpfname,
 #endif
 
   return 0;
-
- fail:
-  if(secret)
-    {
-      log_info(_("WARNING: 2 files with confidential information exists.\n"));
-      log_info(_("%s is the unchanged one\n"), fname );
-      log_info(_("%s is the new one\n"), tmpfname );
-      log_info(_("Please fix this possible security flaw\n"));
-    }
-
-  return rc;
 }
 
 
 static int
-write_keyblock (IOBUF fp, KBNODE keyblock)
+write_keyblock (iobuf_t fp, KBNODE keyblock)
 {
   KBNODE kbctx = NULL, node;
   int rc;
@@ -1307,7 +1282,7 @@ write_keyblock (IOBUF fp, KBNODE keyblock)
       if ( (rc = build_packet (fp, node->pkt) ))
         {
           log_error ("build_packet(%d) failed: %s\n",
-                     node->pkt->pkttype, g10_errstr(rc) );
+                     node->pkt->pkttype, gpg_strerror (rc) );
           return rc;
         }
       if (node->pkt->pkttype == PKT_SIGNATURE) 
@@ -1325,8 +1300,9 @@ write_keyblock (IOBUF fp, KBNODE keyblock)
           iobuf_put (fp, 2);    /* 2 bytes */
           iobuf_put (fp, 0);    /* unused */
           if (iobuf_put (fp, cacheval)) {
+            int tmperr = gpg_error_from_errno (errno);
             log_error ("writing sigcache packet failed\n");
-            return G10ERR_WRITE_FILE;
+            return tmperr;
           }
         }
     }
@@ -1339,13 +1315,13 @@ write_keyblock (IOBUF fp, KBNODE keyblock)
  * This is only done for the public keyrings.
  */
 int
-keyring_rebuild_cache (void *token,int noisy)
+keyring_rebuild_cache (void *token)
 {
   KEYRING_HANDLE hd;
   KEYDB_SEARCH_DESC desc;
   KBNODE keyblock = NULL, node;
   const char *lastresname = NULL, *resname;
-  IOBUF tmpfp = NULL;
+  iobuf_t tmpfp = NULL;
   char *tmpfilename = NULL;
   char *bakfilename = NULL;
   int rc;
@@ -1369,9 +1345,9 @@ keyring_rebuild_cache (void *token,int noisy)
             {
               if (iobuf_close (tmpfp))
                 {
+                  rc = gpg_error_from_errno (errno);
                   log_error ("error closing `%s': %s\n",
                              tmpfilename, strerror (errno));
-                  rc = G10ERR_CLOSE_FILE;
                   goto leave;
                 }
               /* because we have switched resources, we can be sure that
@@ -1385,8 +1361,8 @@ keyring_rebuild_cache (void *token,int noisy)
           if (rc)
             goto leave;
           lastresname = resname;
-          if (noisy && !opt.quiet)
-            log_info (_("caching keyring `%s'\n"), resname);
+          if (!opt.quiet)
+            log_info (_("checking keyring `%s'\n"), resname);
           rc = create_tmp_file (resname, &bakfilename, &tmpfilename, &tmpfp);
           if (rc)
             goto leave;
@@ -1396,45 +1372,24 @@ keyring_rebuild_cache (void *token,int noisy)
       rc = keyring_get_keyblock (hd, &keyblock);
       if (rc) 
         {
-          log_error ("keyring_get_keyblock failed: %s\n", g10_errstr(rc));
-          goto leave;
-        }
-      if ( keyblock->pkt->pkttype != PKT_PUBLIC_KEY)
-        {
-          /* We had a few reports about corrupted keyrings; if we have
-             been called directly from the command line we delete such
-             a keyblock instead of bailing out.  */
-          log_error ("unexpected keyblock found (pkttype=%d)%s\n",
-                     keyblock->pkt->pkttype, noisy? " - deleted":"");
-          if (noisy)
-            continue;
-          log_info ("Hint: backup your keys and try running `%s'\n",
-                    "gpg --rebuild-keydb-caches");
-          rc = G10ERR_INV_KEYRING;
+          log_error ("keyring_get_keyblock failed: %s\n", gpg_strerror (rc));
           goto leave;
         }
+      assert (keyblock->pkt->pkttype == PKT_PUBLIC_KEY);
 
       /* check all signature to set the signature's cache flags */
       for (node=keyblock; node; node=node->next)
         {
-	  /* Note that this doesn't cache the result of a revocation
-	     issued by a designated revoker.  This is because the pk
-	     in question does not carry the revkeys as we haven't
-	     merged the key and selfsigs.  It is questionable whether
-	     this matters very much since there are very very few
-	     designated revoker revocation packets out there. */
-
           if (node->pkt->pkttype == PKT_SIGNATURE)
             {
-	      PKT_signature *sig=node->pkt->pkt.signature;
-
-	      if(!opt.no_sig_cache && sig->flags.checked && sig->flags.valid
-		 && (check_digest_algo(sig->digest_algo)
-		     || check_pubkey_algo(sig->pubkey_algo)))
-		sig->flags.checked=sig->flags.valid=0;
-	      else
-		check_key_signature (keyblock, node, NULL);
-
+	      /* Note that this doesn't cache the result of a
+		 revocation issued by a designated revoker.  This is
+		 because the pk in question does not carry the revkeys
+		 as we haven't merged the key and selfsigs.  It is
+		 questionable whether this matters very much since
+		 there are very very few designated revoker revocation
+		 packets out there. */
+              check_key_signature (keyblock, node, NULL);
               sigcount++;
             }
         }
@@ -1444,8 +1399,8 @@ keyring_rebuild_cache (void *token,int noisy)
       if (rc)
         goto leave;
 
-      if ( !(++count % 50) && noisy && !opt.quiet)
-        log_info(_("%lu keys cached so far (%lu signatures)\n"),
+      if ( !(++count % 50) && !opt.quiet)
+        log_info(_("%lu keys checked so far (%lu signatures)\n"),
                  count, sigcount );
 
     } /* end main loop */ 
@@ -1453,18 +1408,17 @@ keyring_rebuild_cache (void *token,int noisy)
     rc = 0;
   if (rc) 
     {
-      log_error ("keyring_search failed: %s\n", g10_errstr(rc));
+      log_error ("keyring_search failed: %s\n", gpg_strerror (rc));
       goto leave;
     }
-  if(noisy || opt.verbose)
-    log_info(_("%lu keys cached (%lu signatures)\n"), count, sigcount );
+  log_info(_("%lu keys checked (%lu signatures)\n"), count, sigcount );
   if (tmpfp)
     {
       if (iobuf_close (tmpfp))
         {
+          rc = gpg_error_from_errno (errno);
           log_error ("error closing `%s': %s\n",
                      tmpfilename, strerror (errno));
-          rc = G10ERR_CLOSE_FILE;
           goto leave;
         }
       /* because we have switched resources, we can be sure that
@@ -1498,15 +1452,16 @@ static int
 do_copy (int mode, const char *fname, KBNODE root, int secret,
          off_t start_offset, unsigned int n_packets )
 {
-    IOBUF fp, newfp;
+    iobuf_t fp, newfp;
     int rc=0;
     char *bakfname = NULL;
     char *tmpfname = NULL;
 
-    /* Open the source file. Because we do a rename, we have to check the 
+    /* Open the source file. Because we do a rname, we have to check the 
        permissions of the file */
     if (access (fname, W_OK))
-      return G10ERR_WRITE_FILE;
+      return gpg_error_from_errno (errno);
+
 
     fp = iobuf_open (fname);
     if (mode == 1 && !fp && errno == ENOENT) { 
@@ -1515,18 +1470,14 @@ do_copy (int mode, const char *fname, KBNODE root, int secret,
 	mode_t oldmask;
 
 	oldmask=umask(077);
-        if (!secret && is_secured_filename (fname)) {
-            newfp = NULL;
-            errno = EPERM;
-        }
-        else
-            newfp = iobuf_create (fname);
+	newfp = iobuf_create (fname);
 	umask(oldmask);
-	if( !newfp )
-	  {
-	    log_error (_("can't create `%s': %s\n"), fname, strerror(errno));
-	    return G10ERR_OPEN_FILE;
-	  }
+	if( !newfp ) {
+            int tmperr = gpg_error_from_errno (errno);
+	    log_error (_("%s: can't create: %s\n"),
+                       fname, strerror(errno));
+	    return tmperr;
+	}
 	if( !opt.quiet )
 	    log_info(_("%s: keyring created\n"), fname );
 
@@ -1534,43 +1485,38 @@ do_copy (int mode, const char *fname, KBNODE root, int secret,
 	while ( (node = walk_kbnode( root, &kbctx, 0 )) ) {
 	    if( (rc = build_packet( newfp, node->pkt )) ) {
 		log_error("build_packet(%d) failed: %s\n",
-			    node->pkt->pkttype, g10_errstr(rc) );
+			    node->pkt->pkttype, gpg_strerror (rc) );
 		iobuf_cancel(newfp);
-		return G10ERR_WRITE_FILE;
+		return rc;
 	    }
 	}
-	if( iobuf_close(newfp) ) {
+	if (iobuf_close(newfp)) {
+            int tmperr = gpg_error_from_errno (errno);
 	    log_error ("%s: close failed: %s\n", fname, strerror(errno));
-	    return G10ERR_CLOSE_FILE;
+	    return tmperr;
 	}
 	return 0; /* ready */
     }
 
-    if( !fp )
-      {
-	log_error(_("can't open `%s': %s\n"), fname, strerror(errno) );
-	rc = G10ERR_OPEN_FILE;
+    if( !fp ) {
+        rc = gpg_error_from_errno (errno);
+	log_error ("%s: can't open: %s\n", fname, strerror(errno) );
 	goto leave;
-      }
+    }
 
-    /* Create the new file.  */
+    /* create the new file */
     rc = create_tmp_file (fname, &bakfname, &tmpfname, &newfp);
     if (rc) {
 	iobuf_close(fp);
 	goto leave;
     }
-    if (secret)
-      register_secured_file (tmpfname);
-
     if( mode == 1 ) { /* insert */
 	/* copy everything to the new file */
 	rc = copy_all_packets (fp, newfp);
 	if( rc != -1 ) {
 	    log_error("%s: copy to `%s' failed: %s\n",
-		      fname, tmpfname, g10_errstr(rc) );
+		      fname, tmpfname, gpg_strerror (rc) );
 	    iobuf_close(fp);
-            if (secret)
-              unregister_secured_file (tmpfname);
 	    iobuf_cancel(newfp);
 	    goto leave;
 	}
@@ -1582,10 +1528,8 @@ do_copy (int mode, const char *fname, KBNODE root, int secret,
 	rc = copy_some_packets( fp, newfp, start_offset );
 	if( rc ) { /* should never get EOF here */
 	    log_error ("%s: copy to `%s' failed: %s\n",
-                       fname, tmpfname, g10_errstr(rc) );
+                       fname, tmpfname, gpg_strerror (rc) );
 	    iobuf_close(fp);
-            if (secret)
-              unregister_secured_file (tmpfname);
 	    iobuf_cancel(newfp);
 	    goto leave;
 	}
@@ -1594,10 +1538,8 @@ do_copy (int mode, const char *fname, KBNODE root, int secret,
 	rc = skip_some_packets( fp, n_packets );
 	if( rc ) {
 	    log_error("%s: skipping %u packets failed: %s\n",
-			    fname, n_packets, g10_errstr(rc));
+			    fname, n_packets, gpg_strerror (rc));
 	    iobuf_close(fp);
-            if (secret)
-              unregister_secured_file (tmpfname);
 	    iobuf_cancel(newfp);
 	    goto leave;
 	}
@@ -1607,8 +1549,6 @@ do_copy (int mode, const char *fname, KBNODE root, int secret,
         rc = write_keyblock (newfp, root);
         if (rc) {
           iobuf_close(fp);
-          if (secret)
-            unregister_secured_file (tmpfname);
           iobuf_cancel(newfp);
           goto leave;
         }
@@ -1619,10 +1559,8 @@ do_copy (int mode, const char *fname, KBNODE root, int secret,
 	rc = copy_all_packets( fp, newfp );
 	if( rc != -1 ) {
 	    log_error("%s: copy to `%s' failed: %s\n",
-		      fname, tmpfname, g10_errstr(rc) );
+		      fname, tmpfname, gpg_strerror (rc) );
 	    iobuf_close(fp);
-            if (secret)
-              unregister_secured_file (tmpfname);
 	    iobuf_cancel(newfp);
 	    goto leave;
 	}
@@ -1631,20 +1569,20 @@ do_copy (int mode, const char *fname, KBNODE root, int secret,
 
     /* close both files */
     if( iobuf_close(fp) ) {
+        rc = gpg_error_from_errno (errno);
 	log_error("%s: close failed: %s\n", fname, strerror(errno) );
-	rc = G10ERR_CLOSE_FILE;
 	goto leave;
     }
     if( iobuf_close(newfp) ) {
+        rc = gpg_error_from_errno (errno);
 	log_error("%s: close failed: %s\n", tmpfname, strerror(errno) );
-	rc = G10ERR_CLOSE_FILE;
 	goto leave;
     }
 
     rc = rename_tmp_file (bakfname, tmpfname, fname, secret);
 
   leave:
-    xfree(bakfname);
-    xfree(tmpfname);
+    xfree (bakfname);
+    xfree (tmpfname);
     return rc;
 }
