@@ -15,8 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #ifndef GPG_KEYRING_H
@@ -24,6 +23,7 @@
 
 #include "global.h"
 
+
 typedef struct keyring_handle *KEYRING_HANDLE;
 
 int keyring_register_filename (const char *fname, int secret, void **ptr);
@@ -41,6 +41,6 @@ int keyring_delete_keyblock (KEYRING_HANDLE hd);
 int keyring_search_reset (KEYRING_HANDLE hd);
 int keyring_search (KEYRING_HANDLE hd, KEYDB_SEARCH_DESC *desc,
 		    size_t ndesc, size_t *descindex);
-int keyring_rebuild_cache (void *token,int noisy);
+int keyring_rebuild_cache (void *);
 
 #endif /*GPG_KEYRING_H*/
