@@ -1,6 +1,6 @@
 /* packet.h - packet definitions
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003,
- *               2004 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 
+ *               2003  Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -22,9 +22,12 @@
 #ifndef G10_PACKET_H
 #define G10_PACKET_H
 
+#include "gpg.h"
+#include <gcrypt.h>
+
 #include "types.h"
-#include "iobuf.h"
-#include "mpi.h"
+#include "../common/iobuf.h"
+#include "../jnlib/strlist.h"
 #include "cipher.h"
 #include "filter.h"
 #include "global.h"
@@ -96,7 +99,7 @@ typedef struct {
     byte    version;
     byte    pubkey_algo;    /* algorithm used for public key scheme */
     byte    throw_keyid;
-    MPI     data[PUBKEY_MAX_NENC];
+    gcry_mpi_t     data[PUBKEY_MAX_NENC];
 } PKT_pubkey_enc;
 
 
@@ -122,16 +125,16 @@ struct revocation_key {
 };
 
 typedef struct {
+    ulong   local_id;	    /* internal use, valid if > 0 */
     struct {
 	unsigned checked:1; /* signature has been checked */
 	unsigned valid:1;   /* signature is good (if checked is set) */
-        unsigned chosen_selfsig:1; /* a selfsig that is the chosen one */
 	unsigned unknown_critical:1;
         unsigned exportable:1;
         unsigned revocable:1;
         unsigned policy_url:1; /* At least one policy URL is present */
-        unsigned notation:1; /* At least one notation is present */
-        unsigned pref_ks:1;  /* At least one preferred keyserver is present */
+        unsigned notation:1;   /* At least one notation is present */
+        unsigned pref_ks:1;    /* At least one preferred keyserver is present */
         unsigned expired:1;
     } flags;
     u32     keyid[2];	    /* 64 bit keyid */
@@ -150,7 +153,7 @@ typedef struct {
     subpktarea_t *hashed;    /* all subpackets with hashed  data (v4 only) */
     subpktarea_t *unhashed;  /* ditto for unhashed data */
     byte digest_start[2];   /* first 2 bytes of the digest */
-    MPI  data[PUBKEY_MAX_NSIG];
+    gcry_mpi_t  data[PUBKEY_MAX_NSIG];
 } PKT_signature;
 
 #define ATTRIB_IMAGE 1
@@ -186,27 +189,17 @@ typedef struct {
     char name[1];
 } PKT_user_id;
 
-struct revoke_info
-{
-  /* revoked at this date */
-  u32 date;
-  /* the keyid of the revoking key (selfsig or designated revoker) */
-  u32 keyid[2];
-  /* the algo of the revoking key */
-  byte algo;
-};
 
 /****************
  * Note about the pkey/skey elements:  We assume that the secret keys
  * has the same elemts as the public key at the begin of the array, so
  * that npkey < nskey and it is possible to compare the secret and
- * public keys by comparing the first npkey elements of pkey againts skey.
+ * public keys by comparing the first npkey elements of pkey against skey.
  */
 typedef struct {
     u32     timestamp;	    /* key made */
     u32     expiredate;     /* expires at this date or 0 if not at all */
     u32     max_expiredate; /* must not expire past this date */
-    struct revoke_info revoked;
     byte    hdrbytes;	    /* number of header bytes */
     byte    version;
     byte    selfsigversion; /* highest version of all of the self-sigs */
@@ -215,13 +208,10 @@ typedef struct {
     byte    req_usage;      /* hack to pass a request to getkey() */
     byte    req_algo;       /* Ditto */
     u32     has_expired;    /* set to the expiration date if expired */ 
-    int     is_revoked;     /* key has been revoked, 1 if by the
-			       owner, 2 if by a designated revoker */
-    int     maybe_revoked;  /* a designated revocation is present, but
-			       without the key to check it */
+    int     is_revoked;     /* key has been revoked */
     int     is_valid;       /* key (especially subkey) is valid */
     int     dont_cache;     /* do not cache this */
-    byte    backsig;        /* 0=none, 1=bad, 2=good */
+    ulong   local_id;	    /* internal use, valid if > 0 */
     u32     main_keyid[2];  /* keyid of the primary key */
     u32     keyid[2];	    /* calculated by keyid_from_pk() */
     byte    is_primary;
@@ -235,7 +225,7 @@ typedef struct {
     byte    trust_depth;
     byte    trust_value;
     const byte *trust_regexp;
-    MPI     pkey[PUBKEY_MAX_NPKEY];
+    gcry_mpi_t     pkey[PUBKEY_MAX_NPKEY];
 } PKT_public_key;
 
 /* Evaluates as true if the pk is disabled, and false if it isn't.  If
@@ -269,7 +259,7 @@ typedef struct {
 	byte ivlen;  /* used length of the iv */
 	byte iv[16]; /* initialization vector for CFB mode */
     } protect;
-    MPI skey[PUBKEY_MAX_NSKEY];
+    gcry_mpi_t skey[PUBKEY_MAX_NSKEY];
     u16 csum;		/* checksum */
 } PKT_secret_key;
 
@@ -283,16 +273,15 @@ typedef struct {
     u32  len;		  /* reserved */
     byte  new_ctb;
     byte  algorithm;
-    IOBUF buf;		  /* IOBUF reference */
+    iobuf_t buf;		  /* iobuf_t reference */
 } PKT_compressed;
 
 typedef struct {
     u32  len;		  /* length of encrypted data */
     int  extralen;        /* this is (blocksize+2) */
     byte new_ctb;	  /* uses a new CTB */
-    byte is_partial;      /* partial length encoded */
     byte mdc_method;	  /* > 0: integrity protected encrypted data packet */
-    IOBUF buf;		  /* IOBUF reference */
+    iobuf_t buf;		  /* iobuf_t reference */
 } PKT_encrypted;
 
 typedef struct {
@@ -306,7 +295,7 @@ typedef struct {
 
 typedef struct {
     u32  len;		  /* length of encrypted data */
-    IOBUF buf;		  /* IOBUF reference */
+    iobuf_t buf;		  /* iobuf_t reference */
     byte new_ctb;
     byte is_partial;      /* partial length encoded */
     int mode;
@@ -375,32 +364,30 @@ typedef enum {
     SIGSUBPKT_REVOC_REASON =29, /* reason for revocation */
     SIGSUBPKT_FEATURES     =30, /* feature flags */
 
-    SIGSUBPKT_SIGNATURE    =32, /* embedded signature */
-
     SIGSUBPKT_FLAG_CRITICAL=128
 } sigsubpkttype_t;
 
 
 /*-- mainproc.c --*/
-int proc_packets( void *ctx, IOBUF a );
-int proc_signature_packets( void *ctx, IOBUF a,
+int proc_packets( void *ctx, iobuf_t a );
+int proc_signature_packets( void *ctx, iobuf_t a,
 			    STRLIST signedfiles, const char *sigfile );
-int proc_encryption_packets( void *ctx, IOBUF a );
-int list_packets( IOBUF a );
+int proc_encryption_packets( void *ctx, iobuf_t a );
+int list_packets( iobuf_t a );
 
 /*-- parse-packet.c --*/
 int set_packet_list_mode( int mode );
 
 #if DEBUG_PARSE_PACKET
-int dbg_search_packet( IOBUF inp, PACKET *pkt, off_t *retpos, int with_uid,
+int dbg_search_packet( iobuf_t inp, PACKET *pkt, off_t *retpos, int with_uid,
                        const char* file, int lineno  );
-int dbg_parse_packet( IOBUF inp, PACKET *ret_pkt,
+int dbg_parse_packet( iobuf_t inp, PACKET *ret_pkt,
                       const char* file, int lineno );
-int dbg_copy_all_packets( IOBUF inp, IOBUF out,
+int dbg_copy_all_packets( iobuf_t inp, iobuf_t out,
                           const char* file, int lineno  );
-int dbg_copy_some_packets( IOBUF inp, IOBUF out, off_t stopoff,
+int dbg_copy_some_packets( iobuf_t inp, iobuf_t out, off_t stopoff,
                            const char* file, int lineno  );
-int dbg_skip_some_packets( IOBUF inp, unsigned n,
+int dbg_skip_some_packets( iobuf_t inp, unsigned n,
                            const char* file, int lineno	);
 #define search_packet( a,b,c,d )   \
              dbg_search_packet( (a), (b), (c), (d), __FILE__, __LINE__ )
@@ -413,15 +400,13 @@ int dbg_skip_some_packets( IOBUF inp, unsigned n,
 #define skip_some_packets( a,b ) \
              dbg_skip_some_packets((a),(b), __FILE__, __LINE__ )
 #else
-int search_packet( IOBUF inp, PACKET *pkt, off_t *retpos, int with_uid );
-int parse_packet( IOBUF inp, PACKET *ret_pkt);
-int copy_all_packets( IOBUF inp, IOBUF out );
-int copy_some_packets( IOBUF inp, IOBUF out, off_t stopoff );
-int skip_some_packets( IOBUF inp, unsigned n );
+int search_packet( iobuf_t inp, PACKET *pkt, off_t *retpos, int with_uid );
+int parse_packet( iobuf_t inp, PACKET *ret_pkt);
+int copy_all_packets( iobuf_t inp, iobuf_t out );
+int copy_some_packets( iobuf_t inp, iobuf_t out, off_t stopoff );
+int skip_some_packets( iobuf_t inp, unsigned n );
 #endif
 
-int parse_signature( IOBUF inp, int pkttype, unsigned long pktlen,
-		     PKT_signature *sig );
 const byte *enum_sig_subpkt ( const subpktarea_t *subpkts,
                               sigsubpkttype_t reqtype,
                               size_t *ret_n, int *start, int *critical );
@@ -440,8 +425,9 @@ PACKET *create_gpg_control ( ctrlpkttype_t type,
                              size_t datalen );
 
 /*-- build-packet.c --*/
-int build_packet( IOBUF inp, PACKET *pkt );
+int build_packet( iobuf_t inp, PACKET *pkt );
 u32 calc_packet_length( PACKET *pkt );
+void hash_public_key( MD_HANDLE md, PKT_public_key *pk );
 void build_sig_subpkt( PKT_signature *sig, sigsubpkttype_t type,
 			const byte *buffer, size_t buflen );
 void build_sig_subpkt_from_sig( PKT_signature *sig );
@@ -492,19 +478,19 @@ int get_override_session_key( DEK *dek, const char *string );
 
 /*-- compress.c --*/
 int handle_compressed( void *ctx, PKT_compressed *cd,
-		       int (*callback)(IOBUF, void *), void *passthru );
+		       int (*callback)(iobuf_t, void *), void *passthru );
 
 /*-- encr-data.c --*/
 int decrypt_data( void *ctx, PKT_encrypted *ed, DEK *dek );
 
 /*-- plaintext.c --*/
 int handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
-					int nooutput, int clearsig );
+                      int nooutput, int clearsig, int *create_failed );
 int ask_for_detached_datafile( MD_HANDLE md, MD_HANDLE md2,
 			       const char *inname, int textmode );
 
 /*-- comment.c --*/
-int write_comment( IOBUF out, const char *s );
+int write_comment( iobuf_t out, const char *s );
 
 /*-- sign.c --*/
 int make_keysig_packet( PKT_signature **ret_sig, PKT_public_key *pk,
