@@ -15,13 +15,13 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #include <config.h>
 #include <stdio.h>
 
+#include "gpg.h"
 #include "iobuf.h"
 #include "filter.h"
 #include "status.h"
@@ -33,7 +33,7 @@
  */
 int
 progress_filter (void *opaque, int control,
-		 IOBUF a, byte *buf, size_t *ret_len)
+		 iobuf_t a, byte *buf, size_t *ret_len)
 {
   int rc = 0;
   progress_filter_context_t *pfx = opaque;
@@ -96,7 +96,7 @@ progress_filter (void *opaque, int control,
 }
 
 void
-handle_progress (progress_filter_context_t *pfx, IOBUF inp, const char *name)
+handle_progress (progress_filter_context_t *pfx, iobuf_t inp, const char *name)
 {
   off_t filesize = 0;
 
@@ -106,8 +106,8 @@ handle_progress (progress_filter_context_t *pfx, IOBUF inp, const char *name)
   if (!is_status_enabled ())
     return;
 
-  if ( !iobuf_is_pipe_filename (name) && *name )
-    filesize = iobuf_get_filelength (inp, NULL);
+  if (name && *name && !(*name == '-' && !name[1]))
+    filesize = iobuf_get_filelength (inp);
   else if (opt.set_filesize)
     filesize = opt.set_filesize;
 
