@@ -1,12 +1,12 @@
 /* sig-check.c -  Check a signature
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
- *               2007 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002,
+ *               2003  Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,7 +15,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #include <config.h>
@@ -23,6 +24,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
+
+#include "gpg.h"
 #include "util.h"
 #include "packet.h"
 #include "memory.h"
@@ -33,12 +36,14 @@
 #include "status.h"
 #include "i18n.h"
 #include "options.h"
+#include "pkglue.h"
 
 struct cmp_help_context_s {
     PKT_signature *sig;
     MD_HANDLE md;
 };
 
+
 static int do_check( PKT_public_key *pk, PKT_signature *sig, MD_HANDLE digest,
 		     int *r_expired, int *r_revoked, PKT_public_key *ret_pk);
 
@@ -54,66 +59,31 @@ signature_check( PKT_signature *sig, MD_HANDLE digest )
 }
 
 int
-signature_check2( PKT_signature *sig, MD_HANDLE digest, u32 *r_expiredate,
+signature_check2( PKT_signature *sig, MD_HANDLE digest, u32 *r_expiredate, 
 		  int *r_expired, int *r_revoked, PKT_public_key *ret_pk )
 {
-    PKT_public_key *pk = xmalloc_clear( sizeof *pk );
+    PKT_public_key *pk = xcalloc (1, sizeof *pk );
     int rc=0;
 
-    if( (rc=check_digest_algo(sig->digest_algo)) )
-      ; /* we don't have this digest */
-    else if((rc=check_pubkey_algo(sig->pubkey_algo)))
-      ; /* we don't have this pubkey algo */
-    else if(!md_algo_present(digest,sig->digest_algo))
-      {
-	/* Sanity check that the md has a context for the hash that the
-	   sig is expecting.  This can happen if a onepass sig header does
-	   not match the actual sig, and also if the clearsign "Hash:"
-	   header is missing or does not match the actual sig. */
+    /* Sanity check that the md has a context for the hash that the
+       sig is expecting.  This can happen if a onepass sig header does
+       not match the actual sig, and also if the clearsign "Hash:"
+       header is missing or does not match the actual sig. */
 
+    if(!gcry_md_is_enabled (digest,sig->digest_algo)) {
         log_info(_("WARNING: signature digest conflict in message\n"));
-	rc=G10ERR_GENERAL;
-      }
+	rc=GPG_ERR_GENERAL;
+    }
     else if( get_pubkey( pk, sig->keyid ) )
-	rc = G10ERR_NO_PUBKEY;
+	rc = GPG_ERR_NO_PUBKEY;
     else if(!pk->is_valid && !pk->is_primary)
-        rc=G10ERR_BAD_PUBKEY; /* you cannot have a good sig from an
+        rc=GPG_ERR_BAD_PUBKEY; /* you cannot have a good sig from an
 				 invalid subkey */
-    else
-      {
-        if(r_expiredate)
-	  *r_expiredate = pk->expiredate;
-
+    else {
+        if (r_expiredate)
+	    *r_expiredate = pk->expiredate;
 	rc = do_check( pk, sig, digest, r_expired, r_revoked, ret_pk );
-
-	/* Check the backsig.  This is a 0x19 signature from the
-	   subkey on the primary key.  The idea here is that it should
-	   not be possible for someone to "steal" subkeys and claim
-	   them as their own.  The attacker couldn't actually use the
-	   subkey, but they could try and claim ownership of any
-	   signaures issued by it. */
-	if(rc==0 && !pk->is_primary && pk->backsig<2)
-	  {
-	    if(pk->backsig==0)
-	      {
-		log_info(_("WARNING: signing subkey %s is not"
-			   " cross-certified\n"),keystr_from_pk(pk));
-		log_info(_("please see %s for more information\n"),
-			 "https://gnupg.org/faq/subkey-cross-certify.html");
-		/* --require-cross-certification makes this warning an
-                     error.  TODO: change the default to require this
-                     after more keys have backsigs. */
-		if(opt.flags.require_cross_cert)
-		  rc=G10ERR_GENERAL;
-	      }
-	    else if(pk->backsig==1)
-	      {
-		log_info(_("WARNING: signing subkey %s has an invalid"
-			   " cross-certification\n"),keystr_from_pk(pk));
-		rc=G10ERR_GENERAL;
-	      }
-	  }
-      }
+    }
 
     free_public_key( pk );
 
@@ -124,49 +94,38 @@ signature_check2( PKT_signature *sig, MD_HANDLE digest, u32 *r_expiredate,
 	 * and the timestamp, but the drawback of this is, that it is
 	 * not possible to sign more than one identical document within
 	 * one second.	Some remote batch processing applications might
-	 * like this feature here.
-         *
-         * Note that before 1.4.10, we used RIPE-MD160 for the hash
-         * and accidently didn't include the timestamp and algorithm
-         * information in the hash.  Given that this feature is not
-         * commonly used and that a replay attacks detection should
-         * not solely be based on this feature (because it does not
-         * work with RSA), we take the freedom and switch to SHA-1
-         * with 1.4.10 to take advantage of hardware supported SHA-1
-         * implementations and to match the 2.0.10 behaviour.  We also
-         * include the missing information in the hash.  Note also the
-         * SIG_ID as computed by gpg 1.x and gpg 2.x didn't matched
-         * either because 2.x used to print MPIs not in PGP format.  */
-	MD_HANDLE md;
+	 * like this feature here */
+	gcry_md_hd_t md;
 	u32 a = sig->timestamp;
 	int i, nsig = pubkey_get_nsig( sig->pubkey_algo );
 	byte *p, *buffer;
 
-	md = md_open (DIGEST_ALGO_SHA1, 0);
-	md_putc (md, sig->pubkey_algo);
-	md_putc (md, sig->digest_algo);
-	md_putc (md, (a >> 24) & 0xff);
-	md_putc (md, (a >> 16) & 0xff);
-	md_putc (md, (a >>	8) & 0xff);
-	md_putc (md,  a	& 0xff);
+	gcry_md_open (&md, GCRY_MD_RMD160, 0);
+	gcry_md_putc( digest, sig->pubkey_algo );
+	gcry_md_putc( digest, sig->digest_algo );
+	gcry_md_putc( digest, (a >> 24) & 0xff );
+	gcry_md_putc( digest, (a >> 16) & 0xff );
+	gcry_md_putc( digest, (a >>  8) & 0xff );
+	gcry_md_putc( digest,  a        & 0xff );
 	for(i=0; i < nsig; i++ ) {
-	    unsigned n = mpi_get_nbits( sig->data[i]);
+	    size_t n;
+            unsigned char *tmp;
 
-	    md_putc( md, n>>8);
-	    md_putc( md, n );
-	    p = mpi_get_buffer( sig->data[i], &n, NULL );
-	    md_write( md, p, n );
-	    xfree(p);
+	    if (gcry_mpi_aprint (GCRYMPI_FMT_USG, &tmp, &n, sig->data[i]))
+		BUG();
+            
+	    gcry_md_write (md, tmp, n);
+	    xfree (tmp);
 	}
-	md_final( md );
-	p = make_radix64_string( md_read( md, 0 ), 20 );
-	buffer = xmalloc( strlen(p) + 60 );
+	gcry_md_final( md );
+	p = make_radix64_string( gcry_md_read( md, 0 ), 20 );
+	buffer = xmalloc ( strlen(p) + 60 );
 	sprintf( buffer, "%s %s %lu",
 		 p, strtimestamp( sig->timestamp ), (ulong)sig->timestamp );
 	write_status_text( STATUS_SIG_ID, buffer );
-	xfree(buffer);
-	xfree(p);
-	md_close(md);
+	xfree (buffer);
+	xfree (p);
+	gcry_md_close(md);
     }
 
     return rc;
@@ -175,55 +134,58 @@ signature_check2( PKT_signature *sig, MD_HANDLE digest, u32 *r_expiredate,
 
 static int
 do_check_messages( PKT_public_key *pk, PKT_signature *sig,
-		   int *r_expired, int *r_revoked )
+                   int *r_expired, int *r_revoked )
 {
     u32 cur_time;
 
-    if(r_expired)
+    if (r_expired)
       *r_expired = 0;
-    if(r_revoked)
+    if (r_revoked)
       *r_revoked = 0;
+    if( pk->version == 4 && pk->pubkey_algo == PUBKEY_ALGO_ELGAMAL_E ) {
+	log_info(_("key %08lX: this is a PGP generated "
+		   "ElGamal key which is NOT secure for signatures!\n"),
+ 		  (ulong)keyid_from_pk(pk,NULL));
+	return GPG_ERR_PUBKEY_ALGO;
+    }
 
-    if( pk->timestamp > sig->timestamp )
-      {
+    if( pk->timestamp > sig->timestamp ) {
 	ulong d = pk->timestamp - sig->timestamp;
-	log_info(d==1
-		 ?_("public key %s is %lu second newer than the signature\n")
-		 :_("public key %s is %lu seconds newer than the signature\n"),
-		 keystr_from_pk(pk),d );
+	log_info( d==1
+	     ? _("public key %08lX is %lu second newer than the signature\n")
+	     : _("public key %08lX is %lu seconds newer than the signature\n"),
+	        (ulong)keyid_from_pk(pk,NULL),d );
 	if( !opt.ignore_time_conflict )
-	  return G10ERR_TIME_CONFLICT; /* pubkey newer than signature */
-      }
+	    return GPG_ERR_TIME_CONFLICT; /* pubkey newer than signature */
+    }
 
     cur_time = make_timestamp();
-    if( pk->timestamp > cur_time )
-      {
+    if( pk->timestamp > cur_time ) {
 	ulong d = pk->timestamp - cur_time;
-	log_info( d==1
-		  ? _("key %s was created %lu second"
-		      " in the future (time warp or clock problem)\n")
-		  : _("key %s was created %lu seconds"
-		      " in the future (time warp or clock problem)\n"),
-		  keystr_from_pk(pk),d );
+	log_info( d==1 ? _("key %08lX has been created %lu second "
+			   "in future (time warp or clock problem)\n")
+		       : _("key %08lX has been created %lu seconds "
+			   "in future (time warp or clock problem)\n"),
+		       (ulong)keyid_from_pk(pk,NULL),d );
 	if( !opt.ignore_time_conflict )
-	  return G10ERR_TIME_CONFLICT;
-      }
+	    return GPG_ERR_TIME_CONFLICT;
+    }
 
-    /* Check whether the key has expired.  We check the has_expired
-       flag which is set after a full evaluation of the key (getkey.c)
-       as well as a simple compare to the current time in case the
-       merge has for whatever reasons not been done.  */
-    if (pk->has_expired || (pk->expiredate && pk->expiredate < cur_time)) {
+    if( pk->expiredate && pk->expiredate < cur_time ) {
         char buf[11];
-        if (opt.verbose)
-	  log_info(_("NOTE: signature key %s expired %s\n"),
-		   keystr_from_pk(pk), asctimestamp( pk->expiredate ) );
+        if (opt.verbose) {
+	    u32 tmp_kid[2];
+
+	    keyid_from_pk( pk, tmp_kid );
+            log_info(_("NOTE: signature key %08lX expired %s\n"),
+                     (ulong)tmp_kid[1], asctimestamp( pk->expiredate ) );
+        }
 	/* SIGEXPIRED is deprecated.  Use KEYEXPIRED. */
 	sprintf(buf,"%lu",(ulong)pk->expiredate);
 	write_status_text(STATUS_KEYEXPIRED,buf);
 	write_status(STATUS_SIGEXPIRED);
-	if(r_expired)
-	  *r_expired = 1;
+        if (r_expired)
+          *r_expired = 1;
     }
 
     if(pk->is_revoked && r_revoked)
@@ -237,43 +199,49 @@ static int
 do_check( PKT_public_key *pk, PKT_signature *sig, MD_HANDLE digest,
 	  int *r_expired, int *r_revoked, PKT_public_key *ret_pk )
 {
-    MPI result = NULL;
+    gcry_mpi_t result = NULL;
     int rc=0;
+    struct cmp_help_context_s ctx;
 
     if( (rc=do_check_messages(pk,sig,r_expired,r_revoked)) )
         return rc;
+    if( (rc=gcry_md_test_algo(sig->digest_algo)) )
+	return rc;
+    if( (rc=gcry_pk_test_algo(sig->pubkey_algo)) )
+	return rc;
 
-    /* make sure the digest algo is enabled (in case of a detached signature)*/
-    md_enable( digest, sig->digest_algo );
+    /* make sure the digest algo is enabled (in case of a detached
+       signature)*/
+    gcry_md_enable( digest, sig->digest_algo );
 
     /* complete the digest */
     if( sig->version >= 4 )
-	md_putc( digest, sig->version );
-    md_putc( digest, sig->sig_class );
+	gcry_md_putc( digest, sig->version );
+    gcry_md_putc( digest, sig->sig_class );
     if( sig->version < 4 ) {
 	u32 a = sig->timestamp;
-	md_putc( digest, (a >> 24) & 0xff );
-	md_putc( digest, (a >> 16) & 0xff );
-	md_putc( digest, (a >>	8) & 0xff );
-	md_putc( digest,  a	   & 0xff );
+	gcry_md_putc( digest, (a >> 24) & 0xff );
+	gcry_md_putc( digest, (a >> 16) & 0xff );
+	gcry_md_putc( digest, (a >>	8) & 0xff );
+	gcry_md_putc( digest,  a	   & 0xff );
     }
     else {
 	byte buf[6];
 	size_t n;
-	md_putc( digest, sig->pubkey_algo );
-	md_putc( digest, sig->digest_algo );
+	gcry_md_putc( digest, sig->pubkey_algo );
+	gcry_md_putc( digest, sig->digest_algo );
 	if( sig->hashed ) {
 	    n = sig->hashed->len;
-            md_putc (digest, (n >> 8) );
-            md_putc (digest,  n       );
-	    md_write (digest, sig->hashed->data, n);
+            gcry_md_putc (digest, (n >> 8) );
+            gcry_md_putc (digest,  n       );
+	    gcry_md_write (digest, sig->hashed->data, n);
 	    n += 6;
 	}
 	else {
 	  /* Two octets for the (empty) length of the hashed
              section. */
-          md_putc (digest, 0);
-	  md_putc (digest, 0);
+          gcry_md_putc (digest, 0);
+	  gcry_md_putc (digest, 0);
 	  n = 6;
 	}
 	/* add some magic */
@@ -283,54 +251,40 @@ do_check( PKT_public_key *pk, PKT_signature *sig, MD_HANDLE digest,
 	buf[3] = n >> 16;
 	buf[4] = n >>  8;
 	buf[5] = n;
-	md_write( digest, buf, 6 );
+	gcry_md_write( digest, buf, 6 );
     }
-    md_final( digest );
+    gcry_md_final (digest);
 
-    result = encode_md_value( pk, NULL, digest, sig->digest_algo );
+    result = encode_md_value( pk->pubkey_algo, digest, sig->digest_algo,
+			      mpi_get_nbits(pk->pkey[0]), 0 );
     if (!result)
-        return G10ERR_GENERAL;
-    rc = pubkey_verify( pk->pubkey_algo, result, sig->data, pk->pkey );
-    mpi_free( result );
+        return GPG_ERR_GENERAL;
+    ctx.sig = sig;
+    ctx.md = digest;
+    rc = pk_verify ( pk->pubkey_algo, result, sig->data, pk->pkey);
+    gcry_mpi_release ( result );
+    if( (opt.emulate_bugs & EMUBUG_MDENCODE)
+	&& gpg_err_code (rc) == GPG_ERR_BAD_SIGNATURE
+        && is_ELGAMAL(pk->pubkey_algo) ) {
+	/* In this case we try again because old GnuPG versions didn't encode
+	 * the hash right. There is no problem with DSA however  */
+	result = encode_md_value( pk->pubkey_algo, digest, sig->digest_algo,
+			      mpi_get_nbits(pk->pkey[0]), (sig->version < 5) );
+        if (!result)
+            rc = GPG_ERR_GENERAL;
+        else {
+            ctx.sig = sig;
+            ctx.md = digest;
+            rc = pk_verify (pk->pubkey_algo, result, sig->data, pk->pkey);
+        }
+    }
 
-    if(rc==G10ERR_BAD_SIGN && is_RSA(pk->pubkey_algo)
-       && sig->digest_algo==DIGEST_ALGO_SHA224)
-      {
-	/* This code is to work around a SHA-224 problem.  RFC-4880
-	   and the drafts leading up to it were published with the
-	   wrong DER prefix for SHA-224.  Unfortunately, GPG pre-1.4.8
-	   used this wrong prefix.  What this code does is take all
-	   bad RSA signatures that use SHA-224, and re-checks them
-	   using the old, incorrect, DER prefix.  Someday we should
-	   remove this code, and when we do remove it, pkcs1_encode_md
-	   can be made into a static function again.  Note that GPG2
-	   does not have this issue as it uses libgcrypt, which is
-	   being fixed while it is still a development version. */
-
-	/* The incorrect SHA-224 DER prefix used in pre-1.4.8 */
-	static byte asn[]={0x30, 0x31, 0x30, 0x0d, 0x06, 0x09, 0x60, 0x86,
-			   0x48, 0x01, 0x65, 0x03, 0x04, 0x02, 0x04, 0x05,
-			   0x00, 0x04, 0x20};
-
-	result=pkcs1_encode_md(digest,DIGEST_ALGO_SHA224,28,
-			       mpi_get_nbits(pk->pkey[0]),asn,DIM(asn));
-
-	rc=pubkey_verify(pk->pubkey_algo,result,sig->data,pk->pkey);
-	mpi_free(result);
-      }
-
-    /* Print the MD5 warning if not yet done.  Thus at most we get one
-       warning during signature checking.  Note that while validating
-       a key we might have already checked MD5 key signatures.  */
-    if (sig->digest_algo == DIGEST_ALGO_MD5)
-      md5_digest_warn (0);
-
-    if( !rc && sig->flags.unknown_critical )
-      {
-	log_info(_("assuming bad signature from key %s"
-		   " due to an unknown critical bit\n"),keystr_from_pk(pk));
-	rc = G10ERR_BAD_SIGN;
-      }
+    if( !rc && sig->flags.unknown_critical ) {
+      log_info(_("assuming bad signature from key %08lX "
+                 "due to an unknown critical bit\n"),
+               (ulong)keyid_from_pk(pk,NULL));
+	rc = gpg_error (GPG_ERR_BAD_SIGNATURE);
+    }
 
     if(!rc && ret_pk)
       copy_public_key(ret_pk,pk);
@@ -353,9 +307,9 @@ hash_uid_node( KBNODE unode, MD_HANDLE md, PKT_signature *sig )
 	    buf[2] = uid->attrib_len >> 16;
 	    buf[3] = uid->attrib_len >>  8;
 	    buf[4] = uid->attrib_len;
-	    md_write( md, buf, 5 );
+	    gcry_md_write( md, buf, 5 );
 	}
-	md_write( md, uid->attrib_data, uid->attrib_len );
+	gcry_md_write( md, uid->attrib_data, uid->attrib_len );
     }
     else {
 	if( sig->version >=4 ) {
@@ -365,9 +319,9 @@ hash_uid_node( KBNODE unode, MD_HANDLE md, PKT_signature *sig )
 	    buf[2] = uid->len >> 16;
 	    buf[3] = uid->len >>  8;
 	    buf[4] = uid->len;
-	    md_write( md, buf, 5 );
+	    gcry_md_write( md, buf, 5 );
 	}
-	md_write( md, uid->name, uid->len );
+	gcry_md_write( md, uid->name, uid->len );
     }
 }
 
@@ -378,7 +332,7 @@ cache_sig_result ( PKT_signature *sig, int result )
         sig->flags.checked = 1;
         sig->flags.valid = 1;
     }
-    else if ( result == G10ERR_BAD_SIGN ) {
+    else if ( gpg_err_code (result) == GPG_ERR_BAD_SIGNATURE ) {
         sig->flags.checked = 1;
         sig->flags.valid = 0;
     }
@@ -388,36 +342,34 @@ cache_sig_result ( PKT_signature *sig, int result )
     }
 }
 
+
 /* Check the revocation keys to see if any of them have revoked our
    pk.  sig is the revocation sig.  pk is the key it is on.  This code
    will need to be modified if gpg ever becomes multi-threaded.  Note
    that this guarantees that a designated revocation sig will never be
    considered valid unless it is actually valid, as well as being
-   issued by a revocation key in a valid direct signature.  Note also
-   that this is written so that a revoked revoker can still issue
+   issued by a revocation key in a valid direct signature.  Note that
+   this is written so that a revoked revoker can still issue
    revocations: i.e. If A revokes B, but A is revoked, B is still
    revoked.  I'm not completely convinced this is the proper behavior,
    but it matches how PGP does it. -dms */
 
 /* Returns 0 if sig is valid (i.e. pk is revoked), non-0 if not
-   revoked.  It is important that G10ERR_NO_PUBKEY is only returned
-   when a revocation signature is from a valid revocation key
-   designated in a revkey subpacket, but the revocation key itself
-   isn't present. */
+   revoked */
 int
 check_revocation_keys(PKT_public_key *pk,PKT_signature *sig)
 {
   static int busy=0;
-  int i,rc=G10ERR_GENERAL;
+  int i,rc=GPG_ERR_GENERAL;
 
   assert(IS_KEY_REV(sig));
   assert((sig->keyid[0]!=pk->keyid[0]) || (sig->keyid[0]!=pk->keyid[1]));
 
   if(busy)
     {
-      /* return an error (i.e. not revoked), but mark the pk as
-         uncacheable as we don't really know its revocation status
-         until it is checked directly. */
+      /* return -1 (i.e. not revoked), but mark the pk as uncacheable
+         as we don't really know its revocation status until it is
+         checked directly. */
 
       pk->dont_cache=1;
       return rc;
@@ -435,18 +387,17 @@ check_revocation_keys(PKT_public_key *pk,PKT_signature *sig)
       for(i=0;i<pk->numrevkeys;i++)
 	{
           u32 keyid[2];
-
+    
           keyid_from_fingerprint(pk->revkey[i].fpr,MAX_FINGERPRINT_LEN,keyid);
-
+    
           if(keyid[0]==sig->keyid[0] && keyid[1]==sig->keyid[1])
 	    {
-              MD_HANDLE md;
-
-              md=md_open(sig->digest_algo,0);
+              gcry_md_hd_t md;
+    
+              gcry_md_open (&md, sig->digest_algo,0);
               hash_public_key(md,pk);
               rc=signature_check(sig,md);
 	      cache_sig_result(sig,rc);
-              md_close (md);
 	      break;
 	    }
 	}
@@ -454,39 +405,7 @@ check_revocation_keys(PKT_public_key *pk,PKT_signature *sig)
   busy=0;
 
   return rc;
-}
-
-/* Backsigs (0x19) have the same format as binding sigs (0x18), but
-   this function is simpler than check_key_signature in a few ways.
-   For example, there is no support for expiring backsigs since it is
-   questionable what such a thing actually means.  Note also that the
-   sig cache check here, unlike other sig caches in GnuPG, is not
-   persistent. */
-int
-check_backsig(PKT_public_key *main_pk,PKT_public_key *sub_pk,
-	      PKT_signature *backsig)
-{
-  MD_HANDLE md;
-  int rc;
-
-  if(!opt.no_sig_cache && backsig->flags.checked)
-    {
-      if((rc=check_digest_algo(backsig->digest_algo)))
-	return rc;
-
-      return backsig->flags.valid? 0 : G10ERR_BAD_SIGN;
-    }
-
-  md=md_open(backsig->digest_algo,0);
-  hash_public_key(md,main_pk);
-  hash_public_key(md,sub_pk);
-  rc=do_check(sub_pk,backsig,md,NULL,NULL,NULL);
-  cache_sig_result(backsig,rc);
-  md_close(md);
-
-  return rc;
-}
-
+} 
 
 /****************
  * check the signature pointed to by NODE. This is a key signature.
@@ -496,7 +415,7 @@ check_backsig(PKT_public_key *main_pk,PKT_public_key *sub_pk,
 int
 check_key_signature( KBNODE root, KBNODE node, int *is_selfsig )
 {
-  return check_key_signature2(root, node, NULL, NULL, is_selfsig, NULL, NULL );
+    return check_key_signature2(root, node, NULL, NULL, is_selfsig, NULL, NULL);
 }
 
 /* If check_pk is set, then use it to check the signature in node
@@ -508,7 +427,7 @@ check_key_signature( KBNODE root, KBNODE node, int *is_selfsig )
 int
 check_key_signature2( KBNODE root, KBNODE node, PKT_public_key *check_pk,
 		      PKT_public_key *ret_pk, int *is_selfsig,
-		      u32 *r_expiredate, int *r_expired )
+                      u32 *r_expiredate, int *r_expired )
 {
     MD_HANDLE md;
     PKT_public_key *pk;
@@ -529,34 +448,29 @@ check_key_signature2( KBNODE root, KBNODE node, PKT_public_key *check_pk,
     sig = node->pkt->pkt.signature;
     algo = sig->digest_algo;
 
-    /* Check whether we have cached the result of a previous signature
-       check.  Note that we may no longer have the pubkey or hash
-       needed to verify a sig, but can still use the cached value.  A
-       cache refresh detects and clears these cases. */
+    /* check whether we have cached the result of a previous signature check.*/
     if ( !opt.no_sig_cache ) {
         if (sig->flags.checked) { /*cached status available*/
-	    if( is_selfsig ) {
-		u32 keyid[2];
+	    if( is_selfsig ) {	
+		u32 keyid[2];	
 
 		keyid_from_pk( pk, keyid );
 		if( keyid[0] == sig->keyid[0] && keyid[1] == sig->keyid[1] )
 		    *is_selfsig = 1;
 	    }
-	    /* BUG: This is wrong for non-self-sigs.. needs to be the
+	    /* BUG: This is wrong for non-self-sigs. Needs to be the
 	       actual pk */
 	    if((rc=do_check_messages(pk,sig,r_expired,NULL)))
 	      return rc;
-            return sig->flags.valid? 0 : G10ERR_BAD_SIGN;
+            return sig->flags.valid? 0 : gpg_error (GPG_ERR_BAD_SIGNATURE);
         }
     }
 
-    if( (rc=check_pubkey_algo(sig->pubkey_algo)) )
-	return rc;
-    if( (rc=check_digest_algo(algo)) )
-	return rc;
+    if( (rc=gcry_md_test_algo(algo)) )
+      return rc;
 
     if( sig->sig_class == 0x20 ) { /* key revocation */
-        u32 keyid[2];
+        u32 keyid[2];	
 	keyid_from_pk( pk, keyid );
 
 	/* is it a designated revoker? */
@@ -564,31 +478,31 @@ check_key_signature2( KBNODE root, KBNODE node, PKT_public_key *check_pk,
 	  rc=check_revocation_keys(pk,sig);
 	else
 	  {
-	    md = md_open( algo, 0 );
+	    gcry_md_open (&md, algo, 0 );
 	    hash_public_key( md, pk );
 	    rc = do_check( pk, sig, md, r_expired, NULL, ret_pk );
 	    cache_sig_result ( sig, rc );
-	    md_close(md);
+	    gcry_md_close(md);
 	  }
     }
     else if( sig->sig_class == 0x28 ) { /* subkey revocation */
 	KBNODE snode = find_prev_kbnode( root, node, PKT_PUBLIC_SUBKEY );
 
 	if( snode ) {
-	    md = md_open( algo, 0 );
+	    gcry_md_open (&md, algo, 0 );
 	    hash_public_key( md, pk );
 	    hash_public_key( md, snode->pkt->pkt.public_key );
 	    rc = do_check( pk, sig, md, r_expired, NULL, ret_pk );
             cache_sig_result ( sig, rc );
-	    md_close(md);
+	    gcry_md_close(md);
 	}
-	else
-	  {
+	else {
             if (opt.verbose)
-	      log_info (_("key %s: no subkey for subkey"
-			  " revocation signature\n"),keystr_from_pk(pk));
-	    rc = G10ERR_SIG_CLASS;
-	  }
+                log_info (_("key %08lX: no subkey for subkey "
+			    "revocation signature\n"),
+                          (ulong)keyid_from_pk (pk, NULL));
+	    rc = GPG_ERR_SIG_CLASS;
+	}
     }
     else if( sig->sig_class == 0x18 ) { /* key binding */
 	KBNODE snode = find_prev_kbnode( root, node, PKT_PUBLIC_SUBKEY );
@@ -601,27 +515,27 @@ check_key_signature2( KBNODE root, KBNODE node, PKT_public_key *check_pk,
 		if( keyid[0] == sig->keyid[0] && keyid[1] == sig->keyid[1] )
 		    *is_selfsig = 1;
 	    }
-	    md = md_open( algo, 0 );
+	    gcry_md_open (&md, algo, 0 );
 	    hash_public_key( md, pk );
 	    hash_public_key( md, snode->pkt->pkt.public_key );
 	    rc = do_check( pk, sig, md, r_expired, NULL, ret_pk );
             cache_sig_result ( sig, rc );
-	    md_close(md);
+	    gcry_md_close(md);
 	}
-	else
-	  {
+	else {
             if (opt.verbose)
-	      log_info(_("key %s: no subkey for subkey"
-			 " binding signature\n"),keystr_from_pk(pk));
-	    rc = G10ERR_SIG_CLASS;
-	  }
+                log_info(_("key %08lX: no subkey for subkey "
+			   "binding signature\n"),
+			 (ulong)keyid_from_pk (pk, NULL));
+	    rc = GPG_ERR_SIG_CLASS;
+	}
     }
     else if( sig->sig_class == 0x1f ) { /* direct key signature */
-	md = md_open( algo, 0 );
+	gcry_md_open (&md, algo, 0 );
 	hash_public_key( md, pk );
 	rc = do_check( pk, sig, md, r_expired, NULL, ret_pk );
         cache_sig_result ( sig, rc );
-	md_close(md);
+	gcry_md_close(md);
     }
     else { /* all other classes */
 	KBNODE unode = find_prev_kbnode( root, node, PKT_USER_ID );
@@ -630,7 +544,7 @@ check_key_signature2( KBNODE root, KBNODE node, PKT_public_key *check_pk,
 	    u32 keyid[2];
 
 	    keyid_from_pk( pk, keyid );
-	    md = md_open( algo, 0 );
+	    gcry_md_open (&md, algo, 0 );
 	    hash_public_key( md, pk );
 	    hash_uid_node( unode, md, sig );
 	    if( keyid[0] == sig->keyid[0] && keyid[1] == sig->keyid[1] )
@@ -640,20 +554,21 @@ check_key_signature2( KBNODE root, KBNODE node, PKT_public_key *check_pk,
 		rc = do_check( pk, sig, md, r_expired, NULL, ret_pk );
 	      }
 	    else if (check_pk)
-	      rc=do_check(check_pk,sig,md,r_expired,NULL,ret_pk);
+	      rc=do_check(check_pk,sig,md,r_expired, NULL, ret_pk);
 	    else
-	      rc=signature_check2(sig,md,r_expiredate,r_expired,NULL,ret_pk);
+	      rc = signature_check2( sig, md, r_expiredate, r_expired,
+                                     NULL, ret_pk);
 
             cache_sig_result ( sig, rc );
-	    md_close(md);
+	    gcry_md_close(md);
 	}
-	else
-	  {
+	else {
             if (!opt.quiet)
-	      log_info ("key %s: no user ID for key signature packet"
-			" of class %02x\n",keystr_from_pk(pk),sig->sig_class);
-	    rc = G10ERR_SIG_CLASS;
-	  }
+                log_info ("key %08lX: no user ID for key signature packet "
+                          "of class %02x\n",
+                          (ulong)keyid_from_pk (pk, NULL), sig->sig_class );
+	    rc = GPG_ERR_SIG_CLASS;
+	}
     }
 
     return rc;
