@@ -1,12 +1,11 @@
 /* status.h
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003,
- *               2004 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,11 +14,13 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifndef G10_STATUS_H
 #define G10_STATUS_H
 
+
 #define STATUS_ENTER	 1
 #define STATUS_LEAVE	 2
 #define STATUS_ABORT	 3
@@ -28,6 +29,7 @@
 #define STATUS_BADSIG	 5
 #define STATUS_ERRSIG	 6
 
+
 #define STATUS_BADARMOR  7
 
 #define STATUS_RSA_OR_IDEA 8
@@ -98,30 +100,6 @@
 #define STATUS_IMPORT_OK 	68
 #define STATUS_IMPORT_CHECK     69
 #define STATUS_REVKEYSIG        70
-#define STATUS_CARDCTRL         71
-#define STATUS_NEWSIG           72
-#define STATUS_PLAINTEXT        73
-#define STATUS_PLAINTEXT_LENGTH 74
-#define STATUS_KEY_NOT_CREATED  75
-#define STATUS_NEED_PASSPHRASE_PIN 76
-#define STATUS_SIG_SUBPACKET    77
-
-/* Extra status codes for certain smartcard operations.  Primary
-   useful to double check that change PIN worked as expected.  */
-#define STATUS_SC_OP_FAILURE    79
-#define STATUS_SC_OP_SUCCESS    80
-
-#define STATUS_BACKUP_KEY_CREATED 81
-
-#define STATUS_PKA_TRUST_BAD    82
-#define STATUS_PKA_TRUST_GOOD   83
-
-#define STATUS_BEGIN_SIGNING    84
-
-#define STATUS_ERROR  85
-
-#define STATUS_DECRYPTION_INFO  86
-
 
 /*-- status.c --*/
 void set_status_fd ( int fd );
@@ -133,10 +111,6 @@ void write_status_buffer ( int no,
 void write_status_text_and_buffer ( int no, const char *text,
                                     const char *buffer, size_t len, int wrap );
 
-#ifdef USE_SHM_COPROCESSING
-  void init_shm_coprocessing ( ulong requested_shm_size, int lock_mem );
-#endif /*USE_SHM_COPROCESSING*/
-
 int cpr_enabled(void);
 char *cpr_get( const char *keyword, const char *prompt );
 char *cpr_get_no_help( const char *keyword, const char *prompt );
@@ -145,8 +119,6 @@ char *cpr_get_hidden( const char *keyword, const char *prompt );
 void cpr_kill_prompt(void);
 int  cpr_get_answer_is_yes( const char *keyword, const char *prompt );
 int  cpr_get_answer_yes_no_quit( const char *keyword, const char *prompt );
-int  cpr_get_answer_okay_cancel (const char *keyword,
-                                 const char *prompt,
-                                 int def_answer);
+
 
 #endif /*G10_STATUS_H*/
