@@ -1,11 +1,11 @@
 /* tdbio.c
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2012 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -14,7 +14,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #include <config.h>
@@ -28,6 +29,7 @@
 #include <fcntl.h>
 #include <unistd.h>
 
+#include "gpg.h"
 #include "errors.h"
 #include "iobuf.h"
 #include "memory.h"
@@ -38,7 +40,7 @@
 #include "trustdb.h"
 #include "tdbio.h"
 
-#if defined(HAVE_DOSISH_SYSTEM) && !defined(ftruncate)
+#if defined(HAVE_DOSISH_SYSTEM)
 #define ftruncate chsize
 #endif
 
@@ -86,7 +88,7 @@ struct cmp_xdir_struct {
 
 
 static char *db_name;
-static dotlock_t lockhandle;
+static DOTLOCK lockhandle;
 static int is_locked;
 static int  db_fd = -1;
 static int in_transaction;
@@ -94,33 +96,7 @@ static int in_transaction;
 static void open_db(void);
 static void migrate_from_v2 (void);
 
-static int
-take_write_lock (void)
-{
-  if (!lockhandle)
-    lockhandle = dotlock_create (db_name, 0);
-  if (!lockhandle)
-    log_fatal ( _("can't create lock for `%s'\n"), db_name );
 
-  if (!is_locked)
-    {
-      if (dotlock_take (lockhandle, -1) )
-        log_fatal ( _("can't lock `%s'\n"), db_name );
-      else
-        is_locked = 1;
-      return 0;
-    }
-  else
-    return 1;
-}
-
-static void
-release_write_lock (void)
-{
-  if (!opt.lock_once)
-    if (!dotlock_release (lockhandle))
-      is_locked = 0;
-}
 
 /*************************************
  ************* record cache **********
@@ -148,17 +124,20 @@ static int
 write_cache_item( CACHE_CTRL r )
 {
     int n;
+    gpg_error_t rc;
 
     if( lseek( db_fd, r->recno * TRUST_RECORD_LEN, SEEK_SET ) == -1 ) {
+        rc = gpg_error_from_errno (errno);
 	log_error(_("trustdb rec %lu: lseek failed: %s\n"),
 					    r->recno, strerror(errno) );
-	return G10ERR_WRITE_FILE;
+	return rc;
     }
     n = write( db_fd, r->data, TRUST_RECORD_LEN);
     if( n != TRUST_RECORD_LEN ) {
+        rc = gpg_error_from_errno (errno);
 	log_error(_("trustdb rec %lu: write failed (n=%d): %s\n"),
 					    r->recno, n, strerror(errno) );
-	return G10ERR_WRITE_FILE;
+	return rc;
     }
     r->flags.dirty = 0;
     return 0;
@@ -212,7 +191,7 @@ put_record_into_cache( ulong recno, const char *data )
     }
     /* see whether we reached the limit */
     if( cache_entries < MAX_CACHE_ENTRIES_SOFT ) { /* no */
-	r = xmalloc( sizeof *r );
+	r = xmalloc ( sizeof *r );
 	r->flags.used = 1;
 	r->recno = recno;
 	memcpy( r->data, data, TRUST_RECORD_LEN );
@@ -255,7 +234,7 @@ put_record_into_cache( ulong recno, const char *data )
 	if( cache_entries < MAX_CACHE_ENTRIES_HARD ) { /* no */
 	    if( opt.debug && !(cache_entries % 100) )
 		log_debug("increasing tdbio cache size\n");
-	    r = xmalloc( sizeof *r );
+	    r = xmalloc ( sizeof *r );
 	    r->flags.used = 1;
 	    r->recno = recno;
 	    memcpy( r->data, data, TRUST_RECORD_LEN );
@@ -267,13 +246,18 @@ put_record_into_cache( ulong recno, const char *data )
 	    return 0;
 	}
 	log_info(_("trustdb transaction too large\n"));
-	return G10ERR_RESOURCE_LIMIT;
+	return GPG_ERR_RESOURCE_LIMIT;
     }
     if( dirty_count ) {
 	int n = dirty_count / 5; /* discard some dirty entries */
 	if( !n )
 	    n = 1;
-        take_write_lock ();
+	if( !is_locked ) {
+	    if( make_dotlock( lockhandle, -1 ) )
+		log_fatal("can't acquire lock - giving up\n");
+	    else
+		is_locked = 1;
+	}
 	for( unused = NULL, r = cache_list; r; r = r->next ) {
 	    if( r->flags.used && r->flags.dirty ) {
 		int rc = write_cache_item( r );
@@ -287,7 +271,10 @@ put_record_into_cache( ulong recno, const char *data )
 		    break;
 	    }
 	}
-        release_write_lock ();
+	if( !opt.lock_once ) {
+	    if( !release_dotlock( lockhandle ) )
+		is_locked = 0;
+	}
 	assert( unused );
 	r = unused;
 	r->flags.used = 1;
@@ -326,9 +313,13 @@ tdbio_sync()
     if( !cache_is_dirty )
 	return 0;
 
-    if (!take_write_lock ())
-        did_lock = 1;
-
+    if( !is_locked ) {
+	if( make_dotlock( lockhandle, -1 ) )
+	    log_fatal("can't acquire lock - giving up\n");
+	else
+	    is_locked = 1;
+	did_lock = 1;
+    }
     for( r = cache_list; r; r = r->next ) {
 	if( r->flags.used && r->flags.dirty ) {
 	    int rc = write_cache_item( r );
@@ -337,12 +328,15 @@ tdbio_sync()
 	}
     }
     cache_is_dirty = 0;
-    if (did_lock)
-        release_write_lock ();
+    if( did_lock && !opt.lock_once ) {
+	if( !release_dotlock( lockhandle ) )
+	    is_locked = 0;
+    }
 
     return 0;
 }
 
+
 #if 0
 /* The transaction code is disabled in the 1.2.x branch, as it is not
    yet used.  It will be enabled in 1.3.x. */
@@ -375,12 +369,21 @@ tdbio_end_transaction()
 
     if( !in_transaction )
 	log_bug("tdbio: no active transaction\n");
-    take_write_lock ();
-    block_all_signals();
+    if( !is_locked ) {
+	if( make_dotlock( lockhandle, -1 ) )
+	    log_fatal("can't acquire lock - giving up\n");
+	else
+	    is_locked = 1;
+    }
+#warning block_all_signals is not yet available in ../common/signals.c
+    /*    block_all_signals(); */
     in_transaction = 0;
     rc = tdbio_sync();
-    unblock_all_signals();
-    release_write_lock ();
+/*      unblock_all_signals(); */
+    if( !opt.lock_once ) {
+	if( !release_dotlock( lockhandle ) )
+	    is_locked = 0;
+    }
     return rc;
 }
 
@@ -407,7 +410,9 @@ tdbio_cancel_transaction()
     in_transaction = 0;
     return 0;
 }
-#endif
+
+#endif /* transaction code */
+
 
 
 /********************************************************
@@ -418,7 +423,7 @@ static void
 cleanup(void)
 {
     if( is_locked ) {
-	if (!dotlock_release (lockhandle))
+	if( !release_dotlock(lockhandle) )
 	    is_locked = 0;
     }
 }
@@ -440,7 +445,6 @@ tdbio_update_version_record (void)
       rec.r.ver.completes   = opt.completes_needed;
       rec.r.ver.cert_depth  = opt.max_cert_depth;
       rec.r.ver.trust_model = opt.trust_model;
-      rec.r.ver.min_cert_level = opt.min_cert_level;
       rc=tdbio_write_record(&rec);
     }
 
@@ -452,7 +456,7 @@ create_version_record (void)
 {
   TRUSTREC rec;
   int rc;
-
+  
   memset( &rec, 0, sizeof rec );
   rec.r.ver.version     = 3;
   rec.r.ver.created     = make_timestamp();
@@ -463,7 +467,6 @@ create_version_record (void)
     rec.r.ver.trust_model = opt.trust_model;
   else
     rec.r.ver.trust_model = TM_PGP;
-  rec.r.ver.min_cert_level = opt.min_cert_level;
   rec.rectype = RECTYPE_VER;
   rec.recnum = 0;
   rc = tdbio_write_record( &rec );
@@ -475,10 +478,9 @@ create_version_record (void)
 
 
 int
-tdbio_set_dbname( const char *new_dbname, int create, int *r_nofile)
+tdbio_set_dbname( const char *new_dbname, int create )
 {
     char *fname;
-    struct stat statbuf;
     static int initialized = 0;
 
     if( !initialized ) {
@@ -486,8 +488,6 @@ tdbio_set_dbname( const char *new_dbname, int create, int *r_nofile)
 	initialized = 1;
     }
 
-    *r_nofile = 0;
-
     if(new_dbname==NULL)
       fname=make_filename(opt.homedir,"trustdb" EXTSEP_S "gpg", NULL);
     else if (*new_dbname != DIRSEP_C )
@@ -500,28 +500,13 @@ tdbio_set_dbname( const char *new_dbname, int create, int *r_nofile)
     else
       fname = xstrdup (new_dbname);
 
-    xfree (db_name);
-    db_name = fname;
-
-    /*
-     * Quick check for (likely) case where there is trustdb.gpg
-     * already.  This check is not required in theory, but it helps in
-     * practice, avoiding costly operations of preparing and taking
-     * the lock.
-     */
-    if (stat (fname, &statbuf) == 0 && statbuf.st_size > 0)
-      /* OK, we have the valid trustdb.gpg already.  */
-      return 0;
-
-    take_write_lock ();
-
     if( access( fname, R_OK ) ) {
-        if( errno != ENOENT )
-            log_fatal( _("can't access `%s': %s\n"), fname, strerror(errno) );
-
-	if (!create)
-          *r_nofile = 1;
-        else {
+	if( errno != ENOENT ) {
+	    log_error( _("%s: can't access: %s\n"), fname, strerror(errno) );
+	    xfree (fname);
+	    return GPG_ERR_TRUSTDB;
+	}
+	if( create ) {
 	    FILE *fp;
 	    TRUSTREC rec;
 	    int rc;
@@ -532,40 +517,53 @@ tdbio_set_dbname( const char *new_dbname, int create, int *r_nofile)
 	    *p = 0;
 	    if( access( fname, F_OK ) ) {
 		try_make_homedir( fname );
-                if (access (fname, F_OK ))
-                  log_fatal (_("%s: directory does not exist!\n"), fname);
+		log_fatal( _("%s: directory does not exist!\n"), fname );
 	    }
 	    *p = DIRSEP_C;
 
+	    xfree (db_name);
+	    db_name = fname;
+#ifdef __riscos__
+	    if( !lockhandle )
+		lockhandle = create_dotlock( db_name );
+	    if( !lockhandle )
+		log_fatal( _("%s: can't create lock\n"), db_name );
+            if( make_dotlock( lockhandle, -1 ) )
+                log_fatal( _("%s: can't make lock\n"), db_name );
+#endif /* __riscos__ */
 	    oldmask=umask(077);
-            if (is_secured_filename (fname)) {
-                fp = NULL;
-                errno = EPERM;
-            }
-            else
-                fp =fopen( fname, "wb" );
+	    fp =fopen( fname, "wb" );
 	    umask(oldmask);
 	    if( !fp )
-		log_fatal( _("can't create `%s': %s\n"), fname, strerror(errno) );
+		log_fatal( _("%s: can't create: %s\n"), fname, strerror(errno) );
 	    fclose(fp);
 	    db_fd = open( db_name, O_RDWR | MY_O_BINARY );
 	    if( db_fd == -1 )
-		log_fatal( _("can't open `%s': %s\n"), db_name, strerror(errno) );
+		log_fatal( _("%s: can't open: %s\n"), db_name, strerror(errno) );
+
+#ifndef __riscos__
+	    if( !lockhandle )
+		lockhandle = create_dotlock( db_name );
+	    if( !lockhandle )
+		log_fatal( _("%s: can't create lock\n"), db_name );
+#endif /* !__riscos__ */
 
             rc = create_version_record ();
 	    if( rc )
 		log_fatal( _("%s: failed to create version record: %s"),
-						   fname, g10_errstr(rc));
+						   fname, gpg_strerror (rc));
 	    /* and read again to check that we are okay */
 	    if( tdbio_read_record( 0, &rec, RECTYPE_VER ) )
 		log_fatal( _("%s: invalid trustdb created\n"), db_name );
 
 	    if( !opt.quiet )
 		log_info(_("%s: trustdb created\n"), db_name);
+
+	    return 0;
 	}
     }
-
-    release_write_lock ();
+    xfree (db_name);
+    db_name = fname;
     return 0;
 }
 
@@ -587,20 +585,22 @@ open_db()
 
   assert( db_fd == -1 );
 
+  if (!lockhandle )
+    lockhandle = create_dotlock( db_name );
+  if (!lockhandle )
+    log_fatal( _("%s: can't create lock\n"), db_name );
+#ifdef __riscos__
+  if (make_dotlock( lockhandle, -1 ) )
+    log_fatal( _("%s: can't make lock\n"), db_name );
+#endif /* __riscos__ */
   db_fd = open (db_name, O_RDWR | MY_O_BINARY );
-  if (db_fd == -1 && (errno == EACCES
-#ifdef EROFS
-                      || errno == EROFS
-#endif
-                      )
-      ) {
+  if (db_fd == -1 && errno == EACCES) {
       db_fd = open (db_name, O_RDONLY | MY_O_BINARY );
       if (db_fd != -1)
           log_info (_("NOTE: trustdb not writable\n"));
   }
   if ( db_fd == -1 )
-    log_fatal( _("can't open `%s': %s\n"), db_name, strerror(errno) );
-  register_secured_file (db_name);
+    log_fatal( _("%s: can't open: %s\n"), db_name, strerror(errno) );
 
   /* check whether we need to do a version migration */
   do
@@ -610,7 +610,7 @@ open_db()
     {
       migrate_from_v2 ();
     }
-
+  
   /* read the version record */
   if (tdbio_read_record (0, &rec, RECTYPE_VER ) )
     log_fatal( _("%s: invalid trustdb\n"), db_name );
@@ -646,7 +646,7 @@ create_hashtable( TRUSTREC *vr, int type )
 	 rc = tdbio_write_record( &rec );
 	 if( rc )
 	     log_fatal( _("%s: failed to create hashtable: %s\n"),
-					db_name, g10_errstr(rc));
+					db_name, gpg_strerror (rc));
     }
     /* update the version record */
     rc = tdbio_write_record( vr );
@@ -654,7 +654,7 @@ create_hashtable( TRUSTREC *vr, int type )
 	rc = tdbio_sync();
     if( rc )
 	log_fatal( _("%s: error updating version record: %s\n"),
-						  db_name, g10_errstr(rc));
+						  db_name, gpg_strerror (rc));
 }
 
 
@@ -671,13 +671,12 @@ tdbio_db_matches_options()
       rc = tdbio_read_record( 0, &vr, RECTYPE_VER );
       if( rc )
 	log_fatal( _("%s: error reading version record: %s\n"),
-		   db_name, g10_errstr(rc) );
+		   db_name, gpg_strerror (rc) );
 
       yes_no = vr.r.ver.marginals == opt.marginals_needed
 	&& vr.r.ver.completes == opt.completes_needed
 	&& vr.r.ver.cert_depth == opt.max_cert_depth
-	&& vr.r.ver.trust_model == opt.trust_model
-	&& vr.r.ver.min_cert_level == opt.min_cert_level;
+	&& vr.r.ver.trust_model == opt.trust_model;
     }
 
   return yes_no;
@@ -688,11 +687,11 @@ tdbio_read_model(void)
 {
   TRUSTREC vr;
   int rc;
-
+ 
   rc = tdbio_read_record( 0, &vr, RECTYPE_VER );
   if( rc )
     log_fatal( _("%s: error reading version record: %s\n"),
-	       db_name, g10_errstr(rc) );
+	       db_name, gpg_strerror (rc) );
   return vr.r.ver.trust_model;
 }
 
@@ -708,7 +707,7 @@ tdbio_read_nextcheck ()
     rc = tdbio_read_record( 0, &vr, RECTYPE_VER );
     if( rc )
 	log_fatal( _("%s: error reading version record: %s\n"),
-						    db_name, g10_errstr(rc) );
+						    db_name, gpg_strerror (rc) );
     return vr.r.ver.nextcheck;
 }
 
@@ -722,7 +721,7 @@ tdbio_write_nextcheck (ulong stamp)
     rc = tdbio_read_record( 0, &vr, RECTYPE_VER );
     if( rc )
 	log_fatal( _("%s: error reading version record: %s\n"),
-				       db_name, g10_errstr(rc) );
+				       db_name, gpg_strerror (rc) );
 
     if (vr.r.ver.nextcheck == stamp)
       return 0;
@@ -731,7 +730,7 @@ tdbio_write_nextcheck (ulong stamp)
     rc = tdbio_write_record( &vr );
     if( rc )
 	log_fatal( _("%s: error writing version record: %s\n"),
-				       db_name, g10_errstr(rc) );
+				       db_name, gpg_strerror (rc) );
     return 1;
 }
 
@@ -752,7 +751,7 @@ get_trusthashrec(void)
 	rc = tdbio_read_record( 0, &vr, RECTYPE_VER );
 	if( rc )
 	    log_fatal( _("%s: error reading version record: %s\n"),
-					    db_name, g10_errstr(rc) );
+					    db_name, gpg_strerror (rc) );
 	if( !vr.r.ver.trusthashtbl )
 	    create_hashtable( &vr, 0 );
 
@@ -783,8 +782,9 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
     hashrec += msb / ITEMS_PER_HTBL_RECORD;
     rc = tdbio_read_record( hashrec, &rec, RECTYPE_HTBL );
     if( rc ) {
-	log_error("upd_hashtable: read failed: %s\n",	g10_errstr(rc) );
-	return rc;
+        log_error ("upd_hashtable in `%s': read failed: %s\n", db_name,
+                   gpg_strerror (rc) );
+        return rc;
     }
 
     item = rec.r.htbl.item[msb % ITEMS_PER_HTBL_RECORD];
@@ -792,8 +792,8 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
 	rec.r.htbl.item[msb % ITEMS_PER_HTBL_RECORD] = newrecnum;
 	rc = tdbio_write_record( &rec );
 	if( rc ) {
-	    log_error("upd_hashtable: write htbl failed: %s\n",
-							    g10_errstr(rc) );
+            log_error ("upd_hashtable in `%s': write htbl failed: %s\n",
+                       db_name, gpg_strerror (rc) );
 	    return rc;
 	}
     }
@@ -802,7 +802,7 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
 	rc = tdbio_read_record( item, &rec, 0 );
 	if( rc ) {
 	    log_error( "upd_hashtable: read item failed: %s\n",
-							    g10_errstr(rc) );
+							    gpg_strerror (rc) );
 	    return rc;
 	}
 
@@ -811,7 +811,7 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
 	    level++;
 	    if( level >= keylen ) {
 		log_error( "hashtable has invalid indirections.\n");
-		return G10ERR_TRUSTDB;
+		return GPG_ERR_TRUSTDB;
 	    }
 	    goto next_level;
 	}
@@ -828,7 +828,7 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
 						       &rec, RECTYPE_HLST);
 		    if( rc ) {
 			log_error( "upd_hashtable: read hlst failed: %s\n",
-							     g10_errstr(rc) );
+							     gpg_strerror (rc) );
 			return rc;
 		    }
 		}
@@ -843,7 +843,7 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
 			rc = tdbio_write_record( &rec );
 			if( rc )
 			    log_error( "upd_hashtable: write hlst failed: %s\n",
-							      g10_errstr(rc) );
+							      gpg_strerror (rc) );
 			return rc; /* done */
 		    }
 		}
@@ -852,7 +852,7 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
 						      &rec, RECTYPE_HLST );
 		    if( rc ) {
 			log_error( "upd_hashtable: read hlst failed: %s\n",
-							     g10_errstr(rc) );
+							     gpg_strerror (rc) );
 			return rc;
 		    }
 		}
@@ -861,7 +861,7 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
 		    rc = tdbio_write_record( &rec );
 		    if( rc ) {
 			log_error( "upd_hashtable: write hlst failed: %s\n",
-							  g10_errstr(rc) );
+							  gpg_strerror (rc) );
 			return rc;
 		    }
 		    memset( &rec, 0, sizeof rec );
@@ -871,7 +871,7 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
 		    rc = tdbio_write_record( &rec );
 		    if( rc )
 			log_error( "upd_hashtable: write ext hlst failed: %s\n",
-							  g10_errstr(rc) );
+							  gpg_strerror (rc) );
 		    return rc; /* done */
 		}
 	    } /* end loop over hlst slots */
@@ -889,7 +889,7 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
 	    rc = tdbio_write_record( &rec );
 	    if( rc ) {
 		log_error( "upd_hashtable: write new hlst failed: %s\n",
-						  g10_errstr(rc) );
+						  gpg_strerror (rc) );
 		return rc;
 	    }
 	    /* update the hashtable record */
@@ -897,14 +897,14 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
 	    rc = tdbio_write_record( &lastrec );
 	    if( rc )
 		log_error( "upd_hashtable: update htbl failed: %s\n",
-							     g10_errstr(rc) );
+							     gpg_strerror (rc) );
 	    return rc; /* ready */
 	}
 	else {
 	    log_error( "hashtbl %lu: %lu/%d points to an invalid record %lu\n",
 		       table, hashrec, (msb % ITEMS_PER_HTBL_RECORD), item);
 	    list_trustdb(NULL);
-	    return G10ERR_TRUSTDB;
+	    return GPG_ERR_TRUSTDB;
 	}
     }
 
@@ -931,8 +931,8 @@ drop_from_hashtable( ulong table, byte *key, int keylen, ulong recnum )
     hashrec += msb / ITEMS_PER_HTBL_RECORD;
     rc = tdbio_read_record( hashrec, &rec, RECTYPE_HTBL );
     if( rc ) {
-	log_error("drop_from_hashtable: read failed: %s\n",
-							g10_errstr(rc) );
+	log_error ("drop_from_hashtable `%s': read failed: %s\n",
+                   db_name, gpg_strerror (rc) );
 	return rc;
     }
 
@@ -944,15 +944,15 @@ drop_from_hashtable( ulong table, byte *key, int keylen, ulong recnum )
 	rec.r.htbl.item[msb % ITEMS_PER_HTBL_RECORD] = 0;
 	rc = tdbio_write_record( &rec );
 	if( rc )
-	    log_error("drop_from_hashtable: write htbl failed: %s\n",
-							    g10_errstr(rc) );
+	    log_error ("drop_from_hashtable `%s': write htbl failed: %s\n",
+                       db_name, gpg_strerror (rc) );
 	return rc;
     }
 
     rc = tdbio_read_record( item, &rec, 0 );
     if( rc ) {
 	log_error( "drop_from_hashtable: read item failed: %s\n",
-							g10_errstr(rc) );
+							gpg_strerror (rc) );
 	return rc;
     }
 
@@ -961,7 +961,7 @@ drop_from_hashtable( ulong table, byte *key, int keylen, ulong recnum )
 	level++;
 	if( level >= keylen ) {
 	    log_error( "hashtable has invalid indirections.\n");
-	    return G10ERR_TRUSTDB;
+	    return GPG_ERR_TRUSTDB;
 	}
 	goto next_level;
     }
@@ -973,8 +973,9 @@ drop_from_hashtable( ulong table, byte *key, int keylen, ulong recnum )
 		    rec.r.hlst.rnum[i] = 0; /* drop */
 		    rc = tdbio_write_record( &rec );
 		    if( rc )
-			log_error("drop_from_hashtable: write htbl failed: %s\n",
-									g10_errstr(rc) );
+			log_error ("drop_from_hashtable `%s': "
+                                   "write htbl failed: %s\n",
+                                   db_name, gpg_strerror (rc) );
 		    return rc;
 		}
 	    }
@@ -983,7 +984,7 @@ drop_from_hashtable( ulong table, byte *key, int keylen, ulong recnum )
 						   &rec, RECTYPE_HLST);
 		if( rc ) {
 		    log_error( "drop_from_hashtable: read hlst failed: %s\n",
-							 g10_errstr(rc) );
+							 gpg_strerror (rc) );
 		    return rc;
 		}
 	    }
@@ -994,7 +995,7 @@ drop_from_hashtable( ulong table, byte *key, int keylen, ulong recnum )
 
     log_error( "hashtbl %lu: %lu/%d points to wrong record %lu\n",
 		    table, hashrec, (msb % ITEMS_PER_HTBL_RECORD), item);
-    return G10ERR_TRUSTDB;
+    return GPG_ERR_TRUSTDB;
 }
 
 
@@ -1006,8 +1007,8 @@ drop_from_hashtable( ulong table, byte *key, int keylen, ulong recnum )
  */
 static int
 lookup_hashtable( ulong table, const byte *key, size_t keylen,
-		  int (*cmpfnc)(const void*, const TRUSTREC *),
-                  const void *cmpdata, TRUSTREC *rec )
+		  int (*cmpfnc)(void*, const TRUSTREC *), void *cmpdata,
+						TRUSTREC *rec )
 {
     int rc;
     ulong hashrec, item;
@@ -1020,7 +1021,8 @@ lookup_hashtable( ulong table, const byte *key, size_t keylen,
     hashrec += msb / ITEMS_PER_HTBL_RECORD;
     rc = tdbio_read_record( hashrec, rec, RECTYPE_HTBL );
     if( rc ) {
-	log_error("lookup_hashtable failed: %s\n", g10_errstr(rc) );
+	log_error ("lookup_hashtable in `%s' failed: %s\n",
+                   db_name, gpg_strerror (rc) );
 	return rc;
     }
 
@@ -1030,15 +1032,16 @@ lookup_hashtable( ulong table, const byte *key, size_t keylen,
 
     rc = tdbio_read_record( item, rec, 0 );
     if( rc ) {
-	log_error( "hashtable read failed: %s\n", g10_errstr(rc) );
+	log_error ("hashtable `%s' read failed: %s\n",
+                   db_name, gpg_strerror (rc) );
 	return rc;
     }
     if( rec->rectype == RECTYPE_HTBL ) {
 	hashrec = item;
 	level++;
 	if( level >= keylen ) {
-	    log_error("hashtable has invalid indirections\n");
-	    return G10ERR_TRUSTDB;
+	    log_error ("hashtable `%s' has invalid indirections\n", db_name);
+	    return GPG_ERR_TRUSTDB;
 	}
 	goto next_level;
     }
@@ -1053,7 +1056,7 @@ lookup_hashtable( ulong table, const byte *key, size_t keylen,
 		    rc = tdbio_read_record( rec->r.hlst.rnum[i], &tmp, 0 );
 		    if( rc ) {
 			log_error( "lookup_hashtable: read item failed: %s\n",
-							      g10_errstr(rc) );
+							      gpg_strerror (rc) );
 			return rc;
 		    }
 		    if( (*cmpfnc)( cmpdata, &tmp ) ) {
@@ -1066,7 +1069,7 @@ lookup_hashtable( ulong table, const byte *key, size_t keylen,
 		rc = tdbio_read_record( rec->r.hlst.next, rec, RECTYPE_HLST );
 		if( rc ) {
 		    log_error( "lookup_hashtable: read hlst failed: %s\n",
-							 g10_errstr(rc) );
+							 gpg_strerror (rc) );
 		    return rc;
 		}
 	    }
@@ -1107,14 +1110,13 @@ tdbio_dump_record( TRUSTREC *rec, FILE *fp  )
       case 0: fprintf(fp, "blank\n");
 	break;
       case RECTYPE_VER: fprintf(fp,
-	    "version, td=%lu, f=%lu, m/c/d=%d/%d/%d tm=%d mcl=%d nc=%lu (%s)\n",
+	    "version, td=%lu, f=%lu, m/c/d=%d/%d/%d tm=%d nc=%lu (%s)\n",
                                    rec->r.ver.trusthashtbl,
 				   rec->r.ver.firstfree,
 				   rec->r.ver.marginals,
 				   rec->r.ver.completes,
 				   rec->r.ver.cert_depth,
 				   rec->r.ver.trust_model,
-				   rec->r.ver.min_cert_level,
                                    rec->r.ver.nextcheck,
 				   strtimestamp(rec->r.ver.nextcheck)
                                  );
@@ -1170,17 +1172,19 @@ tdbio_read_record( ulong recnum, TRUSTREC *rec, int expected )
     buf = get_record_from_cache( recnum );
     if( !buf ) {
 	if( lseek( db_fd, recnum * TRUST_RECORD_LEN, SEEK_SET ) == -1 ) {
+            rc = gpg_error_from_errno (errno);
 	    log_error(_("trustdb: lseek failed: %s\n"), strerror(errno) );
-	    return G10ERR_READ_FILE;
+	    return rc;
 	}
 	n = read( db_fd, readbuf, TRUST_RECORD_LEN);
 	if( !n ) {
 	    return -1; /* eof */
 	}
 	else if( n != TRUST_RECORD_LEN ) {
+            rc = gpg_error_from_errno (errno);
 	    log_error(_("trustdb: read failed (n=%d): %s\n"), n,
 							strerror(errno) );
-	    return G10ERR_READ_FILE;
+	    return rc;
 	}
 	buf = readbuf;
     }
@@ -1191,7 +1195,7 @@ tdbio_read_record( ulong recnum, TRUSTREC *rec, int expected )
     if( expected && rec->rectype != expected ) {
 	log_error("%lu: read expected rec type %d, got %d\n",
 		    recnum, expected, rec->rectype );
-	return G10ERR_TRUSTDB;
+	return GPG_ERR_TRUSTDB;
     }
     p++;    /* skip reserved byte */
     switch( rec->rectype ) {
@@ -1200,7 +1204,7 @@ tdbio_read_record( ulong recnum, TRUSTREC *rec, int expected )
       case RECTYPE_VER: /* version record */
 	if( memcmp(buf+1, "gpg", 3 ) ) {
 	    log_error( _("%s: not a trustdb file\n"), db_name );
-	    rc = G10ERR_TRUSTDB;
+	    rc = GPG_ERR_TRUSTDB;
 	}
 	p += 2; /* skip "gpg" */
 	rec->r.ver.version  = *p++;
@@ -1208,38 +1212,37 @@ tdbio_read_record( ulong recnum, TRUSTREC *rec, int expected )
 	rec->r.ver.completes = *p++;
 	rec->r.ver.cert_depth = *p++;
 	rec->r.ver.trust_model = *p++;
-	rec->r.ver.min_cert_level = *p++;
-	p += 2;
-	rec->r.ver.created  = buf32_to_ulong (p); p += 4;
-	rec->r.ver.nextcheck = buf32_to_ulong (p); p += 4;
+	p += 3;
+	rec->r.ver.created  = buftoulong(p); p += 4;
+	rec->r.ver.nextcheck = buftoulong(p); p += 4;
 	p += 4;
 	p += 4;
-	rec->r.ver.firstfree =buf32_to_ulong (p); p += 4;
+	rec->r.ver.firstfree =buftoulong(p); p += 4;
 	p += 4;
-	rec->r.ver.trusthashtbl =buf32_to_ulong (p); p += 4;
+	rec->r.ver.trusthashtbl =buftoulong(p); p += 4;
 	if( recnum ) {
 	    log_error( _("%s: version record with recnum %lu\n"), db_name,
 							     (ulong)recnum );
-	    rc = G10ERR_TRUSTDB;
+	    rc = GPG_ERR_TRUSTDB;
 	}
 	else if( rec->r.ver.version != 3 ) {
 	    log_error( _("%s: invalid file version %d\n"), db_name,
 							rec->r.ver.version );
-	    rc = G10ERR_TRUSTDB;
+	    rc = GPG_ERR_TRUSTDB;
 	}
 	break;
       case RECTYPE_FREE:
-	rec->r.free.next  = buf32_to_ulong (p); p += 4;
+	rec->r.free.next  = buftoulong(p); p += 4;
 	break;
       case RECTYPE_HTBL:
 	for(i=0; i < ITEMS_PER_HTBL_RECORD; i++ ) {
-	    rec->r.htbl.item[i] = buf32_to_ulong (p); p += 4;
+	    rec->r.htbl.item[i] = buftoulong(p); p += 4;
 	}
 	break;
       case RECTYPE_HLST:
-	rec->r.hlst.next = buf32_to_ulong (p); p += 4;
+	rec->r.hlst.next = buftoulong(p); p += 4;
 	for(i=0; i < ITEMS_PER_HLST_RECORD; i++ ) {
-	    rec->r.hlst.rnum[i] = buf32_to_ulong (p); p += 4;
+	    rec->r.hlst.rnum[i] = buftoulong(p); p += 4;
 	}
 	break;
       case RECTYPE_TRUST:
@@ -1248,19 +1251,19 @@ tdbio_read_record( ulong recnum, TRUSTREC *rec, int expected )
         rec->r.trust.depth = *p++;
         rec->r.trust.min_ownertrust = *p++;
         p++;
-	rec->r.trust.validlist = buf32_to_ulong (p); p += 4;
+	rec->r.trust.validlist = buftoulong(p); p += 4;
 	break;
       case RECTYPE_VALID:
 	memcpy( rec->r.valid.namehash, p, 20); p+=20;
         rec->r.valid.validity = *p++;
-	rec->r.valid.next = buf32_to_ulong (p); p += 4;
+	rec->r.valid.next = buftoulong(p); p += 4;
 	rec->r.valid.full_count = *p++;
 	rec->r.valid.marginal_count = *p++;
 	break;
       default:
 	log_error( "%s: invalid record type %d at recnum %lu\n",
 				   db_name, rec->rectype, (ulong)recnum );
-	rc = G10ERR_TRUSTDB;
+	rc = GPG_ERR_TRUSTDB;
 	break;
     }
 
@@ -1296,8 +1299,7 @@ tdbio_write_record( TRUSTREC *rec )
 	*p++ = rec->r.ver.completes;
 	*p++ = rec->r.ver.cert_depth;
 	*p++ = rec->r.ver.trust_model;
-	*p++ = rec->r.ver.min_cert_level;
-	p += 2;
+	p += 3;
 	ulongtobuf(p, rec->r.ver.created); p += 4;
 	ulongtobuf(p, rec->r.ver.nextcheck); p += 4;
 	p += 4;
@@ -1377,7 +1379,7 @@ tdbio_delete_record( ulong recnum )
     rc = tdbio_read_record( 0, &vr, RECTYPE_VER );
     if( rc )
 	log_fatal( _("%s: error reading version record: %s\n"),
-				       db_name, g10_errstr(rc) );
+				       db_name, gpg_strerror (rc) );
 
     rec.recnum = recnum;
     rec.rectype = RECTYPE_FREE;
@@ -1404,13 +1406,13 @@ tdbio_new_recnum()
     rc = tdbio_read_record( 0, &vr, RECTYPE_VER );
     if( rc )
 	log_fatal( _("%s: error reading version record: %s\n"),
-					     db_name, g10_errstr(rc) );
+					     db_name, gpg_strerror (rc) );
     if( vr.r.ver.firstfree ) {
 	recnum = vr.r.ver.firstfree;
 	rc = tdbio_read_record( recnum, &rec, RECTYPE_FREE );
 	if( rc ) {
 	    log_error( _("%s: error reading free record: %s\n"),
-						  db_name,  g10_errstr(rc) );
+						  db_name,  gpg_strerror (rc) );
 	    return rc;
 	}
 	/* update dir record */
@@ -1418,7 +1420,7 @@ tdbio_new_recnum()
 	rc = tdbio_write_record( &vr );
 	if( rc ) {
 	    log_error( _("%s: error writing dir record: %s\n"),
-						     db_name, g10_errstr(rc) );
+						     db_name, gpg_strerror (rc) );
 	    return rc;
 	}
 	/*zero out the new record */
@@ -1428,7 +1430,7 @@ tdbio_new_recnum()
 	rc = tdbio_write_record( &rec );
 	if( rc )
 	    log_fatal(_("%s: failed to zero a record: %s\n"),
-				       db_name, g10_errstr(rc));
+				       db_name, gpg_strerror (rc));
     }
     else { /* not found, append a new record */
 	offset = lseek( db_fd, 0, SEEK_END );
@@ -1443,22 +1445,22 @@ tdbio_new_recnum()
 	rec.recnum = recnum;
 	rc = 0;
 	if( lseek( db_fd, recnum * TRUST_RECORD_LEN, SEEK_SET ) == -1 ) {
+            rc = gpg_error_from_errno (errno);
 	    log_error(_("trustdb rec %lu: lseek failed: %s\n"),
 						recnum, strerror(errno) );
-	    rc = G10ERR_WRITE_FILE;
 	}
 	else {
 	    int n = write( db_fd, &rec, TRUST_RECORD_LEN);
 	    if( n != TRUST_RECORD_LEN ) {
+                rc = gpg_error_from_errno (errno);
 		log_error(_("trustdb rec %lu: write failed (n=%d): %s\n"),
 						 recnum, n, strerror(errno) );
-		rc = G10ERR_WRITE_FILE;
 	    }
 	}
 
 	if( rc )
 	    log_fatal(_("%s: failed to append a record: %s\n"),
-				    db_name,	g10_errstr(rc));
+				    db_name,	gpg_strerror (rc));
     }
     return recnum ;
 }
@@ -1466,10 +1468,10 @@ tdbio_new_recnum()
 
 
 static int
-cmp_trec_fpr (const void *fpr, const TRUSTREC *rec )
+cmp_trec_fpr ( void *fpr, const TRUSTREC *rec )
 {
-    return (rec->rectype == RECTYPE_TRUST
-            && !memcmp( rec->r.trust.fingerprint, fpr, 20));
+    return rec->rectype == RECTYPE_TRUST
+	   && !memcmp( rec->r.trust.fingerprint, fpr, 20);
 }
 
 
@@ -1478,9 +1480,9 @@ tdbio_search_trust_byfpr( const byte *fingerprint, TRUSTREC *rec )
 {
     int rc;
 
-    /* Locate the trust record using the hash table.  */
-    rc = lookup_hashtable (get_trusthashrec(), fingerprint, 20,
-			   cmp_trec_fpr, fingerprint, rec);
+    /* locate the trust record using the hash table */
+    rc = lookup_hashtable( get_trusthashrec(), fingerprint, 20,
+			   cmp_trec_fpr, (void*)fingerprint, rec );
     return rc;
 }
 
@@ -1501,9 +1503,9 @@ tdbio_search_trust_bypk (PKT_public_key *pk, TRUSTREC *rec)
 void
 tdbio_invalid(void)
 {
-  log_error (_("Error: The trustdb is corrupted.\n"));
-  how_to_fix_the_trustdb ();
-  g10_exit (2);
+    log_error(_(
+	"the trustdb is corrupted; please run \"gpg --fix-trustdb\".\n") );
+    g10_exit(2);
 }
 
 /*
@@ -1535,12 +1537,12 @@ migrate_from_v2 ()
   /* We have some restrictions here.  We can't use the version record
    * and we can't use any of the old hashtables because we dropped the
    * code.  So we first collect all ownertrusts and then use a second
-   * pass fo find the associated keys.  We have to do this all without using
+   * pass fo find the associated keys.  We have to do this all without using 
    * the regular record read functions.
    */
 
   /* get all the ownertrusts */
-  if (lseek (db_fd, 0, SEEK_SET ) == -1 )
+  if (lseek (db_fd, 0, SEEK_SET ) == -1 ) 
       log_fatal ("migrate_from_v2: lseek failed: %s\n", strerror (errno));
   for (recno=0;;recno++)
     {
@@ -1554,14 +1556,14 @@ migrate_from_v2 ()
 
       if (*oldbuf != 2)
         continue;
-
+      
       /* v2 dir record */
       if (ottable_used == ottable_size)
         {
           ottable_size += 1000;
           ottable = xrealloc (ottable, ottable_size * sizeof *ottable);
         }
-      ottable[ottable_used].keyrecno = buf32_to_ulong (oldbuf+6);
+      ottable[ottable_used].keyrecno = buftoulong (oldbuf+6);
       ottable[ottable_used].ot = oldbuf[18];
       ottable[ottable_used].okay = 0;
       memset (ottable[ottable_used].fpr,0, 20);
@@ -1571,7 +1573,7 @@ migrate_from_v2 ()
   log_info ("found %d ownertrust records\n", ottable_used);
 
   /* Read again and find the fingerprints */
-  if (lseek (db_fd, 0, SEEK_SET ) == -1 )
+  if (lseek (db_fd, 0, SEEK_SET ) == -1 ) 
       log_fatal ("migrate_from_v2: lseek failed: %s\n", strerror (errno));
   for (recno=0;;recno++)
     {
@@ -1583,7 +1585,7 @@ migrate_from_v2 ()
       if (n != 40)
         log_fatal ("migrate_from_v2: read error or short read\n");
 
-      if (*oldbuf != 3)
+      if (*oldbuf != 3) 
         continue;
 
       /* v2 key record */
@@ -1604,7 +1606,7 @@ migrate_from_v2 ()
   if (create_version_record ())
     log_fatal ("failed to recreate version record of `%s'\n", db_name);
 
-  /* access the hash table, so it is store just after the version record,
+  /* access the hash table, so it is store just after the version record, 
    * this is not needed put a dump is more pretty */
   get_trusthashrec ();
 
@@ -1614,7 +1616,7 @@ migrate_from_v2 ()
     {
       if (!ottable[i].okay)
         continue;
-
+      
       memset (&rec, 0, sizeof rec);
       rec.recnum = tdbio_new_recnum ();
       rec.rectype = RECTYPE_TRUST;
