@@ -1,11 +1,11 @@
 /* textfilter.c
- * Copyright (C) 1998, 1999, 2000, 2001, 2004 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -14,7 +14,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #include <config.h>
@@ -24,6 +25,7 @@
 #include <errno.h>
 #include <assert.h>
 
+#include "gpg.h"
 #include "errors.h"
 #include "iobuf.h"
 #include "memory.h"
@@ -31,7 +33,6 @@
 #include "filter.h"
 #include "i18n.h"
 #include "options.h"
-#include "status.h"
 
 #ifdef HAVE_DOSISH_SYSTEM
 #define LF "\r\n"
@@ -61,9 +62,17 @@ len_without_trailing_chars( byte *line, unsigned len, const char *trimchars )
     return mark? (mark - line) : len;
 }
 
+unsigned
+len_without_trailing_ws( byte *line, unsigned len )
+{
+    return len_without_trailing_chars( line, len, " \t\r\n" );
+}
+
+
+
 
 static int
-standard( text_filter_context_t *tfx, IOBUF a,
+standard( text_filter_context_t *tfx, iobuf_t a,
 	  byte *buf, size_t size, size_t *ret_len)
 {
     int rc=0;
@@ -93,30 +102,7 @@ standard( text_filter_context_t *tfx, IOBUF a,
 	    break;
 	}
 	lf_seen = tfx->buffer[tfx->buffer_len-1] == '\n';
-
-	/* The story behind this is that 2440 says that textmode
-	   hashes should canonicalize line endings to CRLF and remove
-	   spaces and tabs.  2440bis-12 says to just canonicalize to
-	   CRLF.  1.4.0 was released using the bis-12 behavior, but it
-	   was discovered that many mail clients do not canonicalize
-	   PGP/MIME signature text appropriately (and were relying on
-	   GnuPG to handle trailing spaces).  So, we default to the
-	   2440 behavior, but use the 2440bis-12 behavior if the user
-	   specifies --no-rfc2440-text.  The default will be changed
-	   at some point in the future when the mail clients have been
-	   upgraded.  Aside from PGP/MIME and broken mail clients,
-	   this makes no difference to any signatures in the real
-	   world except for a textmode detached signature.  PGP always
-	   used the 2440bis-12 behavior (ignoring 2440 itself), so
-	   this actually makes us compatible with PGP textmode
-	   detached signatures for the first time. */
-	if(opt.rfc2440_text)
-	  tfx->buffer_len=trim_trailing_chars(tfx->buffer,tfx->buffer_len,
-					      " \t\r\n");
-	else
-	  tfx->buffer_len=trim_trailing_chars(tfx->buffer,tfx->buffer_len,
-					      "\r\n");
-
+	tfx->buffer_len = trim_trailing_ws( tfx->buffer, tfx->buffer_len );
 	if( lf_seen ) {
 	    tfx->buffer[tfx->buffer_len++] = '\r';
 	    tfx->buffer[tfx->buffer_len++] = '\n';
@@ -127,13 +113,15 @@ standard( text_filter_context_t *tfx, IOBUF a,
 }
 
 
+
+
 /****************
  * The filter is used to make canonical text: Lines are terminated by
  * CR, LF, trailing white spaces are removed.
  */
 int
 text_filter( void *opaque, int control,
-	     IOBUF a, byte *buf, size_t *ret_len)
+	     iobuf_t a, byte *buf, size_t *ret_len)
 {
     size_t size = *ret_len;
     text_filter_context_t *tfx = opaque;
@@ -146,7 +134,7 @@ text_filter( void *opaque, int control,
 	if( tfx->truncated )
 	    log_error(_("can't handle text lines longer than %d characters\n"),
 			MAX_LINELEN );
-	xfree( tfx->buffer );
+	xfree ( tfx->buffer );
 	tfx->buffer = NULL;
     }
     else if( control == IOBUFCTRL_DESC )
@@ -160,7 +148,7 @@ text_filter( void *opaque, int control,
  * md is updated as required by rfc2440
  */
 int
-copy_clearsig_text( IOBUF out, IOBUF inp, MD_HANDLE md,
+copy_clearsig_text( iobuf_t out, iobuf_t inp, MD_HANDLE md,
 		    int escape_dash, int escape_from, int pgp2mode )
 {
     unsigned maxlen;
@@ -176,8 +164,6 @@ copy_clearsig_text( IOBUF out, IOBUF inp, MD_HANDLE md,
     if( !escape_dash )
 	escape_from = 0;
 
-    write_status (STATUS_BEGIN_SIGNING);
-
     for(;;) {
 	maxlen = MAX_LINELEN;
 	n = iobuf_read_line( inp, &buffer, &bufsize, &maxlen );
@@ -190,15 +176,15 @@ copy_clearsig_text( IOBUF out, IOBUF inp, MD_HANDLE md,
 	/* update the message digest */
 	if( escape_dash ) {
 	    if( pending_lf ) {
-		md_putc( md, '\r' );
-		md_putc( md, '\n' );
+		gcry_md_putc( md, '\r' );
+		gcry_md_putc( md, '\n' );
 	    }
-	    md_write( md, buffer,
+	    gcry_md_write( md, buffer,
 		     len_without_trailing_chars( buffer, n,
 						 pgp2mode? " \r\n":" \t\r\n"));
 	}
 	else
-	    md_write( md, buffer, n );
+	    gcry_md_write( md, buffer, n );
 	pending_lf = buffer[n-1] == '\n';
 
 	/* write the output */
@@ -239,7 +225,7 @@ copy_clearsig_text( IOBUF out, IOBUF inp, MD_HANDLE md,
     if( !pending_lf ) { /* make sure that the file ends with a LF */
 	iobuf_writestr( out, LF );
 	if( !escape_dash )
-	    md_putc( md, '\n' );
+	    gcry_md_putc( md, '\n' );
     }
 
     if( truncated )
