@@ -1,6 +1,6 @@
 /* trustdb.h - Trust database
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004,
- *               2005 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003
+ *                                             Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -22,6 +22,7 @@
 #ifndef G10_TRUSTDB_H
 #define G10_TRUSTDB_H
 
+
 /* Trust values must be sorted in ascending order */
 #define TRUST_MASK	 15
 #define TRUST_UNKNOWN	  0  /* o: not yet calculated/assigned */
@@ -38,22 +39,18 @@
 #define TRUST_FLAG_PENDING_CHECK 256 /* a check-trustdb is pending */
 
 /*-- trustdb.c --*/
-void register_trusted_keyid(u32 *keyid);
 void register_trusted_key( const char *string );
 void check_trustdb (void);
 void update_trustdb (void);
 int setup_trustdb( int level, const char *dbname );
 void init_trustdb( void );
-void check_trustdb_stale(void);
 void sync_trustdb( void );
 
-const char *uid_trust_string_fixed(PKT_public_key *key,PKT_user_id *uid);
 const char *trust_value_to_string (unsigned int value);
 int string_to_trust_value (const char *str);
 
 void revalidation_mark (void);
 int trustdb_pending_check(void);
-void trustdb_check_or_update(void);
 
 int cache_disabled_value(PKT_public_key *pk);
 
