@@ -1,5 +1,5 @@
 /* verify.c - verify signed data
- * Copyright (C) 1998, 1999, 2000, 2001, 2004 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -15,8 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #include <config.h>
@@ -55,7 +54,7 @@
 int
 verify_signatures( int nfiles, char **files )
 {
-    IOBUF fp;
+    iobuf_t fp;
     armor_filter_context_t afx;
     progress_filter_context_t pfx;
     const char *sigfile;
@@ -92,15 +91,11 @@ verify_signatures( int nfiles, char **files )
 
     /* open the signature file */
     fp = iobuf_open(sigfile);
-    if (fp && is_secured_file (iobuf_get_fd (fp)))
-      {
-        iobuf_close (fp);
-        fp = NULL;
-        errno = EPERM;
-      }
     if( !fp ) {
-	log_error(_("can't open `%s'\n"), print_fname_stdin(sigfile));
-	return G10ERR_OPEN_FILE;
+        rc = gpg_error_from_errno (errno);
+	log_error(_("can't open `%s': %s\n"),
+                  print_fname_stdin(sigfile), strerror (errno));
+        return rc;
     }
     handle_progress (&pfx, fp, sigfile);
 
@@ -108,12 +103,12 @@ verify_signatures( int nfiles, char **files )
 	iobuf_push_filter( fp, armor_filter, &afx );
 
     sl = NULL;
-    for(i=nfiles-1 ; i > 0 ; i-- )
+    for(i=1 ; i < nfiles; i++ )
 	add_to_strlist( &sl, files[i] );
     rc = proc_signature_packets( NULL, fp, sl, sigfile );
     free_strlist(sl);
     iobuf_close(fp);
-    if( (afx.no_openpgp_data && rc == -1) || rc == G10ERR_NO_DATA ) {
+    if( afx.no_openpgp_data && rc == -1 ) {
 	log_error(_("the signature could not be verified.\n"
 		   "Please remember that the signature file (.sig or .asc)\n"
 		   "should be the first file given on the command line.\n") );
@@ -127,35 +122,29 @@ verify_signatures( int nfiles, char **files )
 void
 print_file_status( int status, const char *name, int what )
 {
-    char *p = m_alloc(strlen(name)+10);
+    char *p = xmalloc (strlen(name)+10);
     sprintf(p, "%d %s", what, name );
     write_status_text( status, p );
-    m_free(p);
+    xfree (p);
 }
 
 
 static int
 verify_one_file( const char *name )
 {
-    IOBUF fp;
+    iobuf_t fp;
     armor_filter_context_t afx;
     progress_filter_context_t pfx;
     int rc;
 
     print_file_status( STATUS_FILE_START, name, 1 );
     fp = iobuf_open(name);
-    if (fp)
-      iobuf_ioctl (fp,3,1,NULL); /* disable fd caching */
-    if (fp && is_secured_file (iobuf_get_fd (fp)))
-      {
-        iobuf_close (fp);
-        fp = NULL;
-        errno = EPERM;
-      }
     if( !fp ) {
+        rc = gpg_error_from_errno (errno);
+	log_error(_("can't open `%s': %s\n"),
+                  print_fname_stdin(name), strerror (errno));
 	print_file_status( STATUS_FILE_ERROR, name, 1 );
-	log_error(_("can't open `%s'\n"), print_fname_stdin(name));
-	return G10ERR_OPEN_FILE;
+	return rc;
     }
     handle_progress (&pfx, fp, name);
 
@@ -190,7 +179,7 @@ verify_files( int nfiles, char **files )
 	    lno++;
 	    if( !*line || line[strlen(line)-1] != '\n' ) {
 		log_error(_("input line %u too long or missing LF\n"), lno );
-		return G10ERR_GENERAL;
+		return GPG_ERR_GENERAL;
 	    }
 	    /* This code does not work on MSDOS but how cares there are
 	     * also no script languages available.  We don't strip any
