@@ -1,11 +1,11 @@
-/* errors.h - erro code
+/* errors.h - error code
  *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GNUPG.
  *
  * GNUPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * GNUPG is distributed in the hope that it will be useful,
@@ -14,11 +14,19 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifndef G10_ERRORS_H
-#define G10_ERRORS_H
+#ifndef GNUPG_INCLUDE_ERRORS_H
+#define GNUPG_INCLUDE_ERRORS_H
 
+#if 0
+#error Remove this file after replacing all error codes with those
+#error from libgpg-error.  The numerical values are identical, though.
+#endif
+
+
+#if 0 /* Not used anymore. */
 #define G10ERR_GENERAL	       1
 #define G10ERR_UNKNOWN_PACKET  2
 #define G10ERR_UNKNOWN_VERSION 3 /* Unknown version (in packet) */
@@ -74,16 +82,20 @@
 #define G10ERR_UNU_PUBKEY     53
 #define G10ERR_UNU_SECKEY     54
 #define G10ERR_KEYSERVER      55
-#define G10ERR_CANCELED       56
-#define G10ERR_NO_CARD        57
-#define G10ERR_NO_DATA        58
+#endif
 
 #ifndef HAVE_STRERROR
-char *strerror (int n);
+char *strerror( int n );
 #endif
 
-#ifdef _WIN32
-const char * w32_strerror (int w32_errno);
-#endif
+#endif /*GNUPG_INCLUDE_ERRORS_H*/
+
+
+
+
+
+
+
+
+
 
-#endif /*G10_ERRORS_H*/
