@@ -6,7 +6,7 @@
  *
  * GNUPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * GNUPG is distributed in the hope that it will be useful,
@@ -15,7 +15,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  *
  * Note: This code is heavily based on the GNU MP Library.
  *	 Actually it's the same code with only minor changes in the
@@ -29,12 +30,34 @@
 #ifndef G10_MPI_H
 #define G10_MPI_H
 
+#include <gcrypt.h>
+
+#if 0
+
+
 #include <config.h>
 #include <stdio.h>
 #include "iobuf.h"
 #include "types.h"
 #include "memory.h"
 
+#if BYTES_PER_MPI_LIMB == SIZEOF_UNSIGNED_INT
+  typedef unsigned int mpi_limb_t;
+  typedef   signed int mpi_limb_signed_t;
+#elif BYTES_PER_MPI_LIMB == SIZEOF_UNSIGNED_LONG
+  typedef unsigned long int mpi_limb_t;
+  typedef   signed long int mpi_limb_signed_t;
+#elif BYTES_PER_MPI_LIMB == SIZEOF_UNSIGNED_LONG_LONG
+  typedef unsigned long long int mpi_limb_t;
+  typedef   signed long long int mpi_limb_signed_t;
+#elif BYTES_PER_MPI_LIMB == SIZEOF_UNSIGNED_SHORT
+  typedef unsigned short int mpi_limb_t;
+  typedef   signed short int mpi_limb_signed_t;
+#else
+#error BYTES_PER_MPI_LIMB does not match any C type
+#endif
+#define BITS_PER_MPI_LIMB    (8*BYTES_PER_MPI_LIMB)
+
 #ifndef EXTERN_UNLESS_MAIN_MODULE
 #if defined (__riscos__) && !defined (INCLUDED_BY_MAIN_MODULE)
 #define EXTERN_UNLESS_MAIN_MODULE extern
@@ -47,9 +70,23 @@
 EXTERN_UNLESS_MAIN_MODULE int mpi_debug_mode;
 
 
-struct gcry_mpi; 
+struct gcry_mpi {
+    int alloced;    /* array size (# of allocated limbs) */
+    int nlimbs;     /* number of valid limbs */
+    int nbits;	    /* the real number of valid bits (info only) */
+    int sign;	    /* indicates a negative number */
+    unsigned flags; /* bit 0: array must be allocated in secure memory space */
+		    /* bit 1: not used */
+		    /* bit 2: the limb is a pointer to some m_alloced data */
+    mpi_limb_t *d;  /* array with the limbs */
+};
+
 typedef struct gcry_mpi *MPI;
 
+#define MPI_NULL NULL
+
+#define mpi_get_nlimbs(a)     ((a)->nlimbs)
+#define mpi_is_neg(a)	      ((a)->sign)
 
 /*-- mpiutil.c --*/
 
@@ -74,23 +111,17 @@ void mpi_free( MPI a );
 void mpi_resize( MPI a, unsigned nlimbs );
 MPI  mpi_copy( MPI a );
 #endif
-#define mpi_is_opaque(a) ((a) && (mpi_get_flags (a)&4))
-MPI mpi_set_opaque( MPI a, void *p, unsigned int len );
-void *mpi_get_opaque( MPI a, unsigned int *len );
-#define mpi_is_secure(a) ((a) && (mpi_get_flags (a)&1))
+#define mpi_is_opaque(a) ((a) && ((a)->flags&4))
+MPI mpi_set_opaque( MPI a, void *p, int len );
+void *mpi_get_opaque( MPI a, int *len );
+#define mpi_is_secure(a) ((a) && ((a)->flags&1))
 void mpi_set_secure( MPI a );
 void mpi_clear( MPI a );
 void mpi_set( MPI w, MPI u);
-void mpi_set_cond( MPI w, MPI u, unsigned long set);
 void mpi_set_ui( MPI w, ulong u);
 MPI  mpi_alloc_set_ui( unsigned long u);
 void mpi_m_check( MPI a );
 void mpi_swap( MPI a, MPI b);
-int  mpi_get_nlimbs (MPI a);
-int  mpi_is_neg (MPI a);
-unsigned int mpi_nlimb_hint_from_nbytes (unsigned int nbytes);
-unsigned int mpi_nlimb_hint_from_nbits (unsigned int nbits);
-unsigned int mpi_get_flags (MPI a);
 
 /*-- mpicoder.c --*/
 int mpi_write( IOBUF out, MPI a );
@@ -140,7 +171,7 @@ int mpi_gcd( MPI g, MPI a, MPI b );
 
 /*-- mpi-pow.c --*/
 void mpi_pow( MPI w, MPI u, MPI v);
-void mpi_powm( MPI res, MPI base, MPI exponent, MPI mod);
+void mpi_powm( MPI res, MPI base, MPI exp, MPI mod);
 
 /*-- mpi-mpow.c --*/
 void mpi_mulpowm( MPI res, MPI *basearray, MPI *exparray, MPI mod);
@@ -166,5 +197,5 @@ void mpi_rshift( MPI x, MPI a, unsigned n );
 
 /*-- mpi-inv.c --*/
 void mpi_invm( MPI x, MPI u, MPI v );
-
+#endif
 #endif /*G10_MPI_H*/
