@@ -5,7 +5,7 @@
  *
  * GNUPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * GNUPG is distributed in the hope that it will be useful,
@@ -14,29 +14,31 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #ifndef G10_TYPES_H
 #define G10_TYPES_H
 
 #ifdef HAVE_INTTYPES_H
-# include <inttypes.h>   /* For uint64_t */
+/* For uint64_t */
+#include <inttypes.h>
 #endif
 
 /* The AC_CHECK_SIZEOF() in configure fails for some machines.
  * we provide some fallback values here */
 #if !SIZEOF_UNSIGNED_SHORT
-# undef SIZEOF_UNSIGNED_SHORT
-# define SIZEOF_UNSIGNED_SHORT 2
+#undef SIZEOF_UNSIGNED_SHORT
+#define SIZEOF_UNSIGNED_SHORT 2
 #endif
 #if !SIZEOF_UNSIGNED_INT
-# undef SIZEOF_UNSIGNED_INT
-# define SIZEOF_UNSIGNED_INT 4
+#undef SIZEOF_UNSIGNED_INT
+#define SIZEOF_UNSIGNED_INT 4
 #endif
 #if !SIZEOF_UNSIGNED_LONG
-# undef SIZEOF_UNSIGNED_LONG
-# define SIZEOF_UNSIGNED_LONG 4
+#undef SIZEOF_UNSIGNED_LONG
+#define SIZEOF_UNSIGNED_LONG 4
 #endif
 
 
@@ -44,52 +46,52 @@
 
 
 #ifndef HAVE_BYTE_TYPEDEF
-# undef byte	    /* maybe there is a macro with this name */
-# ifndef __riscos__
+#undef byte	    /* maybe there is a macro with this name */
+#ifndef __riscos__
 typedef unsigned char byte;
-# else
+#else 
 /* Norcroft treats char  = unsigned char  as legal assignment
                but char* = unsigned char* as illegal assignment
    and the same applies to the signed variants as well  */
 typedef char byte;
-# endif
-# define HAVE_BYTE_TYPEDEF
+#endif
+#define HAVE_BYTE_TYPEDEF
 #endif
 
 #ifndef HAVE_USHORT_TYPEDEF
-# undef ushort     /* maybe there is a macro with this name */
+#undef ushort     /* maybe there is a macro with this name */
 typedef unsigned short ushort;
-# define HAVE_USHORT_TYPEDEF
+#define HAVE_USHORT_TYPEDEF
 #endif
 
 #ifndef HAVE_ULONG_TYPEDEF
-# undef ulong	    /* maybe there is a macro with this name */
+#undef ulong	    /* maybe there is a macro with this name */
 typedef unsigned long ulong;
-# define HAVE_ULONG_TYPEDEF
+#define HAVE_ULONG_TYPEDEF
 #endif
 
 #ifndef HAVE_U16_TYPEDEF
-# undef u16	    /* maybe there is a macro with this name */
-# if SIZEOF_UNSIGNED_INT == 2
+#undef u16	    /* maybe there is a macro with this name */
+#if SIZEOF_UNSIGNED_INT == 2
 typedef unsigned int   u16;
-# elif SIZEOF_UNSIGNED_SHORT == 2
+#elif SIZEOF_UNSIGNED_SHORT == 2
 typedef unsigned short u16;
-# else
-#  error no typedef for u16
-# endif
-# define HAVE_U16_TYPEDEF
+#else
+#error no typedef for u16
+#endif
+#define HAVE_U16_TYPEDEF
 #endif
 
 #ifndef HAVE_U32_TYPEDEF
-# undef u32	    /* maybe there is a macro with this name */
-# if SIZEOF_UNSIGNED_INT == 4
+#undef u32	    /* maybe there is a macro with this name */
+#if SIZEOF_UNSIGNED_INT == 4
 typedef unsigned int u32;
-# elif SIZEOF_UNSIGNED_LONG == 4
+#elif SIZEOF_UNSIGNED_LONG == 4
 typedef unsigned long u32;
-# else
-#  error no typedef for u32
-# endif
-# define HAVE_U32_TYPEDEF
+#else
+#error no typedef for u32
+#endif
+#define HAVE_U32_TYPEDEF
 #endif
 
 /****************
@@ -98,29 +100,24 @@ typedef unsigned long u32;
  * Solaris and IRIX.
  */
 #ifndef HAVE_U64_TYPEDEF
-# undef u64	    /* maybe there is a macro with this name */
-# if SIZEOF_UINT64_T == 8
+#undef u64	    /* maybe there is a macro with this name */
+#if SIZEOF_UINT64_T == 8
 typedef uint64_t u64;
-# ifdef UINT64_C
-#  define U64_C(c) (UINT64_C(c))
-# else
-   /* make a best guess, could happen with UNIX98 <inttypes.h> */
-#  define U64_C(c) (c)
-# endif
-# define HAVE_U64_TYPEDEF
+#define U64_C(c) (UINT64_C(c))
+#define HAVE_U64_TYPEDEF
 #elif SIZEOF_UNSIGNED_INT == 8
 typedef unsigned int u64;
-#  define U64_C(c) (c ## U)
-#  define HAVE_U64_TYPEDEF
-# elif SIZEOF_UNSIGNED_LONG == 8
+#define U64_C(c) (c ## U)
+#define HAVE_U64_TYPEDEF
+#elif SIZEOF_UNSIGNED_LONG == 8
 typedef unsigned long u64;
-#  define U64_C(c) (c ## UL)
-#  define HAVE_U64_TYPEDEF
+#define U64_C(c) (c ## UL)
+#define HAVE_U64_TYPEDEF
 #elif SIZEOF_UNSIGNED_LONG_LONG == 8
 typedef unsigned long long u64;
-#  define U64_C(c) (c ## ULL)
-#  define HAVE_U64_TYPEDEF
-# endif
+#define U64_C(c) (c ## ULL)
+#define HAVE_U64_TYPEDEF
+#endif
 #endif
 
 typedef union {
@@ -135,12 +132,4 @@ typedef union {
     double g;
 } PROPERLY_ALIGNED_TYPE;
 
-struct string_list {
-    struct string_list *next;
-    unsigned int flags;
-    char d[1];
-};
-typedef struct string_list *STRLIST;
-typedef struct string_list *strlist_t;
-
 #endif /*G10_TYPES_H*/
