@@ -1,5 +1,6 @@
 /* [argparse.c wk 17.06.97] Argument Parser for option handling
- *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *
  *  This file is part of GnuPG.
  *
  *  GnuPG is free software; you can redistribute it and/or modify
@@ -164,35 +165,37 @@ initialize( ARGPARSE_ARGS *arg, const char *filename, unsigned *lineno )
 
 	if( filename ) {
 	    if( arg->r_opt == -6 )
-		s = "%s:%u: argument not expected\n";
+		s = "argument not expected\n";
 	    else if( arg->r_opt == -5 )
-		s = "%s:%u: read error\n";
+		s = "read error\n";
 	    else if( arg->r_opt == -4 )
-		s = "%s:%u: keyword too long\n";
+		s = "keyword too long\n";
 	    else if( arg->r_opt == -3 )
-		s = "%s:%u: missing argument\n";
+		s = "missing argument\n";
 	    else if( arg->r_opt == -7 )
-		s = "%s:%u: invalid command\n";
+		s = "invalid command\n";
 	    else if( arg->r_opt == -10 )
-		s = "%s:%u: invalid alias definition\n";
+		s = "invalid alias definition\n";
 	    else
-		s = "%s:%u: invalid option\n";
-	    jnlib_log_error(s, filename, *lineno );
+		s = "invalid option\n";
+	    jnlib_log_error("%s:%u: %s\n", filename, *lineno, s);
 	}
 	else {
+            s = arg->internal.last? arg->internal.last:"[??]";
+            
 	    if( arg->r_opt == -3 )
-		s = "Missing argument for option \"%.50s\"\n";
+              jnlib_log_error ("Missing argument for option \"%.50s\"\n", s);
 	    else if( arg->r_opt == -6 )
-		s = "Option \"%.50s\" does not expect an argument\n";
+              jnlib_log_error ("Option \"%.50s\" does not expect an argument\n",
+                               s );
 	    else if( arg->r_opt == -7 )
-		s = "Invalid command \"%.50s\"\n";
+              jnlib_log_error ("Invalid command \"%.50s\"\n", s);
 	    else if( arg->r_opt == -8 )
-		s = "Option \"%.50s\" is ambiguous\n";
+              jnlib_log_error ("Option \"%.50s\" is ambiguous\n", s);
 	    else if( arg->r_opt == -9 )
-		s = "Command \"%.50s\" is ambiguous\n";
+              jnlib_log_error ("Command \"%.50s\" is ambiguous\n",s );
 	    else
-		s = "Invalid option \"%.50s\"\n";
-	    jnlib_log_error(s, arg->internal.last? arg->internal.last:"[??]" );
+              jnlib_log_error ("Invalid option \"%.50s\"\n", s);
 	}
 	if( arg->err != 1 )
 	    exit(2);
@@ -275,10 +278,12 @@ optfile_parse( FILE *fp, const char *filename, unsigned *lineno,
 		    arg->r_opt = -arg->r_opt;
 		if( !opts[idx].short_opt )   /* unknown command/option */
 		    arg->r_opt = (opts[idx].flags & 256)? -7:-2;
-		else if( (opts[idx].flags & 8) ) /* no argument */
-		    arg->r_opt = -3;	       /* error */
-		else			       /* no or optional argument */
+		else if( !(opts[idx].flags & 7) ) /* does not take an arg */
 		    arg->r_type = 0;	       /* okay */
+		else if( (opts[idx].flags & 8) )  /* argument is optional */
+                    arg->r_type = 0;	       /* okay */
+		else			       /* required argument */
+		    arg->r_opt = -3;	       /* error */
 		break;
 	    }
 	    else if( state == 3 ) {	       /* no argument found */
@@ -899,7 +904,7 @@ strusage( int level )
     switch( level ) {
       case 11: p = "foo"; break;
       case 13: p = "0.0"; break;
-      case 14: p = "Copyright (C) 2000 Free Software Foundation, Inc."; break;
+      case 14: p = "Copyright (C) 2004 Free Software Foundation, Inc."; break;
       case 15: p =
 "This program comes with ABSOLUTELY NO WARRANTY.\n"
 "This is free software, and you are welcome to redistribute it\n"
