@@ -1,5 +1,5 @@
 /* gpgkeys_curl.c - fetch a key via libcurl
- * Copyright (C) 2004, 2005 Free Software Foundation, Inc.
+ * Copyright (C) 2004 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -15,8 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #include <config.h>
@@ -28,66 +27,87 @@
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
-#ifdef FAKE_CURL
-#include "curl-shim.h"
-#else
 #include <curl/curl.h>
-#endif
 #include "keyserver.h"
 #include "ksutil.h"
 
 extern char *optarg;
 extern int optind;
 
-static FILE *input,*output,*console;
+#define GET           0
+#define MAX_LINE     80
+
+#define MAX_SCHEME   20
+#define MAX_AUTH    128
+#define MAX_HOST     80
+#define MAX_PORT     10
+#define MAX_PATH   1024
+#define MAX_PROXY   128
+#define MAX_URL (MAX_SCHEME+1+3+MAX_AUTH+1+1+MAX_HOST+1+1+MAX_PORT+1+1+MAX_PATH+1+50)
+
+#define STRINGIFY(x) #x
+#define MKSTRING(x) STRINGIFY(x)
+
+static int verbose=0;
+static char scheme[MAX_SCHEME+1],auth[MAX_AUTH+1],host[MAX_HOST+1]={'\0'},port[MAX_PORT+1]={'\0'},path[MAX_PATH+1]={'\0'},proxy[MAX_PROXY+1]={'\0'};
+static FILE *input=NULL,*output=NULL,*console=NULL;
 static CURL *curl;
-static struct ks_options *opt;
+static char request[MAX_URL]={'\0'};
+
+static int
+curl_err_to_gpg_err(CURLcode error)
+{
+  switch(error)
+    {
+    case CURLE_FTP_COULDNT_RETR_FILE: return KEYSERVER_KEY_NOT_FOUND;
+    default: return KEYSERVER_INTERNAL_ERROR;
+    }
+}
+
+/* We wrap fwrite so to avoid DLL problems on Win32 (see curl faq for
+   more). */
+static size_t
+writer(const void *ptr,size_t size,size_t nmemb,void *stream)
+{
+  return fwrite(ptr,size,nmemb,stream);
+}
 
 static int
 get_key(char *getkey)
 {
   CURLcode res;
   char errorbuffer[CURL_ERROR_SIZE];
-  char request[MAX_URL];
-  struct curl_writer_ctx ctx;
-
-  memset(&ctx,0,sizeof(ctx));
 
   if(strncmp(getkey,"0x",2)==0)
     getkey+=2;
 
   fprintf(output,"KEY 0x%s BEGIN\n",getkey);
 
-  sprintf(request,"%s://%s%s%s%s",opt->scheme,opt->host,
-	  opt->port?":":"",opt->port?opt->port:"",opt->path?opt->path:"/");
+  sprintf(request,"%s://%s%s%s%s%s%s%s",scheme,auth[0]?auth:"",auth[0]?"@":"",
+	  host,port[0]?":":"",port[0]?port:"",path[0]?"":"/",path);
 
   curl_easy_setopt(curl,CURLOPT_URL,request);
-  curl_easy_setopt(curl,CURLOPT_WRITEFUNCTION,curl_writer);
-  ctx.stream=output;
-  curl_easy_setopt(curl,CURLOPT_FILE,&ctx);
+  curl_easy_setopt(curl,CURLOPT_WRITEFUNCTION,writer);
+  curl_easy_setopt(curl,CURLOPT_FILE,output);
   curl_easy_setopt(curl,CURLOPT_ERRORBUFFER,errorbuffer);
 
-  res=curl_easy_perform(curl);
-  if(res!=CURLE_OK)
+  if(verbose>1)
     {
-      fprintf(console,"gpgkeys: %s fetch error %d: %s\n",opt->scheme,
-	      res,errorbuffer);
-      fprintf(output,"\nKEY 0x%s FAILED %d\n",getkey,curl_err_to_gpg_err(res));
-    }
-  else
-    {
-      curl_writer_finalize(&ctx);
-      if(!ctx.flags.done)
-	{
-	  fprintf(console,"gpgkeys: no key data found for %s\n",request);
-	  fprintf(output,"\nKEY 0x%s FAILED %d\n",
-		  getkey,KEYSERVER_KEY_NOT_FOUND);
-	}
-      else
-	fprintf(output,"\nKEY 0x%s END\n",getkey);
+      curl_easy_setopt(curl,CURLOPT_STDERR,console);
+      curl_easy_setopt(curl,CURLOPT_VERBOSE,1);
     }
 
-  return curl_err_to_gpg_err(res);
+  res=curl_easy_perform(curl);
+  if(res!=0)
+    {
+      fprintf(console,"gpgkeys: %s fetch error %d: %s\n",scheme,
+	      res,errorbuffer);
+      fprintf(output,"KEY 0x%s FAILED %d\n",getkey,curl_err_to_gpg_err(res));
+    }
+  else
+    fprintf(output,"KEY 0x%s END\n",getkey);
+
+  return KEYSERVER_OK;
 }
 
 static void 
@@ -101,11 +121,11 @@ show_help (FILE *fp)
 int
 main(int argc,char *argv[])
 {
-  int arg,ret=KEYSERVER_INTERNAL_ERROR;
+  int arg,action=-1,ret=KEYSERVER_INTERNAL_ERROR;
   char line[MAX_LINE];
   char *thekey=NULL;
+  unsigned int timeout=DEFAULT_KEYSERVER_TIMEOUT;
   long follow_redirects=5;
-  char *proxy=NULL;
 
   console=stderr;
 
@@ -162,60 +182,111 @@ main(int argc,char *argv[])
   if(output==NULL)
     output=stdout;
 
-  opt=init_ks_options();
-  if(!opt)
-    return KEYSERVER_NO_MEMORY;
-
   /* Get the command and info block */
 
   while(fgets(line,MAX_LINE,input)!=NULL)
     {
-      int err;
-      char option[MAX_OPTION+1];
+      int version;
+      char commandstr[7];
+      char optionstr[256];
+      char hash;
 
       if(line[0]=='\n')
 	break;
 
-      err=parse_ks_options(line,opt);
-      if(err>0)
-	{
-	  ret=err;
-	  goto fail;
-	}
-      else if(err==0)
+      if(sscanf(line,"%c",&hash)==1 && hash=='#')
 	continue;
 
-      if(sscanf(line,"OPTION %" MKSTRING(MAX_OPTION) "s\n",option)==1)
+      if(sscanf(line,"COMMAND %6s\n",commandstr)==1)
 	{
-	  int no=0;
-	  char *start=&option[0];
+	  commandstr[6]='\0';
 
-	  option[MAX_OPTION]='\0';
+	  if(strcasecmp(commandstr,"get")==0)
+	    action=GET;
 
-	  if(strncasecmp(option,"no-",3)==0)
+	  continue;
+	}
+
+      if(sscanf(line,"SCHEME %" MKSTRING(MAX_SCHEME) "s\n",scheme)==1)
+	{
+	  scheme[MAX_SCHEME]='\0';
+	  continue;
+	}
+
+      if(sscanf(line,"AUTH %" MKSTRING(MAX_AUTH) "s\n",auth)==1)
+	{
+	  auth[MAX_AUTH]='\0';
+	  continue;
+	}
+
+      if(sscanf(line,"HOST %" MKSTRING(MAX_HOST) "s\n",host)==1)
+	{
+	  host[MAX_HOST]='\0';
+	  continue;
+	}
+
+      if(sscanf(line,"PORT %" MKSTRING(MAX_PORT) "s\n",port)==1)
+	{
+	  port[MAX_PORT]='\0';
+	  continue;
+	}
+
+      if(sscanf(line,"PATH %" MKSTRING(MAX_PATH) "s\n",path)==1)
+	{
+	  path[MAX_PATH]='\0';
+	  continue;
+	}
+
+      if(sscanf(line,"VERSION %d\n",&version)==1)
+	{
+	  if(version!=KEYSERVER_PROTO_VERSION)
 	    {
-	      no=1;
-	      start=&option[3];
+	      ret=KEYSERVER_VERSION_ERROR;
+	      goto fail;
 	    }
 
-	  if(strncasecmp(start,"http-proxy",10)==0)
+	  continue;
+	}
+
+      if(sscanf(line,"OPTION %255s\n",optionstr)==1)
+	{
+	  int no=0;
+	  char *start=&optionstr[0];
+
+	  optionstr[255]='\0';
+
+	  if(strncasecmp(optionstr,"no-",3)==0)
+	    {
+	      no=1;
+	      start=&optionstr[3];
+	    }
+
+	  if(strcasecmp(start,"verbose")==0)
 	    {
-	      /* Safe to not check the return code of strdup() here.
-		 If it fails, we simply won't use a proxy. */
 	      if(no)
-		{
-		  free(proxy);
-		  proxy=strdup("");
-		}
+		verbose--;
+	      else
+		verbose++;
+	    }
+	  else if(strncasecmp(start,"http-proxy",10)==0)
+	    {
+	      if(no)
+		proxy[0]='\0';
 	      else if(start[10]=='=')
 		{
-		  if(strlen(&start[11])<MAX_PROXY)
-		    {
-		      free(proxy);
-		      proxy=strdup(&start[11]);
-		    }
+		  strncpy(proxy,&start[11],MAX_PROXY);
+		  proxy[MAX_PROXY]='\0';
 		}
 	    }
+	  else if(strncasecmp(start,"timeout",7)==0)
+	    {
+	      if(no)
+		timeout=0;
+	      else if(start[7]=='=')
+		timeout=atoi(&start[8]);
+	      else if(start[7]=='\0')
+		timeout=DEFAULT_KEYSERVER_TIMEOUT;
+	    }
 	  else if(strncasecmp(start,"follow-redirects",16)==0)
 	    {
 	      if(no)
@@ -230,20 +301,27 @@ main(int argc,char *argv[])
 	}
     }
 
-  if(!opt->scheme)
+  if(scheme[0]=='\0')
     {
       fprintf(console,"gpgkeys: no scheme supplied!\n");
-      ret=KEYSERVER_SCHEME_NOT_FOUND;
-      goto fail;
+      return KEYSERVER_SCHEME_NOT_FOUND;
     }
-
-  if(!opt->host)
+#ifndef HTTP_VIA_LIBCURL
+  else if(strcasecmp(scheme,"http")==0)
     {
-      fprintf(console,"gpgkeys: no keyserver host provided\n");
-      goto fail;
+      fprintf(console,"gpgkeys: scheme `%s' not supported\n",scheme);
+      return KEYSERVER_SCHEME_NOT_FOUND;
     }
+#endif /* HTTP_VIA_LIBCURL */
+#ifndef FTP_VIA_LIBCURL
+  else if(strcasecmp(scheme,"ftp")==0)
+    {
+      fprintf(console,"gpgkeys: scheme `%s' not supported\n",scheme);
+      return KEYSERVER_SCHEME_NOT_FOUND;
+    }
+#endif /* FTP_VIA_LIBCURL */
 
-  if(opt->timeout && register_timeout()==-1)
+  if(timeout && register_timeout()==-1)
     {
       fprintf(console,"gpgkeys: unable to register timeout handler\n");
       return KEYSERVER_INTERNAL_ERROR;
@@ -265,26 +343,13 @@ main(int argc,char *argv[])
 	curl_easy_setopt(curl,CURLOPT_MAXREDIRS,follow_redirects);
     }
 
-  if(opt->auth)
-    curl_easy_setopt(curl,CURLOPT_USERPWD,opt->auth);
-
-  if(opt->debug)
-    {
-      fprintf(console,"gpgkeys: curl version = %s\n",curl_version());
-      curl_easy_setopt(curl,CURLOPT_STDERR,console);
-      curl_easy_setopt(curl,CURLOPT_VERBOSE,1);
-    }
-
-  curl_easy_setopt(curl,CURLOPT_SSL_VERIFYPEER,opt->flags.check_cert);
-  curl_easy_setopt(curl,CURLOPT_CAINFO,opt->ca_cert_file);
-
-  if(proxy)
+  if(proxy[0])
     curl_easy_setopt(curl,CURLOPT_PROXY,proxy);
 
   /* If it's a GET or a SEARCH, the next thing to come in is the
      keyids.  If it's a SEND, then there are no keyids. */
 
-  if(opt->action==KS_GET)
+  if(action==GET)
     {
       /* Eat the rest of the file */
       for(;;)
@@ -320,7 +385,7 @@ main(int argc,char *argv[])
       goto fail;
     }
 
-  if(!thekey)
+  if(!thekey || !host[0])
     {
       fprintf(console,"gpgkeys: invalid keyserver instructions\n");
       goto fail;
@@ -331,21 +396,23 @@ main(int argc,char *argv[])
   fprintf(output,"VERSION %d\n",KEYSERVER_PROTO_VERSION);
   fprintf(output,"PROGRAM %s\n\n",VERSION);
 
-  if(opt->verbose)
+  if(verbose)
     {
-      fprintf(console,"Scheme:\t\t%s\n",opt->scheme);
-      fprintf(console,"Host:\t\t%s\n",opt->host);
-      if(opt->port)
-	fprintf(console,"Port:\t\t%s\n",opt->port);
-      if(opt->path)
-	fprintf(console,"Path:\t\t%s\n",opt->path);
+      fprintf(console,"Scheme:\t\t%s\n",scheme);
+      fprintf(console,"Host:\t\t%s\n",host);
+      if(port[0])
+	fprintf(console,"Port:\t\t%s\n",port);
+      if(path[0])
+	fprintf(console,"Path:\t\t%s\n",path);
       fprintf(console,"Command:\tGET\n");
     }
 
-  set_timeout(opt->timeout);
+  set_timeout(timeout);
 
   ret=get_key(thekey);
 
+  curl_easy_cleanup(curl);
+
  fail:
 
   free(thekey);
@@ -356,13 +423,6 @@ main(int argc,char *argv[])
   if(output!=stdout)
     fclose(output);
 
-  free_ks_options(opt);
-
-  if(curl)
-    curl_easy_cleanup(curl);
-
-  free(proxy);
-
   curl_global_cleanup();
 
   return ret;
