@@ -25,6 +25,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <unistd.h>
+#include <signal.h>
 
 #include <assuan.h>
 
@@ -33,14 +34,21 @@
 #include "app-common.h"
 #include "apdu.h" /* Required for apdu_*_reader (). */
 
-/* maximum length aloowed as a PIN; used for INQUIRE NEEDPIN */
+/* Maximum length allowed as a PIN; used for INQUIRE NEEDPIN */
 #define MAXLEN_PIN 100
 
+
+/* We keep track of the primary client using scdaemon.  This one will
+   for example receive signal on card change. */
+static ctrl_t primary_connection;
+
+
 #define set_error(e,t) assuan_set_error (ctx, ASSUAN_ ## e, (t))
 
 /* Data used to associate an Assuan context with local server data */
 struct server_local_s {
   ASSUAN_CONTEXT assuan_ctx;
+  int event_signal;        /* Or 0 if not used. */
 };
 
 
@@ -56,14 +64,11 @@ has_option (const char *line, const char *name)
 }
 
 
-
-
-/* Note, that this reset_notify is also used for cleanup purposes. */
+/* Reset the card and free the application context.  With DO_CLOSE set
+   to true, close the reader and don't do just a reset. */
 static void
-reset_notify (ASSUAN_CONTEXT ctx)
+do_reset (ctrl_t ctrl, int do_close)
 {
-  CTRL ctrl = assuan_get_pointer (ctx); 
-
   if (ctrl->card_ctx)
     {
       card_close (ctrl->card_ctx);
@@ -73,18 +78,44 @@ reset_notify (ASSUAN_CONTEXT ctx)
     }
   if (ctrl->app_ctx)
     {
-      int slot = ctrl->app_ctx->slot;
       release_application (ctrl->app_ctx);
       ctrl->app_ctx = NULL;
-      apdu_close_reader (slot);
     }
+  if (ctrl->reader_slot != -1)
+    {
+      if (do_close || apdu_reset (ctrl->reader_slot))
+        {
+          apdu_close_reader (ctrl->reader_slot);
+          ctrl->reader_slot = -1;
+        }
+    }
+}
+
+
+static void
+reset_notify (ASSUAN_CONTEXT ctx)
+{
+  CTRL ctrl = assuan_get_pointer (ctx); 
+
+  do_reset (ctrl, 0);
 }
 
 
 static int
 option_handler (ASSUAN_CONTEXT ctx, const char *key, const char *value)
 {
-  return 0;
+  ctrl_t ctrl = assuan_get_pointer (ctx);
+
+  if (!strcmp (key, "event-signal"))
+    {
+      /* A value of 0 is allowed to reset the event signal. */
+      int i = *value? atoi (value) : -1;
+      if (i < 0)
+        return ASSUAN_Parameter_Error;
+      ctrl->server_local->event_signal = i;
+    }
+
+ return 0;
 }
 
 
@@ -92,7 +123,7 @@ option_handler (ASSUAN_CONTEXT ctx, const char *key, const char *value)
    function returns an Assuan error, so don't map the error a second
    time */
 static AssuanError
-open_card (CTRL ctrl, const char *apptype)
+open_card (ctrl_t ctrl, const char *apptype)
 {
   int slot;
 
@@ -101,13 +132,13 @@ open_card (CTRL ctrl, const char *apptype)
   if (ctrl->card_ctx)
     return 0; /* Already initialized using a card context. */
 
-  slot = apdu_open_reader (opt.reader_port);
+  if (ctrl->reader_slot != -1)
+    slot = ctrl->reader_slot;
+  else
+    slot = apdu_open_reader (opt.reader_port);
+  ctrl->reader_slot = slot;
   if (slot != -1)
-    {
-      ctrl->app_ctx = select_application (ctrl, slot, apptype);
-      if (!ctrl->app_ctx)
-        apdu_close_reader (slot);
-    }
+    ctrl->app_ctx = select_application (ctrl, slot, apptype);
   if (!ctrl->app_ctx)
     { /* No application found - fall back to old mode. */
       /* Note that we should rework the old code to use the
@@ -351,7 +382,7 @@ cmd_learn (ASSUAN_CONTEXT ctx, char *line)
 
           buf = xtrymalloc (40 + 1 + strlen (certid) + 1);
           if (!buf)
-            rc = out_of_core ();
+            rc = gpg_error (gpg_err_code_from_errno (errno));
           else
             {
               sprintf (buf, "%d %s", certtype, certid);
@@ -389,7 +420,7 @@ cmd_learn (ASSUAN_CONTEXT ctx, char *line)
 
           buf = p = xtrymalloc (40 + 1 + strlen (keyid) + 1);
           if (!buf)
-            rc = out_of_core ();
+            rc = gpg_error (gpg_err_code_from_errno (errno));
           else
             {
               int i;
@@ -577,7 +608,7 @@ pin_cb (void *opaque, const char *info, char **retstr)
 
   rc = asprintf (&command, "NEEDPIN %s", info);
   if (rc < 0)
-    return out_of_core ();
+    return gpg_error (gpg_err_code_from_errno (errno));
 
   /* FIXME: Write an inquire function which returns the result in
      secure memory */
@@ -1084,6 +1115,16 @@ scd_command_handler (int listen_fd)
   if (DBG_ASSUAN)
     assuan_set_log_stream (ctx, log_get_stream ());
 
+  /* Store the primary connection's assuan context. */
+  if (!primary_connection)
+    primary_connection = &ctrl;
+
+  /* We open the reader right at startup so that the ticker is able to
+     update the status file. */
+  if (ctrl.reader_slot == -1)
+    ctrl.reader_slot = apdu_open_reader (opt.reader_port);
+
+  /* Command processing loop. */
   for (;;)
     {
       rc = assuan_accept (ctx);
@@ -1104,7 +1145,13 @@ scd_command_handler (int listen_fd)
           continue;
         }
     }
-  reset_notify (ctx); /* used for cleanup */
+
+  /* The next client will be the primary conenction if this one
+     terminates. */
+  if (primary_connection == &ctrl)
+    primary_connection = NULL;
+
+  do_reset (&ctrl, 1); /* Cleanup. */
 
   assuan_deinit_server (ctx);
 }
@@ -1156,3 +1203,69 @@ send_status_info (CTRL ctrl, const char *keyword, ...)
   va_end (arg_ptr);
 }
 
+
+
+void
+scd_update_reader_status_file (void)
+{
+  static struct {
+    int any;
+    unsigned int status;
+    unsigned int changed;
+  } last[10];
+  int slot;
+  int used;
+  unsigned int status, changed;
+
+  /* Note, that we only try to get the status, becuase it does not
+     make sense to wait here for a operation to complete.  If we are
+     so busy working with the card, delays in the status file updated
+     are should be acceptable. */
+  for (slot=0; (slot < DIM(last)
+                &&!apdu_enum_reader (slot, &used)); slot++)
+    if (used && !apdu_get_status (slot, 0, &status, &changed))
+      {
+        if (!last[slot].any || last[slot].status != status
+            || last[slot].changed != changed )
+          {
+            char *fname;
+            char templ[50];
+            FILE *fp;
+
+            last[slot].any = 1;
+            last[slot].status = status;
+            last[slot].changed = changed;
+
+            log_info ("updating status of slot %d to 0x%04X\n", slot, status);
+            
+            sprintf (templ, "reader_%d.status", slot);
+            fname = make_filename (opt.homedir, templ, NULL );
+            fp = fopen (fname, "w");
+            if (fp)
+              {
+                fprintf (fp, "%s\n",
+                         (status & 1)? "USABLE":
+                         (status & 4)? "ACTIVE":
+                         (status & 2)? "PRESENT": "NOCARD");
+                fclose (fp);
+              }
+            xfree (fname);
+
+            /* Send a signal to the primary client, if any. */
+            if (primary_connection && primary_connection->server_local
+                && primary_connection->server_local->assuan_ctx)
+              {
+                pid_t pid = assuan_get_pid (primary_connection
+                                            ->server_local->assuan_ctx);
+                int signo = primary_connection->server_local->event_signal;
+
+                log_info ("client pid is %d, sending signal %d\n", pid, signo);
+
+#ifndef HAVE_W32_SYSTEM
+                if (pid != (pid_t)(-1) && pid && signo > 0)
+                  kill (pid, signo);
+#endif
+              }
+          }
+      }
+}
