@@ -16,6 +16,8 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ *
+ * $Id: iso7816.c,v 1.3.2.10 2004/10/14 09:12:36 werner Exp $
  */
 
 #include <config.h>
@@ -24,7 +26,9 @@
 #include <stdlib.h>
 #include <string.h>
 
-#if GNUPG_MAJOR_VERSION == 1
+#if defined(GNUPG_SCD_MAIN_HEADER)
+#include GNUPG_SCD_MAIN_HEADER
+#elif GNUPG_MAJOR_VERSION == 1
 /* This is used with GnuPG version < 1.9.  The code has been source
    copied from the current GnuPG >= 1.9  and is maintained over
    there. */
@@ -80,6 +84,16 @@ map_sw (int sw)
     case SW_HOST_OUT_OF_CORE: ec = GPG_ERR_ENOMEM; break;
     case SW_HOST_INV_VALUE:   ec = GPG_ERR_INV_VALUE; break;
     case SW_HOST_INCOMPLETE_CARD_RESPONSE: ec = GPG_ERR_CARD; break;
+    case SW_HOST_NOT_SUPPORTED: ec = GPG_ERR_NOT_SUPPORTED; break;
+    case SW_HOST_LOCKING_FAILED: ec = GPG_ERR_BUG; break;
+    case SW_HOST_BUSY:           ec = GPG_ERR_EBUSY; break;
+    case SW_HOST_NO_CARD:        ec = GPG_ERR_CARD_NOT_PRESENT; break;
+    case SW_HOST_CARD_INACTIVE:  ec = GPG_ERR_CARD_RESET; break;
+    case SW_HOST_CARD_IO_ERROR:  ec = GPG_ERR_EIO; break;
+    case SW_HOST_GENERAL_ERROR:  ec = GPG_ERR_GENERAL; break;
+    case SW_HOST_NO_READER:      ec = GPG_ERR_ENODEV; break;
+    case SW_HOST_ABORTED:        ec = GPG_ERR_CANCELED; break;
+
     default:
       if ((sw & 0x010000))
         ec = GPG_ERR_GENERAL; /* Should not happen. */
@@ -91,6 +105,17 @@ map_sw (int sw)
   return gpg_error (ec);
 }
 
+/* Map a status word from the APDU layer to a gpg-error code.  */
+gpg_error_t
+iso7816_map_sw (int sw)
+{
+  /* All APDU functions should return 0x9000 on success but for
+     historical reasons of the implementation some return 0 to
+     indicate success.  We allow for that here. */
+  return sw? map_sw (sw) : 0;
+}
+
+
 /* This function is specialized version of the SELECT FILE command.
    SLOT is the card and reader as created for example by
    apdu_open_reader (), AID is a buffer of size AIDLEN holding the
@@ -101,15 +126,8 @@ map_sw (int sw)
 gpg_error_t
 iso7816_select_application (int slot, const char *aid, size_t aidlen)
 {
-  static char const openpgp_aid[] = { 0xD2, 0x76, 0x00, 0x01, 0x24, 0x01 };
   int sw;
-  int p1 = 0x0C; /* No FCI to be returned. */
-  
-  if (aidlen == sizeof openpgp_aid
-      && !memcmp (aid, openpgp_aid, sizeof openpgp_aid))
-    p1 = 0; /* The current openpgp cards don't allow 0x0c. */
-
-  sw = apdu_send_simple (slot, 0x00, CMD_SELECT_FILE, 4, p1, aidlen, aid);
+  sw = apdu_send_simple (slot, 0x00, CMD_SELECT_FILE, 4, 0, aidlen, aid);
   return map_sw (sw);
 }
 
@@ -200,7 +218,7 @@ iso7816_change_reference_data (int slot, int chvno,
 
   buf = xtrymalloc (oldchvlen + newchvlen);
   if (!buf)
-    return out_of_core ();
+    return gpg_error (gpg_err_code_from_errno (errno));
   if (oldchvlen)
     memcpy (buf, oldchv, oldchvlen);
   memcpy (buf+oldchvlen, newchv, newchvlen);
@@ -341,7 +359,8 @@ iso7816_decipher (int slot, const unsigned char *data, size_t datalen,
       /* We need to prepend the padding indicator. */
       buf = xtrymalloc (datalen + 1);
       if (!buf)
-        return out_of_core ();
+        return gpg_error (gpg_err_code_from_errno (errno));
+
       *buf = padind; /* Padding indicator. */
       memcpy (buf+1, data, datalen);
       sw = apdu_send (slot, 0x00, CMD_PSO, 0x80, 0x86, datalen+1, buf,
@@ -485,6 +504,8 @@ iso7816_read_binary (int slot, size_t offset, size_t nmax,
   int sw;
   unsigned char *buffer;
   size_t bufferlen;
+  int read_all = !nmax;
+  size_t n;
 
   if (!result || !resultlen)
     return gpg_error (GPG_ERR_INV_VALUE);
@@ -493,18 +514,22 @@ iso7816_read_binary (int slot, size_t offset, size_t nmax,
 
   /* We can only encode 15 bits in p0,p1 to indicate an offset. Thus
      we check for this limit. */
-  if (offset > 32767 || nmax > 254)
+  if (offset > 32767)
     return gpg_error (GPG_ERR_INV_VALUE);
 
   do
     {
       buffer = NULL;
       bufferlen = 0;
-      /* Fixme: Either the ccid driver of the TCOS cards have problems
+      /* Fixme: Either the ccid driver or the TCOS cards have problems
          with an Le of 0. */
+      if (read_all || nmax > 254)
+        n = 254;
+      else
+        n = nmax;
       sw = apdu_send_le (slot, 0x00, CMD_READ_BINARY,
-                      ((offset>>8) & 0xff), (offset & 0xff) , -1, NULL,
-                      nmax? nmax : 254, &buffer, &bufferlen);
+                         ((offset>>8) & 0xff), (offset & 0xff) , -1, NULL,
+                         n, &buffer, &bufferlen);
 
       if (sw != SW_SUCCESS && sw != SW_EOF_REACHED)
         {
@@ -542,19 +567,25 @@ iso7816_read_binary (int slot, size_t offset, size_t nmax,
       if (offset > 32767)
         break; /* We simply truncate the result for too large
                   files. */
+      if (nmax > bufferlen)
+        nmax -= bufferlen;
+      else
+        nmax = 0;
     }
-  while (!nmax && sw != SW_EOF_REACHED);
+  while ((read_all && sw != SW_EOF_REACHED) || (!read_all && nmax));
   
   return 0;
 }
 
 /* Perform a READ RECORD command. RECNO gives the record number to
    read with 0 indicating the current record.  RECCOUNT must be 1 (not
-   all cards support reading of more than one record).  The result is
-   stored in a newly allocated buffer at the address passed by RESULT.
-   Returns the length of this data at the address of RESULTLEN. */
+   all cards support reading of more than one record).  SHORT_EF
+   should be 0 to read the current EF or contain a short EF. The
+   result is stored in a newly allocated buffer at the address passed
+   by RESULT.  Returns the length of this data at the address of
+   RESULTLEN. */
 gpg_error_t
-iso7816_read_record (int slot, int recno, int reccount,
+iso7816_read_record (int slot, int recno, int reccount, int short_ef,
                      unsigned char **result, size_t *resultlen)
 {
   int sw;
@@ -568,7 +599,8 @@ iso7816_read_record (int slot, int recno, int reccount,
 
   /* We can only encode 15 bits in p0,p1 to indicate an offset. Thus
      we check for this limit. */
-  if (recno < 0 || recno > 255 || reccount != 1)
+  if (recno < 0 || recno > 255 || reccount != 1
+      || short_ef < 0 || short_ef > 254 )
     return gpg_error (GPG_ERR_INV_VALUE);
 
   buffer = NULL;
@@ -577,7 +609,7 @@ iso7816_read_record (int slot, int recno, int reccount,
      with an Le of 0. */
   sw = apdu_send_le (slot, 0x00, CMD_READ_RECORD,
                      recno, 
-                     0x04,
+                     short_ef? short_ef : 0x04,
                      -1, NULL,
                      254, &buffer, &bufferlen);
 
