@@ -29,10 +29,15 @@
 #include <assert.h>
 #include <time.h>
 #include <fcntl.h>
+#ifndef HAVE_W32_SYSTEM
 #include <sys/socket.h>
 #include <sys/un.h>
+#endif /*HAVE_W32_SYSTEM*/
 #include <unistd.h>
 #include <signal.h>
+#ifdef USE_GNU_PTH
+# include <pth.h>
+#endif
 
 #define JNLIB_NEED_LOG_LOGV
 #include "scdaemon.h"
@@ -44,6 +49,9 @@
 #include "i18n.h"
 #include "sysutils.h"
 #include "app-common.h"
+#ifdef HAVE_W32_SYSTEM
+#include "../jnlib/w32-afunix.h"
+#endif
 
 
 enum cmd_and_opt_values 
@@ -77,6 +85,7 @@ enum cmd_and_opt_values
   oDisableOpenSC,
   oAllowAdmin,
   oDenyAdmin,
+  oDisableApplication,
 
 aTest };
 
@@ -103,8 +112,8 @@ static ARGPARSE_OPTS opts[] = {
   { oNoDetach, "no-detach" ,0, N_("do not detach from the console")},
   { oLogFile,  "log-file"   ,2, N_("use a log file for the server")},
   { oReaderPort, "reader-port", 2, N_("|N|connect to reader at port N")},
-  { octapiDriver, "ctapi-driver", 2, N_("NAME|use NAME as ct-API driver")},
-  { opcscDriver, "pcsc-driver", 2, N_("NAME|use NAME as PC/SC driver")},
+  { octapiDriver, "ctapi-driver", 2, N_("|NAME|use NAME as ct-API driver")},
+  { opcscDriver, "pcsc-driver", 2, N_("|NAME|use NAME as PC/SC driver")},
   { oDisableCCID, "disable-ccid", 0,
 #ifdef HAVE_LIBUSB
                                     N_("do not use the internal CCID driver")
@@ -121,19 +130,41 @@ static ARGPARSE_OPTS opts[] = {
                                          /* end --disable-opensc */},
   { oAllowAdmin, "allow-admin", 0, N_("allow the use of admin card commands")},
   { oDenyAdmin,  "deny-admin",  0, "@" },  
+  { oDisableApplication, "disable-application", 2, "@"},
 
   {0}
 };
 
 
+/* The card dirver we use by default for PC/SC.  */
+#ifdef HAVE_W32_SYSTEM
+#define DEFAULT_PCSC_DRIVER "winscard.dll"
+#else
+#define DEFAULT_PCSC_DRIVER "libpcsclite.so"
+#endif
+
+
 static volatile int caught_fatal_sig = 0;
 
+/* Flag to indicate that a shutdown was requested. */
+static int shutdown_pending;
+
 /* It is possible that we are currently running under setuid permissions */
 static int maybe_setuid = 1;
 
 /* Name of the communication socket */
 static char socket_name[128];
 
+
+#ifndef HAVE_OPENSC
+#ifdef USE_GNU_PTH
+/* Pth wrapper function definitions. */
+GCRY_THREAD_OPTION_PTH_IMPL;
+
+static void *ticker_thread (void *arg);
+#endif /*USE_GNU_PTH*/
+#endif /*!HAVE_OPENSC*/
+
 static const char *
 my_strusage (int level)
 {
@@ -164,12 +195,12 @@ static void
 i18n_init (void)
 {
 #ifdef USE_SIMPLE_GETTEXT
-    set_gettext_file( PACKAGE );
+    set_gettext_file( PACKAGE_GT );
 #else
 #ifdef ENABLE_NLS
     setlocale (LC_ALL, "");
-    bindtextdomain (PACKAGE, LOCALEDIR);
-    textdomain (PACKAGE);
+    bindtextdomain (PACKAGE_GT, LOCALEDIR);
+    textdomain (PACKAGE_GT);
 #endif
 #endif
 }
@@ -284,6 +315,7 @@ main (int argc, char **argv )
 {
   ARGPARSE_ARGS pargs;
   int orig_argc;
+  gpg_error_t err;
   int may_coredump;
   char **orig_argv;
   FILE *configfp = NULL;
@@ -302,6 +334,7 @@ main (int argc, char **argv )
   char *logfile = NULL;
   int debug_wait = 0;
   int gpgconf_list = 0;
+  const char *config_filename = NULL;
 
   set_strusage (my_strusage);
   gcry_control (GCRYCTL_SUSPEND_SECMEM_WARN);
@@ -314,7 +347,20 @@ main (int argc, char **argv )
 
   i18n_init ();
 
-  /* check that the libraries are suitable.  Do it here because
+  /* Libgcrypt requires us to register the threading model first.
+     Note that this will also do the pth_init. */
+#ifndef HAVE_OPENSC
+#ifdef USE_GNU_PTH
+  err = gcry_control (GCRYCTL_SET_THREAD_CBS, &gcry_threads_pth);
+  if (err)
+    {
+      log_fatal ("can't register GNU Pth with Libgcrypt: %s\n",
+                 gpg_strerror (err));
+    }
+#endif /*USE_GNU_PTH*/
+#endif /*!HAVE_OPENSC*/
+
+  /* Check that the libraries are suitable.  Do it here because
      the option parsing may need services of the library */
   if (!gcry_check_version (NEED_LIBGCRYPT_VERSION) )
     {
@@ -334,21 +380,16 @@ main (int argc, char **argv )
   may_coredump = disable_core_dumps ();
 
   /* Set default options. */
-  opt.pcsc_driver = "libpcsclite.so"; 
+  opt.pcsc_driver = DEFAULT_PCSC_DRIVER; 
 
 
   shell = getenv ("SHELL");
   if (shell && strlen (shell) >= 3 && !strcmp (shell+strlen (shell)-3, "csh") )
     csh_style = 1;
   
-  /* FIXME: Using this homedir option does only make sense when not
-     running as a system service.  We might want to check for this by
-     looking at the uid or ebtter use an explict option for this */
-  opt.homedir = getenv("GNUPGHOME");
-  if (!opt.homedir || !*opt.homedir)
-    opt.homedir = GNUPG_DEFAULT_HOMEDIR;
+  opt.homedir = default_homedir ();
 
-  /* check whether we have a config file on the commandline */
+  /* Check whether we have a config file on the commandline */
   orig_argc = argc;
   orig_argv = argv;
   pargs.argc = &argc;
@@ -459,6 +500,10 @@ main (int argc, char **argv )
         case oAllowAdmin: opt.allow_admin = 1; break;
         case oDenyAdmin: opt.allow_admin = 0; break;
 
+        case oDisableApplication:
+          add_to_strlist (&opt.disabled_applications, pargs.r.ret_str); 
+          break;
+
         default : pargs.err = configfp? 1:2; break;
 	}
     }
@@ -466,7 +511,8 @@ main (int argc, char **argv )
     {
       fclose( configfp );
       configfp = NULL;
-      xfree(configname);
+      /* Keep a copy of the config name for use by --gpgconf-list. */
+      config_filename = configname;
       configname = NULL;
       goto next_pass;
     }
@@ -506,20 +552,52 @@ main (int argc, char **argv )
     }
   
   if (gpgconf_list)
-    { /* List options and default values in the GPG Conf format.  */
-      char *filename;
+    {
+      /* List options and default values in the GPG Conf format.  */
+
+      /* The following list is taken from gnupg/tools/gpgconf-comp.c.  */
+      /* Option flags.  YOU MUST NOT CHANGE THE NUMBERS OF THE EXISTING
+         FLAGS, AS THEY ARE PART OF THE EXTERNAL INTERFACE.  */
+#define GC_OPT_FLAG_NONE	0UL
+      /* The RUNTIME flag for an option indicates that the option can be
+         changed at runtime.  */
+#define GC_OPT_FLAG_RUNTIME	(1UL << 3)
+      /* The DEFAULT flag for an option indicates that the option has a
+         default value.  */
+#define GC_OPT_FLAG_DEFAULT	(1UL << 4)
+      /* The DEF_DESC flag for an option indicates that the option has a
+         default, which is described by the value of the default field.  */
+#define GC_OPT_FLAG_DEF_DESC	(1UL << 5)
+      /* The NO_ARG_DESC flag for an option indicates that the argument has
+         a default, which is described by the value of the ARGDEF field.  */
+#define GC_OPT_FLAG_NO_ARG_DESC	(1UL << 6)
+      if (!config_filename)
+        config_filename = make_filename (opt.homedir, "scdaemon.conf", NULL );
+
+      printf ("gpgconf-scdaemon.conf:%lu:\"%s\n",
+              GC_OPT_FLAG_DEFAULT, config_filename);
+        
+      printf ("verbose:%lu:\n"
+              "quiet:%lu:\n"
+              "debug-level:%lu:\"none:\n"
+              "log-file:%lu:\n",
+              GC_OPT_FLAG_NONE,
+              GC_OPT_FLAG_NONE,
+              GC_OPT_FLAG_DEFAULT,
+              GC_OPT_FLAG_NONE );
+
+      printf ("reader-port:%lu:\n", GC_OPT_FLAG_NONE );
+      printf ("ctapi-driver:%lu:\n", GC_OPT_FLAG_NONE );
+      printf ("pcsc-driver:%lu:\"%s:\n",
+              GC_OPT_FLAG_DEFAULT, DEFAULT_PCSC_DRIVER );
+#ifdef HAVE_LIBUSB
+      printf ("disable-ccid:%lu:\n", GC_OPT_FLAG_NONE );
+#endif
+#ifdef HAVE_LIBUSB
+      printf ("disable-opensc:%lu:\n", GC_OPT_FLAG_NONE );
+#endif
+      printf ("allow-admin:%lu:\n", GC_OPT_FLAG_NONE );
 
-      filename = make_filename (opt.homedir, "scdaemon.conf", NULL);
-      printf ("gpgconf-scdaemon.conf:\"%s\n", filename);
-      xfree (filename);
-      
-      printf ("verbose:\n"
-              "quiet:\n"
-              "debug-level:none\n"
-              "log-file:\n"
-              "force:\n"
-              "faked-system-time:\n"
-              "no-greeting:\n");
 
       scd_exit (0);
     }
@@ -533,7 +611,23 @@ main (int argc, char **argv )
 
 
   if (pipe_server)
-    { /* this is the simple pipe based server */
+    { /* This is the simple pipe based server */
+#ifndef HAVE_OPENSC
+#ifdef USE_GNU_PTH
+      pth_attr_t tattr;
+ 
+      tattr = pth_attr_new();
+      pth_attr_set (tattr, PTH_ATTR_JOINABLE, 0);
+      pth_attr_set (tattr, PTH_ATTR_STACK_SIZE, 512*1024);
+      pth_attr_set (tattr, PTH_ATTR_NAME, "ticker");
+
+      if (!pth_spawn (tattr, ticker_thread, NULL))
+        {
+          log_error ("error spawning ticker thread: %s\n", strerror (errno));
+          scd_exit (2);
+        }
+#endif /*USE_GNU_PTH*/
+#endif /*!HAVE_OPENSC*/
       scd_command_handler (-1);
     }
   else if (!is_daemon)
@@ -560,12 +654,15 @@ main (int argc, char **argv )
       if (!p)
         BUG ();
       *p = 0;;
+
+#ifndef HAVE_W32_SYSTEM
       if (!mkdtemp(socket_name))
         {
           log_error ("can't create directory `%s': %s\n",
 	             socket_name, strerror(errno) );
           exit (1);
         }
+#endif
       *p = '/';
 
       if (strchr (socket_name, ':') )
@@ -580,7 +677,11 @@ main (int argc, char **argv )
         }
    
 
+#ifdef HAVE_W32_SYSTEM
+      fd = _w32_sock_new (AF_UNIX, SOCK_STREAM, 0);
+#else
       fd = socket (AF_UNIX, SOCK_STREAM, 0);
+#endif
       if (fd == -1)
         {
           log_error ("can't create socket: %s\n", strerror(errno) );
@@ -593,7 +694,13 @@ main (int argc, char **argv )
       len = (offsetof (struct sockaddr_un, sun_path)
              + strlen(serv_addr.sun_path) + 1);
 
-      if (bind (fd, (struct sockaddr*)&serv_addr, len) == -1)
+      if (
+#ifdef HAVE_W32_SYSTEM
+          _w32_sock_bind
+#else
+          bind 
+#endif
+          (fd, (struct sockaddr*)&serv_addr, len) == -1)
         {
           log_error ("error binding socket to `%s': %s\n",
                      serv_addr.sun_path, strerror (errno) );
@@ -613,6 +720,7 @@ main (int argc, char **argv )
 
 
       fflush (NULL);
+#ifndef HAVE_W32_SYSTEM
       pid = fork ();
       if (pid == (pid_t)-1) 
         {
@@ -711,6 +819,8 @@ main (int argc, char **argv )
           exit (1);
         }
 
+#endif /*!HAVE_W32_SYSTEM*/
+
       scd_command_handler (fd);
 
       close (fd);
@@ -745,6 +855,119 @@ scd_exit (int rc)
 void
 scd_init_default_ctrl (CTRL ctrl)
 {
-
+  ctrl->reader_slot = -1;
 }
 
+
+#ifndef HAVE_OPENSC
+#ifdef USE_GNU_PTH
+
+static void
+handle_signal (int signo)
+{
+  switch (signo)
+    {
+#ifndef HAVE_W32_SYSTEM
+    case SIGHUP:
+      log_info ("SIGHUP received - "
+                "re-reading configuration and resetting cards\n");
+/*       reread_configuration (); */
+      break;
+      
+    case SIGUSR1:
+      log_info ("SIGUSR1 received - no action defined\n");
+      break;
+
+    case SIGUSR2:
+      log_info ("SIGUSR2 received - no action defined\n");
+      break;
+
+    case SIGTERM:
+      if (!shutdown_pending)
+        log_info ("SIGTERM received - shutting down ...\n");
+      else
+        log_info ("SIGTERM received - still %ld running threads\n",
+                  pth_ctrl( PTH_CTRL_GETTHREADS ));
+      shutdown_pending++;
+      if (shutdown_pending > 2)
+        {
+          log_info ("shutdown forced\n");
+          log_info ("%s %s stopped\n", strusage(11), strusage(13) );
+          cleanup ();
+          scd_exit (0);
+	}
+      break;
+        
+    case SIGINT:
+      log_info ("SIGINT received - immediate shutdown\n");
+      log_info( "%s %s stopped\n", strusage(11), strusage(13));
+      cleanup ();
+      scd_exit (0);
+      break;
+#endif /*!HAVE_W32_SYSTEM*/
+
+    default:
+      log_info ("signal %d received - no action defined\n", signo);
+    }
+}
+
+static void
+handle_tick (void)
+{
+  scd_update_reader_status_file ();
+}
+
+static void *
+ticker_thread (void *dummy_arg)
+{
+  pth_event_t sigs_ev, time_ev = NULL;
+  sigset_t sigs;
+  int signo;
+
+#ifndef HAVE_W32_SYSTEM /* fixme */
+  sigemptyset (&sigs );
+  sigaddset (&sigs, SIGHUP);
+  sigaddset (&sigs, SIGUSR1);
+  sigaddset (&sigs, SIGUSR2);
+  sigaddset (&sigs, SIGINT);
+  sigaddset (&sigs, SIGTERM);
+  sigs_ev = pth_event (PTH_EVENT_SIGS, &sigs, &signo);
+#else
+  sigs_ev = NULL;
+#endif
+  
+  for (;;)
+    {
+      if (!time_ev)
+        {
+          time_ev = pth_event (PTH_EVENT_TIME, pth_timeout (2, 0));
+          if (time_ev)
+            pth_event_concat (sigs_ev, time_ev, NULL);
+        }
+
+      if (pth_wait (sigs_ev) < 1)
+        continue;
+
+      if (
+#ifdef PTH_STATUS_OCCURRED     /* This is Pth 2 */
+          pth_event_status (sigs_ev) == PTH_STATUS_OCCURRED
+#else
+          pth_event_occurred (sigs_ev)
+#endif
+          )
+        handle_signal (signo);
+
+      /* Always run the ticker. */
+      if (!shutdown_pending)
+        {
+          pth_event_isolate (sigs_ev);
+          pth_event_free (time_ev, PTH_FREE_ALL);
+          time_ev = NULL;
+          handle_tick ();
+        }
+    }
+
+  pth_event_free (sigs_ev, PTH_FREE_ALL);
+}
+#endif /*USE_GNU_PTH*/
+#endif /*!HAVE_OPENSC*/
