@@ -129,9 +129,14 @@ struct reqgen_ctrl_s {
 };
 
 
-static int proc_parameters (struct para_data_s *para,
+static const char oidstr_keyUsage[] = "2.5.29.15";
+
+
+static int proc_parameters (ctrl_t ctrl,
+                            struct para_data_s *para,
                             struct reqgen_ctrl_s *outctrl);
-static int create_request (struct para_data_s *para,
+static int create_request (ctrl_t ctrl,
+                           struct para_data_s *para,
                            ksba_const_sexp_t public,
                            struct reqgen_ctrl_s *outctrl);
 
@@ -177,10 +182,10 @@ get_parameter_algo (struct para_data_s *para, enum para_name key)
   return gcry_pk_map_name (r->u.value); 
 }
 
-/* parse the usage parameter.  Returns 0 on success.  Note that we
+/* Parse the usage parameter.  Returns 0 on success.  Note that we
    only care about sign and encrypt and don't (yet) allow all the
    other X.509 usage to be specified; instead we will use a fixed
-   mapping to the X.509 usage flags */
+   mapping to the X.509 usage flags. */
 static int
 parse_parameter_usage (struct para_data_s *para, enum para_name key)
 {
@@ -220,6 +225,9 @@ get_parameter_uint (struct para_data_s *para, enum para_name key)
   if (!r)
     return 0;
 
+  if (r->key == pKEYUSAGE)
+    return r->u.usage;
+
   return (unsigned int)strtoul (r->u.value, NULL, 10);
 }
 
@@ -228,7 +236,7 @@ get_parameter_uint (struct para_data_s *para, enum para_name key)
 /* Read the certificate generation parameters from FP and generate
    (all) certificate requests.  */
 static int
-read_parameters (FILE *fp, ksba_writer_t writer)
+read_parameters (ctrl_t ctrl, FILE *fp, ksba_writer_t writer)
 {
   static struct {
     const char *name;
@@ -285,7 +293,7 @@ read_parameters (FILE *fp, ksba_writer_t writer)
             outctrl.dryrun = 1;
           else if (!ascii_strcasecmp( keyword, "%commit"))
             {
-              rc = proc_parameters (para, &outctrl);
+              rc = proc_parameters (ctrl, para, &outctrl);
               if (rc)
                 goto leave;
               any = 1;
@@ -332,7 +340,7 @@ read_parameters (FILE *fp, ksba_writer_t writer)
 
       if (keywords[i].key == pKEYTYPE && para)
         {
-          rc = proc_parameters (para, &outctrl);
+          rc = proc_parameters (ctrl, para, &outctrl);
           if (rc)
             goto leave;
           any = 1;
@@ -375,7 +383,7 @@ read_parameters (FILE *fp, ksba_writer_t writer)
     }
   else if (para)
     {
-      rc = proc_parameters (para, &outctrl);
+      rc = proc_parameters (ctrl, para, &outctrl);
       if (rc)
         goto leave;
       any = 1;
@@ -414,7 +422,8 @@ has_invalid_email_chars (const char *s)
 
 /* Check that all required parameters are given and perform the action */
 static int
-proc_parameters (struct para_data_s *para, struct reqgen_ctrl_s *outctrl)
+proc_parameters (ctrl_t ctrl,
+                 struct para_data_s *para, struct reqgen_ctrl_s *outctrl)
 {
   struct para_data_s *r;
   const char *s;
@@ -429,13 +438,13 @@ proc_parameters (struct para_data_s *para, struct reqgen_ctrl_s *outctrl)
   assert (get_parameter (para, pKEYTYPE));
 
   /* We can only use RSA for now.  There is a with pkcs-10 on how to
-     use ElGamal becuase it is expected that a PK algorithm can always
+     use ElGamal because it is expected that a PK algorithm can always
      be used for signing. */
   i = get_parameter_algo (para, pKEYTYPE);
   if (i < 1 || i != GCRY_PK_RSA )
     {
       r = get_parameter (para, pKEYTYPE);
-      log_error ("line %d: invalid algorithm\n", r->lnr);
+      log_error (_("line %d: invalid algorithm\n"), r->lnr);
       return gpg_error (GPG_ERR_INV_PARAMETER);
     }
   
@@ -444,11 +453,12 @@ proc_parameters (struct para_data_s *para, struct reqgen_ctrl_s *outctrl)
     nbits = 1024;
   else
     nbits = get_parameter_uint (para, pKEYLENGTH);
-  if (nbits < 512 || nbits > 4096)
+  if (nbits < 1024 || nbits > 4096)
     {
+      /* The BSI specs dated 2002-11-25 don't allow lengths below 1024. */
       r = get_parameter (para, pKEYTYPE);
-      log_error ("line %d: invalid key length %u (valid are 512 to 4096)\n",
-                 r->lnr, nbits);
+      log_error (_("line %d: invalid key length %u (valid are %d to %d)\n"),
+                 r->lnr, nbits, 1024, 4096);
       return gpg_error (GPG_ERR_INV_PARAMETER);
     }
     
@@ -461,7 +471,7 @@ proc_parameters (struct para_data_s *para, struct reqgen_ctrl_s *outctrl)
   if (!(s=get_parameter_value (para, pNAMEDN)))
     {
       r = get_parameter (para, pKEYTYPE);
-      log_error ("line %d: no subject name given\n", r->lnr);
+      log_error (_("line %d: no subject name given\n"), r->lnr);
       return gpg_error (GPG_ERR_INV_PARAMETER);
     }
   /* fixme check s */
@@ -476,24 +486,24 @@ proc_parameters (struct para_data_s *para, struct reqgen_ctrl_s *outctrl)
           || strstr(s, ".."))
         {
           r = get_parameter (para, pKEYTYPE);
-          log_error ("line %d: not a valid email address\n", r->lnr);
+          log_error (_("line %d: not a valid email address\n"), r->lnr);
           return gpg_error (GPG_ERR_INV_PARAMETER);
         }
     }
 
   sprintf (numbuf, "%u", nbits);
   snprintf (keyparms, DIM (keyparms)-1, 
-            "(6:genkey(3:rsa(5:nbits%d:%s)))", strlen (numbuf), numbuf);
-  rc = gpgsm_agent_genkey (keyparms, &public);
+            "(6:genkey(3:rsa(5:nbits%d:%s)))", (int)strlen (numbuf), numbuf);
+  rc = gpgsm_agent_genkey (ctrl, keyparms, &public);
   if (rc)
     {
       r = get_parameter (para, pKEYTYPE);
-      log_error ("line %d: key generation failed: %s\n",
+      log_error (_("line %d: key generation failed: %s\n"),
                  r->lnr, gpg_strerror (rc));
       return rc;
     }
 
-  rc = create_request (para, public, outctrl);
+  rc = create_request (ctrl, para, public, outctrl);
   xfree (public);
 
   return rc;
@@ -503,7 +513,8 @@ proc_parameters (struct para_data_s *para, struct reqgen_ctrl_s *outctrl)
 /* Parameters are checked, the key pair has been created.  Now
    generate the request and write it out */
 static int
-create_request (struct para_data_s *para, ksba_const_sexp_t public,
+create_request (ctrl_t ctrl,
+                struct para_data_s *para, ksba_const_sexp_t public,
                 struct reqgen_ctrl_s *outctrl)
 {
   ksba_certreq_t cr;
@@ -512,6 +523,7 @@ create_request (struct para_data_s *para, ksba_const_sexp_t public,
   ksba_stop_reason_t stopreason;
   int rc = 0;
   const char *s;
+  unsigned int use;
 
   err = ksba_certreq_new (&cr);
   if (err)
@@ -572,6 +584,35 @@ create_request (struct para_data_s *para, ksba_const_sexp_t public,
       rc = err;
       goto leave;
     }
+
+  
+  use = get_parameter_uint (para, pKEYUSAGE);
+  if (use == GCRY_PK_USAGE_SIGN)
+    {
+      /* For signing only we encode the bits:
+         KSBA_KEYUSAGE_DIGITAL_SIGNATURE
+         KSBA_KEYUSAGE_NON_REPUDIATION */
+      err = ksba_certreq_add_extension (cr, oidstr_keyUsage, 1, 
+                                        "\x03\x02\x06\xC0", 4);
+    }
+  else if (use == GCRY_PK_USAGE_ENCR)
+    {
+      /* For encrypt only we encode the bits:
+         KSBA_KEYUSAGE_KEY_ENCIPHERMENT
+         KSBA_KEYUSAGE_DATA_ENCIPHERMENT */
+      err = ksba_certreq_add_extension (cr, oidstr_keyUsage, 1, 
+                                        "\x03\x02\x04\x30", 4);
+    }
+  else
+    err = 0; /* Both or none given: don't request one. */
+  if (err)
+    {
+      log_error ("error setting the key usage: %s\n",
+                 gpg_strerror (err));
+      rc = err;
+      goto leave;
+    }
+
                
   do
     {
@@ -614,7 +655,7 @@ create_request (struct para_data_s *para, ksba_const_sexp_t public,
           for (n=0; n < 20; n++)
             sprintf (hexgrip+n*2, "%02X", grip[n]);
 
-          rc = gpgsm_agent_pksign (hexgrip, NULL,
+          rc = gpgsm_agent_pksign (ctrl, hexgrip, NULL,
                                    gcry_md_read(md, GCRY_MD_SHA1), 
                                    gcry_md_get_algo_dlen (GCRY_MD_SHA1),
                                    GCRY_MD_SHA1,
@@ -650,7 +691,7 @@ create_request (struct para_data_s *para, ksba_const_sexp_t public,
 /* Create a new key by reading the parameters from in_fd.  Multiple
    keys may be created */
 int
-gpgsm_genkey (CTRL ctrl, int in_fd, FILE *out_fp)
+gpgsm_genkey (ctrl_t ctrl, int in_fd, FILE *out_fp)
 {
   int rc;
   FILE *in_fp;
@@ -665,7 +706,7 @@ gpgsm_genkey (CTRL ctrl, int in_fd, FILE *out_fp)
       return tmperr;
     }
 
-  ctrl->pem_name = "NEW CERTIFICATE REQUEST";
+  ctrl->pem_name = "CERTIFICATE REQUEST";
   rc = gpgsm_create_writer (&b64writer, ctrl, out_fp, &writer);
   if (rc)
     {
@@ -673,7 +714,7 @@ gpgsm_genkey (CTRL ctrl, int in_fd, FILE *out_fp)
       goto leave;
     }
 
-  rc = read_parameters (in_fp, writer);
+  rc = read_parameters (ctrl, in_fp, writer);
   if (rc)
     {
       log_error ("error creating certificate request: %s\n",
