@@ -26,6 +26,9 @@
 #include <ctype.h>
 #include <unistd.h>
 #include <fcntl.h>
+#ifdef USE_GNU_PTH
+# include <pth.h>
+#endif
 
 #include "gpgsm.h"
 #include <gcrypt.h>
@@ -36,6 +39,11 @@
 #include "keydb.h"
 #include "sysutils.h"
 
+
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 enum cmd_and_opt_values {
   aNull = 0,
   oArmor        = 'a',
@@ -82,14 +90,20 @@ enum cmd_and_opt_values {
   aCallProtectTool,
   aPasswd,
   aGPGConfList,
+  aDumpKeys,
+  aDumpSecretKeys,
+  aDumpExternalKeys,
+  aKeydbClearSomeCertFlags,
 
   oOptions,
   oDebug,
   oDebugLevel,
   oDebugAll,
   oDebugWait,
+  oDebugAllowCoreDump,
   oDebugNoChainValidation,
   oDebugIgnoreExpiration,
+  oFixedPassphrase,
   oLogFile,
 
   oEnableSpecialFilenames,
@@ -101,6 +115,7 @@ enum cmd_and_opt_values {
   oLCctype,
   oLCmessages,
 
+  oPreferSystemDirmngr,
   oDirmngrProgram,
   oProtectToolProgram,
   oFakedSystemTime,
@@ -115,10 +130,11 @@ enum cmd_and_opt_values {
 
   oDisableCRLChecks,
   oEnableCRLChecks,
+  oForceCRLRefresh,
+
   oDisableOCSP,
   oEnableOCSP,
 
-
   oIncludeCerts,
   oPolicyFile,
   oDisablePolicyChecks,
@@ -164,6 +180,7 @@ enum cmd_and_opt_values {
   oWithColons,
   oWithKeyData,
   oWithValidation,
+  oWithEphemeralKeys,
   oSkipVerify,
   oCompressKeys,
   oCompressSigs,
@@ -245,6 +262,11 @@ static ARGPARSE_OPTS opts[] = {
     { aPasswd, "passwd",      256, N_("change a passphrase")},
     { aGPGConfList, "gpgconf-list", 256, "@" },
 
+    { aDumpKeys, "dump-keys", 256, "@"},
+    { aDumpExternalKeys, "dump-external-keys", 256, "@"},
+    { aDumpSecretKeys, "dump-secret-keys", 256, "@"},
+    { aKeydbClearSomeCertFlags, "keydb-clear-some-cert-flags", 256, "@"},
+
     { 301, NULL, 0, N_("@\nOptions:\n ") },
 
     { oArmor, "armor",     0, N_("create ascii armored output")},
@@ -259,9 +281,11 @@ static ARGPARSE_OPTS opts[] = {
 
     { oRecipient, "recipient", 2, N_("|NAME|encrypt for NAME")},
 
-
+    { oPreferSystemDirmngr,"prefer-system-dirmngr", 0,
+      N_("use system's dirmngr if available")},
     { oDisableCRLChecks, "disable-crl-checks", 0, N_("never consult a CRL")},
     { oEnableCRLChecks, "enable-crl-checks", 0, "@"},
+    { oForceCRLRefresh, "force-crl-refresh", 0, "@"},
 
     { oDisableOCSP, "disable-ocsp", 0, "@" },
     { oEnableOCSP,  "enable-ocsp", 0, N_("check validity using OCSP")},
@@ -321,11 +345,13 @@ static ARGPARSE_OPTS opts[] = {
     { oOptions, "options"   , 2, N_("read options from file")},
 
     { oDebug, "debug"     ,4|16, "@"},
-    { oDebugLevel, "debug-level" ,2, "@"},
+    { oDebugLevel, "debug-level" ,2, N_("|LEVEL|set the debugging level to LEVEL")},
     { oDebugAll, "debug-all" ,0, "@"},
     { oDebugWait, "debug-wait" ,1, "@"},
+    { oDebugAllowCoreDump, "debug-allow-core-dump", 0, "@" },
     { oDebugNoChainValidation, "debug-no-chain-validation", 0, "@"},
     { oDebugIgnoreExpiration,  "debug-ignore-expiration", 0, "@"},
+    { oFixedPassphrase,        "fixed-passphrase", 2, "@"},
     { oStatusFD, "status-fd" ,1, N_("|FD|write status info to this FD") },
     { aDummy, "no-comment", 0,   "@"},
     { aDummy, "completes-needed", 1, "@"},
@@ -391,6 +417,7 @@ static ARGPARSE_OPTS opts[] = {
     { oWithKeyData,"with-key-data", 0, "@"},
     { oWithValidation, "with-validation", 0, "@"},
     { oWithMD5Fingerprint, "with-md5-fingerprint", 0, "@"},
+    { oWithEphemeralKeys,  "with-ephemeral-keys", 0, "@"},
     { aListKeys, "list-key", 0, "@" }, /* alias */
     { aListSigs, "list-sig", 0, "@" }, /* alias */
     { aListSigs, "check-sig",0, "@" }, /* alias */
@@ -419,6 +446,10 @@ int gpgsm_errors_seen = 0;
 /* It is possible that we are currentlu running under setuid permissions */
 static int maybe_setuid = 1;
 
+/* Helper to implement --debug-level and --debug*/
+static const char *debug_level;
+static unsigned int debug_value;
+
 /* Option --enable-special-filenames */
 static int allow_special_filenames;
 
@@ -536,21 +567,27 @@ build_list (const char *text, const char * (*mapf)(int), int (*chkf)(int))
 }
 
 
+/* Set the file pointer into binary mode if required.  */
+static void
+set_binary (FILE *fp)
+{
+#ifdef HAVE_DOSISH_SYSTEM
+  setmode (fileno (fp), O_BINARY);
+#endif
+}
+
+
+
 static void
 i18n_init(void)
 {
 #ifdef USE_SIMPLE_GETTEXT
-  set_gettext_file (PACKAGE);
+  set_gettext_file (PACKAGE_GT);
 #else
 # ifdef ENABLE_NLS
-#  ifdef HAVE_LC_MESSAGES
-  setlocale (LC_TIME, "");
-  setlocale (LC_MESSAGES, "");
-#  else
   setlocale (LC_ALL, "" );
-#  endif
-  bindtextdomain (PACKAGE, LOCALEDIR);
-  textdomain (PACKAGE);
+  bindtextdomain (PACKAGE_GT, LOCALEDIR);
+  textdomain (PACKAGE_GT);
 # endif
 #endif
 }
@@ -566,45 +603,44 @@ wrong_args (const char *text)
 }
 
 
-/* Setup the debugging.  With a LEVEL of NULL only the active debug
-   flags are propagated to the subsystems.  With LEVEL set, a specific
-   set of debug flags is set; thus overriding all flags already
-   set. */
+/* Setup the debugging.  With a DEBUG_LEVEL of NULL only the active
+   debug flags are propagated to the subsystems.  With DEBUG_LEVEL
+   set, a specific set of debug flags is set; and individual debugging
+   flags will be added on top.  */
 static void
-set_debug (const char *level)
+set_debug (void)
 {
-  if (!level)
+  if (!debug_level)
     ;
-  else if (!strcmp (level, "none"))
+  else if (!strcmp (debug_level, "none"))
     opt.debug = 0;
-  else if (!strcmp (level, "basic"))
+  else if (!strcmp (debug_level, "basic"))
     opt.debug = DBG_ASSUAN_VALUE;
-  else if (!strcmp (level, "advanced"))
+  else if (!strcmp (debug_level, "advanced"))
     opt.debug = DBG_ASSUAN_VALUE|DBG_X509_VALUE;
-  else if (!strcmp (level, "expert"))
+  else if (!strcmp (debug_level, "expert"))
     opt.debug = (DBG_ASSUAN_VALUE|DBG_X509_VALUE
                  |DBG_CACHE_VALUE|DBG_CRYPTO_VALUE);
-  else if (!strcmp (level, "guru"))
+  else if (!strcmp (debug_level, "guru"))
     opt.debug = ~0;
   else
     {
-      log_error (_("invalid debug-level `%s' given\n"), level);
+      log_error (_("invalid debug-level `%s' given\n"), debug_level);
       gpgsm_exit(2);
     }
 
+  opt.debug |= debug_value;
 
   if (opt.debug && !opt.verbose)
-    {
-      opt.verbose = 1;
-      gcry_control (GCRYCTL_SET_VERBOSITY, (int)opt.verbose);
-    }
-  if (opt.debug && opt.quiet)
+    opt.verbose = 1;
+  if (opt.debug)
     opt.quiet = 0;
 
   if (opt.debug & DBG_MPI_VALUE)
     gcry_control (GCRYCTL_SET_DEBUG_FLAGS, 2);
   if (opt.debug & DBG_CRYPTO_VALUE )
     gcry_control (GCRYCTL_SET_DEBUG_FLAGS, 1);
+  gcry_control (GCRYCTL_SET_VERBOSITY, (int)opt.verbose);
 }
  
 
@@ -681,7 +717,6 @@ main ( int argc, char **argv)
   int greeting = 0;
   int nogreeting = 0;
   int debug_wait = 0;
-  const char *debug_level = NULL;
   int use_random_seed = 1;
   int with_fpr = 0;
   char *def_digest_string = NULL;
@@ -691,6 +726,7 @@ main ( int argc, char **argv)
   CERTLIST signerlist = NULL;
   int do_not_setup_keys = 0;
 
+
   /* trap_unaligned ();*/
   set_strusage (my_strusage);
   gcry_control (GCRYCTL_SUSPEND_SECMEM_WARN);
@@ -719,6 +755,12 @@ main ( int argc, char **argv)
                  NEED_KSBA_VERSION, ksba_check_version (NULL) );
     }
 
+#ifdef HAVE_W32_SYSTEM
+  /* For W32 we need pth.  */
+  pth_init ();
+#endif
+
+
   gcry_control (GCRYCTL_USE_SECURE_RNDPOOL);
 
   may_coredump = disable_core_dumps ();
@@ -729,16 +771,13 @@ main ( int argc, char **argv)
   i18n_init();
 
   opt.def_cipher_algoid = "1.2.840.113549.3.7";  /*des-EDE3-CBC*/
-#ifdef __MINGW32__
-  opt.homedir = read_w32_registry_string ( NULL,
-                                           "Software\\GNU\\GnuPG", "HomeDir" );
-#else
-  opt.homedir = getenv ("GNUPGHOME");
-#endif
-  if (!opt.homedir || !*opt.homedir ) 
-    opt.homedir = GNUPG_DEFAULT_HOMEDIR;
 
-  /* first check whether we have a config file on the commandline */
+  opt.homedir = default_homedir ();
+#ifdef HAVE_W32_SYSTEM
+  opt.no_crl_check = 1;
+#endif
+
+  /* First check whether we have a config file on the commandline */
   orig_argc = argc;
   orig_argv = argv;
   pargs.argc = &argc;
@@ -787,10 +826,10 @@ main ( int argc, char **argv)
   ctrl.status_fd = -1; /* not status output */
   ctrl.autodetect_encoding = 1;
 
-  /* set the default option file */
+  /* Set the default option file */
   if (default_config )
     configname = make_filename (opt.homedir, "gpgsm.conf", NULL);
-  /* cet the default policy file */
+  /* Set the default policy file */
   opt.policy_file = make_filename (opt.homedir, "policies.txt", NULL);
   
   argc        = orig_argc;
@@ -875,12 +914,16 @@ main ( int argc, char **argv)
         case aRecvKeys: 
         case aExport: 
         case aExportSecretKeyP12: 
+        case aDumpKeys:
+        case aDumpExternalKeys: 
+        case aDumpSecretKeys: 
         case aListKeys:
         case aListExternalKeys: 
         case aListSecretKeys: 
         case aListSigs: 
         case aLearnCard: 
         case aPasswd: 
+        case aKeydbClearSomeCertFlags:
           do_not_setup_keys = 1;
           set_cmd (&cmd, pargs.r_opt);
           break;
@@ -930,12 +973,15 @@ main ( int argc, char **argv)
         case oEnableCRLChecks:
           opt.no_crl_check = 0;
           break;
+        case oForceCRLRefresh:
+          opt.force_crl_refresh = 1;
+          break;
 
         case oDisableOCSP:
-          opt.enable_ocsp = 0;
+          ctrl.use_ocsp = opt.enable_ocsp = 0;
           break;
         case oEnableOCSP:
-          opt.enable_ocsp = 1;
+          ctrl.use_ocsp = opt.enable_ocsp = 1;
           break;
 
         case oIncludeCerts: ctrl.include_certs = pargs.r.ret_int; break;
@@ -988,12 +1034,16 @@ main ( int argc, char **argv)
 
         case oKeyring: append_to_strlist (&nrings, pargs.r.ret_str); break;
 
-        case oDebug: opt.debug |= pargs.r.ret_ulong; break;
-        case oDebugAll: opt.debug = ~0; break;
+        case oDebug: debug_value |= pargs.r.ret_ulong; break;
+        case oDebugAll: debug_value = ~0; break;
         case oDebugLevel: debug_level = pargs.r.ret_str; break;
         case oDebugWait: debug_wait = pargs.r.ret_int; break;
+        case oDebugAllowCoreDump:
+          may_coredump = enable_core_dumps ();
+          break;
         case oDebugNoChainValidation: opt.no_chain_validation = 1; break;
         case oDebugIgnoreExpiration: opt.ignore_expiration = 1; break;
+        case oFixedPassphrase: opt.fixed_passphrase = pargs.r.ret_str; break;
 
         case oStatusFD: ctrl.status_fd = pargs.r.ret_int; break;
         case oLoggerFD: log_set_fd (pargs.r.ret_int ); break;
@@ -1023,6 +1073,7 @@ main ( int argc, char **argv)
         case oLCctype: opt.lc_ctype = xstrdup (pargs.r.ret_str); break;
         case oLCmessages: opt.lc_messages = xstrdup (pargs.r.ret_str); break;
         case oDirmngrProgram: opt.dirmngr_program = pargs.r.ret_str;  break;
+        case oPreferSystemDirmngr: opt.prefer_system_dirmngr = 1; break;
         case oProtectToolProgram:
           opt.protect_tool_program = pargs.r.ret_str; 
           break;
@@ -1036,6 +1087,7 @@ main ( int argc, char **argv)
 
         case oDefaultKey:
           /* fixme:opt.def_secret_key = pargs.r.ret_str;*/
+          log_info ("WARNING: --default-key has not yet been implemented\n");
           break;
         case oDefRecipient:
           if (*pargs.r.ret_str)
@@ -1055,6 +1107,7 @@ main ( int argc, char **argv)
         case oWithKeyData: opt.with_key_data=1; /* fall thru */
         case oWithColons: ctrl.with_colons = 1; break;
         case oWithValidation: ctrl.with_validation=1; break;
+        case oWithEphemeralKeys: opt.with_ephemeral_keys=1; break;
 
         case oSkipVerify: opt.skip_verify=1; break;
 
@@ -1116,14 +1169,17 @@ main ( int argc, char **argv)
     {
       fclose (configfp);
       configfp = NULL;
-      xfree (configname);
+      /* Keep a copy of the config filename. */
+      opt.config_filename = configname;
       configname = NULL;
       goto next_pass;
     }
-  
   xfree (configname);
   configname = NULL;
 
+  if (!opt.config_filename)
+    opt.config_filename = make_filename (opt.homedir, "gpgsm.conf", NULL);
+
   if (log_get_errorcount(0))
     gpgsm_exit(2);
   
@@ -1169,13 +1225,19 @@ main ( int argc, char **argv)
 
   gcry_control (GCRYCTL_RESUME_SECMEM_WARN);
 
-  set_debug (debug_level);
+  set_debug ();
 
-  /* FIXME: should set filenames of libgcrypt explicitly
-   * gpg_opt_homedir = opt.homedir; */
+  /* Although we alwasy use gpgsm_exit, we better install a regualr
+     exit handler so that at least the secure memory gets wiped
+     out. */
+  if (atexit (emergency_cleanup))
+    {
+      log_error ("atexit failed\n");
+      gpgsm_exit (2);
+    }
 
-  /* must do this after dropping setuid, because the mapping functions
-     may try to load an module and we may have disabled an algorithm */
+  /* Must do this after dropping setuid, because the mapping functions
+     may try to load an module and we may have disabled an algorithm. */
   if ( !gcry_cipher_map_name (opt.def_cipher_algoid)
        || !gcry_cipher_mode_from_oid (opt.def_cipher_algoid))
     log_error (_("selected cipher algorithm is invalid\n"));
@@ -1192,7 +1254,7 @@ main ( int argc, char **argv)
   if (log_get_errorcount(0))
     gpgsm_exit(2);
   
-  /* set the random seed file */
+  /* Set the random seed file. */
   if (use_random_seed) {
     char *p = make_filename (opt.homedir, "random_seed", NULL);
     gcry_control (GCRYCTL_SET_RANDOM_SEED_FILE, p);
@@ -1259,21 +1321,48 @@ main ( int argc, char **argv)
     {
     case aGPGConfList: 
       { /* List options and default values in the GPG Conf format.  */
-        char *filename;
 
-        /* First the default configuration file.  This is not an
-	 option, but it is vital information for GPG Conf.  */
-        filename = make_filename (opt.homedir, "gpgsm.conf", NULL);
-        printf ("gpgconf-gpgsm.conf:\"%s\n", filename);
-        xfree (filename);
+        /* The following list is taken from gnupg/tools/gpgconf-comp.c.  */
+        /* Option flags.  YOU MUST NOT CHANGE THE NUMBERS OF THE EXISTING
+           FLAGS, AS THEY ARE PART OF THE EXTERNAL INTERFACE.  */
+#define GC_OPT_FLAG_NONE	0UL
+        /* The RUNTIME flag for an option indicates that the option can be
+           changed at runtime.  */
+#define GC_OPT_FLAG_RUNTIME	(1UL << 3)
+        /* The DEFAULT flag for an option indicates that the option has a
+           default value.  */
+#define GC_OPT_FLAG_DEFAULT	(1UL << 4)
+        /* The DEF_DESC flag for an option indicates that the option has a
+           default, which is described by the value of the default field.  */
+#define GC_OPT_FLAG_DEF_DESC	(1UL << 5)
+        /* The NO_ARG_DESC flag for an option indicates that the argument has
+           a default, which is described by the value of the ARGDEF field.  */
+#define GC_OPT_FLAG_NO_ARG_DESC	(1UL << 6)
+
+        printf ("gpgconf-gpgsm.conf:%lu:\"%s\n",
+                GC_OPT_FLAG_DEFAULT, opt.config_filename);
+        
+        printf ("verbose:%lu:\n"
+                "quiet:%lu:\n"
+                "debug-level:%lu:\"none:\n"
+                "log-file:%lu:\n",
+                GC_OPT_FLAG_NONE,
+                GC_OPT_FLAG_NONE,
+                GC_OPT_FLAG_DEFAULT,
+                GC_OPT_FLAG_NONE );
+        printf ("disable-crl-checks:%lu:\n",
+                GC_OPT_FLAG_NONE );
+        printf ("enable-ocsp:%lu:\n",
+                GC_OPT_FLAG_NONE );
+        printf ("include-certs:%lu:1:\n",
+                GC_OPT_FLAG_DEFAULT );
+        printf ("disable-policy-checks:%lu:\n",
+                GC_OPT_FLAG_NONE );
+        printf ("auto-issuer-key-retrieve:%lu:\n",
+                GC_OPT_FLAG_NONE );
+        printf ("prefer-system-dirmngr:%lu:\n",
+                GC_OPT_FLAG_NONE );
 
-        printf ("verbose:\n"
-                "quiet:\n"
-                "debug-level:none\n"
-                "log-file:\n"
-                "force:\n"
-                "faked-system-time:\n"
-                "no-greeting:\n");
       }
       break;
 
@@ -1290,7 +1379,7 @@ main ( int argc, char **argv)
 
     case aCallDirmngr:
       if (!argc)
-        wrong_args (_("--call-dirmngr <command> {args}"));
+        wrong_args ("--call-dirmngr <command> {args}");
       else
         if (gpgsm_dirmngr_run_command (&ctrl, *argv, argc-1, argv+1))
           gpgsm_exit (1);
@@ -1301,18 +1390,22 @@ main ( int argc, char **argv)
       break;
 
     case aEncr: /* encrypt the given file */
+      set_binary (stdin);
+      set_binary (stdout);
       if (!argc)
         gpgsm_encrypt (&ctrl, recplist, 0, stdout); /* from stdin */
       else if (argc == 1)
         gpgsm_encrypt (&ctrl, recplist, open_read (*argv), stdout); /* from file */
       else
-        wrong_args (_("--encrypt [datafile]"));
+        wrong_args ("--encrypt [datafile]");
       break;
 
     case aSign: /* sign the given file */
       /* FIXME: We don't handle --output yet. We should also allow
          to concatenate multiple files for signing because that is
          what gpg does.*/
+      set_binary (stdin);
+      set_binary (stdout);
       if (!argc)
         gpgsm_sign (&ctrl, signerlist,
                     0, detached_sig, stdout); /* create from stdin */
@@ -1320,7 +1413,7 @@ main ( int argc, char **argv)
         gpgsm_sign (&ctrl, signerlist,
                     open_read (*argv), detached_sig, stdout); /* from file */
       else
-        wrong_args (_("--sign [datafile]"));
+        wrong_args ("--sign [datafile]");
       break;
         
     case aSignEncr: /* sign and encrypt the given file */
@@ -1335,6 +1428,7 @@ main ( int argc, char **argv)
       {
         FILE *fp = NULL;
 
+        set_binary (stdin);
         if (argc == 2 && opt.outfile)
           log_info ("option --output ignored for a detached signature\n");
         else if (opt.outfile)
@@ -1347,7 +1441,7 @@ main ( int argc, char **argv)
         else if (argc == 2) /* detached signature (sig, detached) */
           gpgsm_verify (&ctrl, open_read (*argv), open_read (argv[1]), NULL); 
         else
-          wrong_args (_("--verify [signature [detached_data]]"));
+          wrong_args ("--verify [signature [detached_data]]");
 
         if (fp && fp != stdout)
           fclose (fp);
@@ -1355,16 +1449,18 @@ main ( int argc, char **argv)
       break;
 
     case aVerifyFiles:
-      log_error ("this command has not yet been implemented\n");
+      log_error (_("this command has not yet been implemented\n"));
       break;
 
     case aDecrypt:
+      set_binary (stdin);
+      set_binary (stdout);
       if (!argc)
         gpgsm_decrypt (&ctrl, 0, stdout); /* from stdin */
       else if (argc == 1)
         gpgsm_decrypt (&ctrl, open_read (*argv), stdout); /* from file */
       else
-        wrong_args (_("--decrypt [filename]"));
+        wrong_args ("--decrypt [filename]");
       break;
 
     case aDeleteKey:
@@ -1383,6 +1479,13 @@ main ( int argc, char **argv)
       free_strlist(sl);
       break;
 
+    case aDumpKeys:
+      for (sl=NULL; argc; argc--, argv++)
+        add_to_strlist (&sl, *argv);
+      gpgsm_list_keys (&ctrl, sl, stdout, (256 | (1<<6)));
+      free_strlist(sl);
+      break;
+
     case aListExternalKeys:
       for (sl=NULL; argc; argc--, argv++)
         add_to_strlist (&sl, *argv);
@@ -1391,6 +1494,14 @@ main ( int argc, char **argv)
       free_strlist(sl);
       break;
 
+    case aDumpExternalKeys:
+      for (sl=NULL; argc; argc--, argv++)
+        add_to_strlist (&sl, *argv);
+      gpgsm_list_keys (&ctrl, sl, stdout,
+                       (256 | (1<<7)));
+      free_strlist(sl);
+      break;
+
     case aListSecretKeys:
       for (sl=NULL; argc; argc--, argv++)
         add_to_strlist (&sl, *argv);
@@ -1398,6 +1509,13 @@ main ( int argc, char **argv)
       free_strlist(sl);
       break;
 
+    case aDumpSecretKeys:
+      for (sl=NULL; argc; argc--, argv++)
+        add_to_strlist (&sl, *argv);
+      gpgsm_list_keys (&ctrl, sl, stdout, (256 | 2 | (1<<6)));
+      free_strlist(sl);
+      break;
+
     case aKeygen: /* generate a key */
       log_error ("this function is not yet available from the commandline\n");
       break;
@@ -1407,6 +1525,7 @@ main ( int argc, char **argv)
       break;
 
     case aExport:
+      set_binary (stdout);
       for (sl=NULL; argc; argc--, argv++)
         add_to_strlist (&sl, *argv);
       gpgsm_export (&ctrl, sl, stdout);
@@ -1414,10 +1533,11 @@ main ( int argc, char **argv)
       break;
 
     case aExportSecretKeyP12:
+      set_binary (stdout);
       if (argc == 1)
         gpgsm_p12_export (&ctrl, *argv, stdout);
       else
-        wrong_args (_("--export-secret-key-p12 KEY-ID"));
+        wrong_args ("--export-secret-key-p12 KEY-ID");
       break;
       
     case aSendKeys:
@@ -1431,7 +1551,7 @@ main ( int argc, char **argv)
         wrong_args ("--learn-card");
       else
         {
-          int rc = gpgsm_agent_learn ();
+          int rc = gpgsm_agent_learn (&ctrl);
           if (rc)
             log_error ("error learning card: %s\n", gpg_strerror (rc));
         }
@@ -1454,7 +1574,7 @@ main ( int argc, char **argv)
           else 
             {
               char *desc = gpgsm_format_keydesc (cert);
-              rc = gpgsm_agent_passwd (grip, desc);
+              rc = gpgsm_agent_passwd (&ctrl, grip, desc);
               xfree (desc);
             }
           if (rc)
@@ -1464,6 +1584,14 @@ main ( int argc, char **argv)
         }
       break;
 
+    case aKeydbClearSomeCertFlags:
+      for (sl=NULL; argc; argc--, argv++)
+        add_to_strlist (&sl, *argv);
+      keydb_clear_some_cert_flags (&ctrl, sl);
+      free_strlist(sl);
+      break;
+
+
     default:
         log_error ("invalid command (there is no implicit command)\n");
 	break;
@@ -1541,11 +1669,14 @@ open_read (const char *filename)
   int fd;
 
   if (filename[0] == '-' && !filename[1])
-    return 0; /* stdin */
+    {
+      set_binary (stdin);
+      return 0; /* stdin */
+    }
   fd = check_special_filename (filename);
   if (fd != -1)
     return fd;
-  fd = open (filename, O_RDONLY);
+  fd = open (filename, O_RDONLY | O_BINARY);
   if (fd == -1)
     {
       log_error (_("can't open `%s': %s\n"), filename, strerror (errno));
@@ -1565,7 +1696,10 @@ open_fwrite (const char *filename)
   FILE *fp;
 
   if (filename[0] == '-' && !filename[1])
-    return stdout;
+    {
+      set_binary (stdout);
+      return stdout;
+    }
 
   fd = check_special_filename (filename);
   if (fd != -1)
@@ -1576,6 +1710,7 @@ open_fwrite (const char *filename)
           log_error ("fdopen(%d) failed: %s\n", fd, strerror (errno));
           gpgsm_exit (2);
         }
+      set_binary (fp);
       return fp;
     }
   fp = fopen (filename, "wb");
@@ -1591,6 +1726,7 @@ open_fwrite (const char *filename)
 static void
 run_protect_tool (int argc, char **argv)
 {
+#ifndef HAVE_W32_SYSTEM
   const char *pgm;
   char **av;
   int i;
@@ -1609,5 +1745,6 @@ run_protect_tool (int argc, char **argv)
   av[i] = NULL;
   execv (pgm, av); 
   log_error ("error executing `%s': %s\n", pgm, strerror (errno));
+#endif /*HAVE_W32_SYSTEM*/
   gpgsm_exit (2);
 }
