@@ -33,8 +33,6 @@
 #include "keydb.h" 
 #include "i18n.h"
 
-#define DIRSEP_C '/'
-
 static int active_handles;
 
 typedef enum {
@@ -218,10 +216,25 @@ keydb_add_resource (const char *url, int force, int secret)
                 = create_dotlock (filename);
               if (!all_resources[used_resources].lockhandle)
                 log_fatal ( _("can't create lock for `%s'\n"), filename);
+
+              /* Do a compress run if needed and the file is not locked. */
+              if (!make_dotlock (all_resources[used_resources].lockhandle, 0))
+                {
+                  KEYBOX_HANDLE kbxhd = keybox_new (token, secret);
+                  
+                  if (kbxhd)
+                    {
+                      keybox_compress (kbxhd);
+                      keybox_release (kbxhd);
+                    }
+                  release_dotlock (all_resources[used_resources].lockhandle);
+                }
                   
               used_resources++;
             }
         }
+
+
 	break;
     default:
       log_error ("resource type of `%s' not supported\n", url);
@@ -397,7 +410,7 @@ lock_all (KEYDB_HANDLE hd)
   int i, rc = 0;
 
   /* Fixme: This locking scheme may lead to deadlock if the resources
-     are not added in the same order all processes.  We are
+     are not added in the same order by all processes.  We are
      currently only allowing one resource so it is not a problem. */
   for (i=0; i < hd->used; i++) 
     {
@@ -511,7 +524,7 @@ keydb_update_keyblock (KEYDB_HANDLE hd, KBNODE kb)
 	return 0;
 
     if (!hd->locked)
-      return gpg_error (GPG_ERR_CONFLICT);
+      return gpg_error (GPG_ERR_NOT_LOCKED);
 
     switch (hd->active[hd->found].type) {
       case KEYDB_RESOURCE_TYPE_NONE:
@@ -630,7 +643,7 @@ keydb_get_flags (KEYDB_HANDLE hd, int which, int idx, unsigned int *value)
 /* Set a flag of the last found object. WHICH is the flag to be set; it
    should be one of the KEYBOX_FLAG_ values.  If the operation is
    successful, the flag value will be stored in the keybox.  Note,
-   that some flag values can't be updated and thus may retrun an
+   that some flag values can't be updated and thus may return an
    error, some other flag values may be masked out before an update.
    Returns 0 on success or an error code. */
 gpg_error_t
@@ -645,7 +658,7 @@ keydb_set_flags (KEYDB_HANDLE hd, int which, int idx, unsigned int value)
     return gpg_error (GPG_ERR_NOTHING_FOUND);
   
   if (!hd->locked)
-    return gpg_error (GPG_ERR_CONFLICT);
+    return gpg_error (GPG_ERR_NOT_LOCKED);
 
   switch (hd->active[hd->found].type) 
     {
@@ -760,7 +773,7 @@ keydb_delete (KEYDB_HANDLE hd)
     return 0;
 
   if (!hd->locked)
-    return gpg_error (GPG_ERR_CONFLICT); /* ...NOT_LOCKED would be better. */
+    return gpg_error (GPG_ERR_NOT_LOCKED); 
 
   switch (hd->active[hd->found].type)
     {
@@ -1036,7 +1049,7 @@ classify_user_id (const char *name,
    * we set it to the correct value right at the end of this function */
   memset (desc, 0, sizeof *desc);
   *force_exact = 0;
-  /* skip leading spaces.  Fixme: what about trailing white space? */
+  /* Skip leading spaces.  Fixme: what about trailing white space? */
   for(s = name; *s && spacep (s); s++ )
     ;
 
@@ -1359,7 +1372,7 @@ keydb_store_cert (ksba_cert_t cert, int ephemeral, int *existed)
 
 
 /* This is basically keydb_set_flags but it implements a complete
-   transaction by locating teh certificate in the DB and updating the
+   transaction by locating the certificate in the DB and updating the
    flags. */
 gpg_error_t
 keydb_set_cert_flags (ksba_cert_t cert, int which, int idx, unsigned int value)
@@ -1420,3 +1433,98 @@ keydb_set_cert_flags (ksba_cert_t cert, int which, int idx, unsigned int value)
   return 0;
 }
 
+
+/* Reset all the certificate flags we have stored with the certificates
+   for performance reasons. */
+void
+keydb_clear_some_cert_flags (ctrl_t ctrl, STRLIST names)
+{
+  gpg_error_t err;
+  KEYDB_HANDLE hd = NULL;
+  KEYDB_SEARCH_DESC *desc = NULL;
+  int ndesc;
+  STRLIST sl;
+  int rc=0;
+  unsigned int old_value, value;
+  
+  hd = keydb_new (0);
+  if (!hd)
+    {
+      log_error ("keydb_new failed\n");
+      goto leave;
+    }
+
+  if (!names)
+    ndesc = 1;
+  else
+    {
+      for (sl=names, ndesc=0; sl; sl = sl->next, ndesc++) 
+        ;
+    }
+
+  desc = xtrycalloc (ndesc, sizeof *desc);
+  if (!ndesc)
+    {
+      log_error ("allocating memory failed: %s\n",
+                 gpg_strerror (OUT_OF_CORE (errno)));
+      goto leave;
+    }
+
+  if (!names)
+    desc[0].mode = KEYDB_SEARCH_MODE_FIRST;
+  else 
+    {
+      for (ndesc=0, sl=names; sl; sl = sl->next) 
+        {
+          rc = keydb_classify_name (sl->d, desc+ndesc);
+          if (rc)
+            {
+              log_error ("key `%s' not found: %s\n",
+                         sl->d, gpg_strerror (rc));
+              rc = 0;
+            }
+          else
+            ndesc++;
+        }
+    }
+
+  err = keydb_lock (hd);
+  if (err)
+    {
+      log_error (_("error locking keybox: %s\n"), gpg_strerror (err));
+      goto leave;
+    }
+
+  while (!(rc = keydb_search (hd, desc, ndesc)))
+    {
+      if (!names) 
+        desc[0].mode = KEYDB_SEARCH_MODE_NEXT;
+
+      err = keydb_get_flags (hd, KEYBOX_FLAG_VALIDITY, 0, &old_value);
+      if (err)
+        {
+          log_error (_("error getting stored flags: %s\n"),
+                     gpg_strerror (err));
+          goto leave;
+        }
+ 
+      value = (old_value & ~VALIDITY_REVOKED);
+      if (value != old_value)
+        {
+          err = keydb_set_flags (hd, KEYBOX_FLAG_VALIDITY, 0, value);
+          if (err)
+            {
+              log_error (_("error storing flags: %s\n"), gpg_strerror (err));
+              goto leave;
+            }
+        }
+    }
+  if (rc && rc != -1)
+    log_error ("keydb_search failed: %s\n", gpg_strerror (rc));
+  
+ leave:
+  xfree (desc);
+  keydb_release (hd);
+}
+
+
