@@ -1,5 +1,5 @@
 /* bftest.c - Blowfish test program
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -23,12 +23,12 @@
 #include <stdlib.h>
 #include <string.h>
 #ifdef HAVE_DOSISH_SYSTEM
-  #include <io.h>
-  #include <fcntl.h>
+#include <io.h>
+#include <fcntl.h>
 #endif
 
-#include <gcrypt.h>
 #include "util.h"
+#include "cipher.h"
 #include "i18n.h"
 
 static void
@@ -38,34 +38,35 @@ my_usage(void)
     exit(1);
 }
 
+const char *
+strusage( int level )
+{
+    return default_strusage(level);
+}
 
 static void
 i18n_init(void)
 {
-  #ifdef ENABLE_NLS
-    #ifdef HAVE_LC_MESSAGES
-       setlocale( LC_MESSAGES, "" );
-    #else
-       setlocale( LC_ALL, "" );
-    #endif
-    bindtextdomain( PACKAGE, GNUPG_LOCALEDIR );
-    textdomain( PACKAGE );
-  #endif
+#ifdef ENABLE_NLS
+  setlocale( LC_ALL, "" );
+  bindtextdomain( PACKAGE, G10_LOCALEDIR );
+  textdomain( PACKAGE );
+#endif
 }
 
 int
 main(int argc, char **argv)
 {
     int encode=0;
-    GCRY_CIPHER_HD hd;
+    CIPHER_HANDLE hd;
     char buf[4096];
     int n, size=4096;
     int algo;
 
-  #ifdef HAVE_DOSISH_SYSTEM
+#ifdef HAVE_DOSISH_SYSTEM
     setmode( fileno(stdin), O_BINARY );
     setmode( fileno(stdout), O_BINARY );
-  #endif
+#endif
 
     i18n_init();
     if( argc > 1 && !strcmp(argv[1], "-e") ) {
@@ -87,21 +88,20 @@ main(int argc, char **argv)
     if( argc != 3 )
 	my_usage();
     argc--; argv++;
-    algo = gcry_cipher_map_name( *argv );
+    algo = string_to_cipher_algo( *argv );
     argc--; argv++;
 
-    hd = gcry_cipher_open( algo, GCRY_CIPHER_MODE_CFB, 0 );
-    gcry_cipher_setkey( hd, *argv, strlen(*argv) );
-    gcry_cipher_setiv( hd, NULL, 0 );
+    hd = cipher_open( algo, CIPHER_MODE_CFB, 0 );
+    cipher_setkey( hd, *argv, strlen(*argv) );
+    cipher_setiv( hd, NULL, 0 );
     while( (n = fread( buf, 1, size, stdin )) > 0 ) {
 	if( encode )
-	    gcry_cipher_encrypt( hd, buf, n, buf, n );
+	    cipher_encrypt( hd, buf, buf, n );
 	else
-	    gcry_cipher_decrypt( hd, buf, n, buf, n );
+	    cipher_decrypt( hd, buf, buf, n );
 	if( fwrite( buf, 1, n, stdout) != n )
 	    log_fatal("write error\n");
     }
-    gcry_cipher_close(hd);
+    cipher_close(hd);
     return 0;
 }
-
