@@ -23,8 +23,6 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-/* FIXME use gettext.h */
-#include <libintl.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <sys/types.h>
@@ -32,17 +30,18 @@
 #include <errno.h>
 #include <time.h>
 #include <stdarg.h>
+#include <signal.h>
 
 /* For log_logv(), asctimestamp(), gnupg_get_time ().  */
 #define JNLIB_NEED_LOG_LOGV
 #include "util.h"
+#include "i18n.h"
 
 #include "gpgconf.h"
 
+
 
 /* TODO:
-   Portability - Add gnulib replacements for getline, etc.
-   Backend: File backend must be able to write out changes !!!
    Components: Add more components and their options.
    Robustness: Do more validation.  Call programs to do validation for us.
    Don't use popen, as this will not tell us if the program had a
@@ -52,8 +51,7 @@
 */
 
 
-#if defined (__riscos__) \
-    || (__GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 5 ))
+#if (__GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 5 ))
 void gc_error (int status, int errnum, const char *fmt, ...) \
   __attribute__ ((format (printf, 3, 4)));
 #endif
@@ -87,6 +85,9 @@ gc_error (int status, int errnum, const char *fmt, ...)
 }
 
 
+/* Forward declaration.  */
+void gpg_agent_runtime_change (void);
+
 /* Backend configuration.  Backends are used to decide how the default
    and current value of an option can be determined, and how the
    option can be changed.  To every option in every component belongs
@@ -108,6 +109,9 @@ typedef enum
     /* The GPG Agent.  */
     GC_BACKEND_GPG_AGENT,
 
+    /* The GnuPG SCDaemon.  */
+    GC_BACKEND_SCDAEMON,
+
     /* The Aegypten directory manager.  */
     GC_BACKEND_DIRMNGR,
 
@@ -131,6 +135,9 @@ static struct
      GPGConf.  In this case, PROGRAM is NULL.  */
   char *program;
 
+  /* The runtime change callback.  */
+  void (*runtime_change) (void);
+
   /* The option name for the configuration filename of this backend.
      This must be an absolute pathname.  It can be an option from a
      different backend (but then ordering of the options might
@@ -142,12 +149,15 @@ static struct
   const char *option_name;
 } gc_backend[GC_BACKEND_NR] =
   {
-    { NULL, NULL, NULL },		/* GC_BACKEND_ANY dummy entry.  */
-    { "GnuPG", "gpg", "gpgconf-gpg.conf" },
-    { "GPGSM", "gpgsm", "gpgconf-gpgsm.conf" },
-    { "GPG Agent", "gpg-agent", "gpgconf-gpg-agent.conf" },
-    { "DirMngr", "dirmngr", "gpgconf-dirmngr.conf" },
-    { "DirMngr LDAP Server List", NULL, "ldapserverlist-file", "LDAP Server" },
+    { NULL },		/* GC_BACKEND_ANY dummy entry.  */
+    { "GnuPG", "gpg", NULL, "gpgconf-gpg.conf" },
+    { "GPGSM", "gpgsm", NULL, "gpgconf-gpgsm.conf" },
+    { "GPG Agent", "gpg-agent", gpg_agent_runtime_change,
+      "gpgconf-gpg-agent.conf" },
+    { "SCDaemon", "scdaemon", NULL, "gpgconf-scdaemon.conf" },
+    { "DirMngr", "dirmngr", NULL, "gpgconf-dirmngr.conf" },
+    { "DirMngr LDAP Server List", NULL, NULL, "ldapserverlist-file",
+      "LDAP Server" },
   };
 
 
@@ -319,7 +329,10 @@ static struct
     { "group" },
     { "optional arg" },
     { "list" },
-    { "runtime" }
+    { "runtime" },
+    { "default" },
+    { "default desc" },
+    { "no arg desc" }
   };
 
 
@@ -348,12 +361,22 @@ struct gc_option
 
   /* A gettext domain in which the following description can be found.
      If this is NULL, then DESC is not translated.  Valid for groups
-     and options.  */
+     and options.
+     
+     Note that we try to keep the description of groups within the
+     gnupg domain. 
+     
+     IMPORTANT: If you add a new domain please make sure to add a code
+     set switching call to the function my_dgettext further below.  */
   const char *desc_domain;
 
   /* A gettext description for this group or option.  If it starts
      with a '|', then the string up to the next '|' describes the
-     argument, and the description follows the second '|'.  */
+     argument, and the description follows the second '|'. 
+
+     In general enclosing these description in N_() is not required
+     because the description should be identical to the one in the
+     help menu of the respective program. */
   const char *desc;
 
   /* The following fields are only valid for options.  */
@@ -403,6 +426,60 @@ typedef struct gc_option gc_option_t;
 /* The options of the GC_COMPONENT_GPG_AGENT component.  */
 static gc_option_t gc_options_gpg_agent[] =
  {
+   /* The configuration file to which we write the changes.  */
+   { "gpgconf-gpg-agent.conf", GC_OPT_FLAG_NONE, GC_LEVEL_INTERNAL,
+     NULL, NULL, GC_ARG_TYPE_PATHNAME, GC_BACKEND_GPG_AGENT },
+
+   { "Monitor",
+     GC_OPT_FLAG_GROUP, GC_LEVEL_BASIC,
+     "gnupg", N_("Options controlling the diagnostic output") },
+   { "verbose", GC_OPT_FLAG_LIST|GC_OPT_FLAG_RUNTIME, GC_LEVEL_BASIC,
+     "gnupg", "verbose",
+     GC_ARG_TYPE_NONE, GC_BACKEND_GPG_AGENT },
+   { "quiet", GC_OPT_FLAG_NONE|GC_OPT_FLAG_RUNTIME, GC_LEVEL_BASIC,
+     "gnupg", "be somewhat more quiet",
+     GC_ARG_TYPE_NONE, GC_BACKEND_GPG_AGENT },
+   { "no-greeting", GC_OPT_FLAG_NONE, GC_LEVEL_INVISIBLE,
+     NULL, NULL,
+     GC_ARG_TYPE_NONE, GC_BACKEND_GPG_AGENT },
+
+   { "Configuration",
+     GC_OPT_FLAG_GROUP, GC_LEVEL_EXPERT,
+     "gnupg", N_("Options controlling the configuration") },
+   { "options", GC_OPT_FLAG_NONE, GC_LEVEL_EXPERT,
+     "gnupg", "|FILE|read options from FILE",
+     GC_ARG_TYPE_PATHNAME, GC_BACKEND_GPG_AGENT },
+
+   { "Debug",
+     GC_OPT_FLAG_GROUP, GC_LEVEL_ADVANCED,
+     "gnupg", N_("Options useful for debugging") },
+   { "debug-level", GC_OPT_FLAG_ARG_OPT|GC_OPT_FLAG_RUNTIME, GC_LEVEL_ADVANCED,
+     "gnupg", "|LEVEL|set the debugging level to LEVEL",
+     GC_ARG_TYPE_STRING, GC_BACKEND_GPG_AGENT },
+   { "log-file", GC_OPT_FLAG_RUNTIME, GC_LEVEL_ADVANCED,
+     "gnupg", N_("|FILE|write server mode logs to FILE"),
+     GC_ARG_TYPE_PATHNAME, GC_BACKEND_GPG_AGENT },
+   { "faked-system-time", GC_OPT_FLAG_NONE, GC_LEVEL_INVISIBLE,
+     NULL, NULL,
+     GC_ARG_TYPE_UINT32, GC_BACKEND_GPG_AGENT },
+
+   { "Security",
+     GC_OPT_FLAG_GROUP, GC_LEVEL_BASIC,
+     "gnupg", N_("Options controlling the security") },
+   { "default-cache-ttl", GC_OPT_FLAG_RUNTIME, GC_LEVEL_BASIC,
+     "gnupg", "|N|expire cached PINs after N seconds",
+     GC_ARG_TYPE_UINT32, GC_BACKEND_GPG_AGENT },
+   { "ignore-cache-for-signing", GC_OPT_FLAG_RUNTIME, GC_LEVEL_BASIC,
+     "gnupg", "do not use the PIN cache when signing",
+     GC_ARG_TYPE_NONE, GC_BACKEND_GPG_AGENT },
+   { "allow-mark-trusted", GC_OPT_FLAG_RUNTIME, GC_LEVEL_ADVANCED,
+     "gnupg", "allow clients to mark keys as \"trusted\"",
+     GC_ARG_TYPE_NONE, GC_BACKEND_GPG_AGENT },
+   { "no-grab", GC_OPT_FLAG_RUNTIME, GC_LEVEL_EXPERT,
+     "gnupg", "do not grab keyboard and mouse",
+     GC_ARG_TYPE_NONE, GC_BACKEND_GPG_AGENT },
+
+
    GC_OPTION_NULL
  };
 
@@ -410,13 +487,183 @@ static gc_option_t gc_options_gpg_agent[] =
 /* The options of the GC_COMPONENT_SCDAEMON component.  */
 static gc_option_t gc_options_scdaemon[] =
  {
+   /* The configuration file to which we write the changes.  */
+   { "gpgconf-scdaemon.conf", GC_OPT_FLAG_NONE, GC_LEVEL_INTERNAL,
+     NULL, NULL, GC_ARG_TYPE_PATHNAME, GC_BACKEND_SCDAEMON },
+
+   { "Monitor",
+     GC_OPT_FLAG_GROUP, GC_LEVEL_BASIC,
+     "gnupg", N_("Options controlling the diagnostic output") },
+   { "verbose", GC_OPT_FLAG_LIST, GC_LEVEL_BASIC,
+     "gnupg", "verbose",
+     GC_ARG_TYPE_NONE, GC_BACKEND_SCDAEMON },
+   { "quiet", GC_OPT_FLAG_NONE, GC_LEVEL_BASIC,
+     "gnupg", "be somewhat more quiet",
+     GC_ARG_TYPE_NONE, GC_BACKEND_SCDAEMON },
+   { "no-greeting", GC_OPT_FLAG_NONE, GC_LEVEL_INVISIBLE,
+     NULL, NULL,
+     GC_ARG_TYPE_NONE, GC_BACKEND_SCDAEMON },
+
+   { "Configuration",
+     GC_OPT_FLAG_GROUP, GC_LEVEL_EXPERT,
+     "gnupg", N_("Options controlling the configuration") },
+   { "options", GC_OPT_FLAG_NONE, GC_LEVEL_EXPERT,
+     "gnupg", "|FILE|read options from FILE",
+     GC_ARG_TYPE_PATHNAME, GC_BACKEND_SCDAEMON },
+   { "reader-port", GC_OPT_FLAG_NONE, GC_LEVEL_BASIC,
+     "gnupg", "|N|connect to reader at port N",
+     GC_ARG_TYPE_STRING, GC_BACKEND_SCDAEMON },
+   { "ctapi-driver", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
+     "gnupg", "|NAME|use NAME as ct-API driver",
+     GC_ARG_TYPE_STRING, GC_BACKEND_SCDAEMON },
+   { "pcsc-driver", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
+     "gnupg", "|NAME|use NAME as PC/SC driver",
+     GC_ARG_TYPE_STRING, GC_BACKEND_SCDAEMON },
+   { "disable-opensc", GC_OPT_FLAG_NONE, GC_LEVEL_EXPERT,
+     "gnupg", "do not use the OpenSC layer",
+     GC_ARG_TYPE_NONE, GC_BACKEND_SCDAEMON },
+   { "disable-ccid", GC_OPT_FLAG_NONE, GC_LEVEL_EXPERT,
+     "gnupg", "do not use the internal CCID driver",
+     GC_ARG_TYPE_NONE, GC_BACKEND_SCDAEMON },
+
+
+   { "Debug",
+     GC_OPT_FLAG_GROUP, GC_LEVEL_ADVANCED,
+     "gnupg", N_("Options useful for debugging") },
+   { "debug-level", GC_OPT_FLAG_ARG_OPT, GC_LEVEL_ADVANCED,
+     "gnupg", "|LEVEL|set the debugging level to LEVEL",
+     GC_ARG_TYPE_STRING, GC_BACKEND_SCDAEMON },
+   { "log-file", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
+     "gnupg", N_("|FILE|write server mode logs to FILE"),
+     GC_ARG_TYPE_PATHNAME, GC_BACKEND_SCDAEMON },
+
+   { "Security",
+     GC_OPT_FLAG_GROUP, GC_LEVEL_BASIC,
+     "gnupg", N_("Options controlling the security") },
+   { "allow-admin", GC_OPT_FLAG_NONE, GC_LEVEL_BASIC,
+     "gnupg", "allow the use of admin card commands",
+     GC_ARG_TYPE_NONE, GC_BACKEND_SCDAEMON },
+
+
    GC_OPTION_NULL
  };
 
 
+/* The options of the GC_COMPONENT_GPG component.  */
+static gc_option_t gc_options_gpg[] =
+ {
+   /* The configuration file to which we write the changes.  */
+   { "gpgconf-gpg.conf", GC_OPT_FLAG_NONE, GC_LEVEL_INTERNAL,
+     NULL, NULL, GC_ARG_TYPE_PATHNAME, GC_BACKEND_GPG },
+
+   { "Monitor",
+     GC_OPT_FLAG_GROUP, GC_LEVEL_BASIC,
+     "gnupg", N_("Options controlling the diagnostic output") },
+   { "verbose", GC_OPT_FLAG_LIST, GC_LEVEL_BASIC,
+     "gnupg", "verbose",
+     GC_ARG_TYPE_NONE, GC_BACKEND_GPG },
+   { "quiet", GC_OPT_FLAG_NONE, GC_LEVEL_BASIC,
+     "gnupg", "be somewhat more quiet",
+     GC_ARG_TYPE_NONE, GC_BACKEND_GPG },
+   { "no-greeting", GC_OPT_FLAG_NONE, GC_LEVEL_INVISIBLE,
+     NULL, NULL,
+     GC_ARG_TYPE_NONE, GC_BACKEND_GPG },
+
+   { "Configuration",
+     GC_OPT_FLAG_GROUP, GC_LEVEL_EXPERT,
+     "gnupg", N_("Options controlling the configuration") },
+   { "options", GC_OPT_FLAG_NONE, GC_LEVEL_EXPERT,
+     "gnupg", "|FILE|read options from FILE",
+     GC_ARG_TYPE_PATHNAME, GC_BACKEND_GPG },
+
+   { "Debug",
+     GC_OPT_FLAG_GROUP, GC_LEVEL_ADVANCED,
+     "gnupg", N_("Options useful for debugging") },
+   { "debug-level", GC_OPT_FLAG_ARG_OPT, GC_LEVEL_ADVANCED,
+     "gnupg", "|LEVEL|set the debugging level to LEVEL",
+     GC_ARG_TYPE_STRING, GC_BACKEND_GPG },
+   { "log-file", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
+     "gnupg", N_("|FILE|write server mode logs to FILE"),
+     GC_ARG_TYPE_PATHNAME, GC_BACKEND_GPG },
+/*    { "faked-system-time", GC_OPT_FLAG_NONE, GC_LEVEL_INVISIBLE, */
+/*      NULL, NULL, */
+/*      GC_ARG_TYPE_UINT32, GC_BACKEND_GPG }, */
+
+   { "Keyserver",
+     GC_OPT_FLAG_GROUP, GC_LEVEL_BASIC,
+     "gnupg", N_("Configuration for Keyservers") },
+   { "keyserver", GC_OPT_FLAG_NONE, GC_LEVEL_BASIC,
+     "gnupg", "|URL|use keyserver at URL",
+     GC_ARG_TYPE_STRING, GC_BACKEND_GPG },
+
+
+   GC_OPTION_NULL
+ };
+
+
+
 /* The options of the GC_COMPONENT_GPGSM component.  */
 static gc_option_t gc_options_gpgsm[] =
  {
+   /* The configuration file to which we write the changes.  */
+   { "gpgconf-gpgsm.conf", GC_OPT_FLAG_NONE, GC_LEVEL_INTERNAL,
+     NULL, NULL, GC_ARG_TYPE_PATHNAME, GC_BACKEND_GPGSM },
+
+   { "Monitor",
+     GC_OPT_FLAG_GROUP, GC_LEVEL_BASIC,
+     "gnupg", N_("Options controlling the diagnostic output") },
+   { "verbose", GC_OPT_FLAG_LIST, GC_LEVEL_BASIC,
+     "gnupg", "verbose",
+     GC_ARG_TYPE_NONE, GC_BACKEND_GPGSM },
+   { "quiet", GC_OPT_FLAG_NONE, GC_LEVEL_BASIC,
+     "gnupg", "be somewhat more quiet",
+     GC_ARG_TYPE_NONE, GC_BACKEND_GPGSM },
+   { "no-greeting", GC_OPT_FLAG_NONE, GC_LEVEL_INVISIBLE,
+     NULL, NULL,
+     GC_ARG_TYPE_NONE, GC_BACKEND_GPGSM },
+
+   { "Configuration",
+     GC_OPT_FLAG_GROUP, GC_LEVEL_EXPERT,
+     "gnupg", N_("Options controlling the configuration") },
+   { "options", GC_OPT_FLAG_NONE, GC_LEVEL_EXPERT,
+     "gnupg", "|FILE|read options from FILE",
+     GC_ARG_TYPE_PATHNAME, GC_BACKEND_GPGSM },
+   { "prefer-system-dirmngr", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
+     "gnupg", "use system's dirmngr if available",
+     GC_ARG_TYPE_NONE, GC_BACKEND_GPGSM },
+
+   { "Debug",
+     GC_OPT_FLAG_GROUP, GC_LEVEL_ADVANCED,
+     "gnupg", N_("Options useful for debugging") },
+   { "debug-level", GC_OPT_FLAG_ARG_OPT, GC_LEVEL_ADVANCED,
+     "gnupg", "|LEVEL|set the debugging level to LEVEL",
+     GC_ARG_TYPE_STRING, GC_BACKEND_GPGSM },
+   { "log-file", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
+     "gnupg", N_("|FILE|write server mode logs to FILE"),
+     GC_ARG_TYPE_PATHNAME, GC_BACKEND_GPGSM },
+   { "faked-system-time", GC_OPT_FLAG_NONE, GC_LEVEL_INVISIBLE,
+     NULL, NULL,
+     GC_ARG_TYPE_UINT32, GC_BACKEND_GPGSM },
+
+   { "Security",
+     GC_OPT_FLAG_GROUP, GC_LEVEL_BASIC,
+     "gnupg", N_("Options controlling the security") },
+   { "disable-crl-checks", GC_OPT_FLAG_NONE, GC_LEVEL_BASIC,
+     "gnupg", "never consult a CRL",
+     GC_ARG_TYPE_NONE, GC_BACKEND_GPGSM },
+   { "enable-ocsp", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
+     "gnupg", "check validity using OCSP",
+     GC_ARG_TYPE_NONE, GC_BACKEND_GPGSM },
+   { "include-certs", GC_OPT_FLAG_NONE, GC_LEVEL_EXPERT,
+     "gnupg", "|N|number of certificates to include",
+     GC_ARG_TYPE_INT32, GC_BACKEND_GPGSM },
+   { "disable-policy-checks", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
+     "gnupg", "do not check certificate policies",
+     GC_ARG_TYPE_NONE, GC_BACKEND_GPGSM },
+   { "auto-issuer-key-retrieve", GC_OPT_FLAG_NONE, GC_LEVEL_BASIC,
+     "gnupg", "fetch missing issuer certificates",
+     GC_ARG_TYPE_NONE, GC_BACKEND_GPGSM },
+
    GC_OPTION_NULL
  };
 
@@ -430,7 +677,7 @@ static gc_option_t gc_options_dirmngr[] =
 
    { "Monitor",
      GC_OPT_FLAG_GROUP, GC_LEVEL_BASIC,
-     NULL, "Options controlling the diagnostic output" },
+     "gnupg", N_("Options controlling the diagnostic output") },
    { "verbose", GC_OPT_FLAG_LIST, GC_LEVEL_BASIC,
      "dirmngr", "verbose",
      GC_ARG_TYPE_NONE, GC_BACKEND_DIRMNGR },
@@ -443,7 +690,7 @@ static gc_option_t gc_options_dirmngr[] =
 
    { "Format",
      GC_OPT_FLAG_GROUP, GC_LEVEL_BASIC,
-     NULL, "Options controlling the format of the output" },
+     "gnupg", N_("Options controlling the format of the output") },
    { "sh", GC_OPT_FLAG_NONE, GC_LEVEL_BASIC,
      "dirmngr", "sh-style command output",
      GC_ARG_TYPE_NONE, GC_BACKEND_DIRMNGR },
@@ -453,25 +700,22 @@ static gc_option_t gc_options_dirmngr[] =
    
    { "Configuration",
      GC_OPT_FLAG_GROUP, GC_LEVEL_EXPERT,
-     NULL, "Options controlling the configuration" },
+     "gnupg", N_("Options controlling the configuration") },
    { "options", GC_OPT_FLAG_NONE, GC_LEVEL_EXPERT,
      "dirmngr", "|FILE|read options from FILE",
      GC_ARG_TYPE_PATHNAME, GC_BACKEND_DIRMNGR },
 
    { "Debug",
      GC_OPT_FLAG_GROUP, GC_LEVEL_ADVANCED,
-     "dirmngr", "Options useful for debugging" },
-   { "debug", GC_OPT_FLAG_ARG_OPT, GC_LEVEL_ADVANCED,
-     "dirmngr", "|FLAGS|set the debugging FLAGS",
-     GC_ARG_TYPE_UINT32, GC_BACKEND_DIRMNGR },
-   { "debug-all", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
-     "dirmngr", "set all debugging flags",
-     GC_ARG_TYPE_NONE, GC_BACKEND_DIRMNGR },
+     "gnupg", N_("Options useful for debugging") },
+   { "debug-level", GC_OPT_FLAG_ARG_OPT, GC_LEVEL_ADVANCED,
+     "dirmngr", "|LEVEL|set the debugging level to LEVEL",
+     GC_ARG_TYPE_STRING, GC_BACKEND_DIRMNGR },
    { "no-detach", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
      "dirmngr", "do not detach from the console",
      GC_ARG_TYPE_NONE, GC_BACKEND_DIRMNGR },
    { "log-file", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
-     "dirmngr", "|FILE|write logs to FILE",
+     "dirmngr", N_("|FILE|write server mode logs to FILE"),
      GC_ARG_TYPE_PATHNAME, GC_BACKEND_DIRMNGR },
    { "debug-wait", GC_OPT_FLAG_NONE, GC_LEVEL_INVISIBLE,
      NULL, NULL,
@@ -482,7 +726,7 @@ static gc_option_t gc_options_dirmngr[] =
 
    { "Enforcement",
      GC_OPT_FLAG_GROUP, GC_LEVEL_BASIC,
-     NULL, "Options controlling the interactivity and enforcement" },
+     "gnupg", N_("Options controlling the interactivity and enforcement") },
    { "batch", GC_OPT_FLAG_NONE, GC_LEVEL_BASIC,
      "dirmngr", "run without asking a user",
      GC_ARG_TYPE_NONE, GC_BACKEND_DIRMNGR },
@@ -490,10 +734,35 @@ static gc_option_t gc_options_dirmngr[] =
      "dirmngr", "force loading of outdated CRLs",
      GC_ARG_TYPE_NONE, GC_BACKEND_DIRMNGR },
 
+   { "HTTP",
+     GC_OPT_FLAG_GROUP, GC_LEVEL_ADVANCED,
+     "gnupg", N_("Configuration for HTTP servers") },
+   { "disable-http", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
+     "dirmngr", "inhibit the use of HTTP",
+      GC_ARG_TYPE_NONE, GC_BACKEND_DIRMNGR },
+   { "ignore-http-dp", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
+     "dirmngr", "ignore HTTP CRL distribution points",
+      GC_ARG_TYPE_NONE, GC_BACKEND_DIRMNGR },
+   { "http-proxy", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
+     "dirmngr", "|URL|redirect all HTTP requests to URL",
+     GC_ARG_TYPE_STRING, GC_BACKEND_DIRMNGR },
+
    { "LDAP",
      GC_OPT_FLAG_GROUP, GC_LEVEL_BASIC,
-     NULL, "Configuration of LDAP servers to use" },
-   { "add-servers", GC_OPT_FLAG_NONE, GC_LEVEL_BASIC,
+     "gnupg", N_("Configuration of LDAP servers to use") },
+   { "disable-ldap", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
+     "dirmngr", "inhibit the use of LDAP",
+      GC_ARG_TYPE_NONE, GC_BACKEND_DIRMNGR },
+   { "ignore-ldap-dp", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
+     "dirmngr", "ignore LDAP CRL distribution points",
+      GC_ARG_TYPE_NONE, GC_BACKEND_DIRMNGR },
+   { "ldap-proxy", GC_OPT_FLAG_NONE, GC_LEVEL_BASIC,
+     "dirmngr", "|HOST|use HOST for LDAP queries",
+     GC_ARG_TYPE_STRING, GC_BACKEND_DIRMNGR },
+   { "only-ldap-proxy", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
+     "dirmngr", "do not use fallback hosts with --ldap-proxy",
+      GC_ARG_TYPE_NONE, GC_BACKEND_DIRMNGR },
+   { "add-servers", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
      "dirmngr", "add new servers discovered in CRL distribution points"
      " to serverlist", GC_ARG_TYPE_NONE, GC_BACKEND_DIRMNGR },
    { "ldaptimeout", GC_OPT_FLAG_NONE, GC_LEVEL_BASIC,
@@ -508,17 +777,30 @@ static gc_option_t gc_options_dirmngr[] =
    /* This entry must come after at least one entry for
       GC_BACKEND_DIRMNGR in this component, so that the entry for
       "ldapserverlist-file will be initialized before this one.  */
-   { "LDAP Server", GC_OPT_FLAG_LIST, GC_LEVEL_BASIC,
+   { "LDAP Server", GC_OPT_FLAG_ARG_OPT|GC_OPT_FLAG_LIST, GC_LEVEL_BASIC,
      NULL, "LDAP server list",
      GC_ARG_TYPE_LDAP_SERVER, GC_BACKEND_DIRMNGR_LDAP_SERVER_LIST },
-
-   { "CRL",
-     GC_OPT_FLAG_GROUP, GC_LEVEL_BASIC,
-     NULL, "Configuration of the CRL" },
    { "max-replies", GC_OPT_FLAG_NONE, GC_LEVEL_BASIC,
      "dirmngr", "|N|do not return more than N items in one query",
      GC_ARG_TYPE_UINT32, GC_BACKEND_DIRMNGR },
 
+   { "OCSP",
+     GC_OPT_FLAG_GROUP, GC_LEVEL_ADVANCED,
+     "gnupg", N_("Configuration for OCSP") },
+   { "allow-ocsp", GC_OPT_FLAG_NONE, GC_LEVEL_BASIC,
+     "dirmngr", "allow sending OCSP requests",
+     GC_ARG_TYPE_NONE, GC_BACKEND_DIRMNGR },
+   { "ignore-ocsp-service-url", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
+     "dirmngr", "ignore certificate contained OCSP service URLs",
+      GC_ARG_TYPE_NONE, GC_BACKEND_DIRMNGR },
+   { "ocsp-responder", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
+     "dirmngr", "|URL|use OCSP responder at URL",
+     GC_ARG_TYPE_STRING, GC_BACKEND_DIRMNGR },
+   { "ocsp-signer", GC_OPT_FLAG_NONE, GC_LEVEL_ADVANCED,
+     "dirmngr", "|FPR|OCSP response signed by FPR",
+     GC_ARG_TYPE_STRING, GC_BACKEND_DIRMNGR },
+
+
    GC_OPTION_NULL
  };
 
@@ -528,6 +810,9 @@ static gc_option_t gc_options_dirmngr[] =
    update GC_COMPONENT below.  */
 typedef enum
   {
+    /* The classic GPG for OpenPGP.  */
+    GC_COMPONENT_GPG,
+
     /* The GPG Agent.  */
     GC_COMPONENT_GPG_AGENT,
 
@@ -564,23 +849,86 @@ static struct
   gc_option_t *options;
 } gc_component[] =
   {
+    { "gpg", NULL,   "GPG for OpenPGP", gc_options_gpg },
     { "gpg-agent", NULL, "GPG Agent", gc_options_gpg_agent },
     { "scdaemon", NULL, "Smartcard Daemon", gc_options_scdaemon },
     { "gpgsm", NULL, "GPG for S/MIME", gc_options_gpgsm },
-    { "dirmngr", NULL, "CRL Manager", gc_options_dirmngr }
+    { "dirmngr", NULL, "Directory Manager", gc_options_dirmngr }
   };
 
 
-/* Robust version of dgettext.  */
+/* Engine specific support.  */
+void
+gpg_agent_runtime_change (void)
+{
+#ifndef HAVE_W32_SYSTEM
+  char *agent = getenv ("GPG_AGENT_INFO");
+  char *pid_str;
+  unsigned long pid_long;
+  char *tail;
+  pid_t pid;
+
+  if (!agent)
+    return;
+
+  pid_str = strchr (agent, ':');
+  if (!pid_str)
+    return;
+
+  pid_str++;
+  errno = 0;
+  pid_long = strtoul (pid_str, &tail, 0);
+  if (errno || (*tail != ':' && *tail != '\0'))
+    return;
+
+  pid = (pid_t) pid_long;
+
+  /* Check for overflow.  */
+  if (pid_long != (unsigned long) pid)
+    return;
+
+  /* Ignore any errors here.  */
+  kill (pid, SIGHUP);
+#endif /*!HAVE_W32_SYSTEM*/
+}
+
+
+/* More or less Robust version of dgettext.  It has the side effect of
+   switching the codeset to utf-8 because this is what we want to
+   output.  In theory it is posible to keep the orginal code set and
+   switch back for regular disgnostic output (redefine "_(" for that)
+   but given the natur of this tool, being something invoked from
+   other pograms, it does not make much sense.  */
 static const char *
 my_dgettext (const char *domain, const char *msgid)
 {
+#ifdef ENABLE_NLS
   if (domain)
     {
-      char *text = dgettext (domain, msgid);
+      static int switched_codeset;
+      char *text;
+      
+      if (!switched_codeset)
+        {
+          switched_codeset = 1;
+          bind_textdomain_codeset (PACKAGE_GT, "utf-8");
+
+          bindtextdomain ("dirmngr", LOCALEDIR);
+          bind_textdomain_codeset ("dirmngr", "utf-8");
+   
+        }
+
+      /* Note: This is a hack to actually use the gnupg2 domain as
+         long we are in a transition phase where gnupg 1.x and 1.9 may
+         coexist. */
+      if (!strcmp (domain, "gnupg"))
+        domain = PACKAGE_GT;
+
+      text = dgettext (domain, msgid);
       return text ? text : msgid;
     }
   else
+#endif
     return msgid;
 }
 
@@ -924,7 +1272,13 @@ get_config_pathname (gc_component_t component, gc_backend_t backend)
   else
     pathname = "";
 
+#ifdef HAVE_DOSISH_SYSTEM
+  if (!(pathname[0] 
+        && pathname[1] == ':'
+        && (pathname[2] == '/' || pathname[2] == '\\')))
+#else
   if (pathname[0] != '/')
+#endif
     gc_error (1, 0, "Option %s, needed by backend %s, is not absolute",
 	      gc_backend[backend].option_config_filename,
 	      gc_backend[backend].name);
@@ -951,7 +1305,7 @@ retrieve_options_from_program (gc_component_t component, gc_backend_t backend)
   if (!config)
     gc_error (1, errno, "could not gather active options from %s", cmd_line);
 
-  while ((length = getline (&line, &line_len, config)) > 0)
+  while ((length = read_line (config, &line, &line_len, NULL)) > 0)
     {
       gc_option_t *option;
       char *linep;
@@ -1018,7 +1372,7 @@ retrieve_options_from_program (gc_component_t component, gc_backend_t backend)
 	    option->default_value = xstrdup (default_value);
 	}
     }
-  if (ferror (config))
+  if (length < 0 || ferror (config))
     gc_error (1, errno, "error reading from %s", cmd_line);
   if (fclose (config) && ferror (config))
     gc_error (1, errno, "error closing %s", cmd_line);
@@ -1033,7 +1387,7 @@ retrieve_options_from_program (gc_component_t component, gc_backend_t backend)
 	      config_pathname);
   else
     {
-      while ((length = getline (&line, &line_len, config)) > 0)
+      while ((length = read_line (config, &line, &line_len, NULL)) > 0)
 	{
 	  char *name;
 	  char *value;
@@ -1114,14 +1468,13 @@ retrieve_options_from_program (gc_component_t component, gc_backend_t backend)
 	    }
 	}
 
-      if (ferror (config))
+      if (length < 0 || ferror (config))
 	gc_error (1, errno, "error reading from %s", config_pathname);
       if (fclose (config) && ferror (config))
 	gc_error (1, errno, "error closing %s", config_pathname);
     }
 
-  if (line)
-    free (line);
+  xfree (line);
 }
 
 
@@ -1150,7 +1503,7 @@ retrieve_options_from_file (gc_component_t component, gc_backend_t backend)
   else
     {
 
-      while ((length = getline (&line, &line_len, list_file)) > 0)
+      while ((length = read_line (list_file, &line, &line_len, NULL)) > 0)
 	{
 	  char *start;
 	  char *end;
@@ -1172,26 +1525,25 @@ retrieve_options_from_file (gc_component_t component, gc_backend_t backend)
 	  while (*(end - 1) == ' ' || *(end - 1) == '\t')
 	    end--;
 	  *end = '\0';
-	  /* FIXME: Oh, no!  This is so lame!  Use realloc and really
-	     append.  */
+	  /* FIXME: Oh, no!  This is so lame!  Should use realloc and
+	     really append.  */
 	  if (list)
 	    {
-	      new_list = xasprintf ("%s,%s", list, percent_escape (start));
+	      new_list = xasprintf ("%s,\"%s", list, percent_escape (start));
 	      xfree (list);
 	      list = new_list;
 	    }
 	  else
 	    list = xasprintf ("\"%s", percent_escape (start));
 	}
-      if (ferror (list_file))
+      if (length < 0 || ferror (list_file))
 	gc_error (1, errno, "can not read list file %s", list_pathname);
     }
 
   list_option->active = 1;
   list_option->value = list;
 
-  if (line)
-    free (line);
+  xfree (line);
 }
 
 
@@ -1367,7 +1719,6 @@ change_options_file (gc_component_t component, gc_backend_t backend,
   assert (option);
   assert (option->active);
   assert (gc_arg_type[option->arg_type].fallback != GC_ARG_TYPE_NONE);
-  assert (!(option->flags & GC_OPT_FLAG_ARG_OPT));
 
   /* FIXME.  Throughout the function, do better error reporting.  */
   /* Note that get_config_pathname() calls percent_deescape(), so we
@@ -1377,7 +1728,9 @@ change_options_file (gc_component_t component, gc_backend_t backend,
   orig_filename = xasprintf ("%s.gpgconf.%i.bak", dest_filename, getpid ());
 
   arg = option->new_value;
-  if (arg)
+  if (arg && arg[0] == '\0')
+    arg = NULL;
+  else if (arg)
     {
       char *end;
 
@@ -1396,7 +1749,12 @@ change_options_file (gc_component_t component, gc_backend_t backend,
 	arg = NULL;
     }
 
+#if HAVE_W32_SYSTEM
+  res = 0; 
+#warning no backups for W32 yet - need to write a copy function
+#else
   res = link (dest_filename, orig_filename);
+#endif
   if (res < 0 && errno != ENOENT)
     return -1;
   if (res < 0)
@@ -1433,7 +1791,7 @@ change_options_file (gc_component_t component, gc_backend_t backend,
       if (!dest_file)
 	goto change_file_one_err;
 
-      while ((length = getline (&line, &line_len, dest_file)) > 0)
+      while ((length = read_line (dest_file, &line, &line_len, NULL)) > 0)
 	{
 	  int disable = 0;
 	  char *start;
@@ -1521,7 +1879,7 @@ change_options_file (gc_component_t component, gc_backend_t backend,
 		goto change_file_one_err;
 	    }
 	}
-      if (ferror (dest_file))
+      if (length < 0 || ferror (dest_file))
 	goto change_file_one_err;
     }
 
@@ -1587,17 +1945,18 @@ change_options_file (gc_component_t component, gc_backend_t backend,
     }
   if (dest_file)
     {
-      while ((length = getline (&line, &line_len, dest_file)) > 0)
+      while ((length = read_line (dest_file, &line, &line_len, NULL)) > 0)
 	{
 	  fprintf (src_file, "%s", line);
 	  if (ferror (src_file))
 	    goto change_file_one_err;
 	}
-      if (ferror (dest_file))
+      if (length < 0 || ferror (dest_file))
 	goto change_file_one_err;
     }
-  if (line)
-    free (line);
+  xfree (line);
+  line = NULL;
+
   res = fclose (src_file);
   if (res)
     {
@@ -1618,8 +1977,7 @@ change_options_file (gc_component_t component, gc_backend_t backend,
   return 0;
 
  change_file_one_err:
-  if (line)
-    free (line);
+  xfree (line);
   res = errno;
   if (src_file)
     {
@@ -1660,7 +2018,12 @@ change_options_program (gc_component_t component, gc_backend_t backend,
   src_filename = xasprintf ("%s.gpgconf.%i.new", dest_filename, getpid ());
   orig_filename = xasprintf ("%s.gpgconf.%i.bak", dest_filename, getpid ());
 
+#if HAVE_W32_SYSTEM
+  res = 0; 
+#warning no backups for W32 yet - need to write a copy function
+#else
   res = link (dest_filename, orig_filename);
+#endif
   if (res < 0 && errno != ENOENT)
     return -1;
   if (res < 0)
@@ -1697,7 +2060,7 @@ change_options_program (gc_component_t component, gc_backend_t backend,
       if (!dest_file)
 	goto change_one_err;
 
-      while ((length = getline (&line, &line_len, dest_file)) > 0)
+      while ((length = read_line (dest_file, &line, &line_len, NULL)) > 0)
 	{
 	  int disable = 0;
 	  char *start;
@@ -1752,7 +2115,7 @@ change_options_program (gc_component_t component, gc_backend_t backend,
 		goto change_one_err;
 	    }
 	}
-      if (ferror (dest_file))
+      if (length < 0 || ferror (dest_file))
 	goto change_one_err;
     }
 
@@ -1772,6 +2135,11 @@ change_options_program (gc_component_t component, gc_backend_t backend,
      Now, dump the changed options (except for those we are going to
      revert to their default), and write the end marker, possibly
      followed by the rest of the original file.  */
+
+  /* We have to turn on UTF8 strings for GnuPG.  */
+  if (backend == GC_BACKEND_GPG)
+    fprintf (src_file, "utf8-strings\n");
+
   option = gc_component[component].options;
   while (option->name)
     {
@@ -1865,17 +2233,18 @@ change_options_program (gc_component_t component, gc_backend_t backend,
     }
   if (dest_file)
     {
-      while ((length = getline (&line, &line_len, dest_file)) > 0)
+      while ((length = read_line (dest_file, &line, &line_len, NULL)) > 0)
 	{
 	  fprintf (src_file, "%s", line);
 	  if (ferror (src_file))
 	    goto change_one_err;
 	}
-      if (ferror (dest_file))
+      if (length < 0 || ferror (dest_file))
 	goto change_one_err;
     }
-  if (line)
-    free (line);
+  xfree (line);
+  line = NULL;
+
   res = fclose (src_file);
   if (res)
     {
@@ -1896,8 +2265,7 @@ change_options_program (gc_component_t component, gc_backend_t backend,
   return 0;
 
  change_one_err:
-  if (line)
-    free (line);
+  xfree (line);
   res = errno;
   if (src_file)
     {
@@ -1916,6 +2284,7 @@ void
 gc_component_change_options (int component, FILE *in)
 {
   int err = 0;
+  int runtime[GC_BACKEND_NR];
   char *src_pathname[GC_BACKEND_NR];
   char *dest_pathname[GC_BACKEND_NR];
   char *orig_pathname[GC_BACKEND_NR];
@@ -1927,12 +2296,13 @@ gc_component_change_options (int component, FILE *in)
 
   for (backend = 0; backend < GC_BACKEND_NR; backend++)
     {
+      runtime[backend] = 0;
       src_pathname[backend] = NULL;
       dest_pathname[backend] = NULL;
       orig_pathname[backend] = NULL;
     }
 
-  while ((length = getline (&line, &line_len, in)) > 0)
+  while ((length = read_line (in, &line, &line_len, NULL)) > 0)
     {
       char *linep;
       unsigned long flags = 0;
@@ -1989,6 +2359,9 @@ gc_component_change_options (int component, FILE *in)
 
       option_check_validity (option, flags, new_value, &new_value_nr);
 
+      if (option->flags & GC_OPT_FLAG_RUNTIME)
+	runtime[option->backend] = 1;
+
       option->new_flags = flags;
       if (!(flags & GC_OPT_FLAG_DEFAULT))
 	{
@@ -2023,7 +2396,8 @@ gc_component_change_options (int component, FILE *in)
     {
       /* Go on if we have already seen this backend, or if there is
 	 nothing to do.  */
-      if (src_pathname[option->backend] || !(option->new_flags || option->new_value))
+      if (src_pathname[option->backend]
+	  || !(option->new_flags || option->new_value))
 	{
 	  option++;
 	  continue;
@@ -2031,14 +2405,14 @@ gc_component_change_options (int component, FILE *in)
 
       if (gc_backend[option->backend].program)
 	err = change_options_program (component, option->backend,
-				      &src_pathname[component],
-				      &dest_pathname[component],
-				      &orig_pathname[component]);
+				      &src_pathname[option->backend],
+				      &dest_pathname[option->backend],
+				      &orig_pathname[option->backend]);
       else
 	err = change_options_file (component, option->backend,
-				   &src_pathname[component],
-				   &dest_pathname[component],
-				   &orig_pathname[component]);
+				   &src_pathname[option->backend],
+				   &dest_pathname[option->backend],
+				   &orig_pathname[option->backend]);
 	
       if (err)
 	break;
@@ -2050,7 +2424,7 @@ gc_component_change_options (int component, FILE *in)
     {
       int i;
 
-      for (i = 0; i < GC_COMPONENT_NR; i++)
+      for (i = 0; i < GC_BACKEND_NR; i++)
 	{
 	  if (src_pathname[i])
 	    {
@@ -2062,12 +2436,18 @@ gc_component_change_options (int component, FILE *in)
 		err = rename (src_pathname[i], dest_pathname[i]);
 	      else
 		{
+#ifdef HAVE_W32_SYSTEM
+                  /* FIXME: Won't work becuase W32 doesn't silently
+                     overwrite. */
+                  err = rename (src_pathname[i], dest_pathname[i]);
+#else /*!HAVE_W32_SYSTEM*/
 		  /* This is a bit safer than rename() because we
 		     expect DEST_PATHNAME not to be there.  If it
 		     happens to be there, this will fail.  */
 		  err = link (src_pathname[i], dest_pathname[i]);
 		  if (!err)
 		    unlink (src_pathname[i]);
+#endif /*!HAVE_W32_SYSTEM*/
 		}
 	      if (err)
 		break;
@@ -2082,7 +2462,7 @@ gc_component_change_options (int component, FILE *in)
       int saved_errno = errno;
 
       /* An error occured.  */
-      for (i = 0; i < GC_COMPONENT_NR; i++)
+      for (i = 0; i < GC_BACKEND_NR; i++)
 	{
 	  if (src_pathname[i])
 	    {
@@ -2105,6 +2485,26 @@ gc_component_change_options (int component, FILE *in)
 	}
       gc_error (1, saved_errno, "could not commit changes");
     }
-  if (line)
-    free (line);
+
+  /* If it all worked, notify the daemons of the changes.  */
+  if (opt.runtime)
+    for (backend = 0; backend < GC_BACKEND_NR; backend++)  
+      {
+	if (runtime[backend] && gc_backend[backend].runtime_change)
+	  (*gc_backend[backend].runtime_change) ();
+      }
+
+  /* Move the per-process backup file into its place.  */
+  for (backend = 0; backend < GC_BACKEND_NR; backend++)  
+    if (orig_pathname[backend])
+      {
+	char *backup_pathname;
+
+	assert (dest_pathname[backend]);
+
+	backup_pathname = xasprintf ("%s.gpgconf.bak", dest_pathname[backend]);
+	rename (orig_pathname[backend], backup_pathname);
+      }
+
+  xfree (line);
 }
