@@ -1,12 +1,11 @@
 /* dotlock.c - dotfile locking
- * Copyright (C) 1998, 1999, 2000, 2001, 2004,
- *               2005 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2004 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,7 +14,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #include <config.h>
@@ -51,6 +51,8 @@ struct dotlock_handle {
 static volatile DOTLOCK all_lockfiles;
 static int never_lock;
 
+static int read_lockfile( const char *name );
+
 void
 disable_dotlock(void)
 {
@@ -78,14 +80,14 @@ create_dotlock( const char *file_to_lock )
 {
     static int initialized;
     DOTLOCK h;
-#if !defined (HAVE_DOSISH_SYSTEM)
-    int fd = -1;
+    int  fd = -1;
     char pidstr[16];
+#if !defined (HAVE_DOSISH_SYSTEM)
     struct utsname utsbuf;
+#endif
     const char *nodename;
     const char *dirpart;
     int dirpartlen;
-#endif
 
     if( !initialized ) {
 	atexit( remove_lockfiles );
@@ -94,7 +96,7 @@ create_dotlock( const char *file_to_lock )
     if( !file_to_lock )
 	return NULL;
 
-    h = xmalloc_clear( sizeof *h );
+    h = m_alloc_clear( sizeof *h );
     if( never_lock ) {
 	h->disable = 1;
 #ifdef _REENTRANT
@@ -140,13 +142,13 @@ create_dotlock( const char *file_to_lock )
     h->next = all_lockfiles;
     all_lockfiles = h;
 
-    h->tname = xmalloc( dirpartlen + 6+30+ strlen(nodename) + 11 );
+    h->tname = m_alloc( dirpartlen + 6+30+ strlen(nodename) + 11 );
 #ifndef __riscos__
     sprintf( h->tname, "%.*s/.#lk%p.%s.%d",
-	     dirpartlen, dirpart, (void *)h, nodename, (int)getpid() );
+	     dirpartlen, dirpart, h, nodename, (int)getpid() );
 #else /* __riscos__ */
     sprintf( h->tname, "%.*s.lk%p/%s/%d",
-	     dirpartlen, dirpart, (void *)h, nodename, (int)getpid() );
+	     dirpartlen, dirpart, h, nodename, (int)getpid() );
 #endif /* __riscos__ */
 
     do {
@@ -158,8 +160,8 @@ create_dotlock( const char *file_to_lock )
 	all_lockfiles = h->next;
 	log_error( "failed to create temporary file `%s': %s\n",
 					    h->tname, strerror(errno));
-	xfree(h->tname);
-	xfree(h);
+	m_free(h->tname);
+	m_free(h);
 	return NULL;
     }
     if( write(fd, pidstr, 11 ) != 11 ) {
@@ -170,8 +172,8 @@ create_dotlock( const char *file_to_lock )
 	log_fatal( "error writing to `%s': %s\n", h->tname, strerror(errno) );
 	close(fd);
 	unlink(h->tname);
-	xfree(h->tname);
-	xfree(h);
+	m_free(h->tname);
+	m_free(h);
 	return NULL;
     }
     if( close(fd) ) {
@@ -181,8 +183,8 @@ create_dotlock( const char *file_to_lock )
 #endif
 	log_error( "error closing `%s': %s\n", h->tname, strerror(errno));
 	unlink(h->tname);
-	xfree(h->tname);
-	xfree(h);
+	m_free(h->tname);
+	m_free(h);
 	return NULL;
     }
 
@@ -190,7 +192,7 @@ create_dotlock( const char *file_to_lock )
     /* release mutex */
 #endif
 #endif
-    h->lockname = xmalloc( strlen(file_to_lock) + 6 );
+    h->lockname = m_alloc( strlen(file_to_lock) + 6 );
     strcpy(stpcpy(h->lockname, file_to_lock), EXTSEP_S "lock");
     return h;
 }
@@ -223,16 +225,16 @@ destroy_dotlock ( DOTLOCK h )
               unlink (h->lockname);
             if (h->tname)
               unlink (h->tname);
-	    xfree (h->tname);
-	    xfree (h->lockname);
+	    m_free (h->tname);
+	    m_free (h->lockname);
           }
-	xfree(h);
+	m_free(h);
 
       }
 #endif
 }
 
-#ifndef HAVE_DOSISH_SYSTEM
+
 
 static int
 maybe_deadlock( DOTLOCK h )
@@ -246,43 +248,6 @@ maybe_deadlock( DOTLOCK h )
     return 0;
 }
 
-/****************
- * Read the lock file and return the pid, returns -1 on error.
- */
-static int
-read_lockfile( const char *name )
-{
-    int fd, pid;
-    char pidstr[16];
-
-    if( (fd = open(name, O_RDONLY)) == -1 ) {
-	int e = errno;
-	log_debug("error opening lockfile `%s': %s\n", name, strerror(errno) );
-	errno = e;
-	return -1;
-    }
-    if( read(fd, pidstr, 10 ) != 10 ) {  /* Read 10 digits w/o newline */
-	log_debug("error reading lockfile `%s'", name );
-	close(fd);
-	errno = 0;
-	return -1;
-    }
-    pidstr[10] = 0;  /* terminate pid string */
-    close(fd);
-    pid = atoi(pidstr);
-#ifndef __riscos__
-    if( !pid || pid == -1 ) {
-#else /* __riscos__ */
-    if( (!pid && riscos_getpid()) || pid == -1 ) {
-#endif /* __riscos__ */
-	log_error("invalid pid %d in lockfile `%s'", pid, name );
-	errno = 0;
-	return -1;
-    }
-    return pid;
-}
-#endif /* !HAVE_DOSISH_SYSTEM */
-
 /****************
  * Do a lock on H. A TIMEOUT of 0 returns immediately,
  * -1 waits forever (hopefully not), other
@@ -434,6 +399,48 @@ release_dotlock( DOTLOCK h )
 #endif
 }
 
+
+/****************
+ * Read the lock file and return the pid, returns -1 on error.
+ */
+static int
+read_lockfile( const char *name )
+{
+#if defined (HAVE_DOSISH_SYSTEM)
+    return 0;
+#else
+    int fd, pid;
+    char pidstr[16];
+
+    if( (fd = open(name, O_RDONLY)) == -1 ) {
+	int e = errno;
+	log_debug("error opening lockfile `%s': %s\n", name, strerror(errno) );
+	errno = e;
+	return -1;
+    }
+    if( read(fd, pidstr, 10 ) != 10 ) {  /* Read 10 digits w/o newline */
+	log_debug("error reading lockfile `%s'", name );
+	close(fd);
+	errno = 0;
+	return -1;
+    }
+    pidstr[10] = 0;  /* terminate pid string */
+    close(fd);
+    pid = atoi(pidstr);
+#ifndef __riscos__
+    if( !pid || pid == -1 ) {
+#else /* __riscos__ */
+    if( (!pid && riscos_getpid()) || pid == -1 ) {
+#endif /* __riscos__ */
+	log_error("invalid pid %d in lockfile `%s'", pid, name );
+	errno = 0;
+	return -1;
+    }
+    return pid;
+#endif
+}
+
+
 void
 remove_lockfiles()
 {
