@@ -1,12 +1,11 @@
 /* http.c  -  HTTP protocol handler
- * Copyright (C) 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
- *               2009, 2012 Free Software Foundation, Inc.
+ * Copyright (C) 1999, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,7 +14,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #include <config.h>
@@ -68,13 +68,12 @@ static int remove_escapes( byte *string );
 static int insert_escapes( byte *buffer, const byte *string,
 					 const byte *special );
 static URI_TUPLE parse_tuple( byte *string );
-static int send_request( HTTP_HD hd, const char *auth, const char *proxy,
-			 struct http_srv *srv, STRLIST headers);
+static int send_request( HTTP_HD hd, const char *proxy );
 static byte *build_rel_path( PARSED_URI uri );
 static int parse_response( HTTP_HD hd );
 
 static int connect_server( const char *server, ushort port, unsigned int flags,
-			   struct http_srv *srv );
+			   const char *srvtag );
 static int write_server( int sock, const char *data, size_t length );
 
 #ifdef _WIN32
@@ -94,7 +93,7 @@ init_sockets (void)
         return;
 
     if( WSAStartup( 0x0101, &wsdata ) ) {
-        log_error ("error initializing socket library: ec=%d\n",
+        log_error ("error initializing socket library: ec=%d\n", 
                     (int)WSAGetLastError () );
         return;
     }
@@ -117,14 +116,14 @@ static byte bintoasc[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
  * create a radix64 encoded string.
  */
 
-/* TODO: This is a duplicate of code in g10/armor.c modified to do the
-   "=" padding.  Better to use a single copy in strgutil.c ? */
+/* TODO: This is a duplicate of code in g10/armor.c.  Better to use a
+   single copy in strgutil.c */
 static char *
 make_radix64_string( const byte *data, size_t len )
 {
     char *buffer, *p;
 
-    buffer = p = xmalloc( (len+2)/3*4 + 1 );
+    buffer = p = m_alloc( (len+2)/3*4 + 1 );
     for( ; len >= 3 ; len -= 3, data += 3 ) {
 	*p++ = bintoasc[(data[0] >> 2) & 077];
 	*p++ = bintoasc[(((data[0] <<4)&060)|((data[1] >> 4)&017))&077];
@@ -135,13 +134,10 @@ make_radix64_string( const byte *data, size_t len )
 	*p++ = bintoasc[(data[0] >> 2) & 077];
 	*p++ = bintoasc[(((data[0] <<4)&060)|((data[1] >> 4)&017))&077];
 	*p++ = bintoasc[((data[1]<<2)&074)];
-	*p++ = '=';
     }
     else if( len == 1 ) {
 	*p++ = bintoasc[(data[0] >> 2) & 077];
 	*p++ = bintoasc[(data[0] <<4)&060];
-	*p++ = '=';
-	*p++ = '=';
     }
     *p = 0;
     return buffer;
@@ -149,8 +145,7 @@ make_radix64_string( const byte *data, size_t len )
 
 int
 http_open( HTTP_HD hd, HTTP_REQ_TYPE reqtype, const char *url,
-	   char *auth, unsigned int flags, const char *proxy,
-	   struct http_srv *srv, STRLIST headers )
+	   unsigned int flags, const char *proxy )
 {
     int rc;
 
@@ -166,7 +161,7 @@ http_open( HTTP_HD hd, HTTP_REQ_TYPE reqtype, const char *url,
 
     rc = parse_uri( &hd->uri, url );
     if( !rc ) {
-        rc = send_request( hd, auth, proxy, srv, headers );
+	rc = send_request( hd, proxy );
 	if( !rc ) {
 	    hd->fp_write = iobuf_sockopen( hd->sock , "w" );
 	    if( hd->fp_write )
@@ -205,19 +200,15 @@ http_wait_response( HTTP_HD hd, unsigned int *ret_status )
     http_start_data( hd ); /* make sure that we are in the data */
 
 #if 0
-    hd->sock = dup( hd->sock );
+    hd->sock = dup( hd->sock ); 
     if( hd->sock == -1 )
 	return G10ERR_GENERAL;
 #endif
     iobuf_ioctl (hd->fp_write, 1, 1, NULL); /* keep the socket open */
     iobuf_close (hd->fp_write);
     hd->fp_write = NULL;
-    /* We do not want the shutdown code anymore.  It used to be there
-       to support old versions of pksd.  These versions are anyway
-       unusable and the latest releases haven been fixed to properly
-       handle HTTP 1.0. */
-    /* if ( !(hd->flags & HTTP_FLAG_NO_SHUTDOWN) ) */
-    /*     shutdown( hd->sock, 1 ); */
+    if ( !(hd->flags & HTTP_FLAG_NO_SHUTDOWN) )
+        shutdown( hd->sock, 1 );
     hd->in_data = 0;
 
     hd->fp_read = iobuf_sockopen( hd->sock , "r" );
@@ -233,14 +224,12 @@ http_wait_response( HTTP_HD hd, unsigned int *ret_status )
 
 
 int
-http_open_document( HTTP_HD hd, const char *document, char *auth,
-		    unsigned int flags, const char *proxy, struct http_srv *srv,
-		    STRLIST headers )
+http_open_document( HTTP_HD hd, const char *document,
+		    unsigned int flags, const char *proxy )
 {
     int rc;
 
-    rc = http_open(hd, HTTP_REQ_GET, document, auth, flags, proxy, srv,
-		   headers );
+    rc = http_open( hd, HTTP_REQ_GET, document, flags, proxy );
     if( rc )
 	return rc;
 
@@ -262,7 +251,7 @@ http_close( HTTP_HD hd )
     iobuf_close( hd->fp_read );
     iobuf_close( hd->fp_write );
     release_parsed_uri( hd->uri );
-    xfree( hd->buffer );
+    m_free( hd->buffer );
     hd->initialized = 0;
 }
 
@@ -276,7 +265,7 @@ http_close( HTTP_HD hd )
 static int
 parse_uri( PARSED_URI *ret_uri, const char *uri )
 {
-   *ret_uri = xmalloc_clear( sizeof(**ret_uri) + strlen(uri) );
+   *ret_uri = m_alloc_clear( sizeof(**ret_uri) + strlen(uri) );
    strcpy( (*ret_uri)->buffer, uri );
    return do_parse_uri( *ret_uri, 0 );
 }
@@ -290,9 +279,9 @@ release_parsed_uri( PARSED_URI uri )
 
 	for( r = uri->query; r; r = r2 ) {
 	    r2 = r->next;
-	    xfree( r );
+	    m_free( r );
 	}
-	xfree( uri );
+	m_free( uri );
     }
 }
 
@@ -323,6 +312,8 @@ do_parse_uri( PARSED_URI uri, int only_local_part )
 	uri->scheme = p;
 	if(strcmp(uri->scheme,"http")==0)
 	  uri->port = 80;
+	else if(strcmp(uri->scheme,"hkp")==0)
+	  uri->port = 11371;
 	else
 	  return G10ERR_INVALID_URI; /* Unsupported scheme */
 
@@ -347,27 +338,16 @@ do_parse_uri( PARSED_URI uri, int only_local_part )
 	      }
 
 	    strlwr( p );
-
-	    /* Handle a host of [IP] so that [IP:V6]:port works */
-	    if( *p == '[' && (p3=strchr( p, ']' )) )
-	      {
-		*p3++ = '\0';
-		/* worst case, uri->host should have length 0, points to \0 */
-		uri->host = p + 1;
-		p = p3;
-	      }
-	    else
-	      uri->host = p;
-
-	    if( (p3=strchr( p, ':' )) )
-	      {
-		*p3++ = '\0';
+	    uri->host = p;
+	    if( (p3=strchr( p, ':' )) ) {
+		*p3++ = 0;
 		uri->port = atoi( p3 );
-	      }
+	    }
 
+	    uri->host = p;
 	    if( (n = remove_escapes( uri->host )) < 0 )
 		return G10ERR_BAD_URI;
-	    if( n != strlen( uri->host ) )
+	    if( n != strlen( p ) )
 		return G10ERR_BAD_URI; /* hostname with a Nul in it */
 	    p = p2 ? p2 : NULL;
 	}
@@ -498,7 +478,7 @@ parse_tuple( byte *string )
 	return NULL; /* bad URI */
     if( n != strlen( p ) )
        return NULL; /* name with a Nul in it */
-    tuple = xmalloc_clear( sizeof *tuple );
+    tuple = m_alloc_clear( sizeof *tuple );
     tuple->name = p;
     if( !p2 )  {
 	/* we have only the name, so we assume an empty value string */
@@ -507,7 +487,7 @@ parse_tuple( byte *string )
     }
     else { /* name and value */
 	if( (n = remove_escapes( p2 )) < 0 ) {
-	    xfree( tuple );
+	    m_free( tuple );
 	    return NULL; /* bad URI */
 	}
 	tuple->value = p2;
@@ -522,19 +502,18 @@ parse_tuple( byte *string )
  * Returns 0 if the request was successful
  */
 static int
-send_request( HTTP_HD hd, const char *auth, const char *proxy,
-	      struct http_srv *srv, STRLIST headers )
+send_request( HTTP_HD hd, const char *proxy )
 {
     const byte *server;
     byte *request, *p;
     ushort port;
     int rc;
-    char *proxy_authstr=NULL,*authstr=NULL;
+    char *auth=NULL;
 
     server = *hd->uri->host? hd->uri->host : "localhost";
     port   = hd->uri->port?  hd->uri->port : 80;
 
-    if(proxy && *proxy)
+    if(proxy)
       {
 	PARSED_URI uri;
 
@@ -546,40 +525,27 @@ send_request( HTTP_HD hd, const char *auth, const char *proxy,
 	    return G10ERR_NETWORK;
 	  }
 	hd->sock = connect_server( *uri->host? uri->host : "localhost",
-				   uri->port? uri->port : 80, 0, srv );
+				   uri->port? uri->port : 80, 0, NULL );
 	if(uri->auth)
 	  {
-	    char *x;
-	    remove_escapes(uri->auth);
-	    x=make_radix64_string(uri->auth,strlen(uri->auth));
-	    proxy_authstr=xmalloc(52+strlen(x));
-	    sprintf(proxy_authstr,"Proxy-Authorization: Basic %s\r\n",x);
-	    xfree(x);
+	    char *x=make_radix64_string(uri->auth,strlen(uri->auth));
+	    auth=m_alloc(50+strlen(x));
+	    sprintf(auth,"Proxy-Authorization: Basic %s\r\n",x);
+	    m_free(x);
 	  }
 
 	release_parsed_uri( uri );
       }
     else
-      hd->sock = connect_server( server, port, hd->flags, srv );
-
-    if(auth || hd->uri->auth)
       {
-	char *x,*tempauth=NULL;
-
-	if(auth)
+	hd->sock = connect_server( server, port, hd->flags, hd->uri->scheme );
+	if(hd->uri->auth)
 	  {
-	    tempauth=xstrdup(auth);
-	    remove_escapes(tempauth);
+	    char *x=make_radix64_string(hd->uri->auth,strlen(hd->uri->auth));
+	    auth=m_alloc(50+strlen(x));
+	    sprintf(auth,"Authorization: Basic %s\r\n",x);
+	    m_free(x);
 	  }
-	else if(hd->uri->auth)
-	  remove_escapes(hd->uri->auth);
-
-	x=make_radix64_string(tempauth?tempauth:hd->uri->auth,
-			      strlen(tempauth?tempauth:hd->uri->auth));
-	authstr=xmalloc(52+strlen(x));
-	sprintf(authstr,"Authorization: Basic %s\r\n",x);
-	xfree(x);
-	xfree(tempauth);
       }
 
     if( hd->sock == -1 )
@@ -587,52 +553,33 @@ send_request( HTTP_HD hd, const char *auth, const char *proxy,
 
     p = build_rel_path( hd->uri );
 
-    request=xmalloc(strlen(server)*2 + strlen(p)
-		    + (authstr?strlen(authstr):0)
-		    + (proxy_authstr?strlen(proxy_authstr):0) + 65);
-    if( proxy && *proxy )
-      sprintf( request, "%s http://%s:%hu%s%s HTTP/1.0\r\n%s%s",
+    request=m_alloc(strlen(server)*2 + strlen(p) + (auth?strlen(auth):0) + 65);
+    if( proxy )
+      sprintf( request, "%s http://%s:%hu%s%s HTTP/1.0\r\n%s",
 	       hd->req_type == HTTP_REQ_GET ? "GET" :
 	       hd->req_type == HTTP_REQ_HEAD? "HEAD":
 	       hd->req_type == HTTP_REQ_POST? "POST": "OOPS",
-	       server, port,  *p == '/'? "":"/", p,
-	       authstr?authstr:"",proxy_authstr?proxy_authstr:"" );
+	       server, port,  *p == '/'? "":"/", p, auth?auth:"" );
     else
       {
-	char portstr[35];
+	char portstr[15];
 
-	if(port == 80 || (srv && srv->used_server))
-	  *portstr = 0;
-	else
+	if(port!=80)
 	  sprintf(portstr,":%u",port);
 
 	sprintf( request, "%s %s%s HTTP/1.0\r\nHost: %s%s\r\n%s",
 		 hd->req_type == HTTP_REQ_GET ? "GET" :
 		 hd->req_type == HTTP_REQ_HEAD? "HEAD":
 		 hd->req_type == HTTP_REQ_POST? "POST": "OOPS",
-		 *p == '/'? "":"/", p, server, portstr,
-		 authstr?authstr:"");
+		 *p == '/'? "":"/", p, server, (port!=80)?portstr:"",
+		 auth?auth:"");
       }
 
-    xfree(p);
+    m_free(p);
 
     rc = write_server( hd->sock, request, strlen(request) );
-
-    if(rc==0)
-      for(;headers;headers=headers->next)
-	{
-	  rc = write_server( hd->sock, headers->d, strlen(headers->d) );
-	  if(rc)
-	    break;
-
-	  rc = write_server( hd->sock, "\r\n", 2 );
-	  if(rc)
-	    break;
-	}
-
-    xfree( request );
-    xfree(proxy_authstr);
-    xfree(authstr);
+    m_free( request );
+    m_free(auth);
 
     return rc;
 }
@@ -661,7 +608,7 @@ build_rel_path( PARSED_URI uri )
     n++;
 
     /* now  allocate and copy */
-    p = rel_path = xmalloc( n );
+    p = rel_path = m_alloc( n );
     n = insert_escapes( p, uri->path, "%;?&" );
     p += n;
     /* todo: add params */
@@ -744,7 +691,7 @@ parse_response( HTTP_HD hd )
 
 #ifdef TEST
 static int
-start_server(void)
+start_server()
 {
     struct sockaddr_in mya;
     struct sockaddr_in peer;
@@ -817,16 +764,10 @@ start_server(void)
 
 static int
 connect_server( const char *server, ushort port, unsigned int flags,
-		struct http_srv *srv )
+		const char *srvtag )
 {
-  int sock = -1;
-  int srvcount = 0;
-  int connected = 0;
-  int hostfound = 0;
-  int chosen = -1;
-  int fakesrv = 0;
-  struct srventry *srvlist = NULL;
-  int srvindex;
+  int sock=-1,srv,srvcount=0,connected=0,hostfound=0;
+  struct srventry *srvlist=NULL;
 
 #ifdef _WIN32
   unsigned long inaddr;
@@ -834,7 +775,7 @@ connect_server( const char *server, ushort port, unsigned int flags,
   init_sockets();
   /* Win32 gethostbyname doesn't handle IP addresses internally, so we
      try inet_addr first on that platform only. */
-  if((inaddr=inet_addr(server))!=INADDR_NONE)
+  if((inaddr=inet_addr(server))!=SOCKET_ERROR)
     {
       struct sockaddr_in addr;
 
@@ -846,9 +787,9 @@ connect_server( const char *server, ushort port, unsigned int flags,
 	  return -1;
 	}
 
-      addr.sin_family=AF_INET;
+      addr.sin_family=AF_INET; 
       addr.sin_port=htons(port);
-      memcpy(&addr.sin_addr,&inaddr,sizeof(inaddr));
+      memcpy(&addr.sin_addr,&inaddr,sizeof(inaddr));      
 
       if(connect(sock,(struct sockaddr *)&addr,sizeof(addr))==0)
 	return sock;
@@ -862,15 +803,15 @@ connect_server( const char *server, ushort port, unsigned int flags,
 
 #ifdef USE_DNS_SRV
   /* Do the SRV thing */
-  if(srv && srv->srvtag)
+  if(flags&HTTP_FLAG_TRY_SRV && srvtag)
     {
       /* We're using SRV, so append the tags */
-      if(1+strlen(srv->srvtag)+6+strlen(server)+1<=MAXDNAME)
+      if(1+strlen(srvtag)+6+strlen(server)+1<=MAXDNAME)
 	{
 	  char srvname[MAXDNAME];
 
 	  strcpy(srvname,"_");
-	  strcat(srvname,srv->srvtag);
+	  strcat(srvname,srvtag);
 	  strcat(srvname,"._tcp.");
 	  strcat(srvname,server);
 	  srvcount=getsrv(srvname,&srvlist);
@@ -882,27 +823,24 @@ connect_server( const char *server, ushort port, unsigned int flags,
     {
       /* Either we're not using SRV, or the SRV lookup failed.  Make
 	 up a fake SRV record. */
-      srvlist=calloc(1,sizeof(struct srventry));
-      if(!srvlist)
-	return -1;
+      srvlist=m_alloc_clear(sizeof(struct srventry));
       srvlist->port=port;
       strncpy(srvlist->target,server,MAXDNAME);
       srvlist->target[MAXDNAME-1]='\0';
-      srvcount = 1;
-      fakesrv = 1;
+      srvcount=1;
     }
 
 #ifdef HAVE_GETADDRINFO
 
-  for(srvindex=0;srvindex<srvcount;srvindex++)
+  for(srv=0;srv<srvcount;srv++)
     {
       struct addrinfo hints,*res,*ai;
       char portstr[6];
 
-      sprintf(portstr,"%u",srvlist[srvindex].port);
+      sprintf(portstr,"%u",srvlist[srv].port);
       memset(&hints,0,sizeof(hints));
       hints.ai_socktype=SOCK_STREAM;
-      if(getaddrinfo(srvlist[srvindex].target,portstr,&hints,&res)==0)
+      if(getaddrinfo(srvlist[srv].target,portstr,&hints,&res)==0)
 	hostfound=1;
       else
 	continue;
@@ -919,11 +857,8 @@ connect_server( const char *server, ushort port, unsigned int flags,
 	  if(connect(sock,ai->ai_addr,ai->ai_addrlen)==0)
 	    {
 	      connected=1;
-	      chosen = srvindex;
 	      break;
 	    }
-
-	  sock_close(sock);
 	}
 
       freeaddrinfo(res);
@@ -934,7 +869,7 @@ connect_server( const char *server, ushort port, unsigned int flags,
 
 #else /* !HAVE_GETADDRINFO */
 
-  for(srvindex=0; srvindex < srvcount; srvindex++)
+  for(srv=0;srv<srvcount;srv++)
     {
       int i=0;
       struct hostent *host=NULL;
@@ -942,11 +877,9 @@ connect_server( const char *server, ushort port, unsigned int flags,
 
       memset(&addr,0,sizeof(addr));
 
-      if((host=gethostbyname(srvlist[srvindex].target))==NULL)
+      if((host=gethostbyname(srvlist[srv].target))==NULL)
 	continue;
 
-      hostfound=1;
-
       if((sock=socket(host->h_addrtype,SOCK_STREAM,0))==-1)
 	{
 	  log_error("error creating socket: %s\n",strerror(errno));
@@ -956,18 +889,18 @@ connect_server( const char *server, ushort port, unsigned int flags,
       addr.sin_family=host->h_addrtype;
       if(addr.sin_family!=AF_INET)
 	{
-	  log_error("%s: unknown address family\n",srvlist[srvindex].target);
+	  log_error("%s: unknown address family\n",srvlist[srv].target);
 	  return -1;
 	}
 
-      addr.sin_port=htons(srvlist[srvindex].port);
+      addr.sin_port=htons(srvlist[srv].port);
 
       /* Try all A records until one responds. */
       while(host->h_addr_list[i])
 	{
 	  if(host->h_length!=4)
 	    {
-	      log_error("%s: illegal address length\n",srvlist[srvindex].target);
+	      log_error("%s: illegal address length\n",srvlist[srv].target);
 	      return -1;
 	    }
 
@@ -976,7 +909,6 @@ connect_server( const char *server, ushort port, unsigned int flags,
 	  if(connect(sock,(struct sockaddr *)&addr,sizeof(addr))==0)
 	    {
 	      connected=1;
-	      chosen = srvindex;
 	      break;
 	    }
 
@@ -985,22 +917,13 @@ connect_server( const char *server, ushort port, unsigned int flags,
 
       if(host->h_addr_list[i])
 	break;
-
-      sock_close(sock);
     }
 #endif /* !HAVE_GETADDRINFO */
 
-  if(!fakesrv && chosen > -1 && srv)
-    {
-      srv->used_server = strdup (srvlist[chosen].target);
-      srv->used_port = srvlist[chosen].port;
-    }
-
-  free(srvlist);
+  m_free(srvlist);
 
   if(!connected)
     {
-      int err=errno;
 #ifdef _WIN32
       if(hostfound)
 	log_error("%s: Unable to connect: ec=%d\n",server,(int)WSAGetLastError());
@@ -1008,13 +931,12 @@ connect_server( const char *server, ushort port, unsigned int flags,
 	log_error("%s: Host not found: ec=%d\n",server,(int)WSAGetLastError());
 #else
       if(hostfound)
-	log_error("%s: %s\n",server,strerror(err));
+	log_error("%s: %s\n",server,strerror(errno));
       else
 	log_error("%s: Host not found\n",server);
 #endif
       if(sock!=-1)
 	sock_close(sock);
-      errno=err;
       return -1;
     }
 
@@ -1029,7 +951,7 @@ write_server( int sock, const char *data, size_t length )
 
     nleft = length;
     while( nleft > 0 ) {
-#ifdef _WIN32
+#ifdef _WIN32  
         int nwritten;
 
         nwritten = send (sock, data, nleft, 0);
@@ -1110,7 +1032,7 @@ main(int argc, char **argv)
     }
     release_parsed_uri( uri ); uri = NULL;
 
-    rc = http_open_document( &hd, *argv, NULL, 0, NULL, NULL, NULL );
+    rc = http_open_document( &hd, *argv, 0, NULL );
     if( rc ) {
 	log_error("can't get `%s': %s\n", *argv, g10_errstr(rc));
 	return 1;
