@@ -1,12 +1,12 @@
 /* strgutil.c -  string utilities
  * Copyright (C) 1994, 1998, 1999, 2000, 2001,
- *               2003, 2004, 2005, 2009 Free Software Foundation, Inc.
+ *               2003 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,7 +15,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #include <config.h>
@@ -47,14 +48,6 @@
 #include "memory.h"
 #include "i18n.h"
 #include "dynload.h"
-#include "estream-printf.h"
-
-/* Our xasprintf replacements are expected to work with our memory
-   allocator.  Let's test for this here.  */
-#if !defined(_ESTREAM_PRINTF_MALLOC) || !defined(_ESTREAM_PRINTF_FREE)
-#error Please define _ESTREAM_PRINTF_MALLOC and _FREE
-#endif
-
 
 
 #ifndef USE_GNUPG_ICONV
@@ -174,7 +167,7 @@ free_strlist( STRLIST sl )
 
     for(; sl; sl = sl2 ) {
 	sl2 = sl->next;
-	xfree(sl);
+	m_free(sl);
     }
 }
 
@@ -184,7 +177,7 @@ add_to_strlist( STRLIST *list, const char *string )
 {
     STRLIST sl;
 
-    sl = xmalloc( sizeof *sl + strlen(string));
+    sl = m_alloc( sizeof *sl + strlen(string));
     sl->flags = 0;
     strcpy(sl->d, string);
     sl->next = *list;
@@ -206,7 +199,7 @@ add_to_strlist2( STRLIST *list, const char *string, int is_utf8 )
     else {
 	char *p = native_to_utf8( string );
 	sl = add_to_strlist( list, p );
-	xfree( p );
+	m_free( p );
     }
     return sl;
 }
@@ -216,7 +209,7 @@ append_to_strlist( STRLIST *list, const char *string )
 {
     STRLIST r, sl;
 
-    sl = xmalloc( sizeof *sl + strlen(string));
+    sl = m_alloc( sizeof *sl + strlen(string));
     sl->flags = 0;
     strcpy(sl->d, string);
     sl->next = NULL;
@@ -240,7 +233,7 @@ append_to_strlist2( STRLIST *list, const char *string, int is_utf8 )
     else {
 	char *p = native_to_utf8( string );
 	sl = append_to_strlist( list, p );
-	xfree( p );
+	m_free( p );
     }
     return sl;
 }
@@ -273,11 +266,11 @@ pop_strlist( STRLIST *list )
 
   if(sl)
     {
-      str=xmalloc(strlen(sl->d)+1);
+      str=m_alloc(strlen(sl->d)+1);
       strcpy(str,sl->d);
 
       *list=sl->next;
-      xfree(sl);
+      m_free(sl);
     }
 
   return str;
@@ -329,7 +322,7 @@ ascii_memistr( const char *buf, size_t buflen, const char *sub )
 
 /* Like strncpy() but copy at max N-1 bytes and append a '\0'.  With
  * N given as 0 nothing is copied at all. With DEST given as NULL
- * sufficient memory is allocated using xmalloc (note that xmalloc is
+ * sufficient memory is allocated using m_alloc (note that m_alloc is
  * guaranteed to succeed or to abort the process).  */
 char *
 mem2str( char *dest , const void *src , size_t n )
@@ -339,7 +332,7 @@ mem2str( char *dest , const void *src , size_t n )
 
     if( n ) {
 	if( !dest )
-	    dest = xmalloc( n ) ;
+	    dest = m_alloc( n ) ;
 	d = dest;
 	s = src ;
 	for(n--; n && *s; n-- )
@@ -460,33 +453,10 @@ static void
 handle_iconv_error (const char *to, const char *from, int use_fallback)
 {
   if (errno == EINVAL)
-    {
-      static int shown1, shown2;
-      int x;
-
-      if (to && !strcmp (to, "utf-8"))
-        {
-          x = shown1;
-          shown1 = 1;
-        }
-      else
-        {
-          x = shown2;
-          shown2 = 1;
-        }
-
-      if (!x)
-        log_info (_("conversion from `%s' to `%s' not available\n"),
-                  from, to);
-    }
+    log_info (_("conversion from `%s' to `%s' not available\n"),
+               from, to);
   else
-    {
-      static int shown;
-
-      if (!shown)
-        log_info (_("iconv_open failed: %s\n"), strerror (errno));
-      shown = 1;
-    }
+    log_info (_("iconv_open failed: %s\n"), strerror (errno));
 
   if (use_fallback)
     {
@@ -512,7 +482,6 @@ set_native_charset( const char *newset )
 #ifdef _WIN32
         static char codepage[30];
         unsigned int cpno;
-        const char *aliases;
 
         /* We are a console program thus we need to use the
            GetConsoleOutputCP function and not the the GetACP which
@@ -525,36 +494,14 @@ set_native_charset( const char *newset )
         if (!cpno)
           cpno = GetACP ();
         sprintf (codepage, "CP%u", cpno );
-        /* Resolve alias.  We use a long string string and not the
-           usual array to optimize if the code is taken to a DSO.
-           Taken from libiconv 1.9.2. */
-        newset = codepage;
-        for (aliases = ("CP936"   "\0" "GBK" "\0"
-                        "CP1361"  "\0" "JOHAB" "\0"
-                        "CP20127" "\0" "ASCII" "\0"
-                        "CP20866" "\0" "KOI8-R" "\0"
-                        "CP21866" "\0" "KOI8-RU" "\0"
-                        "CP28591" "\0" "ISO-8859-1" "\0"
-                        "CP28592" "\0" "ISO-8859-2" "\0"
-                        "CP28593" "\0" "ISO-8859-3" "\0"
-                        "CP28594" "\0" "ISO-8859-4" "\0"
-                        "CP28595" "\0" "ISO-8859-5" "\0"
-                        "CP28596" "\0" "ISO-8859-6" "\0"
-                        "CP28597" "\0" "ISO-8859-7" "\0"
-                        "CP28598" "\0" "ISO-8859-8" "\0"
-                        "CP28599" "\0" "ISO-8859-9" "\0"
-                        "CP28605" "\0" "ISO-8859-15" "\0"
-			"CP65001" "\0" "UTF-8" "\0");
-             *aliases;
-             aliases += strlen (aliases) + 1, aliases += strlen (aliases) + 1)
-          {
-            if (!strcmp (codepage, aliases) ||(*aliases == '*' && !aliases[1]))
-              {
-                newset = aliases + strlen (aliases) + 1;
-                break;
-              }
-          }
-
+        /* If it is the Windows name for Latin-1 we use the standard
+           name instead to avoid loading of iconv.dll.  Unfortunately
+           it is often CP850 and we don't have a custom translation
+           for it. */
+        if (!strcmp (codepage, "CP1252"))
+            newset = "iso-8859-1";
+        else
+            newset = codepage;
 #else
 #ifdef HAVE_LANGINFO_CODESET
         newset = nl_langinfo (CODESET);
@@ -683,7 +630,7 @@ native_to_utf8( const char *string )
   
   if (no_translation)
     { /* Already utf-8 encoded. */
-      buffer = xstrdup (string);
+      buffer = m_strdup (string);
     }
   else if( !active_charset && !use_iconv) /* Shortcut implementation
                                              for Latin-1.  */
@@ -694,7 +641,7 @@ native_to_utf8( const char *string )
           if( *s & 0x80 )
             length++;
 	}
-      buffer = xmalloc( length + 1 );
+      buffer = m_alloc( length + 1 );
       for(p=buffer, s=string; *s; s++ )
         {
           if( *s & 0x80 )
@@ -728,7 +675,7 @@ native_to_utf8( const char *string )
           if ((*s & 0x80))
             length += 5; /* We may need up to 6 bytes for the utf8 output. */
         }
-      buffer = xmalloc (length + 1);
+      buffer = m_alloc (length + 1);
 
       inptr = string;
       inbytes = strlen (string);
@@ -737,12 +684,8 @@ native_to_utf8( const char *string )
       if ( iconv (cd, (ICONV_CONST char **)&inptr, &inbytes,
                   &outptr, &outbytes) == (size_t)-1)
         {
-          static int shown;
-
-          if (!shown)
-            log_info (_("conversion from `%s' to `%s' failed: %s\n"),
-                      active_charset_name, "utf-8", strerror (errno));
-          shown = 1;
+          log_info (_("conversion from `%s' to `%s' failed: %s\n"),
+                       active_charset_name, "utf-8", strerror (errno));
           /* We don't do any conversion at all but use the strings as is. */
           strcpy (buffer, string);
         }
@@ -762,7 +705,7 @@ native_to_utf8( const char *string )
           if( *s & 0x80 )
             length += 2; /* We may need up to 3 bytes. */
         }
-      buffer = xmalloc( length + 1 );
+      buffer = m_alloc( length + 1 );
       for(p=buffer, s=string; *s; s++ ) {
         if( *s & 0x80 ) {
           ushort val = active_charset[ *s & 0x7f ];
@@ -984,7 +927,7 @@ utf8_to_native( const char *string, size_t length, int delim )
 	    }
 	}
 	if( !buffer ) { /* allocate the buffer after the first pass */
-	    buffer = p = xmalloc( n + 1 );
+	    buffer = p = m_alloc( n + 1 );
 	}
 #ifdef USE_GNUPG_ICONV
         else if(use_iconv) {
@@ -1000,7 +943,7 @@ utf8_to_native( const char *string, size_t length, int delim )
             if (cd == (iconv_t)-1)
                 {
                     handle_iconv_error (active_charset_name, "utf-8", 1);
-                    xfree (buffer);
+                    m_free (buffer);
                     return utf8_to_native (string, length, delim);
                 }
 
@@ -1012,20 +955,16 @@ utf8_to_native( const char *string, size_t length, int delim )
             outbytes = n * MB_LEN_MAX;
             if (outbytes / MB_LEN_MAX != n) 
                 BUG (); /* Actually an overflow. */
-            outbuf = outptr = xmalloc (outbytes);
+            outbuf = outptr = m_alloc (outbytes);
             if ( iconv (cd, (ICONV_CONST char **)&inptr, &inbytes,
                         &outptr, &outbytes) == (size_t)-1) {
-                static int shown;
-                
-                if (!shown)
-                  log_info (_("conversion from `%s' to `%s' failed: %s\n"),
-                            "utf-8", active_charset_name, strerror (errno));
-                shown = 1;
+                log_info (_("conversion from `%s' to `%s' failed: %s\n"),
+                           "utf-8", active_charset_name, strerror (errno));
                 /* Didn't worked out.  Temporary disable the use of
                  * iconv and fall back to our old code. */
-                xfree (buffer);
+                m_free (buffer);
                 buffer = NULL;
-                xfree (outbuf);
+                m_free (outbuf);
                 use_iconv = 0;
                 outbuf = utf8_to_native (string, length, delim);
                 use_iconv = 1;
@@ -1035,7 +974,7 @@ utf8_to_native( const char *string, size_t length, int delim )
                 /* We could realloc the buffer now but I doubt that it makes
                    much sense given that it will get freed anyway soon
                    after.  */
-                xfree (buffer);
+                m_free (buffer);
             }
             iconv_close (cd);
             return outbuf;
@@ -1048,111 +987,6 @@ utf8_to_native( const char *string, size_t length, int delim )
     }
 }
 
-/* This is similar to native_to_utf8, except it can take any input
-   (which may or may not be UTF8 encoded) and return something that is
-   (almost) definitely UTF8.  This code is mostly borrowed from
-   GPA. */
-
-char *
-string_to_utf8 (const char *string)
-{
-  const char *s;
-  
-  if (!string)
-    return NULL;
-  
-  /* Due to a bug in old and not so old PGP versions user IDs have
-     been copied verbatim into the key.  Thus many users with Umlauts
-     et al. in their name will see their names garbled.  Although this
-     is not an issue for me (;-)), I have a couple of friends with
-     Umlauts in their name, so let's try to make their life easier by
-     detecting invalid encodings and convert that to Latin-1.  We use
-     this even for X.509 because it may make things even better given
-     all the invalid encodings often found in X.509 certificates.  */
-  for (s = string; *s && !(*s & 0x80); s++)
-    ;
-  if (*s && ((s[1] & 0xc0) == 0x80) && ( ((*s & 0xe0) == 0xc0)
-                                         || ((*s & 0xf0) == 0xe0)
-                                         || ((*s & 0xf8) == 0xf0)
-                                         || ((*s & 0xfc) == 0xf8)
-                                         || ((*s & 0xfe) == 0xfc)) )
-    {  
-      /* Possible utf-8 character followed by continuation byte.
-         Although this might still be Latin-1 we better assume that it
-         is valid utf-8. */
-      return xstrdup (string);
-     }
-  else if (*s && !strchr (string, 0xc3))
-    {
-      size_t length=0;
-      char *buffer,*p;
-
-      /* No 0xC3 character in the string; assume that it is Latin-1.  */
-
-      for(s=string; *s; s++ ) 
-        {
-          length++;
-          if( *s & 0x80 )
-            length++;
-	}
-      buffer = xmalloc( length + 1 );
-      for(p=buffer, s=string; *s; s++ )
-        {
-          if( *s & 0x80 )
-            {
-              *p++ = 0xc0 | ((*s >> 6) & 3);
-              *p++ = 0x80 | ( *s & 0x3f );
-            }
-          else
-            *p++ = *s;
-        }
-      *p = 0;
-
-      return buffer;
-    }
-  else
-    {
-      /* Everything else is assumed to be UTF-8.  We do this even that
-         we know the encoding is not valid.  However as we only test
-         the first non-ascii character, valid encodings might
-         follow.  */
-      return xstrdup (string);
-    }
-}
-
-/* Same as asprintf but return an allocated buffer suitable to be
-   freed using xfree.  This function simply dies on memory failure,
-   thus no extra check is required. */
-char *
-xasprintf (const char *fmt, ...)
-{
-  va_list ap;
-  char *buf;
-
-  va_start (ap, fmt);
-  if (estream_vasprintf (&buf, fmt, ap) < 0)
-    log_fatal ("estream_asprintf failed: %s\n", strerror (errno));
-  va_end (ap);
-  return buf;
-}
-
-/* Same as above but return NULL on memory failure.  */
-char *
-xtryasprintf (const char *fmt, ...)
-{
-  int rc;
-  va_list ap;
-  char *buf;
-
-  va_start (ap, fmt);
-  rc = estream_vasprintf (&buf, fmt, ap);
-  va_end (ap);
-  if (rc < 0)
-    return NULL;
-  return buf;
-}
-
-
 /****************************************************
  ******** locale insensitive ctype functions ********
  ****************************************************/
@@ -1169,6 +1003,76 @@ ascii_islower (int c)
     return c >= 'a' && c <= 'z';
 }
 
+int 
+ascii_toupper (int c)
+{
+    if (c >= 'a' && c <= 'z')
+        c &= ~0x20;
+    return c;
+}
+
+int 
+ascii_tolower (int c)
+{
+    if (c >= 'A' && c <= 'Z')
+        c |= 0x20;
+    return c;
+}
+
+
+int
+ascii_strcasecmp (const char *a, const char *b)
+{
+  const unsigned char *p1 = (const unsigned char *)a;
+  const unsigned char *p2 = (const unsigned char *)b;
+  unsigned char c1, c2;
+
+  if (p1 == p2)
+    return 0;
+
+  do
+    {
+      c1 = ascii_tolower (*p1);
+      c2 = ascii_tolower (*p2);
+
+      if (c1 == '\0')
+	break;
+
+      ++p1;
+      ++p2;
+    }
+  while (c1 == c2);
+  
+  return c1 - c2;
+}
+
+int 
+ascii_strncasecmp (const char *a, const char *b, size_t n)
+{
+  const unsigned char *p1 = (const unsigned char *)a;
+  const unsigned char *p2 = (const unsigned char *)b;
+  unsigned char c1, c2;
+
+  if (p1 == p2 || !n )
+    return 0;
+
+  do
+    {
+      c1 = ascii_tolower (*p1);
+      c2 = ascii_tolower (*p2);
+
+      if ( !--n || c1 == '\0')
+	break;
+
+      ++p1;
+      ++p2;
+    }
+  while (c1 == c2);
+  
+  return c1 - c2;
+}
+
+
 int
 ascii_memcasecmp( const char *a, const char *b, size_t n )
 {
@@ -1199,6 +1103,56 @@ stpcpy(char *a,const char *b)
 }
 #endif
 
+
+#ifndef HAVE_STRSEP
+/* code taken from glibc-2.2.1/sysdeps/generic/strsep.c */
+char *
+strsep (char **stringp, const char *delim)
+{
+  char *begin, *end;
+
+  begin = *stringp;
+  if (begin == NULL)
+    return NULL;
+
+  /* A frequent case is when the delimiter string contains only one
+     character.  Here we don't need to call the expensive `strpbrk'
+     function and instead work using `strchr'.  */
+  if (delim[0] == '\0' || delim[1] == '\0')
+    {
+      char ch = delim[0];
+
+      if (ch == '\0')
+        end = NULL;
+      else
+        {
+          if (*begin == ch)
+            end = begin;
+          else if (*begin == '\0')
+            end = NULL;
+          else
+            end = strchr (begin + 1, ch);
+        }
+    }
+  else
+    /* Find the end of the token.  */
+    end = strpbrk (begin, delim);
+
+  if (end)
+    {
+      /* Terminate the token and set *STRINGP past NUL character.  */
+      *end++ = '\0';
+      *stringp = end;
+    }
+  else
+    /* No more delimiters; this is the last token.  */
+    *stringp = NULL;
+
+  return begin;
+}
+#endif /*HAVE_STRSEP*/
+
+
 #ifndef HAVE_STRLWR
 char *
 strlwr(char *s)
@@ -1240,6 +1194,111 @@ strncasecmp( const char *a, const char *b, size_t n )
 
 
 #ifdef _WIN32
+/* 
+ * Like vsprintf but provides a pointer to malloc'd storage, which
+ * must be freed by the caller (m_free).  Taken from libiberty as
+ * found in gcc-2.95.2 and a little bit modernized.
+ * FIXME: Write a new CRT for W32.
+ */
+int
+vasprintf (char **result, const char *format, va_list args)
+{
+  const char *p = format;
+  /* Add one to make sure that it is never zero, which might cause malloc
+     to return NULL.  */
+  int total_width = strlen (format) + 1;
+  va_list ap;
+
+  /* this is not really portable but works under Windows */
+  memcpy ( &ap, &args, sizeof (va_list));
+
+  while (*p != '\0')
+    {
+      if (*p++ == '%')
+	{
+	  while (strchr ("-+ #0", *p))
+	    ++p;
+	  if (*p == '*')
+	    {
+	      ++p;
+	      total_width += abs (va_arg (ap, int));
+	    }
+	  else
+            {
+              char *endp;  
+              total_width += strtoul (p, &endp, 10);
+              p = endp;
+            }
+	  if (*p == '.')
+	    {
+	      ++p;
+	      if (*p == '*')
+		{
+		  ++p;
+		  total_width += abs (va_arg (ap, int));
+		}
+	      else
+                {
+                  char *endp;
+                  total_width += strtoul (p, &endp, 10);
+                  p = endp;
+                }
+	    }
+	  while (strchr ("hlL", *p))
+	    ++p;
+	  /* Should be big enough for any format specifier except %s
+             and floats.  */
+	  total_width += 30;
+	  switch (*p)
+	    {
+	    case 'd':
+	    case 'i':
+	    case 'o':
+	    case 'u':
+	    case 'x':
+	    case 'X':
+	    case 'c':
+	      (void) va_arg (ap, int);
+	      break;
+	    case 'f':
+	    case 'e':
+	    case 'E':
+	    case 'g':
+	    case 'G':
+	      (void) va_arg (ap, double);
+	      /* Since an ieee double can have an exponent of 307, we'll
+		 make the buffer wide enough to cover the gross case. */
+	      total_width += 307;
+	    
+	    case 's':
+	      total_width += strlen (va_arg (ap, char *));
+	      break;
+	    case 'p':
+	    case 'n':
+	      (void) va_arg (ap, char *);
+	      break;
+	    }
+	}
+    }
+  *result = m_alloc (total_width);
+  if (*result != NULL)
+    return vsprintf (*result, format, args);
+  else
+    return 0;
+}
+
+int
+asprintf (char **buf, const char *fmt, ...)
+{
+  int status;
+  va_list ap;
+
+  va_start (ap, fmt);
+  status = vasprintf (buf, fmt, ap);
+  va_end (ap);
+  return status;  
+}
+
 const char *
 w32_strerror (int w32_errno)
 {
